/*************************************************************************
MGLTargaImage.h

Standard Targa loader.

Author   :   Dave Astle

Written for More OpenGL Game Programming
*************************************************************************/
#ifndef MGLTARGAIMAGE_H_INCLUDED
#define MGLTARGAIMAGE_H_INCLUDED

#ifdef WIN32
  #define WIN32_LEAN_AND_MEAN
  #define WIN32_EXTRA_LEAN
  #include <windows.h>
#endif
#include <GL\glee.h>
#include "MGLTypes.h"

namespace MGL
{
  class TargaImage
  {
  public:
    TargaImage();
    virtual ~TargaImage();

    // loading and unloading
    bool Load(const char* filename);
    void Release();

    // flips image vertically
    bool FlipVertical();

    unsigned short GetWidth() { return m_width; }
    unsigned short GetHeight() { return m_height; }

    unsigned int GetFormat() { return m_imageDataFormat; }
    unsigned int GetType() { return m_imageDataType; }

    // converts RGB format to RGBA format and vice versa
    bool ConvertRGBAToRGB();
    bool ConvertRGBToRGBA(unsigned char alphaValue, rgb_t *colorKey = NULL);

    // returns the current image data
    unsigned char* GetImage() { return m_pImageData; }

  private:
    // swap the red and blue components in the image data
    void SwapRedBlue();

    GLenum          m_imageDataType;
    GLenum          m_imageDataFormat;
    unsigned char   m_colorDepth;
    unsigned char  *m_pImageData;
    int             m_width;
    int             m_height;	
    unsigned int    m_imageSize;
  };
}
#endif // MGLTARGAIMAGE_H_INCLUDED