/*************************************************************************
fur_generator.cpp

Functions for generating fur polygons and textures

Author   :   Lukas Heise

Written for More OpenGL Game Programming
*************************************************************************/

#include "fur_generator.h"

void generateShellTexture(GLuint* ids, int num_shell_textures,int size)
{
	unsigned char* data;
	
	glGenTextures(num_shell_textures,ids);
	data = new unsigned char[size * size];
	ZeroMemory(data,size*size);

	for (int i = num_shell_textures - 1; i >= 0; i--)
	{
		for (int j = 0; j < 700; j++)
			data[rand() % (size * size)] = 255;

		glBindTexture(GL_TEXTURE_2D,ids[i]);
		glPixelStorei(GL_UNPACK_ALIGNMENT,1);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
		glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);

		gluBuild2DMipmaps(GL_TEXTURE_2D,GL_ALPHA4,size,size,GL_ALPHA,GL_UNSIGNED_BYTE,data);
	}

	delete [] data;
}

void buildFins(ASE_Scene* m, vector<GLuint>& material_list, vector<GLuint>& display_lists, float fur_length, float fur_scale, GLuint finTexture)
{
	Vector3 v;
	float z;

	float fin_length = fur_length * 2.5f;

	// skip the first object, we just want to generate fur for the "hair" object and not for the "face"
	for (int k = 1; k < m->objectCount; k++)
	{
		int matID = m->objs[k].material;

		GLuint dl = glGenLists(1);
		glNewList(dl,GL_COMPILE);

		glDisable(GL_CULL_FACE);
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
		glDepthMask(GL_FALSE);

		if (matID > -1)
		{
			glActiveTextureARB(GL_TEXTURE1_ARB);
			glEnable(GL_TEXTURE_2D);
			glBindTexture(GL_TEXTURE_2D,material_list[matID]);
		}

		glActiveTextureARB(GL_TEXTURE0_ARB);
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D,finTexture);

		glBegin(GL_QUADS);
			for (int i = 0; i < m->objs[k].mesh.faceCount; i++)
			{
				int vp0 = m->objs[k].mesh.faces[i].vertex[0];
				int vp1 = m->objs[k].mesh.faces[i].vertex[1];
				int vp2 = m->objs[k].mesh.faces[i].vertex[2];
				int tp0 = m->objs[k].mesh.faces[i].texture_coordinates[0];
				int tp1 = m->objs[k].mesh.faces[i].texture_coordinates[1];
				int tp2 = m->objs[k].mesh.faces[i].texture_coordinates[2];

				Vector3 p1(m->objs[k].mesh.vertices[vp0].x,m->objs[k].mesh.vertices[vp0].y,m->objs[k].mesh.vertices[vp0].z);
				Vector3 p2(m->objs[k].mesh.vertices[vp1].x,m->objs[k].mesh.vertices[vp1].y,m->objs[k].mesh.vertices[vp1].z);
				Vector3 p3(m->objs[k].mesh.vertices[vp2].x,m->objs[k].mesh.vertices[vp2].y,m->objs[k].mesh.vertices[vp2].z);

				Vector3 t1(m->objs[k].mesh.texture_coordinates[tp0].x,-m->objs[k].mesh.texture_coordinates[tp0].y,0.0f);
				Vector3 t2(m->objs[k].mesh.texture_coordinates[tp1].x,-m->objs[k].mesh.texture_coordinates[tp1].y,0.0f);
				Vector3 t3(m->objs[k].mesh.texture_coordinates[tp2].x,-m->objs[k].mesh.texture_coordinates[tp2].y,0.0f);

				Vector3 n1(m->objs[k].mesh.vertex_normals[vp0].x,m->objs[k].mesh.vertex_normals[vp0].y,m->objs[k].mesh.vertex_normals[vp0].z);
				Vector3 n2(m->objs[k].mesh.vertex_normals[vp1].x,m->objs[k].mesh.vertex_normals[vp1].y,m->objs[k].mesh.vertex_normals[vp1].z);
				Vector3 n3(m->objs[k].mesh.vertex_normals[vp2].x,m->objs[k].mesh.vertex_normals[vp2].y,m->objs[k].mesh.vertex_normals[vp2].z);

				v = p1 - p2;
				z = fur_scale * sqrt(v.DotProduct(v));
				glNormal3fv(n1.v);
				glColor4f(0.25f,0.25f,0.25f,0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f,1.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB,t1.x,t1.y);
				glVertex3fv(p1.v);
				glColor4f(1.0f,1.0f,1.0f,0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f,0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB,t1.x,t1.y);
				glVertex3f(p1.x + fin_length * fur_length * n1.x,
						   p1.y + fin_length * fur_length * n1.y,
						   p1.z + fin_length * fur_length * n1.z);
				glNormal3fv(n2.v);
				glColor4f(1.0f,1.0f,1.0f,0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB,z,0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB,t2.x,t2.y);
				glVertex3f(p2.x + fin_length * fur_length * n2.x,
						   p2.y + fin_length * fur_length * n2.y,
						   p2.z + fin_length * fur_length * n2.z);
				glColor4f(0.25f,0.25f,0.25f,0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB,z,1.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB,t2.x,t2.y);
				glVertex3fv(p2.v);


				v = p3 - p1;
				z = fur_scale * sqrt(v.DotProduct(v));
				glNormal3fv(n3.v);
				glColor4f(0.25f,0.25f,0.25f,0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f,1.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB,t3.x,t3.y);
				glVertex3fv(p3.v);
				glColor4f(1.0f,1.0f,1.0f,0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f,0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB,t3.x,t3.y);
				glVertex3f(p3.x + fin_length * fur_length * n3.x,
						   p3.y + fin_length * fur_length * n3.y,
						   p3.z + fin_length * fur_length * n3.z);
				glNormal3fv(n1.v);
				glColor4f(1.0f,1.0f,1.0f,0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB,z,0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB,t1.x,t1.y);
				glVertex3f(p1.x + fin_length * fur_length * n1.x,
						   p1.y + fin_length * fur_length * n1.y,
						   p1.z + fin_length * fur_length * n1.z);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB,z,1.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB,t1.x,t1.y);
				glColor4f(0.25f,0.25f,0.25f,0.5f);
				glVertex3fv(p1.v);

				
				v = p3 - p2;
				z = fur_scale * sqrt(v.DotProduct(v));
				glNormal3fv(n3.v);
				glColor4f(0.25f,0.25f,0.25f,0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f,1.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB,t3.x,t3.y);
				glVertex3fv(p3.v);
				glColor4f(1.0f,1.0f,1.0f,0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f,0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB,t3.x,t3.y);
				glVertex3f(p3.x + fin_length * fur_length * n3.x,
						   p3.y + fin_length * fur_length * n3.y,
						   p3.z + fin_length * fur_length * n3.z);
				glNormal3fv(n2.v);
				glColor4f(1.0f,1.0f,1.0f,0.5f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB,z,0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB,t2.x,t2.y);
				glVertex3f(p2.x + fin_length * fur_length * n2.x,
						   p2.y + fin_length * fur_length * n2.y,
						   p2.z + fin_length * fur_length * n2.z);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB,z,1.0f);
				glMultiTexCoord2fARB(GL_TEXTURE1_ARB,t2.x,t2.y);
				glColor4f(0.25f,0.25f,0.25f,0.5f);
				glVertex3fv(p2.v);
			}
		glEnd();

		if (matID > -1)
		{
			glActiveTextureARB(GL_TEXTURE1_ARB);
			glDisable(GL_TEXTURE_2D);
		}

		glActiveTextureARB(GL_TEXTURE0_ARB);
		glDisable(GL_TEXTURE_2D);
		glDisable(GL_BLEND);
		glEnable(GL_CULL_FACE);
		glDepthMask(GL_TRUE);

		glEndList();

		display_lists.push_back(dl);
	}
}

void buildShells(ASE_Scene* m, vector<GLuint>& material_list, vector<GLuint>& display_lists, GLuint* shell_textures, int num_shell_textures, float fur_length, float fur_scale)
{
	Vector3 v1,v2;
	float l1,l2;

	// skip the first object, we just want to generate fur for the "hair" object and not for the "face"
	for (int k = 1; k < m->objectCount; k++)
	{
		int matID = m->objs[k].material;

		GLuint dl = glGenLists(1);
		glNewList(dl,GL_COMPILE);

		glDepthMask(GL_FALSE);
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

		if (matID > -1)
		{
			glActiveTextureARB(GL_TEXTURE1_ARB);
			glEnable(GL_TEXTURE_2D);
			glBindTexture(GL_TEXTURE_2D,material_list[matID]);
		}

		glActiveTextureARB(GL_TEXTURE0_ARB);
		glEnable(GL_TEXTURE_2D);

		for (int s = 0; s < num_shell_textures; s++)
		{
			float z = (float)s * (fur_length / (float)num_shell_textures);
			float col = 0.25f + 0.75f * ((float)s / (float)num_shell_textures);

			glBindTexture(GL_TEXTURE_2D,shell_textures[s]);
			glColor4f(col,col,col,1.0f);

			glBegin(GL_TRIANGLES);
				for (int i = 0; i < m->objs[k].mesh.faceCount; i++)
				{
					int vp0 = m->objs[k].mesh.faces[i].vertex[0];
					int vp1 = m->objs[k].mesh.faces[i].vertex[1];
					int vp2 = m->objs[k].mesh.faces[i].vertex[2];
					int tp0 = m->objs[k].mesh.faces[i].texture_coordinates[0];
					int tp1 = m->objs[k].mesh.faces[i].texture_coordinates[1];
					int tp2 = m->objs[k].mesh.faces[i].texture_coordinates[2];

					Vector3 p1(m->objs[k].mesh.vertices[vp0].x,m->objs[k].mesh.vertices[vp0].y,m->objs[k].mesh.vertices[vp0].z);
					Vector3 p2(m->objs[k].mesh.vertices[vp1].x,m->objs[k].mesh.vertices[vp1].y,m->objs[k].mesh.vertices[vp1].z);
					Vector3 p3(m->objs[k].mesh.vertices[vp2].x,m->objs[k].mesh.vertices[vp2].y,m->objs[k].mesh.vertices[vp2].z);

					Vector3 t1(m->objs[k].mesh.texture_coordinates[tp0].x,-m->objs[k].mesh.texture_coordinates[tp0].y,0.0f);
					Vector3 t2(m->objs[k].mesh.texture_coordinates[tp1].x,-m->objs[k].mesh.texture_coordinates[tp1].y,0.0f);
					Vector3 t3(m->objs[k].mesh.texture_coordinates[tp2].x,-m->objs[k].mesh.texture_coordinates[tp2].y,0.0f);

					Vector3 n1(m->objs[k].mesh.vertex_normals[vp0].x,m->objs[k].mesh.vertex_normals[vp0].y,m->objs[k].mesh.vertex_normals[vp0].z);
					Vector3 n2(m->objs[k].mesh.vertex_normals[vp1].x,m->objs[k].mesh.vertex_normals[vp1].y,m->objs[k].mesh.vertex_normals[vp1].z);
					Vector3 n3(m->objs[k].mesh.vertex_normals[vp2].x,m->objs[k].mesh.vertex_normals[vp2].y,m->objs[k].mesh.vertex_normals[vp2].z);


					glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f,0.0f);
					glMultiTexCoord2fARB(GL_TEXTURE1_ARB,t1.x,t1.y);
					glNormal3fv(n1.v);
					glVertex3f(p1.x + z * n1.x, p1.y + z * n1.y, p1.z + z * n1.z);

					v1 = p2 - p1;
					l1 = sqrt(v1.DotProduct(v1));
					v1.Normalize();
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB,fur_scale * l1,0.0f);
					glMultiTexCoord2fARB(GL_TEXTURE1_ARB,t2.x,t2.y);
					glNormal3fv(n2.v);
					glVertex3f(p2.x + z * n2.x, p2.y + z * n2.y, p2.z + z * n2.z);

					v2 = p3 - p1;
					l2 = sqrt(v2.DotProduct(v2));
					v2.Normalize();
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB,fur_scale * l2 * v1.DotProduct(v2),fur_scale * l2 * sin(acos(v1.DotProduct(v2))));
					glMultiTexCoord2fARB(GL_TEXTURE1_ARB,t3.x,t3.y);
					glNormal3fv(n3.v);
					glVertex3f(p3.x + z * n3.x,p3.y + z * n3.y,p3.z + z * n3.z);
				}
			glEnd();
		}

		if (matID > -1)
		{
			glActiveTextureARB(GL_TEXTURE1_ARB);
			glDisable(GL_TEXTURE_2D);
		}

		glActiveTextureARB(GL_TEXTURE0_ARB);
		glDisable(GL_TEXTURE_2D);
		glDisable(GL_BLEND);
		glDepthMask(GL_TRUE);

		glEndList();

		display_lists.push_back(dl);
	}
}
