/*************************************************************************
MGLSDL.h

Setup code for a basic SDL application.

Author   :   Dave Astle

Written for More OpenGL Game Programming
*************************************************************************/

#include "sdl.h"
#include "MGLApp.h"
#include <stdlib.h>

void SetGLAttributes()
{
  SDL_GL_SetAttribute(SDL_GL_BUFFER_SIZE, MGL_COLOR_BITS);
  SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, MGL_DEPTH_BITS);
  SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE, MGL_STENCIL_BITS);
  SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
//  SDL_GL_SetAttribute(SDL_GL_MULTISAMPLEBUFFERS, 1);
//  SDL_GL_SetAttribute(SDL_GL_MULTISAMPLESAMPLES, 4);
}


// returns false if a quit event is received
bool HandleEvent(MGLApp* pApp, const SDL_Event& event)
{
  switch (event.type)
  {
  case SDL_QUIT:
    return false;
  case SDL_KEYDOWN:
    {
      switch (event.key.keysym.sym)
      {
      case SDLK_ESCAPE:
        return false;
      }
    } break;

  case SDL_VIDEORESIZE:
    pApp->SetupProjection(event.resize.w, event.resize.h);
    break;
  }

  return true;
}


int main(int argc, char* argv[])
{
  SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER);
  atexit(SDL_Quit);

  SDL_WM_SetCaption(MGL_APP_NAME, MGL_APP_NAME);

  SetGLAttributes();

  Uint32 displayFlags = SDL_OPENGL;

  if (MGL_FORCE_FULLSCREEN)
  {
    displayFlags |= SDL_FULLSCREEN;
  }
  else
  {
    displayFlags |= SDL_RESIZABLE;
  }

  SDL_Surface* pDisplay = SDL_SetVideoMode(MGL_WINDOW_WIDTH, MGL_WINDOW_HEIGHT, MGL_COLOR_BITS, displayFlags);

  if (pDisplay == NULL)
  {
    fprintf(stderr, "Couldn't set GL mode: %s\n", SDL_GetError());
    exit(0);
  }

  MGLApp* pApp = new MGLApp();
  pApp->Init();
  pApp->SetupProjection(MGL_WINDOW_WIDTH, MGL_WINDOW_HEIGHT);

  Uint32 lastTime = SDL_GetTicks();
  
  for (;;)
  {
    SDL_Event event;
    Uint32 currTime;


    if (SDL_PollEvent(&event) == 0)
    {
      currTime = SDL_GetTicks();
      pApp->Update((currTime - lastTime)/1000.0f);
      lastTime = currTime;
      pApp->Render();
      SDL_GL_SwapBuffers();
    }
    else if (!HandleEvent(pApp, event))
      break;
  }
  pApp->Shutdown();
  delete pApp;

  return 0;
}