/*************************************************************************
MGLApp.cpp

Main application class.

Author   :   Dave Astle

Written for More OpenGL Game Programming
*************************************************************************/

#ifdef WIN32
	#define WIN32_LEAN_AND_MEAN
	#define WIN32_EXTRA_LEAN
	#include <windows.h>
#endif

#include "MGLApp.h"

const float MOUSE_SENSITIVITY = 0.01f;

MGLApp::MGLApp()
{
}

MGLApp::~MGLApp()
{
}

bool MGLApp::Init()
{	
  if (!GLeeInit())
    return false;

  if (!GLEE_ARB_occlusion_query)
    return false;

  m_skybox.Initialize(10.0f);
  m_skybox.LoadTextures("skybox\\barrendesertup.tga", "skybox\\barrendesertdn.tga",
                        "skybox\\barrendesertft.tga", "skybox\\barrendesertbk.tga",
                        "skybox\\barrendesertlf.tga", "skybox\\barrendesertrt.tga");
  glEnable(GL_TEXTURE_2D);
  m_brightness = 0.0f;

  m_camera.SetPosition(0.0, 0.0, 0.0);
  m_camera.YawAndPitch(PI, 0);

  glGenQueries(1, &m_query);

  glEnable(GL_DEPTH_TEST);

  return true;
}

bool MGLApp::Shutdown()
{
  glDeleteQueries(1, &m_query);
  m_skybox.Release();
	return true;
}

void MGLApp::SetupProjection(int width, int height)
{

	// avoid div by zero
	if( height == 0 )
		height = 1;					

  m_windowWidth = width;
  m_windowHeight = height;

	// set up GL viewport
	glViewport( 0, 0, width, height );

	// set up a perspective projection
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 52.0f, (GLfloat)width/(GLfloat)height, 1.0f, 1000.0f );
}

void MGLApp::Update( double dt )
{
}

void MGLApp::Render()
{
  glClear( GL_DEPTH_BUFFER_BIT );

  // set up the camera
  glMatrixMode( GL_MODELVIEW );
  glLoadIdentity();
  m_camera.SetView();

  m_brightness = DrawSun();

  glColor3f(m_brightness, m_brightness, m_brightness);
  MGL::Vector3 camPos = m_camera.GetPosition();
  m_skybox.Render(camPos.x, camPos.y, camPos.z);
}

void MGLApp::ProcessMouseInput(int deltaX, int deltaY)
{
  m_camera.YawAndPitch(deltaY * MOUSE_SENSITIVITY, deltaX * MOUSE_SENSITIVITY);
}


GLfloat MGLApp::DrawSun()
{
  glDisable(GL_TEXTURE_2D);

  glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
  glColor3f(1.0, 1.0, 1.0);
  glBeginQuery(GL_SAMPLES_PASSED, m_query);
  glBegin(GL_QUADS);
    glVertex3f(-1.5f, 0.2f, 9.9f);
    glVertex3f(-1.5f, 3.3f, 9.9f);
    glVertex3f(1.5f, 3.3f, 9.9f);
    glVertex3f(1.5f, 0.2f, 9.9f);
  glEnd();
  glEndQuery(GL_SAMPLES_PASSED);
  glEnable(GL_TEXTURE_2D);
  glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);

  GLint result;
  glGetQueryObjectiv(m_query, GL_QUERY_RESULT, &result);

  return (5.0f * (float)result)/(m_windowWidth * m_windowHeight);
}