/*************************************************************************
MGLApp.h

Main application class.

Author   :   Dave Astle

Written for More OpenGL Game Programming
*************************************************************************/

#ifndef MGLAPP_H_INCLUDED
#define MGLAPP_H_INCLUDED

#ifdef WIN32
  #define WIN32_LEAN_AND_MEAN
  #define WIN32_EXTRA_LEAN
  #include <windows.h>
#endif

#include "glee.h"
#include <gl/glu.h>

#include "MGLVector.h"

// values used to set up the window and rendering context
const char MGL_APP_NAME[]	= "MOGLGP - Chapter 1 - Marbles with VBOs";

const int MGL_WINDOW_WIDTH = 1024;
const int MGL_WINDOW_HEIGHT = 768;

const int MGL_COLOR_BITS = 32;
const int MGL_DEPTH_BITS = 24;
const int MGL_STENCIL_BITS = 8;

const bool MGL_FORCE_FULLSCREEN = false;

const int NUM_MARBLES = 120;

struct Marble
{
  MGL::Vector3 position;
  MGL::Vector3 color;
  MGL::Vector3 direction;
};


class MGLApp
{
public:
	MGLApp();
	virtual ~MGLApp();

	bool Init();
	bool Shutdown();
	void SetupProjection( int width, int height );
	void Update( float dt );
	void Render();

  bool ToggleVertexArrays()
  {
    m_useVertexArrays = !m_useVertexArrays;
    return m_useVertexArrays;
  }

  bool ToggleBuffers()
  {
    m_useBuffers = !m_useBuffers;
    return m_useBuffers;
  }

  void RotateRight() { m_angle-=1.0f; }
  void RotateLeft() { m_angle+=1.0f; }

protected:
  void InitializeMarbles();
  void MoveMarbles(float dt);
  bool GenerateSphere(float radius, int resolution);
  void DrawBox();
  void DrawMarbles(bool invert);
  void DrawSphere();
  void DrawFloor();
  void DrawBoxBottom();
  void DrawReflection();

private:
  int m_windowWidth;
  int m_windowHeight;

  int m_numStrips;
  int m_vertsPerStrip;


  GLuint m_buffer;
  GLuint m_indexBuffer;

  GLuint m_numVertices;

  bool m_useVertexArrays;
  bool m_useBuffers;

  GLuint m_marbleTexture;
  GLuint m_brickTexture;

  GLfloat m_angle;

  GLuint *m_indexArray;
  MGL::Vector3  *m_positions;
  MGL::Vector3  *m_texCoords;

  Marble m_marbles[NUM_MARBLES];
};

#endif // MGLAPP_H_INCLUDED