/*************************************************************************
MGLApp.cpp

Main application class.

Author   :   Dave Astle

Written for More OpenGL Game Programming
*************************************************************************/

#ifdef WIN32
	#define WIN32_LEAN_AND_MEAN
	#define WIN32_EXTRA_LEAN
	#include <windows.h>
#endif

#include <cmath>
#include <string>
#include "MGLApp.h"
#include "../_common/Dolphin.h"
#include "../_common/LoadProgram.h"

const GLfloat waterColor[] = {0.0f, 0.3f, 0.6f, 1.0f};
bool useVertexProgram = true;
GLuint vp;
double time = 0.0;

//TODO: Add keyboard handling - press a key to toggle vertex program

MGLApp::MGLApp()
{
}

MGLApp::~MGLApp()
{
}

bool MGLApp::Init()
{	
	if (!GLeeInit())
		return false;

	//Ensure that ARB_vertex_program is supported
	if(!GLEE_ARB_vertex_program)
		return false;

	//Load the vertex program
	vp = LoadProgram(GL_VERTEX_PROGRAM_ARB, "dolphin1.vp");

	if(vp == 0)
		return false;

	//Set OpenGL states
	glClearColor(waterColor[0], waterColor[1], waterColor[2], waterColor[3]);
	glShadeModel(GL_SMOOTH);
	glEnable(GL_DEPTH_TEST);
	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);

	return true;
}

bool MGLApp::Shutdown()
{
	return true;
}

void MGLApp::SetupProjection(int width, int height)
{
	// avoid div by zero
	if( height == 0 )
		height = 1;					

	// set up GL viewport
	glViewport( 0, 0, width, height );

	// set up a perspective projection
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0f, (GLfloat)width/(GLfloat)height, 1.0f, 1000.0f );
}

void MGLApp::Update( double dt )
{
	//Update time
	time += dt;
}

void MGLApp::Render()
{
    //Clear buffers
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	//Apply viewing transformation
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslatef(0.0f, 0.0f, -1.5f);

	glColor4f(0.8f, 0.8f, 0.8f, 1.0f);

	//If using vertex program
	if(useVertexProgram)
	{
		//Bind program, send parameter and enable
		glBindProgramARB(GL_VERTEX_PROGRAM_ARB, vp);
		double t = (sin(2.5*time) + 1.0f) / 2;
		glProgramLocalParameter4dARB(GL_VERTEX_PROGRAM_ARB, 0, t, 1.0-t, 0.0, 0.0);
		glEnable(GL_VERTEX_PROGRAM_ARB);
		
		//Set up texture coordinate arrays
		glTexCoordPointer(3, GL_FLOAT, 0, vertices[1]);
		glEnable(GL_TEXTURE_COORD_ARRAY);
		
		glClientActiveTextureARB(GL_TEXTURE1_ARB);
		glTexCoordPointer(3, GL_FLOAT, 0, vertices[2]);
		glEnable(GL_TEXTURE_COORD_ARRAY);
		glClientActiveTextureARB(GL_TEXTURE0_ARB);
	}

	//Set up vertex array
	glVertexPointer(3, GL_FLOAT, 0, vertices[0]);
	glEnableClientState(GL_VERTEX_ARRAY);

	//Draw the dolphin
	glDrawElements(GL_TRIANGLES, numIndices, GL_UNSIGNED_SHORT, indices);
	
	//Restore states
	glDisableClientState(GL_VERTEX_ARRAY);

	if(useVertexProgram)
	{
		glDisable(GL_VERTEX_PROGRAM_ARB);
		glDisable(GL_TEXTURE_COORD_ARRAY);
		
		glClientActiveTextureARB(GL_TEXTURE1_ARB);
		glDisable(GL_TEXTURE_COORD_ARRAY);
		glClientActiveTextureARB(GL_TEXTURE0_ARB);
	}
}
