/*************************************************************************
MGLApp.cpp

Main application class.

Author   :   Dave Astle

Written for More OpenGL Game Programming
*************************************************************************/

#ifdef WIN32
	#define WIN32_LEAN_AND_MEAN
	#define WIN32_EXTRA_LEAN
	#include <windows.h>
#endif

#include <cmath>
#include <string>
#include "MGLApp.h"
#include "../_common/Dolphin.h"
#include "../_common/LoadProgram.h"

const GLfloat eyeSpaceLightPos[] = {2.0f, 2.0f,-3.0f, 1.0f};
const GLfloat waterColor[] = {0.0f, 0.3f, 0.6f, 1.0f};
GLuint vp;
double time = 0.0;

MGLApp::MGLApp()
{
}

MGLApp::~MGLApp()
{
}

bool MGLApp::Init()
{	
	//Ensure that the necessary extensions are supported
	if(	!GLEE_ARB_vertex_program || !GLEE_EXT_fog_coord)
		return false;

	//Load the vertex program
	vp = LoadProgram(GL_VERTEX_PROGRAM_ARB, "dolphin2.vp");

	if(vp == 0)
		return false;

	//Set OpenGL states
	glClearColor(waterColor[0], waterColor[1], waterColor[2], waterColor[3]);

	glShadeModel(GL_SMOOTH);

	glEnable(GL_DEPTH_TEST);

	glEnable(GL_CULL_FACE);

	glFogi(GL_FOG_COORDINATE_SOURCE_EXT, GL_FOG_COORDINATE_EXT);
	glFogfv(GL_FOG_COLOR, waterColor);
	glFogi(GL_FOG_MODE, GL_LINEAR);
	glFogf(GL_FOG_START, 2.0f);
	glFogf(GL_FOG_END, 8.0f);
	glEnable(GL_FOG);

	return true;
}

bool MGLApp::Shutdown()
{
	return true;
}

void MGLApp::SetupProjection(int width, int height)
{
	// avoid div by zero
	if( height == 0 )
		height = 1;					

	// set up GL viewport
	glViewport( 0, 0, width, height );

	// set up a perspective projection
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0f, (GLfloat)width/(GLfloat)height, 1.0f, 1000.0f );
}

void MGLApp::Update( double dt )
{
	//Update time
	time += dt;
}

void MGLApp::Render()
{
	//Clear buffers
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	//Set movelview transform
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslatef(0.0f, 0.0f, -5.0f);
	glRotatef(-20*time, 0.0f, 1.0f, 0.0f);
	glTranslatef(0.0f, 0.0f, -2.0f);

	glColor4f(0.8f, 0.8f, 0.8f, 1.0f);

	//Bind program, set parameters and enable
	glBindProgramARB(GL_VERTEX_PROGRAM_ARB, vp);
	double t = (sin(2.5*time) + 1.0f) / 2;
	glProgramLocalParameter4dARB(GL_VERTEX_PROGRAM_ARB, 0, t, 1.0-t, 0.0, 0.0);
	glProgramLocalParameter4fvARB(GL_VERTEX_PROGRAM_ARB, 1, eyeSpaceLightPos);
	glEnable(GL_VERTEX_PROGRAM_ARB);
		
	//Set up vertex arrays
	glVertexPointer(3, GL_FLOAT, 0, vertices[0]);
	glEnableClientState(GL_VERTEX_ARRAY);
	glVertexAttribPointerARB(6, 3, GL_FLOAT, GL_FALSE, 0, vertices[1]);
	glEnableVertexAttribArrayARB(6);
	glVertexAttribPointerARB(7, 3, GL_FLOAT, GL_FALSE, 0, vertices[2]);
	glEnableVertexAttribArrayARB(7);

	glNormalPointer(GL_FLOAT, 0, normals[0]);
	glEnableClientState(GL_NORMAL_ARRAY);
	glVertexAttribPointerARB(8, 3, GL_FLOAT, GL_FALSE, 0, normals[2]);
	glEnableVertexAttribArrayARB(8);
	glVertexAttribPointerARB(9, 3, GL_FLOAT, GL_FALSE, 0, normals[2]);
	glEnableVertexAttribArrayARB(9);
	
	//Draw the dolphin
	glDrawElements(GL_TRIANGLES, numIndices, GL_UNSIGNED_SHORT, indices);
	
	//Restore states
	glDisable(GL_VERTEX_PROGRAM_ARB);

	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableVertexAttribArrayARB(6);
	glDisableVertexAttribArrayARB(7);

	glDisableClientState(GL_NORMAL_ARRAY);
	glDisableVertexAttribArrayARB(8);
	glDisableVertexAttribArrayARB(9);
}
