!!ARBfp1.0

ATTRIB iObjPos	= fragment.texcoord[0];
ATTRIB iEyeNorm	= fragment.texcoord[1];

PARAM itModelview[] = { state.matrix.modelview.invtrans};
PARAM modelview[] = { state.matrix.modelview};
PARAM plane0	= program.local[0];
PARAM plane1	= program.local[1];
PARAM plane2	= program.local[2];
PARAM plane3	= program.local[3];
PARAM plane4	= program.local[4];
PARAM eyeLightPos = program.local[5];
PARAM purple	= {0.8, 0.0, 0.6, 0.0};
PARAM yellow	= {1.0, 1.0, 0.0, 0.0};

TEMP distPlane1, distPlane2, classPlane1, classPlane2, inStar, materialColor;
TEMP eyePosition, eyeLight, normEyeLight, normNormal, lighting, reflection, eyeView;

OUTPUT oCol		= result.color;

# Calculate the distance of the fragment from each plane
DP4 distPlane1.x, iObjPos, plane0;
DP4 distPlane1.y, iObjPos, plane1;
DP4 distPlane1.z, iObjPos, plane2;
DP4 distPlane1.w, iObjPos, plane3;
DP4 distPlane2.x, iObjPos, plane4;

#Classify the fragment by each plane
SGE classPlane1, distPlane1, 0.0;
SGE classPlane2, distPlane2, 0.0;

#Calculate whether the fragment is in the star
DP4 inStar, classPlane1, 1.0;
ADD inStar, inStar, classPlane2.x;
SUB_SAT inStar, inStar, 3.0;

#If so, the base color is yellow, otherwise purple
LRP materialColor, inStar, yellow, purple;

# Calculate the eye-space position
DP4 eyePosition.x, modelview[0], iObjPos;
DP4 eyePosition.y, modelview[1], iObjPos;
DP4 eyePosition.z, modelview[2], iObjPos;
DP4 eyePosition.w, modelview[3], iObjPos;

# Calculate the eye-space light vector
SUB eyeLight, eyeLightPos, eyePosition;

# Calculate a normalized light vector
DP3 normEyeLight.w, eyeLight, eyeLight;
RSQ normEyeLight.w, normEyeLight.w;
MUL normEyeLight.xyz, normEyeLight.w, eyeLight;

# Normalize the normal
DP3 normNormal.w, iEyeNorm, iEyeNorm;
RSQ normNormal.w, normNormal.w;
MUL normNormal.xyz, normNormal.w, iEyeNorm;

# Calculate N.L
DP3 lighting.x, normNormal, normEyeLight;

# Calculate the reflection vector, R = 2(N.L)N-L
MUL reflection, lighting.x, 2.0;
MUL reflection, reflection, normNormal;
SUB reflection, reflection, normEyeLight;

# Calculate the normalised eye-space view vector
DP3 eyeView.w, eyePosition, eyePosition;
RSQ eyeView.w, eyeView.w;
MUL eyeView.xyz, eyeView.w, -eyePosition;

# Calculate R.V
DP3 lighting.y, reflection, eyeView;

# Specular exponent 32
MOV lighting.w, 32.0;

# Calculate lighting coefficients
LIT lighting, lighting;

# Add the ambient brightness to the diffuse
ADD lighting.y, lighting.y, 0.1;

# Calculate the final color
MAD oCol, materialColor, lighting.y, lighting.z;

END