#include <vector>
#include "MGLVector.h"
#include "Torus.h"

Torus::Torus(int precision) :	vertices((precision+1)*(precision+1)),
								indices(2*precision*precision*3)
{
	int ring, i;
	//calculate the first ring - inner radius 4, outer radius 1.5
	for(i=0; i<precision+1; i++)
	{
		MGL::Vector3 v1(1.5f, 0.0f, 0.0f);
		MGL::Vector3 v2(4.0f, 0.0f, 0.0f);
		vertices[i].position=v1.Rotate(i*2*PI/precision, MGL::Vector3(0.0f, 0.0f, 1.0f)) + v2;
		
		vertices[i].s=0.0f;
		vertices[i].t=(float)i*2/precision;

		vertices[i].tangent = MGL::Vector3(0.0f, 0.0f, -1.0f);
		MGL::Vector3 v3(0.0f, -1.0f, 0.0f);
		vertices[i].binormal = v3.Rotate(i*2*PI/precision, MGL::Vector3(0.0f, 0.0f, 1.0f));
		vertices[i].normal=vertices[i].binormal.CrossProduct(vertices[i].tangent);
	}

	//rotate this to get other rings
	for(ring=1; ring<precision+1; ring++)
	{
		for(i=0; i<precision+1; i++)
		{
			vertices[ring*(precision+1)+i].position=vertices[i].position.Rotate(ring*2*PI/precision, MGL::Vector3(0.0f, 1.0f, 0.0f));
			
			vertices[ring*(precision+1)+i].s=8.0f*ring/precision;
			vertices[ring*(precision+1)+i].t=vertices[i].t;

			vertices[ring*(precision+1)+i].tangent=vertices[i].tangent.Rotate(ring*2*PI/precision, MGL::Vector3(0.0f, 1.0f, 0.0f));
			vertices[ring*(precision+1)+i].binormal=vertices[i].binormal.Rotate(ring*2*PI/precision, MGL::Vector3(0.0f, 1.0f, 0.0f));
			vertices[ring*(precision+1)+i].normal=vertices[i].normal.Rotate(ring*2*PI/precision, MGL::Vector3(0.0f, 1.0f, 0.0f));
		}
	}

	//calculate the indices
	for(ring=0; ring<precision; ring++)
	{
		for(i=0; i<precision; i++)
		{
			indices[((ring*precision+i)*2)*3+0]=ring*(precision+1)+i;
			indices[((ring*precision+i)*2)*3+1]=(ring+1)*(precision+1)+i;
			indices[((ring*precision+i)*2)*3+2]=ring*(precision+1)+i+1;
			indices[((ring*precision+i)*2+1)*3+0]=ring*(precision+1)+i+1;
			indices[((ring*precision+i)*2+1)*3+1]=(ring+1)*(precision+1)+i;
			indices[((ring*precision+i)*2+1)*3+2]=(ring+1)*(precision+1)+i+1;
		}
	}

	//Calculate the rows of the matrix with tangent, binormal, normal as columns
	for(std::vector<TorusVertex>::size_type n=0; n<vertices.size(); ++n)
	{
		vertices[i].tbn[0].x = vertices[n].tangent.x;
		vertices[i].tbn[0].y = vertices[n].binormal.x;
		vertices[i].tbn[0].z = vertices[n].normal.x;

		vertices[i].tbn[1].x = vertices[n].tangent.y;
		vertices[i].tbn[1].y = vertices[n].binormal.y;
		vertices[i].tbn[1].z = vertices[n].normal.y;

		vertices[i].tbn[2].x = vertices[n].tangent.z;
		vertices[i].tbn[2].y = vertices[n].binormal.z;
		vertices[i].tbn[2].z = vertices[n].normal.z;
	}
}

Torus::~Torus()
{

}
