#ifndef __PRIMITIVEFACTORY_H__
#define __PRIMITIVEFACTORY_H__

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#define WIN32_EXTRA_LEAN
#include <windows.h>
#endif

#include <gl/gl.h>


// Structure for holding a simple model
struct Primitive
{
	GLenum nElemType;	// GL_TRIANGLES, etc
	int nVertices;		// total unique vertex count
	int nIndices;		// index count
	GLfloat *pVert;		// x,y,z,...
	GLubyte *pCol;		// r,g,b,a,...
	GLfloat *pNormal;	// x,y,z,...
	GLfloat *pTexCoord;	// u,v,...
	GLushort *pIdx;		// index data

	Primitive( GLenum nType, int nVerts, int nIdxs );
	~Primitive();

	// Sets up clientstate and renders the primitive
	void Draw( bool bWireframe = false, 
	           bool bNormalArray = true, 
	           bool bColorArray = true, 
	           bool bTexCoordArray = true );
};

// Primitive factory class
class CPrimitiveFactory
{
public:

	// Deletes a primitive object that was allocated by a factory method
	static void DeletePrimitive( Primitive **ppPrim );

	// Generates a patch on the XZ plane, centered on the origin
	static Primitive* MakePatch( float fWidth, float fHeight, int nCols, 
		int nRows );

	// Generates a quad on the XY plane
	static Primitive* MakeQuad( float fWidth, float fHeight );

};

#endif //__PRIMITIVEFACTORY_H__
