/*************************************************************************
MGLApp.cpp

Main application class.

Author   :   Dave Astle

Written for More OpenGL Game Programming
*************************************************************************/

#ifdef WIN32
	#define WIN32_LEAN_AND_MEAN
	#define WIN32_EXTRA_LEAN
	#include <windows.h>
#endif

#include "MGLApp.h"
#include "MGLTargaImage.h"

MGLApp::MGLApp()
{
  m_translateT = 0.0f;
}

MGLApp::~MGLApp()
{
}

bool MGLApp::Init()
{	
	if (!GLeeInit())
		return false;

  m_useAnisotropy = GL_TRUE;

  glGetFloatv(GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT, &m_maxAnisotropy);

  glGenTextures(2, m_textures);

  MGL::TargaImage image;

  image.Load("road.tga");
  glBindTexture(GL_TEXTURE_2D, m_textures[0]);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, m_maxAnisotropy);
  gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB, image.GetWidth(), image.GetHeight(), GL_RGB, GL_UNSIGNED_BYTE, image.GetImage());

  image.Load("grass.tga");
  glBindTexture(GL_TEXTURE_2D, m_textures[1]);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, m_maxAnisotropy);
  gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB, image.GetWidth(), image.GetHeight(), GL_RGB, GL_UNSIGNED_BYTE, image.GetImage());

  glEnable(GL_TEXTURE_2D);
  glClearColor(0.4f, 0.4f, 0.9f, 1.0f);

	return true;
}

bool MGLApp::Shutdown()
{
  glDeleteTextures(2, m_textures);
	return true;
}

void MGLApp::SetupProjection(int width, int height)
{
	// avoid div by zero
	if( height == 0 )
		height = 1;					

	// set up GL viewport
	glViewport( 0, 0, width, height );

	// set up a perspective projection
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0f, (GLfloat)width/(GLfloat)height, 0.2f, 1000.0f );
}

void MGLApp::Update( double dt )
{
  m_translateT += dt;
}

void MGLApp::Render()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// set up the camera
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
  gluLookAt(0.0, 0.1, 0.0, 0.0, 0.0, -10.0, 0.0, 1.0, 0.0);

  glMatrixMode(GL_TEXTURE);
  glLoadIdentity();
  glTranslated(0.0, m_translateT, 0.0);

  glBindTexture(GL_TEXTURE_2D, m_textures[1]);
  glBegin(GL_QUADS);
    glTexCoord2i(0, 0); glVertex3f(-10.0f, 0.0f, 10.0f);
    glTexCoord2i(40, 0); glVertex3f(10.0f, 0.0f, 10.0f);
    glTexCoord2i(40, 40); glVertex3f(10.0f, 0.0f, -10.0f);
    glTexCoord2i(0, 40); glVertex3f(-10.0f, 0.0f, -10.0f);
  glEnd();

  glLoadIdentity();
  glTranslated(0.0, m_translateT/4.0, 0.0);

  glBindTexture(GL_TEXTURE_2D, m_textures[0]);
  glBegin(GL_QUADS);
    glTexCoord2i(0, 0); glVertex3f(-0.3f, 0.0f, 10.0f);
    glTexCoord2i(1, 0); glVertex3f(0.3f, 0.0f, 10.0f);
    glTexCoord2i(1, 10); glVertex3f(0.3f, 0.0f, -10.0f);
    glTexCoord2i(0, 10); glVertex3f(-0.3f, 0.0f, -10.0f);
  glEnd();

  glMatrixMode( GL_MODELVIEW );
}


void MGLApp::ToggleAnisotropy()
{
  m_useAnisotropy = !m_useAnisotropy;

  if (m_useAnisotropy)
  {
    glBindTexture(GL_TEXTURE_2D, m_textures[0]);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, m_maxAnisotropy);
    glBindTexture(GL_TEXTURE_2D, m_textures[1]);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, m_maxAnisotropy);
  }
  else
  {
    glBindTexture(GL_TEXTURE_2D, m_textures[0]);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, 0);
    glBindTexture(GL_TEXTURE_2D, m_textures[1]);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, 0);
  }
}
