/*************************************************************************
MGLSDL.h

Setup code for a basic SDL application.

Author   :   Dave Astle

Written for More OpenGL Game Programming
*************************************************************************/

#include "SDL.h"
#include "MGLApp.h"
#include <stdlib.h>

void SetGLAttributes()
{
  SDL_GL_SetAttribute(SDL_GL_BUFFER_SIZE, MGL_COLOR_BITS);
  SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, MGL_DEPTH_BITS);
  SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE, MGL_STENCIL_BITS);
  SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
//  SDL_GL_SetAttribute(SDL_GL_MULTISAMPLEBUFFERS, 1);
//  SDL_GL_SetAttribute(SDL_GL_MULTISAMPLESAMPLES, 4);
}

int main(int argc, char* argv[])
{
  SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER);
  atexit(SDL_Quit);

  SDL_WM_SetCaption(MGL_APP_NAME, MGL_APP_NAME);

  SetGLAttributes();

  Uint32 displayFlags = SDL_OPENGL;
  
  if (MGL_FULLSCREEN_MODE == MGL_FULLSCREEN_TRUE)
  {
    displayFlags |= SDL_FULLSCREEN;
  }
  else
  {
    displayFlags |= SDL_RESIZABLE;
  }

  SDL_Surface* pDisplay = SDL_SetVideoMode(MGL_WINDOW_WIDTH, MGL_WINDOW_HEIGHT, MGL_COLOR_BITS, displayFlags);

  if (pDisplay == NULL)
  {
    fprintf(stderr, "Couldn't set GL mode: %s\n", SDL_GetError());
    exit(0);
  }

  //DAVE: added init failure check  --Andres
  MGLApp* pApp = new MGLApp();
  if( !pApp->Init() )
  {
	  fprintf(stderr, "Couldn't initialize demo\n");
	  exit(0);
  }
  pApp->SetupProjection(MGL_WINDOW_WIDTH, MGL_WINDOW_HEIGHT);

  for (;;)
  {
    SDL_Event event;
    static Uint32 lastTime = SDL_GetTicks(), currTime;


    if (SDL_PollEvent(&event) == 0)
    {
      currTime = SDL_GetTicks();
      pApp->Update((currTime - lastTime)/1000.0);
      lastTime = currTime;
      pApp->Render();
      SDL_GL_SwapBuffers();
    }
    else
    {
      if (SDL_QUIT == event.type ||
          SDL_KEYDOWN == event.type && SDLK_ESCAPE == event.key.keysym.sym)
        break;

	  if( event.type == SDL_KEYDOWN )
	  {
		  if( event.key.keysym.sym == SDLK_LEFT ) pApp->PrevEnv();
		  if( event.key.keysym.sym == SDLK_RIGHT ) pApp->NextEnv();

		  if( event.key.keysym.sym == SDLK_UP ) pApp->PrevShader();
		  if( event.key.keysym.sym == SDLK_DOWN ) pApp->NextShader();

		  if( event.key.keysym.sym == SDLK_MINUS ) pApp->ChangeExposure(0.9f);
		  if( event.key.keysym.sym == SDLK_EQUALS ) pApp->ChangeExposure(1.1f);
		  if( event.key.keysym.sym == SDLK_F1 ) pApp->ShowHelp();
	  }

	  if( event.type == SDL_KEYUP )
	  {
		  if( event.key.keysym.sym == SDLK_F1 ) pApp->HideHelp();
	  }

	  //handle mouse movement
	  if (event.type == SDL_MOUSEMOTION)
	  {
		  // get info about the mouse movement from SDL
		  unsigned int x = event.motion.x;
		  unsigned int y = event.motion.y;
		  int dx = event.motion.xrel;
		  int dy = event.motion.yrel;
		  unsigned int state = event.motion.state;

		  // if mouse is dragged with the left button down, orbit camera
		  if (state & SDL_BUTTON(SDL_BUTTON_LEFT))
		  {
			  pApp->RotateCamera( (float)dy, (float)dx );
		  }

		  // middle button dragging rotates the model
		  if (state&SDL_BUTTON(SDL_BUTTON_MIDDLE))
		  {	
			  pApp->RotateEnvironment( (float)dx );
		  }

		  // right button dragging zooms in and out
		  if (state&SDL_BUTTON(SDL_BUTTON_RIGHT))
		  {	
			  pApp->ZoomCamera( (float)dy );
		  }
	  }
    }
  }
  pApp->Shutdown();
  delete pApp;

  return 0;
}