#include "Mesh.h"
#include <iostream>
#include <fstream>

namespace ibl
{
	Mesh::Mesh()
	{
		mVertexBuffer = 0;
		mTexCoordBuffer = 0;
		mNormalBuffer = 0;
	}

	Mesh::~Mesh()
	{
		if( mVertexBuffer ) delete [] mVertexBuffer;
		if( mTexCoordBuffer ) delete [] mTexCoordBuffer;
		if( mNormalBuffer ) delete [] mNormalBuffer;

		mVertexBuffer = 0;
		mTexCoordBuffer = 0;
		mNormalBuffer = 0;
	}

	void Mesh::load( const std::string &filename )
	{
		std::cout<<"loading mesh file "<<filename<<" ... ";
		std::cout.flush();

		FILE * file = fopen( filename.c_str(), "rb" );
		if( !file )
		{
			std::cerr<<"mesh error: failed to open file "<<filename<<std::endl;
			return;
		}

		if( mVertexBuffer ) delete [] mVertexBuffer;
		if( mTexCoordBuffer ) delete [] mTexCoordBuffer;
		if( mNormalBuffer ) delete [] mNormalBuffer;

		fread( &mVertexCount, sizeof(size_t), 1, file );
		mVertexBuffer = new float[mVertexCount * 3];
		mNormalBuffer = new float[mVertexCount * 3];
		mTexCoordBuffer = new float[mVertexCount * 2];

		fread( mVertexBuffer, sizeof( float ), mVertexCount * 3, file );
		fread( mNormalBuffer, sizeof( float ), mVertexCount * 3, file );
		fread( mTexCoordBuffer, sizeof( float ), mVertexCount * 2, file );

		fclose( file );
		std::cout<<"success"<<std::endl;
	}

	void Mesh::draw()const
	{
		// if the vertex buffer is empty don't even bother
		if( mVertexBuffer )
		{
			glEnableClientState( GL_VERTEX_ARRAY );		
			glVertexPointer( 3, GL_FLOAT, 0, &mVertexBuffer[0] );	

			// if texture coordinates exist, prepare them
			if( mTexCoordBuffer )
			{
				glEnableClientState( GL_TEXTURE_COORD_ARRAY );
				glTexCoordPointer( 2, GL_FLOAT, 0, &mTexCoordBuffer[0] );
			}

			// if normals exist, prepare them
			if( mNormalBuffer )
			{
				glEnableClientState( GL_NORMAL_ARRAY );
				glNormalPointer(GL_FLOAT, 0, &mNormalBuffer[0] );
			}

			// draw all arrays
			glDrawArrays( GL_TRIANGLES, 0, (GLsizei)mVertexCount );

			glDisableClientState( GL_VERTEX_ARRAY );
			glDisableClientState( GL_TEXTURE_COORD_ARRAY );
			glDisableClientState( GL_NORMAL_ARRAY );
		}
	}
}