#ifndef SKYBOX_H
#define SKYBOX_H

#include "glee.h"
#include <gl/glu.h>

namespace ibl
{
	// This class draws a 200x200x200 box at the origin with texture coordinates
	// as rays eminating from the origin which makes it easy to sample an environment
	// cubemap.
	class Skybox
	{
	private:
		float mVerts[24];

	public:
		Skybox()
		{
			float size = 100;
			mVerts[0] =  size; mVerts[1] =  size; mVerts[2] = -size;
			mVerts[3] = -size; mVerts[4] =  size; mVerts[5] = -size;
			mVerts[6] = -size; mVerts[7] =  size; mVerts[8] =  size;
			mVerts[9] =  size; mVerts[10] =  size; mVerts[11] =  size;

			mVerts[12] = -size; mVerts[13] = -size; mVerts[14] =  size;
			mVerts[15] =  size; mVerts[16] = -size; mVerts[17] =  size;
			mVerts[18] = -size; mVerts[19] = -size; mVerts[20] = -size;
			mVerts[21] =  size; mVerts[22] = -size; mVerts[23] = -size;
		}

		void draw()const
		{
			//don't bother writing to the depth buffer, the skybox should always be background
			glDepthMask( GL_FALSE );

			//cull away the front faces of the box because we'll be inside it
			glCullFace( GL_FRONT );
			glBegin( GL_QUADS );
			glTexCoord3fv( &mVerts[0] );
			glVertex3fv( &mVerts[0] );
			glTexCoord3fv( &mVerts[3] );
			glVertex3fv( &mVerts[3] );
			glTexCoord3fv( &mVerts[6] );
			glVertex3fv( &mVerts[6] );
			glTexCoord3fv( &mVerts[9] );
			glVertex3fv( &mVerts[9] );
			
			glTexCoord3fv( &mVerts[21] );
			glVertex3fv( &mVerts[21] );
			glTexCoord3fv( &mVerts[15] );
			glVertex3fv( &mVerts[15] );
			glTexCoord3fv( &mVerts[12] );
			glVertex3fv( &mVerts[12] );
			glTexCoord3fv( &mVerts[18] );
			glVertex3fv( &mVerts[18] );
			
			glTexCoord3fv( &mVerts[0] );
			glVertex3fv( &mVerts[0] );
			glTexCoord3fv( &mVerts[9] );
			glVertex3fv( &mVerts[9] );
			glTexCoord3fv( &mVerts[15] );
			glVertex3fv( &mVerts[15] );
			glTexCoord3fv( &mVerts[21] );
			glVertex3fv( &mVerts[21] );
			
			glTexCoord3fv( &mVerts[6] );
			glVertex3fv( &mVerts[6] );
			glTexCoord3fv( &mVerts[3] );
			glVertex3fv( &mVerts[3] );
			glTexCoord3fv( &mVerts[18] );
			glVertex3fv( &mVerts[18] );
			glTexCoord3fv( &mVerts[12] );
			glVertex3fv( &mVerts[12] );
			
			glTexCoord3fv( &mVerts[9] );
			glVertex3fv( &mVerts[9] );
			glTexCoord3fv( &mVerts[6] );
			glVertex3fv( &mVerts[6] );
			glTexCoord3fv( &mVerts[12] );
			glVertex3fv( &mVerts[12] );
			glTexCoord3fv( &mVerts[15] );
			glVertex3fv( &mVerts[15] );

			glTexCoord3fv( &mVerts[3] );
			glVertex3fv( &mVerts[3] );
			glTexCoord3fv( &mVerts[0] );
			glVertex3fv( &mVerts[0] );
			glTexCoord3fv( &mVerts[21] );
			glVertex3fv( &mVerts[21] );
			glTexCoord3fv( &mVerts[18] );
			glVertex3fv( &mVerts[18] );

			glEnd();
			glCullFace( GL_BACK );
			glDepthMask( GL_TRUE );
		}
	};
}
#endif

