/*************************************************************************
MGLApp.cpp

Main application class.

Author   :   Dave Astle

Written for More OpenGL Game Programming
*************************************************************************/

#ifdef WIN32
	#define WIN32_LEAN_AND_MEAN
	#define WIN32_EXTRA_LEAN
	#include <windows.h>
#endif

#include "MGLApp.h"
#include "MGLTargaImage.h"
#include "MD2Model.h"
#include "GLSLProgram.h"

MGLApp::MGLApp()
: mModel(0)
{
}

MGLApp::~MGLApp()
{
}

bool MGLApp::Init()
{
	if (!GLeeInit())
		return false;
  
  mModel = new MD2Model("Assets/Models/pknight/tris.md2");
  if(!mModel)
    return false;
  std::string err;
  mShader  = new GLSLProgram("Assets/Shaders/cel.vsh","Assets/Shaders/cel.fsh");
  mShader->GetProgramLog(err);
  

  mSkinLoc = mShader->getUniformLocation("texd");
  mCelLoc  = mShader->getUniformLocation("texc");

  
  glEnable(GL_TEXTURE_2D);
  glEnable(GL_CULL_FACE);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  
  glClearColor(1.0f,1.0f,1.0f,1.0f);
  
MGL::TargaImage tex;

  tex.Load("Assets/Textures/cel_detail.tga");
  glGenTextures(1,&mSkinTexture);
  glBindTexture(GL_TEXTURE_2D,mSkinTexture);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexImage2D(GL_TEXTURE_2D,0,3,tex.GetWidth(),tex.GetHeight(),0,tex.GetFormat(),tex.GetType(),tex.GetImage());

  tex.Load("Assets/Textures/cel.tga");
  glGenTextures(1,&mCelTexture);
  glBindTexture(GL_TEXTURE_2D,mCelTexture);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexImage2D(GL_TEXTURE_2D,0,3,tex.GetWidth(),tex.GetHeight(),0,tex.GetFormat(),tex.GetType(),tex.GetImage());
	return true;
}

bool MGLApp::Shutdown()
{
  delete mShader;
  delete mModel;
	return true;
}

void MGLApp::SetupProjection(int width, int height)
{
	// avoid div by zero
	if( height == 0 )
		height = 1;					

	// set up GL viewport
	glViewport( 0, 0, width, height );

	// set up a perspective projection
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0f, (GLfloat)width/(GLfloat)height, 1.0f, 1000.0f );
}

void MGLApp::Update( double dt )
{
  mModel->Update(dt * 5.0f);
}

void MGLApp::Render()
{
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	// set up the camera
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	glTranslatef(0.0f,0.0f,-100.0f);
	
	mShader->use();
	
	glActiveTexture(GL_TEXTURE0);
  glBindTexture(GL_TEXTURE_2D,mSkinTexture);
	glActiveTexture(GL_TEXTURE1);
	glBindTexture(GL_TEXTURE_2D,mCelTexture);
  mShader->sendUniform(mSkinLoc,0);
  mShader->sendUniform(mCelLoc,1);
	
	mModel->Render();
}
