/*************************************************************************
MGLApp.h

Main application class.

Author   :   Dave Astle

Written for More OpenGL Game Programming
*************************************************************************/

#ifndef MGLAPP_H_INCLUDED
#define MGLAPP_H_INCLUDED

#ifdef WIN32
  #define WIN32_LEAN_AND_MEAN
  #define WIN32_EXTRA_LEAN
  #include <windows.h>
#endif

#include <gl/glee.h>
#include <gl/glu.h>

// values used to set up the window and rendering context
const char MGL_APP_NAME[]	= "MOGLGP - NPR - Cel Shading";

const int MGL_WINDOW_WIDTH = 800;
const int MGL_WINDOW_HEIGHT = 600;

const int MGL_COLOR_BITS = 32;
const int MGL_DEPTH_BITS = 24;
const int MGL_STENCIL_BITS = 0;

enum
{
  MGL_FULLSCREEN_FALSE = 0,
  MGL_FULLSCREEN_TRUE = 1,
  MGL_FULLSCREEN_CHOOSE = 2
};

const int  MGL_FULLSCREEN_MODE = MGL_FULLSCREEN_CHOOSE;
const bool MGL_FORCE_FULLSCREEN = false;

class MD2Model;
class GLSLProgram;

class MGLApp
{
public:
	MGLApp();
	virtual ~MGLApp();

	bool Init();
	bool Shutdown();
	void SetupProjection( int width, int height );
	void Update( double dt );
	void Render();

private:
  MD2Model     *mModel;
  GLSLProgram  *mShader;
  GLuint        mSkinTexture;
  GLuint        mCelTexture;
  GLuint        mSkinLoc;
  GLuint        mCelLoc;
};

#endif // MGLAPP_H_INCLUDED