/* ==========================================================================

     File    edge.vsh
     Author  Josh Petrie (jpetrie@mac.com)
     
             This file is part of a example program written
             for the book "More OpenGL Game Programming."

   ========================================================================== */

attribute float i;
attribute vec4  v1;
attribute vec3  n0;
attribute vec3  n1;
attribute vec3  nA;
attribute vec3  nB;

uniform float  sw;
uniform float  sh;

varying  vec2  texc;

bool xor(bool a,bool b)
{
  return ((a || b) && (!a || !b));
}

float sgn(float v)
{
float o = 1.0;

  if(v < 0.0)
    o = -1.0;
  return (o);
}

void main(void)
{
  // Compute the screen-space positions of v0 and v1.
vec4  c0 = gl_ModelViewProjectionMatrix * gl_Vertex;
vec4  c1 = gl_ModelViewProjectionMatrix * v1;
vec3  s0 = vec3(c0.xyz / c0.w);
vec3  s1 = vec3(c1.xyz / c1.w);

  // Compute screen-space perpendicular of the edge.
vec2  p  = normalize(vec2(s0.y - s1.y,s1.x - s0.x));
vec4  d0 = gl_ModelViewProjectionMatrix * vec4((gl_Vertex + vec4(n0,0.0)).xy,0,1);
vec4  d1 = gl_ModelViewProjectionMatrix * vec4((v1 + vec4(n1,0.0)).xy,0,1);
vec2  m0 = vec2(d0.xy / d0.w);
vec2  m1 = vec2(d1.xy / d1.w);

  // Transform the eye into object space and computer a vector from the vertex
  // to the eye.
vec4  eye = gl_ModelViewMatrixInverse * vec4(0.0,0.0,0,1.0);
vec3  v2e = vec3(eye) - vec3(gl_Vertex);

  // Normalize all vectors that need it.
  nA  = normalize(nA);
  nB  = normalize(nB);
  v2e = normalize(v2e);

float k = 4.0;

  p.x  = p.x * (k / 400);
  p.y  = p.y * (k / 300);
  
vec2  q0 = s0 - p;
vec2  q1 = s1 - p;
vec2  q2 = s1 + p;
vec2  q3 = s0 + p;
 
float  u0 = 0.1;
float  u1 = 0.9;
float  v0 = 0.1;
float  v1 = 0.9;

  // Flip u coordinates if needed.
  if(sgn(dot(p,m0)) > 0.0)
  {
     u0 = 0.9;
     u1 = 0.1;
  }

  // Determine if the vertex is on a feature edge.
  if(xor(dot(nA,v2e) < 0.0,dot(nB,v2e) < 0.0))
  {
    if(i == 0.0)
    {
      gl_Position = vec4(q0.x,q0.y,s0.z,1.0);
      texc.x = u0;
      texc.y = v0;
    }
    else if(i == 1.0)
    {
      gl_Position = vec4(q1.x,q1.y,s1.z,1.0);
      texc.x = u1;
      texc.y = v0;
    }
    else if(i == 2.0)
    {
      gl_Position = vec4(q2.x,q2.y,s1.z,1.0);
      texc.x = u1;
      texc.y = v1;
    }
    else 
    {
      gl_Position = vec4(q3.x,q3.y,s0.z,1.0);
      texc.x = u0;
      texc.y = v1;
    }
  }
  else
    gl_Position = vec4(0.0,0.0,-1.0,1.0);
}