/* ==========================================================================

     File    EdgeMesh.cpp
     Author  Josh Petrie (jpetrie@mac.com)
     
             This file is part of a example program written
             for the book "More OpenGL Game Programming."

   ========================================================================== */
/*                                                                   includes
----------------------------------------------------------------------------- */

#include "EdgeMesh.h"


/*                                                                  functions
----------------------------------------------------------------------------- */

/*  _________________________________________________________________________ */
EdgeMesh BuildEdgeMesh(const MD2Model &source)
{
EdgeMesh                          edges;
std::set< std::pair< int,int > >  seenEdges;

  // For each triangle...
  for(int i = 0; i < source.GetTriangleCount(); ++i)
  {
  MD2Model::Triangle   t = source.GetTriangle(i);
  MD2Model::Adjacency  a = source.GetTriangleAdj(i);
  
    // For each edge...
    for(int j = 0; j < 3; ++j)
    {
      // Find the index of the edge on the triangle adjacent to this one.
    MD2Model::Triangle  ta     = source.GetTriangle(a.neighbor[j]);
    unsigned short      p0     = t.vtxIdx[j];
    unsigned short      p1     = t.vtxIdx[(j + 1) % 3];
    int                 adjIdx = -1;
    
      // On the adjacent triangle, the first vertex of the edge will be p1,
      // not p0, so the index at which p1 appears is the index of the edge.
      for(int k = 0; k < 3; ++k)
        if(p1 == ta.vtxIdx[k])
          adjIdx = k;
      
      // See if the edge from p0 to p1, or the other way around, has been seen
      // already or not. If neither form of the edge exist in the map, we have
      // not yet seen this edge.
    std::pair< int,int >  edge0 = std::make_pair(p0,p1);
    std::pair< int,int >  edge1 = std::make_pair(p1,p0);
    
      if(seenEdges.find(edge0) == seenEdges.end() &&
         seenEdges.find(edge1) == seenEdges.end())
      {
      EdgeInfo  e;
      
        // v0 and v1 are the indices of the actual edge vertices.
        e.v0 = p0;
        e.v1 = p1;
        
        // v2 is on triangle t.
        e.v2 = t.vtxIdx[(j + 2) % 3];
        
        // v3 is on triangle ta.
        e.v3 = t.vtxIdx[(adjIdx + 2) % 3];
        
        // a and b are indices of the main and adjacent triangle, respectively.
        e.a = i;
        e.b = a.neighbor[j];
        
        // Add the edge. The edge is added four times, each of which is
        // identical except for the value of i which ranges from 0 to 3.
        e.i = 0;
        edges.push_back(e);
        e.i = 1;
        edges.push_back(e);
        e.i = 2;
        edges.push_back(e);
        e.i = 3;
        edges.push_back(e);
        
        // Insert both edges into the map to ensure we don't add this
        // edge to the edge mesh again.
        seenEdges.insert(edge0);
        seenEdges.insert(edge1);
      }
    }
  }
  
  return (edges);
}