/*************************************************************************
MGLApp.cpp

Main application class.

Author   :   Dave Astle

Written for More OpenGL Game Programming
*************************************************************************/

#ifdef WIN32
	#define WIN32_LEAN_AND_MEAN
	#define WIN32_EXTRA_LEAN
	#include <windows.h>
#endif

#include "MGLApp.h"
#include "MGLTargaImage.h"
#include "MD2Model.h"
#include "GLSLProgram.h"

MGLApp::MGLApp()
: mModel(0)
{
}

MGLApp::~MGLApp()
{
}

bool MGLApp::Init()
{
	if (!GLeeInit())
		return false;
  
  mModel = new MD2Model("Assets/Models/pknight/tris.md2");
  if(!mModel)
    return false;
  std::string err;
  mShader  = new GLSLProgram("Assets/Shaders/edge.vsh","Assets/Shaders/edge.fsh");
  mShader->GetProgramLog(err);
  
  mMesh = BuildEdgeMesh(*mModel);

  mTexLoc = mShader->getUniformLocation("tex");

  
  glEnable(GL_TEXTURE_2D);
  glEnable(GL_CULL_FACE);
  glEnable(GL_DEPTH_TEST);
  glDisable(GL_LIGHTING);
  
  glClearColor(1.0f,1.0f,1.0f,1.0f);
  
MGL::TargaImage tex;

  tex.Load("Assets/Textures/stroke.tga");
  glGenTextures(1,&mTexture);
  glBindTexture(GL_TEXTURE_2D,mTexture);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexImage2D(GL_TEXTURE_2D,0,3,tex.GetWidth(),tex.GetHeight(),0,tex.GetFormat(),tex.GetType(),tex.GetImage());
	return true;
}

bool MGLApp::Shutdown()
{
  delete mShader;
  delete mModel;
	return true;
}

void MGLApp::SetupProjection(int width, int height)
{
	// avoid div by zero
	if( height == 0 )
		height = 1;					

	// set up GL viewport
	glViewport( 0, 0, width, height );

	// set up a perspective projection
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0f, (GLfloat)width/(GLfloat)height, 1.0f, 1000.0f );
}

void MGLApp::Update( double dt )
{
  mModel->Update(dt * 5.0f);
}

void MGLApp::Render()
{
GLuint  loc_i  = mShader->getAttributeLocation("i");
GLuint  loc_v1 = mShader->getAttributeLocation("v1");
GLuint  loc_n0 = mShader->getAttributeLocation("n0");
GLuint  loc_n1 = mShader->getAttributeLocation("n1");
GLuint  loc_nA = mShader->getAttributeLocation("nA");
GLuint  loc_nB = mShader->getAttributeLocation("nB");

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	// set up the camera
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	glTranslatef(0.0f,0.0f,-100.0f);
	
	glColorMask(GL_FALSE,GL_FALSE,GL_FALSE,GL_FALSE);
  mShader->disable();
  mModel->Render();
  glColorMask(GL_TRUE,GL_TRUE,GL_TRUE,GL_TRUE);
	
	mShader->use();
	
	glActiveTexture(GL_TEXTURE0);
  glBindTexture(GL_TEXTURE_2D,mTexture);
  mShader->sendUniform(mTexLoc,0);
	
	glBegin(GL_QUADS);
  for(unsigned int i = 0; i < mMesh.size(); ++i)
  {
  MGL::Vector3  v0 = mModel->GetVertex(mMesh[i].v0);
  MGL::Vector3  v1 = mModel->GetVertex(mMesh[i].v1);
  MGL::Vector3  n0 = mModel->GetNormal(mMesh[i].v0);
  MGL::Vector3  n1 = mModel->GetNormal(mMesh[i].v1);
  MGL::Vector3  nA = mModel->GetFaceNormal(mMesh[i].a);
  MGL::Vector3  nB = mModel->GetFaceNormal(mMesh[i].b);
  
    // Set i.
    glVertexAttrib1f(loc_i,static_cast< GLfloat >(mMesh[i].i));
    
    // Send face normals of triangles bordering the edge.
    glVertexAttrib3f(loc_nA,nA.x,nA.y,nA.z);
    glVertexAttrib3f(loc_nB,nB.x,nB.y,nB.z);
    
    // 
    glVertexAttrib3f(loc_n0,n0.x,n0.y,n0.z);
    glVertexAttrib3f(loc_n1,n1.x,n1.y,n1.z);
    
    // Send edge vertices. v0 is sent via gl_Position.
    // v1 is sent via an attribute.
    glVertexAttrib4f(loc_v1,v1.x,v1.y,v1.z,1.0f);
    glVertex4f(v0.x,v0.y,v0.z,1.0f);
  }
  glEnd();
}
