/*************************************************************************
MGLSDL.h

Setup code for a basic SDL application.

Author   :   Dave Astle
Author	 :	 Lee Thomason

Written for More OpenGL Game Programming
*************************************************************************/

#include "sdl.h"
#include "MGLBillboards.h"
#include <stdlib.h>

void SetGLAttributes()
{
  SDL_GL_SetAttribute(SDL_GL_BUFFER_SIZE, MGL_COLOR_BITS);
  SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, MGL_DEPTH_BITS);
  SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE, MGL_STENCIL_BITS);
  SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
}

int main(int argc, char* argv[])
{
  SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER);
  atexit(SDL_Quit);

  SDL_WM_SetCaption(MGL_APP_NAME, MGL_APP_NAME);

  SetGLAttributes();

  Uint32 displayFlags = SDL_OPENGL;

  if (MGL_FORCE_FULLSCREEN)
  {
    displayFlags |= SDL_FULLSCREEN;
  }
  else
  {
    displayFlags |= SDL_RESIZABLE;
  }

  SDL_Surface* pDisplay = SDL_SetVideoMode(MGL_WINDOW_WIDTH, MGL_WINDOW_HEIGHT, MGL_COLOR_BITS, displayFlags);

  if (pDisplay == NULL)
  {
    fprintf(stderr, "Couldn't set GL mode: %s\n", SDL_GetError());
    exit(0);
  }

  MGLBillboards* pApp = new MGLBillboards();
  pApp->Init();
  pApp->SetupProjection(MGL_WINDOW_WIDTH, MGL_WINDOW_HEIGHT);

  for (;;)
  {
    SDL_Event event;
    Uint32 lastTime = SDL_GetTicks(), currTime;


    if (SDL_PollEvent(&event) == 0)
    {
      currTime = SDL_GetTicks();
      pApp->Update((currTime - lastTime)/1000.0f);
      lastTime = currTime;
      pApp->Render();
      SDL_GL_SwapBuffers();
    }
    else
    {
      if (SDL_QUIT == event.type ||
          SDL_KEYDOWN == event.type && SDLK_ESCAPE == event.key.keysym.sym)
        break;

      if (SDL_KEYDOWN == event.type ) {
	    if ( pApp->GetMode() == MGLBillboards::SHOW_BILLBOARD_TEXTURES ) {
		  pApp->SetMode( MGLBillboards::SHOW_ANIMATION, MGL_WINDOW_WIDTH, MGL_WINDOW_HEIGHT );
		}
		else {
		  if ( event.key.keysym.sym == SDLK_1 )		 pApp->SetTransition( MGLBillboards::ALPHA1 );
		  else if ( event.key.keysym.sym == SDLK_2 ) pApp->SetTransition( MGLBillboards::ALPHA2 );
		  else if ( event.key.keysym.sym == SDLK_3 ) pApp->SetTransition( MGLBillboards::STIPPLE );
		  else if ( event.key.keysym.sym == SDLK_m ) pApp->SetAnimation( MGLBillboards::RENDER_MODEL );
		  else if ( event.key.keysym.sym == SDLK_t ) pApp->SetAnimation( MGLBillboards::RENDER_TRANSITION );
		  else if ( event.key.keysym.sym == SDLK_b ) pApp->SetAnimation( MGLBillboards::RENDER_BILLBOARD );
		  else if ( event.key.keysym.sym == SDLK_UP ) pApp->SetView( MGLBillboards::HIGH );
		  else if ( event.key.keysym.sym == SDLK_DOWN ) pApp->SetView( MGLBillboards::LOW );
		  else if ( event.key.keysym.sym == SDLK_LEFT ) pApp->SetView( MGLBillboards::LEFT );
		  else if ( event.key.keysym.sym == SDLK_RIGHT ) pApp->SetView( MGLBillboards::RIGHT );
		  else if ( event.key.keysym.sym == SDLK_a ) pApp->SetBBType( MGLBillboards::ASYMMETRIC );
		  else if ( event.key.keysym.sym == SDLK_s ) pApp->SetBBType( MGLBillboards::SYMMETRIC );
	    }
	  }
    }
  }
  pApp->Shutdown();
  delete pApp;

  return 0;
}