/*************************************************************************
MGLApp.cpp

Demo for "Glow Effect" section

Author   :   Andrei Gradinari
Based on Dave Astle's application template

Written for More OpenGL Game Programming
August, 2005
*************************************************************************/

#ifdef WIN32
	#define WIN32_LEAN_AND_MEAN
	#define WIN32_EXTRA_LEAN
	#include <windows.h>
#endif

#include <stdio.h>

#include "MGLApp.h"
using namespace MGL;

enum EDemoModes {
	RENDER_ALL = 0, 
	RENDER_WITHOUT_GLOW_EFFECT = 1, 
	SHOW_RENDER_TEXTURE = 2, 
	SHOW_BLURRED_TEXTURE = 3, 
	SHOW_BLURRED_GLOW_TEXTURE = 4
};
EDemoModes demo_mode = RENDER_ALL;


MGLApp::MGLApp()
{
	m_dElapTime = 0;
	for(int i=0; i<3; i++) m_pWheel[i] = NULL;
	for(	i=0; i<2; i++) pbuffer[i] = new PBuffer("rgb texture2D");
}

MGLApp::~MGLApp()
{
	for(int i=0; i<2; i++) delete pbuffer[i];
	for(	i=0; i<3; i++) delete m_pWheel[i];
}

bool MGLApp::Init()
{	
	// PBUFFERS INITIALIZATION
	for(int i=0; i<2; i++) if(! pbuffer[i]->Initialize(256, 256, true, false)) return false;

	// CREATE OUR RENDER TEXTURE OBJECT
    glGenTextures(1, &m_iRenderTexture);
	glBindTexture(GL_TEXTURE_2D, m_iRenderTexture);
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);//_MIPMAP_LINEAR);
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	// notice that we don't need to call glTexImage2D for this texture object
	
	if(! LoadData()) // LOAD MODELS, SHADERS AND TEXTURES
	{
		MessageBox(NULL, "Error loading data. Exiting", "Error", MB_OK);
		return false;
	}

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);
    
    return true;
}

void MGLApp::OnKeyDown(int nVirtKey, int lKeyData)
{	
	int t  = demo_mode;
	
	switch(nVirtKey)
	{
		case VK_UP: 
			t = t - 1 % 5;
			if(t < 0) t = t + 5;
		break;
			
		case VK_DOWN: t = (t + 1) % 5;
		break;
	}

	demo_mode = EDemoModes(t);
}

bool MGLApp::Shutdown()
{
	for(int i=0; i<2; i++) pbuffer[i]->Destroy();
	return true;
}

void MGLApp::OnSize(int width, int height )
{
	m_nWindowWidth = width; 
	m_nWindowHeight = height; 
	SetupProjection(m_nWindowWidth, m_nWindowHeight);
}

void MGLApp::SetupProjection(int width, int height)
{
	// avoid div by zero
	if( height == 0 )
		height = 1;					

	// set up GL viewport
	glViewport( 0, 0, width, height );

	// set up a perspective projection
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0f, (GLfloat)width/(GLfloat)height, 0.5f, 100.0f );

	glMatrixMode(GL_MODELVIEW);
}

void MGLApp::Update( double dt )
{
	m_dElapTime += dt;
}

void MGLApp :: SetupLighting()
{
	float pos[4] = {1, 0, 0, 0};
	glLightfv(GL_LIGHT0, GL_POSITION, pos);
}

void MGLApp :: RenderGUI()
{
	glPushAttrib(GL_COLOR_BUFFER_BIT);
	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE, GL_ONE);	
	glEnable(GL_TEXTURE_2D);
	glUseProgramObjectARB(0);
	
	glMatrixMode(GL_MODELVIEW); glPushMatrix(); glLoadIdentity();
	glMatrixMode(GL_PROJECTION); glPushMatrix();  glLoadIdentity();
	glOrtho(0, m_nWindowWidth, 0, m_nWindowHeight, 0.1, 10);	
	glMatrixMode(GL_MODELVIEW);
	glTranslated(0, 0,-0.5);
	
	glBindTexture(GL_TEXTURE_2D, m_iCaptionTexture );
	glBegin(GL_QUADS);
	glTexCoord2d(0, 1); glVertex2d(0, m_nWindowHeight - 32);
	glTexCoord2d(1, 1); glVertex2d(512, m_nWindowHeight - 32);
	glTexCoord2d(1, 0); glVertex2d(512, m_nWindowHeight);
	glTexCoord2d(0, 0); glVertex2d(0, m_nWindowHeight);
	glEnd();

	glBindTexture(GL_TEXTURE_2D, m_iRenderModeTextures[demo_mode] );
	glBegin(GL_QUADS);
	glTexCoord2d(0, 1); glVertex2d(0, m_nWindowHeight - 65);
	glTexCoord2d(1, 1); glVertex2d(512, m_nWindowHeight - 65);
	glTexCoord2d(1, 0); glVertex2d(512, m_nWindowHeight-33);
	glTexCoord2d(0, 0); glVertex2d(0, m_nWindowHeight-33);
	glEnd();
	
	glMatrixMode(GL_PROJECTION); glPopMatrix();
	glMatrixMode(GL_MODELVIEW); glPopMatrix();

	glPopAttrib();
}

void MGLApp :: RenderGeometry(bool b_specular_only = false)
{
	if(b_specular_only)glUseProgramObjectARB(m_iSpecularOnlyShader);
	else glUseProgramObjectARB(m_iMaterialShader);
	for(int i=0; i<3; i++) m_pWheel[i]->Draw();

	glUseProgramObjectARB(0);
}

void MGLApp::Render()
{
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	glPushMatrix();
	
	glLoadIdentity();
	gluLookAt(7, 7, 7, 0,0,0, 0,1,0);
	SetupLighting();
	
	glRotatef( 20.f * cos((float)m_dElapTime) - 25, 0.f, 1.f, 0.f );
	
	if( demo_mode == RENDER_ALL || demo_mode == RENDER_WITHOUT_GLOW_EFFECT )
		RenderGeometry();
	
	// RENDER GEOMETRY TO PBUFFER. PBUFFER CONTENT WILL BE BLURRED LATER
	pbuffer[0]->Activate();
	glPushAttrib(GL_VIEWPORT_BIT | GL_POLYGON_BIT);
		glViewport( 0, 0, pbuffer[0]->GetWidth(), pbuffer[0]->GetHeight());
		glClearColor(0.0f, 0.0f, 0.0f, 1); 
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
				
		glEnable(GL_TEXTURE_2D);
		glPushMatrix();

		RenderGeometry(true);

		glPopMatrix();
	
		pbuffer[0]->SwapBuffers();
	glPopAttrib();
	pbuffer[0]->Deactivate();
	

	// BLURRING SCREEN SPACE TEXTURE
	glUseProgramObjectARB(m_iBlurShader);
	glBindTexture(GL_TEXTURE_2D, m_iRenderTexture);

	glMatrixMode(GL_MODELVIEW); glPushMatrix();	glLoadIdentity();
	glMatrixMode(GL_PROJECTION); glPushMatrix(); glLoadIdentity();
	glOrtho(0, 1, 0, 1, 0.01, 100);
	glPushAttrib(GL_DEPTH_BUFFER_BIT | GL_VIEWPORT_BIT); 
	glViewport( 0, 0, pbuffer[0]->GetWidth(), pbuffer[0]->GetHeight());
	glDisable(GL_DEPTH_TEST); 
	
	int buff_index=0;
	
	if(demo_mode != SHOW_RENDER_TEXTURE)
		for(int i=0; i<5; i++, buff_index=!buff_index)
		{
			pbuffer[buff_index]->Bind(WGL_FRONT_LEFT_ARB); // binding buffer to a texture
			pbuffer[!buff_index]->Activate();
		
			glBegin(GL_QUADS);
			glTexCoord2d(0,	0);	glVertex3d(0, 0, -1); 
			glTexCoord2d(1, 0);	glVertex3d(1, 0, -1);
			glTexCoord2d(1, 1);	glVertex3d(1, 1, -1);
			glTexCoord2d(0, 1);	glVertex3d(0, 1, -1);
			glEnd();

			pbuffer[buff_index]->Release(WGL_FRONT_LEFT_ARB);
			
			pbuffer[!buff_index]->SwapBuffers();
			pbuffer[!buff_index]->Deactivate();
		}
		glPopAttrib();
	glMatrixMode(GL_PROJECTION); glPopMatrix();
	glMatrixMode(GL_MODELVIEW); glPopMatrix();

	// RENDERING SCREEN SPACE TEXTURE TO FRAMEBUFFER
	if(demo_mode != RENDER_WITHOUT_GLOW_EFFECT)
	{
		if(demo_mode == SHOW_RENDER_TEXTURE || demo_mode == SHOW_BLURRED_TEXTURE ) glUseProgramObjectARB(0);
		else glUseProgramObjectARB(m_iGlowShader);


		glBindTexture(GL_TEXTURE_2D, m_iRenderTexture);
		pbuffer[buff_index]->Bind(WGL_FRONT_LEFT_ARB);

		glMatrixMode(GL_MODELVIEW); glPushMatrix();	glLoadIdentity();
		glMatrixMode(GL_PROJECTION); glPushMatrix(); glLoadIdentity();
		glOrtho(0, 1, 0, 1, 0.01, 100);
		
		glPushAttrib(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT); 
		glDisable(GL_DEPTH_TEST); 
		
		// setup additive blending
		glEnable(GL_BLEND); glBlendFunc(GL_ONE, GL_ONE);
		
		float offset[4][2] = {
			{-0.01, -0.01},	{0.01, -0.01},
			{0.01, 0.01},	{-0.01, 0.01} };
		
		for(int i=0; i<4; i++)
		{
		glPushMatrix();
			glTranslated(offset[i][0], offset[i][1], 0);
			glBegin(GL_QUADS);
			glTexCoord2d(0,	0);	glVertex3d(0, 0, -1); 
			glTexCoord2d(1, 0);	glVertex3d(1, 0, -1);
			glTexCoord2d(1, 1);	glVertex3d(1, 1, -1);
			glTexCoord2d(0, 1);	glVertex3d(0, 1, -1);
			glEnd();
		glPopMatrix();
		}
		
		glPopAttrib();
		glMatrixMode(GL_PROJECTION); glPopMatrix();
		glMatrixMode(GL_MODELVIEW); glPopMatrix();
		
		pbuffer[buff_index]->Release(WGL_FRONT_LEFT_ARB); // release pbuffer texture for further rendering	
	}

	glPopMatrix();

	RenderGUI();
}

GLhandleARB MGLApp::LoadShader(const char* s_vert, const char* s_frag)
{
	GLhandleARB shader_object = 0;
	
	char *vs_src = textFileRead(s_vert);
	char *fs_src = textFileRead(s_frag);
	
	GLint vs_len, fs_len;

	if(vs_src==NULL || fs_src==NULL) goto label_end;
	
	vs_len = (GLint)strlen(vs_src);
	fs_len = (GLint)strlen(fs_src);

	GLhandleARB vertex_shader, fragment_shader;
	shader_object = glCreateProgramObjectARB();
	vertex_shader = glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
	fragment_shader = glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
	
	
	glShaderSourceARB(vertex_shader, 1, (const char**) &vs_src, NULL);
	glShaderSourceARB(fragment_shader, 1, (const char**) &fs_src, NULL);

	GLint t;
	glCompileShaderARB(vertex_shader);
	glGetObjectParameterivARB(vertex_shader, GL_OBJECT_COMPILE_STATUS_ARB, &t); 
	if(t==0)
	{
		glDeleteObjectARB(shader_object);
		shader_object = 0;
		goto label_end;
	}
	
	glCompileShaderARB(fragment_shader);
	glGetObjectParameterivARB(fragment_shader, GL_OBJECT_COMPILE_STATUS_ARB, &t); 
	if(t==0)
	{
		glDeleteObjectARB(shader_object);
		shader_object = 0;
		goto label_end;
	}
	
	glAttachObjectARB(shader_object, vertex_shader);
	glAttachObjectARB(shader_object, fragment_shader);

	 // LINKING
	glLinkProgramARB(shader_object);
	glGetObjectParameterivARB(shader_object, GL_OBJECT_LINK_STATUS_ARB, &t); 
	if(t==0)
	{
		glDeleteObjectARB(shader_object);
		shader_object = 0;
		goto label_end;
	}

	//SETTING UNIFORMS
	glUseProgramObjectARB(shader_object);
	GLint attr_loc;
	attr_loc = glGetUniformLocationARB(shader_object, "tex_unit_0");
	if(attr_loc!=-1) glUniform1iARB(attr_loc, 0);
	
label_end:
	if(vs_src!=NULL)delete vs_src;
	if(fs_src!=NULL)delete fs_src;
	return shader_object;
}

GLuint MGLApp :: LoadTexture(char* fname, GLenum wrapst, bool mipmap)
{
	TargaImage img;
	if(!img.Load(fname)) return 0;
	img.FlipVertical();
	int width = img.GetWidth();
	int height = img.GetHeight();

	GLuint tex_id = 0;
	glGenTextures(1, &tex_id );
	glBindTexture(GL_TEXTURE_2D, tex_id);
	
	// mipmap ? GL_LINEAR_MIPMAP_LINEAR : GL_LINEAR
	// this fragment disables mippaping for those texture objects, which do not need it
	// here we need to disable mipmapping for projective textures
	
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, mipmap ? GL_LINEAR_MIPMAP_LINEAR : GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, wrapst);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, wrapst);
	gluBuild2DMipmaps(GL_TEXTURE_2D, 3, width, height, GL_RGB, GL_UNSIGNED_BYTE, img.GetImage() );
	
	return tex_id;
}

bool MGLApp::LoadData()
{
// LOADING TEXTURES	
	m_iCaptionTexture = LoadTexture("Data/Textures/caption.tga");
	m_iRenderModeTextures[0] = LoadTexture("Data/Textures/0.tga");
	m_iRenderModeTextures[1] = LoadTexture("Data/Textures/1.tga");
	m_iRenderModeTextures[2] = LoadTexture("Data/Textures/2.tga");
	m_iRenderModeTextures[3] = LoadTexture("Data/Textures/3.tga");
	m_iRenderModeTextures[4] = LoadTexture("Data/Textures/4.tga");

// LOADING MODELS	
	m_pWheel[0] = CPrimitiveFactory::LoadPrimitive("Data/0.txt");
	m_pWheel[1] = CPrimitiveFactory::LoadPrimitive("Data/1.txt");
	m_pWheel[2] = CPrimitiveFactory::LoadPrimitive("Data/2.txt");

	for(int i=0; i<3; i++) if (m_pWheel[i] == NULL) return false;
	
// LOADING SHADERS
	m_iMaterialShader	= LoadShader("Data/Shaders/material.vert", "Data/Shaders/material.frag");
	m_iSpecularOnlyShader = LoadShader("Data/Shaders/specular_only.vert", "Data/Shaders/specular_only.frag");
	m_iBlurShader		= LoadShader("Data/Shaders/blur.vert", "Data/Shaders/blur.frag");
	m_iGlowShader		= LoadShader("Data/Shaders/glow.vert", "Data/Shaders/glow.frag");

	GLint	radius_x = glGetUniformLocationARB(m_iBlurShader, "radius_x"),
			radius_y = glGetUniformLocationARB(m_iBlurShader, "radius_y");
	
	glUseProgramObjectARB(m_iBlurShader);
	if(radius_x>=0 && radius_y>=0)
	{
		glUniform1fARB(radius_x, 1.5 / pbuffer[0]->GetWidth() );
		glUniform1fARB(radius_y, 1.5 / pbuffer[0]->GetHeight() );
	}

	return m_iMaterialShader&& m_iSpecularOnlyShader && m_iBlurShader && m_iGlowShader;
}
