/*************************************************************************
MGLApp.h

Demo for "Glow Effect" section

Author   :   Andrei Gradinari
Based on Dave Astle's application template

Written for More OpenGL Game Programming
August, 2005
*************************************************************************/

#ifndef MGLAPP_H_INCLUDED
#define MGLAPP_H_INCLUDED

#ifdef WIN32
  #define WIN32_LEAN_AND_MEAN
  #define WIN32_EXTRA_LEAN
  #include <windows.h>
#endif

#include <gl/glee.h>
#include <gl/glu.h>

#include "../../_common/MGLTargaImage.h"
#include "../../_common/MGLTimer.h"
#include "../../_common/MGLTypes.h"
#include "../../_common/MGLVector.h"
#include "../../_common/MGLUtil.h"
#include "../../_common/prim.h"

#include "pbuffer.h"


// values used to set up the window and rendering context
const char MGL_APP_NAME[]	= "MOGLGP - Chapter Special Effects - Glow Effect Demo";

const int MGL_WINDOW_WIDTH = 800;
const int MGL_WINDOW_HEIGHT = 600;

const int MGL_COLOR_BITS = 32;
const int MGL_DEPTH_BITS = 32;
const int MGL_STENCIL_BITS = 0;

const bool MGL_FORCE_FULLSCREEN = false;

class MGLApp
{
public:
	MGLApp();
	virtual ~MGLApp();

	bool Init();
	bool Shutdown();
	
	void OnSize(int width, int height );
	void OnKeyDown(int nVirtKey, int lKeyData);

	void SetupProjection( int width, int height );
	void Update( double dt );
	void Render();
	void RenderGUI();



	static GLuint LoadTexture(char* fname, GLenum wrapst = GL_REPEAT, bool mipmap = true);

private:
	
	void SetupLighting();
	void RenderGeometry(bool lit = true);
	
	static GLhandleARB LoadShader(const char* s_vert, const char* s_frag);
	bool LoadData(); // load shaders and textures
	
	double m_dElapTime;
	int m_nWindowWidth, m_nWindowHeight;
	
	GLuint m_iEarthTexture, 
		m_iCaptionTexture, m_iRenderModeTextures[5];
	GLhandleARB m_iMaterialShader, m_iBlurShader, m_iGlowShader;
	
	PBuffer *pbuffer[2];
	GLuint m_iRenderTexture;

	Primitive * m_pEarth;
};

#endif // MGLAPP_H_INCLUDED