#include "Camera.h"

namespace MGL {

CCamera :: CCamera()
{
	p[0] = Vector3(0,0,0);
	p[1] = Vector3(1,0,0);
	p[2] = Vector3(0,1,0);
	p[3] = Vector3(0,0,1);
}

// N - plane normal
// A - arbitrary point belonging to plane
void CCamera :: Reflect(Vector3 A, Vector3 N)
{
	for(int i=0; i<4; i++)
		p[i] = reflect_point_about_plane(p[i], A, N);
}

void CCamera :: Apply()
{
	Vector3	v_right(p[0],p[1]), 
			v_up(p[0],p[2]),
			v_view(p[0],p[3]), // vector to viewer
			pos(p[0]);

	float mat[16] =	{
		v_right[0],	v_up[0], v_view[0], pos[0], 
		v_right[1],	v_up[1], v_view[1], pos[1], 
		v_right[2],	v_up[2], v_view[2], pos[2], 
		0, 0, 0, 1
	};

	mat4 bas(mat);
	bas = bas.transpose();
	bas = bas.inverse();
	glMultMatrixf((GLfloat*)&bas);
}

void CCamera :: LookAt(Vector3 pos, Vector3 at, Vector3 v_up)
{
	Vector3 v_view(at, pos);
	Vector3 v_right = v_up.CrossProduct(v_view);
	v_up = v_view.CrossProduct(v_right);
	
	v_view.Normalize();
	v_right.Normalize();	
	v_up.Normalize();

	p[0] = pos;
	p[1] = pos + v_right;
	p[2] = pos + v_up;
	p[3] = pos + v_view;
}

} // namespace MGL