/*
 *	This file is part of demo program made to accompany
 *	"More OpenGL Game Programming" book's chapter
 *	titled "Robust Multi-Level Reflections".
 *
 *	Author: Maciej Sawitus, 2005
 *	E-mail: msawitus@gmail.com
 *
 */

#include "common.h"

#include "Camera.h"
#include "Application.h"

cCamera::cCamera(const string& name, cApplication* app) :
	Coords(0, 0, 0),
	Speed(0, 0, 0),
	Angles(0, 0, 0),
	MouseSensitivity(30),
	Name(name),
	App(app)
{
	loadState();
}

cCamera::~cCamera()
{
	saveState();
}

void cCamera::update()
{
	// Viewing angles
	Angles[0] -= App->getMouseMovement()[1] * MouseSensitivity * App->getFrameTime();
	Angles[1] -= App->getMouseMovement()[0] * MouseSensitivity * App->getFrameTime();

	// Clamp angles
	if (Angles[0] < -90)
		Angles[0] = -90;
	if (Angles[0] > 90)
		Angles[0] = 90;
	if (Angles[1] > 360)
	{
		double a;
		float b = (float) modf(Angles[1], &a);
		Angles[1] = (((int) a) % 360) + b;
	}
	if (Angles[1] < 0)
	{
		double a;
		float b = (float) modf(-Angles[1], &a);
		Angles[1] = -(((int) a) % 360) - b;
	}

	cVec3 direction(0, 0, 0);

	// Forward / Back
	if (App->isKeyPressed(SDLK_s))
	{
		direction[0] += -(float) sin(Angles[1] * PIOVER180);
		direction[2] += -(float) cos(Angles[1] * PIOVER180);
	}
	else if (App->isKeyPressed(SDLK_x))
	{
		direction[0] += (float) sin(Angles[1] * PIOVER180);
		direction[2] += (float) cos(Angles[1] * PIOVER180);
	}

	// Strafe right / Strafe left
	if (App->isKeyPressed(SDLK_c))
	{
		direction[0] +=  (float) cos(Angles[1] * PIOVER180);
		direction[2] += -(float) sin(Angles[1] * PIOVER180);
	}
	else if (App->isKeyPressed(SDLK_z))
	{
		direction[0] += -(float) cos(Angles[1] * PIOVER180);
		direction[2] += (float) sin(Angles[1] * PIOVER180);
	}

	// Up / Down
	if (App->isKeyPressed(SDLK_q))
		direction[1] += 1;
	else if (App->isKeyPressed(SDLK_a))
		direction[1] += -1;

	// Normalize direction
	direction.normalize();

	// Update camera position
	Coords += direction * 10 * App->getFrameTime();
}

void cCamera::setView() const
{
	glLoadIdentity();
	glRotatef(-Angles[2], 0, 0, 1);
	glRotatef(-Angles[0], 1, 0, 0);
	glRotatef(-Angles[1], 0, 1, 0);
	glTranslatef(-Coords[0], -Coords[1], -Coords[2]);
}

bool cCamera::loadState()
{
	FILE* f = fopen(("../media/conf/" + Name + ".camera").data(), "rb");
	if (f == NULL)
		return false;
	fread(&Coords, sizeof(cVec3), 1, f);
	fread(&Angles, sizeof(cVec3), 1, f);
	fclose(f);
	return true;
}

bool cCamera::saveState()
{
	FILE* f = fopen(("../media/conf/" + Name + ".camera").data(), "wb");
	if (f == NULL)
		return false;
	fwrite(&Coords, sizeof(cVec3), 1, f);
	fwrite(&Angles, sizeof(cVec3), 1, f);
	fclose(f);
	return true;
}