/*
 *	This file is part of demo program made to accompany
 *	"More OpenGL Game Programming" book's chapter
 *	titled "Robust Multi-Level Reflections".
 *
 *	Author: Maciej Sawitus, 2005
 *	E-mail: msawitus@gmail.com
 *
 */

// Predefinitions
class cApplication;

/**
 *	Simple camera class.
 *	This camera is capable of saving and restoring its position
 *	and viewing angles with each application restart.
 */
class cCamera
{
public:

	/**
	 *	Constructor.
	 *	Loads camera state from file (if such exists).
	 *
	 *	@param name name of the camera (under same name
	 *		camera config file is being expected)
	 *	@param app application this camera works for
	 *		(needed for mouse input and timer)
	 */
	cCamera(const string& name, cApplication* app);

	/**
	 *	Saves camera state to file.
	 */
	~cCamera();

	/**
	 *	Performs update of itself.
	 */
	void update();

	/**
	 *	Sets viewing angles and position of the camera using OpenGL transformations.
	 */
	void setView() const;

	/**
	 *	Returns coordinates of the camera.
	 */
	inline const cVec3& getCoords() const
	{
		return Coords;
	}

	/**
	 *	Returns viewing angles of the camera.
	 */
	inline const cVec3& getAngles() const
	{
		return Angles;
	}

	/**
	 *	Returns application this camera was created for.
	 */
	inline cApplication* getApp() const
	{
		return App;
	}

protected:

	/**
	 *	Loads state from file.
	 *	State includes: coordinates, speed and viewing angles.
	 *	In case state file doesn't exist, it's created.
	 *	@return true on success, false otherwise.
	 */
	bool loadState();

	/**
	 *	Saves state to file.
	 *	@return true on success, false otherwise.
	 */
	bool saveState();

	/// Application this camera works for
	cApplication* App;

	/// Name of the camera (and so the name of camera config file)
	string Name;

	/// World coordinates
	cVec3 Coords;

	/// Speed vector
	cVec3 Speed;

	/// Viewing angles (x, y, z)
	cVec3 Angles;

	/// Mouse sensitivity value (10 is reasonable)
	float MouseSensitivity;
};