/*
 *	This file is part of demo program made to accompany
 *	"More OpenGL Game Programming" book's chapter
 *	titled "Robust Multi-Level Reflections".
 *
 *	Author: Maciej Sawitus, 2005
 *	E-mail: msawitus@gmail.com
 *
 */

#ifndef MGLAPP_H_INCLUDED
#define MGLAPP_H_INCLUDED

// Values used to set up the window and rendering context
const char MGL_APP_NAME[] = "MOGLGP - Part II, Chapter 7 - Robust Multi-Level Reflections";

const int MGL_WINDOW_WIDTH = 800;
const int MGL_WINDOW_HEIGHT = 600;

const int MGL_COLOR_BITS = 32;
const int MGL_DEPTH_BITS = 24;
const int MGL_STENCIL_BITS = 8;

enum eFullScreenMode
{
	MGL_FULLSCREEN_FALSE = 0,
	MGL_FULLSCREEN_TRUE,
	MGL_FULLSCREEN_CHOOSE
};

const int MGL_FULLSCREEN_MODE = MGL_FULLSCREEN_CHOOSE;

// Predefinitions
class cSimpleScene;
class cCamera;
class cRenderer;

/**
 *	Application showing multi-level mirror reflections.
 */
class MGLApp : public cApplication
{
public:

	/**
	 *	Creates my application.
	 *	@param fullScreen true to enable full screen mode, false otherwise
	 */
	MGLApp(bool fullScreen);

	virtual ~MGLApp() {}

	/*------------ cApplication implementation -------------*/

	virtual bool init();

	virtual bool shutdown();

	/*
	 *	This method was excluded from common template, because
	 *	here it's camera object that sets projection.
	 *
	 *	void setupProjection(int width, int height);
	 */

	virtual void update(double deltaTime);

	virtual void render();

protected:

	/**
	 *	Renders basic demo information including keys meaning.
	 */
	void renderInfo();

	/*------------------ Attributes ----------------*/

	/// Renderer used to render the scene
	cRenderer* Renderer;

	/// Current scene
	cSimpleScene* Scene;

	/// Viewer's camera
	cCamera* Camera;

	/**
	 *	Scene and other settings.
	 */
	struct cSettings
	{
		/**
		 *	Sets default settings.
		 */
		void setDefaults();

		/// Shall mirror cube be rendered?
		bool ShowMirrorCube;

		/// Shall mirror cube be rotating?
		bool RotateMirrorCube;

		/// Show info screen?
		bool ShowInfoScreen;
	};

	/// Application settings
	cSettings Settings;

};

#endif // MGLAPP_H_INCLUDED