/*
 *	This file is part of demo program made to accompany
 *	"More OpenGL Game Programming" book's chapter
 *	titled "Robust Multi-Level Reflections".
 *
 *	Author: Maciej Sawitus, 2005
 *	E-mail: msawitus@gmail.com
 *
 */

#include "common.h"

#include "Material.h"
#include "Mesh.h"
#include "Renderer.h"

cMesh::cMesh() :
	Material(NULL)
{
}

void cMesh::render() const
{
	// Wire / debug mode only
	if (cRenderer::RenderMode.IsWireMode)
	{
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
		glDisable(GL_DEPTH_TEST);
		glDepthMask(GL_FALSE);
		glDisable(GL_BLEND);

		if (Material->isMirrorSurface())
		{
			glColor3f(1, 0, 0);
			glDisable(GL_TEXTURE_2D);
		}
	}

	// Set up vertex coordinate, normal and texture coordinate arrays
	glVertexPointer(3, GL_FLOAT, sizeof(cVec3), &Coords.front());
	glTexCoordPointer(2, GL_FLOAT, sizeof(cVec2), &TexCoords.front());

	// Draw triangles using given array as indices
	glDrawElements(GL_TRIANGLES, (GLsizei) Indices.size(), GL_UNSIGNED_SHORT, &Indices.front());

	// Wire / debug mode only
	if (cRenderer::RenderMode.IsWireMode)
	{
		if (Material->isMirrorSurface())
		{
			glColor3f(1, 1, 1);
			glEnable(GL_TEXTURE_2D);
		}

		glDepthMask(GL_TRUE);
		glEnable(GL_DEPTH_TEST);
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	}
}

void cMesh::renderSimple() const
{
	// Disable normal and texture coordinate arrays which are enabled by default
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);

	// Set up vertex coordinate array
	glVertexPointer(3, GL_FLOAT, sizeof(cVec3), &Coords.front());

	// Draw triangles using given array as indices
	glDrawElements(GL_TRIANGLES, (GLsizei) Indices.size(), GL_UNSIGNED_SHORT, &Indices.front());

	// Enable back normal and texture coordinate arrays
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
}

void cMesh::recalculateBoundingVolume()
{
	BoundingVolume.calculate(Coords);
}

void cMesh::flipTriangles()
{
	unsigned int i;
	unsigned short temp;

	for (i = 0; i < Indices.size(); i += 3)
	{
		temp = Indices[i + 1];
		Indices[i + 1] = Indices[i + 2];
		Indices[i + 2] = temp;
	}
}

void cMesh::scaleTextureCoords(float scale)
{
	unsigned int i;
	for (i = 0; i < TexCoords.size(); i++)
		TexCoords[i] *= scale;
}

bool cMesh::greater(cMesh* a, cMesh* b)
{
	return cMaterial::compare(a->Material, b->Material) > 0;
}