/*
 *	This file is part of demo program made to accompany
 *	"More OpenGL Game Programming" book's chapter
 *	titled "Robust Multi-Level Reflections".
 *
 *	Author: Maciej Sawitus, 2005
 *	E-mail: msawitus@gmail.com
 *
 */

#include "common.h"

#include "Mesh.h"
#include "Material.h"
#include "Mirror.h"

cMirror::cMirror(const vector<cVec3>& verts, cMesh* mesh) :
	Mesh(mesh)
{
	setBoundingVerts(verts);
}

cMirror::cMirror(cMesh* mesh) :
	Mesh(mesh)
{
}

void cMirror::setBoundingVerts(const vector<cVec3>& verts)
{
	Verts = verts;

	if (Verts.size() != 4)
		throw cException("cMirror::cMirror(): Invalid bounding verts' count to create mirror (expected 4 verts).");

	// Calculate mirror's plane
	Plane.build(Verts[0], Verts[1], Verts[2]);
}

const cBoundingVolume& cMirror::getBoundingVolume() const
{
	return Mesh->getBoundingVolume();
}

float cMirror::getTransparency() const
{
	return Mesh->getMaterial()->getTransparency();
}