/*
 *	This file is part of demo program made to accompany
 *	"More OpenGL Game Programming" book's chapter
 *	titled "Robust Multi-Level Reflections".
 *
 *	Author: Maciej Sawitus, 2005
 *	E-mail: msawitus@gmail.com
 *
 */

#ifndef _MIRROR_H_
#define _MIRROR_H_

// Predefinitions
class cMesh;
class cApplication;

/**
 *	Mirror object representation.
 *	<p>
 *	Note that mirror surface may consist of multiple co-planar
 *	polygons. To handle all of them at once, one has to specify
 *	bounding quad (4 vertices) of the mirror surface.
 */
class cMirror
{
public:

	/**
	 *	Creates mirror.
	 *	@param verts 4 bounding vertices of a mirror;
	 *		these verts are assumed to be co-planar and also
	 *		they should be bounding rectangle (portal) for all
	 *		of the mirror's polygons
	 *	@param mesh mesh of the mirror (blended on top of the
	 *		mirror surface)
	 */
	cMirror(const cVec3Array& verts, cMesh* mesh);

	/**
	 *	Creates mirror.
	 *	@param mesh mesh of the mirror (blended on top of the
	 *		mirror surface)
	 */
	cMirror(cMesh* mesh);

	/*---------------- Accessors -----------------*/

	void setBoundingVerts(const vector<cVec3>& verts);
	inline const vector<cVec3>& getBoundingVerts() const { return Verts; }
	inline const cPlane& getPlane() const { return Plane; }
	inline cMesh* getMesh() const { return Mesh; }
	const cBoundingVolume& getBoundingVolume() const;
	float getTransparency() const;

protected:

	/// Four corners (vertices) of a bounding mirror's rectangle
	vector<cVec3> Verts;

	/// Plane of the mirror
	cPlane Plane;

	/// Mesh covering the mirror (used only to determine stencil buffer region)
	cMesh* Mesh;
};

#endif //_MIRROR_H_