/*************************************************************************
MGLApp.cpp

Main application class.

Author   :   Dave Astle

Written for More OpenGL Game Programming
*************************************************************************/

#ifdef WIN32
	#define WIN32_LEAN_AND_MEAN
	#define WIN32_EXTRA_LEAN
	#include <windows.h>
#endif

#include "MGLApp.h"
#include <cstdlib>

MGLApp::MGLApp()
{
  m_snowstorm = NULL;
  m_list = 0;
}

MGLApp::~MGLApp()
{
}

bool MGLApp::Init()
{	
	if (!GLeeInit())
		return false;

  m_skybox.Initialize(100.0f);
  if (!m_skybox.LoadTextures("skybox\\snowy_up.tga", "skybox\\snowy_dn.tga", "skybox\\snowy_ft.tga", "skybox\\snowy_bk.tga", "skybox\\snowy_lf.tga", "skybox\\snowy_rt.tga"))
    return false;

  // add fog
  float fogColor[] = {0.5, 0.5, 0.5, 1.0};

  glFogfv(GL_FOG_COLOR, fogColor);
  glFogf(GL_FOG_DENSITY, 0.1f);

  glEnable(GL_DEPTH_TEST);
  glEnable(GL_TEXTURE_2D);

  // set up the particle system
  m_snowstorm = new Snowstorm(5000, Vector3(0.0, 0.0, -5.0), 5.0, 5.0, 5.0);
  m_snowstorm->InitializeSystem();

  // artificially cause time to elapse so we start with snow on the screen
  for (int i = 0; i < 100; i++)
    m_snowstorm->Update(0.1f);

	return true;
}

bool MGLApp::Shutdown()
{
  m_skybox.Release();
  m_snowstorm->KillSystem();
  delete m_snowstorm;
  m_snowstorm = NULL;

  glDeleteLists(m_list, 1);
  m_list = 0;

	return true;
}

void MGLApp::SetupProjection(int width, int height)
{
	// avoid div by zero
	if( height == 0 )
		height = 1;					

	// set up GL viewport
	glViewport( 0, 0, width, height );

	// set up a perspective projection
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0f, (GLfloat)width/(GLfloat)height, 1.0f, 1000.0f );
}

void MGLApp::Update( double dt )
{
  m_snowstorm->Update((float)dt);
}

void MGLApp::Render()
{
	glClear( GL_DEPTH_BUFFER_BIT );

	// set up the camera
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
  gluLookAt( 0.0, 1.0, 1.0, 0.0, 1.0, 0.0, 0.0, 1.0, 0.0);

  m_skybox.Render(0.0, 0.0, 0.0);

  if (m_list == 0)
  {
    m_list = glGenLists(1);
    glNewList(m_list, GL_COMPILE_AND_EXECUTE);

    glEnable(GL_FOG);
    glDisable(GL_TEXTURE_2D);
    // draw a white ground plane
    glColor3f(1.0f, 1.0f, 1.0f);
    glBegin(GL_QUADS);
    for (int x = -50; x < 50; x += 5)
      for (int z = -50; z < 50; z += 5)
      {
        srand(x+z);
        glVertex3i(x, -(rand()%2), z);
        srand(x+z+5);
        glVertex3i(x, -(rand()%2), z + 5);
        srand(x+z+10);
        glVertex3i(x + 5, -(rand()%2), z + 5);
        srand(x+z+5);
        glVertex3i(x + 5, -(rand()%2), z);
      }
    glEnd();
    glEnable(GL_TEXTURE_2D);
    glEndList();
  }
  else
    glCallList(m_list);

  m_snowstorm->Render();
  glDisable(GL_FOG);
}
