/****************************************************************************
 Particles.cpp
 
 Author   :   Dave Astle
 Date     :   2/1/2001

 Written for OpenGL Game Programming
*****************************************************************************/
#include "Particles.h"


/*****************************************************************************
 ParticleSystem()
 
 Store initialization values and set defaults.
*****************************************************************************/
ParticleSystem::ParticleSystem(int maxParticles, Vector3 origin)
{
  m_maxParticles = maxParticles;
  m_origin = origin;
  m_particleList = NULL;
} // end ParticleSystem()


/*****************************************************************************
 ParticleSystem::Emit()
 
 Creates the number of new particles specified by the parameter, using
 the general particle system values with some random element. Note that
 only initial values will be randomized. Final values will not. This may
 be changed in the future.
*****************************************************************************/
int ParticleSystem::Emit(int numParticles)
{
  // create numParticles new particles (if there's room)
  while (numParticles && (m_numParticles < m_maxParticles))
  {
    // initialize the current particle and increase the count
    InitializeParticle(m_numParticles++);
    --numParticles;
  }
  return numParticles;
} // end ParticleSystem::Emit


/*****************************************************************************
 ParticleSystem::InitializeSystem()
 
 Allocate memory for the maximum number of particles in the system
*****************************************************************************/
void ParticleSystem::InitializeSystem()
{
  // if this is just a reset, free the memory
  if (m_particleList)
  {
    delete[] m_particleList;
    m_particleList = NULL;
  }

  // allocate the maximum number of particles
  m_particleList = new Particle[m_maxParticles];

  // reset the number of particles and accumulated time
  m_numParticles = 0;
  m_accumulatedTime = 0.0f;
} // end ParticleSystem::InitializeSystem


/*****************************************************************************
 ParticleSystem::KillSystem()
 
 Tells the emitter to stop emitting.
*****************************************************************************/
void ParticleSystem::KillSystem()
{
  if (m_particleList)
  {
    delete[] m_particleList;
    m_particleList = NULL;
  }

  m_numParticles = 0;
} // end ParticleSystem::KillSystem