/****************************************************************************
 Particles.h  Particle and particle system base classes.
 
 Author   :   Dave Astle
 Date     :   2/1/2001

 Written for OpenGL Game Programming
*****************************************************************************/
#ifndef PARTICLES_H_INCLUDED
#define PARTICLES_H_INCLUDED


/********************************* Includes *********************************/
#include <iostream>
#include <GL/glee.h>
#include <GL/glu.h>
#include "../../_common/MGLVector.h"

using MGL::Vector3;

struct Particle
{
  Vector3  m_pos;           // current position of the particle
  Vector3  m_prevPos;       // last position of the particle
  Vector3  m_velocity;      // direction and speed
  Vector3  m_acceleration;  // acceleration

  float   m_energy;           // determines how long the particle is alive

  float   m_size;             // size of particle
  float   m_sizeDelta;        // amount to change the size over time

  float   m_weight;           // determines how gravity affects the particle
  float   m_weightDelta;      // change over time

  float   m_color[4];         // current color of the particle
  float   m_colorDelta[4];    // how the color changes with time
};


class ParticleSystem
{
public:
	ParticleSystem(int maxParticles, Vector3 origin);

  // abstract functions
  virtual void  Update(float elapsedTime)     = 0;
  virtual void  Render()                      = 0;

  virtual int   Emit(int numParticles);

  virtual void  InitializeSystem();
  virtual void  KillSystem();

protected:
  virtual void  InitializeParticle(int index) = 0;
  Particle *m_particleList;   // particles for this emitter
  int       m_maxParticles;   // maximum number of particles in total
  int       m_numParticles;   // indicies of all free particles
  Vector3   m_origin;         // center of the particle system

  float     m_accumulatedTime;  // used to track how long since the last particle was emitted

  Vector3   m_force;          // force (gravity, wind, etc.) acting on the particle system
};


#endif // PARTICLES_H_INCLUDED
