#ifdef _WINDOWS
  #define WIN32_LEAN_AND_MEAN
  #define WIN32_EXTRA_LEAN
  #include <windows.h>
#endif


#include "../../_common/MGLTargaImage.h"
#include "Skybox.h"
#include <gl/glu.h>


Skybox::Skybox()
{
  for (int idx = 0; idx < 6; idx++)
    m_textures[idx] = 0;
  m_list = 0;
}


Skybox::~Skybox()
{
  Release();
}


void Skybox::Initialize(float size)
{
  m_size = size;
}


bool Skybox::LoadTextures(char* top, char* bottom, char* front, char* back, char* left, char* right)
{
  MGL::TargaImage image;
  glGenTextures(6, m_textures);

  if (!image.Load(top))
    return false;
  glBindTexture(GL_TEXTURE_2D, m_textures[SKY_FRONT]);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, image.GetWidth(), image.GetHeight(), 0, image.GetFormat(), image.GetType(), image.GetImage());
  image.Release();

  if (!image.Load(bottom))
    return false;
  glBindTexture(GL_TEXTURE_2D, m_textures[SKY_FRONT]);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, image.GetWidth(), image.GetHeight(), 0, image.GetFormat(), image.GetType(), image.GetImage());
  image.Release();

  if (!image.Load(front))
    return false;
  glBindTexture(GL_TEXTURE_2D, m_textures[SKY_FRONT]);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, image.GetWidth(), image.GetHeight(), 0, image.GetFormat(), image.GetType(), image.GetImage());
  image.Release();

  if (!image.Load(back))
    return false;
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, image.GetWidth(), image.GetHeight(), 0, image.GetFormat(), image.GetType(), image.GetImage());
  image.Release();

  if (!image.Load(left))
    return false;
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, image.GetWidth(), image.GetHeight(), 0, image.GetFormat(), image.GetType(), image.GetImage());
  image.Release();

  if (!image.Load(right))
    return false;
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, image.GetWidth(), image.GetHeight(), 0, image.GetFormat(), image.GetType(), image.GetImage());
  image.Release();

  return true;
}


void Skybox::Render(float cameraX, float cameraY, float cameraZ)
{
  glPushMatrix();

  // Move the skybox so that it's centered on the camera.
  glTranslatef(cameraX, cameraY, cameraZ);


  glPushAttrib(GL_FOG_BIT | GL_DEPTH_BUFFER_BIT | GL_LIGHTING_BIT | GL_TEXTURE_BIT);

  if (m_list == 0)
  {
    m_list = glGenLists(1);
    glNewList(m_list, GL_COMPILE_AND_EXECUTE);

    glDisable(GL_FOG);
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_LIGHTING);
    glEnable(GL_TEXTURE_2D);
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

    // Top
    glBindTexture(GL_TEXTURE_2D, m_textures[SKY_TOP]);
    glBegin(GL_QUADS);
    glTexCoord2f(1.0f, 1.0f); glVertex3f(-m_size, m_size, -m_size);
    glTexCoord2f(0.0f, 1.0f); glVertex3f(m_size, m_size, -m_size);
    glTexCoord2f(0.0f, 0.0f); glVertex3f(m_size, m_size, m_size);
    glTexCoord2f(1.0f, 0.0f); glVertex3f(-m_size, m_size, m_size);
    glEnd();

    // Bottom
    glBindTexture(GL_TEXTURE_2D, m_textures[SKY_BOTTOM]);
    glBegin(GL_QUADS);
    glTexCoord2f(1.0f, 1.0f); glVertex3f(m_size, -m_size, -m_size);
    glTexCoord2f(0.0f, 1.0f); glVertex3f(-m_size, -m_size, -m_size);
    glTexCoord2f(0.0f, 0.0f); glVertex3f(-m_size, -m_size, m_size);
    glTexCoord2f(1.0f, 0.0f); glVertex3f(m_size, -m_size, m_size);
    glEnd();

    // Front
    glBindTexture(GL_TEXTURE_2D, m_textures[SKY_FRONT]);
    glBegin(GL_QUADS);
    glTexCoord2f(0.0f, 0.0f); glVertex3f(-m_size, -m_size, -m_size);
    glTexCoord2f(1.0f, 0.0f); glVertex3f(m_size, -m_size, -m_size);
    glTexCoord2f(1.0f, 1.0f); glVertex3f(m_size, m_size, -m_size);
    glTexCoord2f(0.0f, 1.0f); glVertex3f(-m_size, m_size, -m_size);
    glEnd();

    // Back
    glBindTexture(GL_TEXTURE_2D, m_textures[SKY_BACK]);
    glBegin(GL_QUADS);
    glTexCoord2f(0.0f, 0.0f); glVertex3f(m_size, -m_size, m_size);
    glTexCoord2f(1.0f, 0.0f); glVertex3f(-m_size, -m_size, m_size);
    glTexCoord2f(1.0f, 1.0f); glVertex3f(-m_size, m_size, m_size);
    glTexCoord2f(0.0f, 1.0f); glVertex3f(m_size, m_size, m_size);
    glEnd();

    // Right
    glBindTexture(GL_TEXTURE_2D, m_textures[SKY_RIGHT]);
    glBegin(GL_QUADS);
    glTexCoord2f(1.0f, 0.0f); glVertex3f(m_size, -m_size, m_size);
    glTexCoord2f(1.0f, 1.0f); glVertex3f(m_size, m_size, m_size);
    glTexCoord2f(0.0f, 1.0f); glVertex3f(m_size, m_size, -m_size);
    glTexCoord2f(0.0f, 0.0f); glVertex3f(m_size, -m_size, -m_size);
    glEnd();

    // Left
    glBindTexture(GL_TEXTURE_2D, m_textures[SKY_LEFT]);
    glBegin(GL_QUADS);
    glTexCoord2f(1.0f, 0.0f); glVertex3f(-m_size, -m_size, -m_size);
    glTexCoord2f(1.0f, 1.0f); glVertex3f(-m_size, m_size, -m_size); 
    glTexCoord2f(0.0f, 1.0f); glVertex3f(-m_size, m_size, m_size);
    glTexCoord2f(0.0f, 0.0f); glVertex3f(-m_size, -m_size, m_size);
    glEnd();

    glEndList();
  }
  else
    glCallList(m_list);
  
  glPopAttrib();
  glPopMatrix();
}


void Skybox::Release()
{
  for (int i = 0; i < 6; ++i)
    glDeleteTextures(6, &m_textures[0]);
  glDeleteLists(m_list, 1);
  m_list = 0;
}