/*************************************************************************
MGLApp.h

Main application class.

Author   :   Dave Astle

Written for More OpenGL Game Programming
-------------------------------------------------------------------------
| dd/mm/yyyy  |  Programmer        | Explanation of change              |
-------------------------------------------------------------------------
| 09/10/2005  |  J.Villagrana   @1 | -Changed MGL_APP_NAME and added    | 
|             |                    |  particle system object			|
| 01/11/2005  |  J.Villagrana   @2 | -Changed hardcoded default psystems| 
*************************************************************************/

#ifndef MGLAPP_H_INCLUDED
#define MGLAPP_H_INCLUDED

#ifdef WIN32
  #define WIN32_LEAN_AND_MEAN
  #define WIN32_EXTRA_LEAN
  #include <windows.h>
#endif

//#include <gl/glee.h> @1D
#include <GLee/glee.h> //@1A
#include <gl/glu.h>

#include "./Common/MGLTimer.h"						//@1A
#include "./Particles/CParticle_Sys.h"				//@1A
#include "./Particles/CParticleSys_Scriptable.h"	//@1A
#include "./Scripting/CParticleScript.h"			//@1A
#include "./Text/GLFont.h"							//@1A

// values used to set up the window and rendering context
const char MGL_APP_NAME[]	= "MOGLGP - Chapter 7 - Scripted Particle Systems - J.Villagrana"; //@1C

const int MGL_WINDOW_WIDTH = 800;
const int MGL_WINDOW_HEIGHT = 600;

const int MGL_COLOR_BITS = 32;
const int MGL_DEPTH_BITS = 32;
const int MGL_STENCIL_BITS = 0;

const bool MGL_FORCE_FULLSCREEN = false; //@1C - changed to false

// ************* @1A BEGINS **************
/* --------------------------------------
#define MAX_INDEX_DEMO_SCRIPTS 5
const char particle_scripts[][30] = {	"data/test1.psys",
										"data/test2.psys",
										"data/test3.psys",
										"data/test4.psys",
										"data/test5.psys",
										"data/test6.psys"};
----------------------------------- @2D	*/
// ************** @1A ENDS ***************

class MGLApp
{
public:
	MGLApp();
	virtual ~MGLApp();

	bool Init();
	bool Shutdown();
	void SetupProjection( int width, int height );
	void Update( double dt );
	void Render();

	void ReinitParticleSystem();			//@1A
	void NextScript(char *pstrFileName, char *pstrTitleName); //@1A //@2C

private:
	bool LoadScript(char *filename);		//@1A
	void Draw2DBegin();						//@1A
	void Draw2DEnd();						//@1A
	void RenderInfo();						//@1A

	CParticleScript		*m_pScript;			//@1A
	CParticle_Sys		*m_pParticleSystem;	//@1A
	MGL::Timer			m_timer;			//@1A
	GLFont				m_font;				//@1A
	int					m_currentScript;	//@1A
	char				m_scriptTitle[100]; //@2A
};

#endif // MGLAPP_H_INCLUDED
