/*************************************************************************/
/*|=====================================================================|*/
/*| Filename: CParticle.cpp                                             |*/
/*| Author: Joel Villagrana    (joel.villagrana@gmail.com)              |*/
/*|                                                                     |*/
/*|%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%|*/
/*| dd/mm/yyyy | Programmer        | Modifications                      |*/
/*|=====================================================================|*/
/*| 17/05/2005 | JVillagrana       | Initial Version                    |*/
/*|---------------------------------------------------------------------|*/
/*************************************************************************/

#include "CParticle.h"

// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
// ========================================================================
// IMPLEMENTATION OF CPARTICLE CLASS 
// ========================================================================
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

// 
// Constructor
// 
CParticle::CParticle()
{
	m_life = 1.0f;
	m_size = 1.0f;
	m_originalSize = 1.0f;
	m_weight = 1.0f;
	m_direction.Set(0.0f, 1.0f, 0.0f);	
	m_timeEmitted = 0.0f;
}

// 
// Destructor
// 
CParticle::~CParticle()
{
}

// 
// Initialize a particle
// 
void CParticle::InitParticle(CColour c, MGL::Vector3 pos, MGL::Vector3 dir,
								float life, float size, float timeEmitted)
{
	m_position	  = pos;
	m_prevPos	  = pos;
	m_direction	  = dir;
	m_colour	  = c;
	m_life		  = life;
	m_size		  = size;
	m_originalSize= size;
	m_timeEmitted = timeEmitted;
}

// 
// Update particle properties
// 
VOID CParticle::Update(GLfloat elapsedTime, int *deadParticle, 
								MGL::Vector3 externalForces, float maxLife)
{
	float timeLived = 0.0f;

	// Update position
	m_position += (m_direction * m_weight * 0.025f);

	// Update direction
	m_direction += (externalForces * 0.05f);
	m_direction.Normalize();

	// Decrease life
	if(elapsedTime > (m_timeEmitted + m_life))
	{
		m_life=0.0f;
		*deadParticle = 1;
	}

	timeLived = elapsedTime - m_timeEmitted;
	if(timeLived < 0.0f)
		timeLived = 0.0f;

	// Decrease alpha according to particle's age
	int newAlpha = (int)(255.0f * ((m_life - (timeLived))/maxLife));

	//int newAlpha = (int)(255.0f * (1.0f - (m_life/maxLife)));
	m_colour.UpdateAlpha(newAlpha);	
}

// 
// Render
// 
VOID CParticle::Render(PARTICLE_REP rep, MGL::Vector3 billbX, 
										MGL::Vector3 billbY, bool textured)
{
	GLfloat r, g, b, a;

	m_colour.Get_RGBA(&r, &g, &b, &a);

	glColor4f( r,g,b,a );
	glPushMatrix();
	
	if((rep==R_POINT_SPRITES)||(rep==R_POINTS))
	{
		glPointSize( m_size );
        glBegin( GL_POINTS );
			glVertex3f( m_position.x,
						m_position.y,
                        m_position.z );
        glEnd();
		glPointSize(1.0f);
	}
	else if(rep==R_QUADS)
	{
		MGL::Vector3 pts[4], mbillbX, mbillbY;

		mbillbX = billbX * (-1);
		mbillbY = billbY * (-1);

		pts[0] = m_position + ((mbillbX + mbillbY) * m_size);
		pts[1] = m_position + ((billbX  + mbillbY) * m_size);
		pts[2] = m_position + ((billbX  + billbY)  * m_size);
		pts[3] = m_position + ((mbillbX + billbY)  * m_size);
		
		glBegin(GL_TRIANGLE_FAN);
			if(textured)
				glTexCoord2f(0.0f, 0.0f);
			glVertex3f(pts[0].x, pts[0].y, pts[0].z);

			if(textured)
				glTexCoord2f(1.0f, 0.0f);
			glVertex3f(pts[1].x, pts[1].y, pts[1].z);

			if(textured)
				glTexCoord2f(1.0f, 1.0f);
			glVertex3f(pts[2].x, pts[2].y, pts[2].z);

			if(textured)
				glTexCoord2f(0.0f, 1.0f);
			glVertex3f(pts[3].x, pts[3].y, pts[3].z);
		glEnd();
	}
	else if(rep==R_LINES)
	{
		glBegin(GL_LINES);
			glColor4f( r, g, b, 0.1f );
			glVertex3f(m_position.x, m_position.y, m_position.z);
			glColor4f( r, g, b, a );
			glVertex3f(	m_position.x + m_direction.x, 
						m_position.y + m_direction.y, 
						m_position.z + m_direction.z);
		glEnd();
	}
	glPopMatrix();
}

// 
// Fade Colour
// 
void CParticle::Fade_Color(CColour finalColor, float elapsedEventTime, float timeRange)
{
	int red = m_colour.Get_RedRGBA();
	int green = m_colour.Get_GreenRGBA();
	int blue = m_colour.Get_BlueRGBA();
	int alpha = m_colour.Get_AlphaRGBA();
	int currRed = red;
	int currGreen = green;
	int currBlue = blue;
	int currAlpha = alpha;

	bool redUp = false;
	bool greenUp = false;
	bool blueUp = false;
	bool alphaUp = false;

	if(red < finalColor.Get_RedRGBA())
		redUp = true;
	if(green < finalColor.Get_GreenRGBA())
		greenUp = true;
	if(blue < finalColor.Get_BlueRGBA())
		blueUp = true;
	if(alpha < finalColor.Get_AlphaRGBA())
		alphaUp = true;

	red	= abs(  (( abs(finalColor.Get_RedRGBA() - red) * elapsedEventTime)  /timeRange) );
	if(redUp)red += currRed;
	else red = currRed - red;
	if(red > 255)red = 255;
	if(red < 0)red = 0;

	green	= abs(  (( abs(finalColor.Get_GreenRGBA() - green) * elapsedEventTime)/timeRange) );
	if(greenUp)green += currGreen;
	else green = currGreen - green;
	if(green > 255)green = 255;
	if(green < 0)green = 0;
	
	blue	= abs(  (( abs(finalColor.Get_BlueRGBA() - blue)  * elapsedEventTime) /timeRange) );
	if(blueUp)blue += currBlue;
	else blue = currBlue - blue;
	if(blue > 255)blue = 255;
	if(blue < 0)blue = 0;
	
	alpha	= abs(  (( abs(finalColor.Get_AlphaRGBA() - alpha) * elapsedEventTime)/timeRange) );
	if(alphaUp)alpha += currAlpha;
	else alpha = currAlpha - alpha;
	if(alpha > 255)alpha = 255;
	if(alpha < 0)alpha = 0;

	m_colour.Set_ColourRGBA(red, green, blue, alpha);
}

// 
// Fade Weight
// 
void CParticle::Fade_Weight(float finalWeight, float elapsedEventTime, float timeRange)
{	
	float newWeight = 0.0f;

	newWeight = ( (elapsedEventTime * (finalWeight))/timeRange );
	if(newWeight > m_weight)
		m_weight = newWeight;

	if(m_weight>finalWeight)
		m_weight = finalWeight;
}

// 
// Fade Size
// 
void CParticle::Fade_Size(float finalSize, float elapsedEventTime, float timeRange)
{	
	float newSize = 0.0f;
	float tempsize = m_size;

	newSize = ( (elapsedEventTime * (finalSize))/timeRange );

	if( ((m_size < finalSize) && (newSize > m_size)) ||
		((m_size > finalSize) && (newSize < m_size)) )
		m_size = newSize;

	if(m_size>finalSize)
		m_size = finalSize;
}


// %%%%%%%%%%%%%%%%%%%%%%%%%%%%% END OF FILE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

