/*************************************************************************/
/*|=====================================================================|*/
/*| Filename: CParticle.h                                               |*/
/*| Author: Joel Villagrana    (joel.villagrana@gmail.com)              |*/
/*|                                                                     |*/
/*|%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%|*/
/*| dd/mm/yyyy | Programmer        | Modifications                      |*/ 
/*|=====================================================================|*/
/*| 17/05/2005 | JVillagrana       | Initial Version                    |*/
/*|---------------------------------------------------------------------|*/
/*************************************************************************/

#ifndef _PARTICLE_H
#define _PARTICLE_H

#include <windows.h>
#include <stdio.h>
#include <assert.h>
#include <glee/glee.h>
#include <gl/gl.h>
#include <gl/glu.h>

#include <vector>
using namespace std;

#include "../Common/MGLVector.h"
//#include "../Math/helper_math.h"
#include "../Util/helper_defs.h"
#include "../Util/CColor.h"

/*$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
=====================================================================================
					Definition of the CParticle Class
=====================================================================================
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$*/
class CParticle
{
	private:
		MGL::Vector3	m_position;			// Position of the particle
		MGL::Vector3	m_prevPos;			// Last position of the particle
		MGL::Vector3	m_direction;		// Direction 
		CColour		m_colour;				// Colour 
		float		m_life;					// Particle Life		
		float		m_size;					// Particle Size
		float		m_originalSize;			// Original particle size
		float		m_weight;				// Particle Weight
		float		m_timeEmitted;			// Emission time

	public:

		CParticle();
		~CParticle();

		void		InitParticle(CColour c, MGL::Vector3 pos, MGL::Vector3 dir, 
									float life, float size, float timeEmitter);

		// CORE METHODS
		// ==============================================================
		void		Update(GLfloat elapsedTime, int *deadParticle, 
									MGL::Vector3 externalForces, float maxLife);
		void		Render(PARTICLE_REP rep, MGL::Vector3 x, MGL::Vector3 y, bool textured);				

		// ACCESSOR FUNCTIONS
		// ==============================================================
		void		Set_Color( CColour c ){			m_colour = c;		}
		void		Get_Color( CColour *c){			*c = m_colour;		}
		void		Fade_Color(CColour finalColor, float elapsedTime, float timeRange);

		void		Set_Position( MGL::Vector3 &v )	{	m_position = v;	}
		void		Get_Position( MGL::Vector3 *v )	{	*v = m_position;}

		void		Set_Direction( MGL::Vector3 &v ){	m_direction = v;}
		void		Get_Direction( MGL::Vector3 *v ){	*v =m_direction;}

		void		Set_Life( float lf )	{		m_life = lf;		}
		float		Get_Life()				{		return	m_life;		}
		bool		IsAlive()	{	return  (m_life>0.0f)?true:false;	}	
		bool		IsDead()	{	return  (m_life==0.0f)?true:false;	}	

		void		Set_Weight( float w )	{		m_weight = w;		}
		void		Get_Weight( float *w )	{		*w = m_weight;		}
		void		Fade_Weight(float finalWeight, float elapsedTime, float timeRange);

		void		Set_Size( float s )		{		m_size = s;			}
		void		Get_Size( float *s)		{		*s = m_size;		}
		void		Fade_Size(float finalSize, float elapsedTime, float timeRange);
};

typedef vector<CParticle*> STLparticleVec;
typedef STLparticleVec::iterator STLparticleVec_it;

#endif _PARTICLE_H


// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% END OF FILE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


