/*************************************************************************/
/*|=====================================================================|*/
/*| Filename: CParticleSys_Scriptable.h                                 |*/
/*| Author: Joel Villagrana    (joel.villagrana@gmail.com)              |*/
/*|                                                                     |*/
/*|%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%|*/
/*| dd/mm/yyyy | Programmer        | Modifications                      |*/ 
/*|=====================================================================|*/
/*| 12/07/2005 | JVillagrana       | Initial Version                    |*/
/*|---------------------------------------------------------------------|*/
/*************************************************************************/

#ifndef _PARTICLESYS_SCRIPTABLE_H
#define _PARTICLESYS_SCRIPTABLE_H

#include <windows.h>
#include <stdio.h>
#include <assert.h>
#include <glee/glee.h>
#include <gl/gl.h>
#include <gl/glu.h>

#include <vector>
using namespace std;

#include "CParticle_Sys.h"
#include "CParticle_Emitter.h"
#include "CParticle_Event.h"
#include "../Scripting/CParticleScript.h"

//#include "../Math/helper_math.h"
#include "../Common/MGLVector.h"
#include "../Util/helper_defs.h"
#include "../Util/CColor.h"
#include "../Texture/textureManager.h"


/*$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
=====================================================================================
					Definition of the CParticleSys_Scriptable Class
=====================================================================================
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$*/
class CParticleSys_Scriptable : public CParticle_Sys
{
	private:
			
		CParticleScript		*m_pScript;			// A pointer to a script
		BOOL				m_scriptApplied;	// Has a script been attached to the system?
		
		STLeventVec			m_events;			// Vector of events
		STLeventVec_It		m_events_it;		// Iterator
		
	public:

		CParticleSys_Scriptable();
		~CParticleSys_Scriptable();

		VOID	DestroyAll();
		VOID	Update(GLfloat elapsedTime);		 
		VOID	Render();

		VOID	ReinitSystem();
		VOID	AttachScript(CParticleScript *scr);
		BOOL	ApplyScript();
		BOOL	AddSystemProperties();
		BOOL	AddEmitters();
		BOOL	AddEvents();		
		BOOL	IsScriptApplied(){	return m_scriptApplied;	}

		// METHODS CALLED BY THE EVENTS TO UPDATE EMITTERS
		BOOL	Fade_ParticleSize(int index, float newSize, float elapsedTime, float timeRange);
		BOOL	Fade_ParticleColor(int index, CColour newColor, float elapsedTime, float timeRange);
		BOOL	Fade_ParticleWeight(int index, float newWeight, float elapsedTime, float timeRange);
		BOOL	Set_EmitterPos(int index, MGL::Vector3 newPos);
		BOOL	Set_EmitterEmissionRate(int index, float newRate);
		BOOL	Set_EmitterBatchSize(int index, int newBatchSize);
		BOOL	Set_EmitterBlends(int index, GLenum src, GLenum dst);
		BOOL	Set_EmitterState(int index, bool newState);		
		BOOL	Set_EmitterForce(int index, int forceIndex, MGL::Vector3 newForce, 
														MGL::Vector3 forcePos, float dst);
};

#endif _PARTICLESYS_SCRIPTABLE_H

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% END OF FILE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

