/*************************************************************************/
/*|=====================================================================|*/
/*| Filename: CParticle_Emitter.h                                       |*/
/*| Author: Joel Villagrana    (joel.villagrana@gmail.com)              |*/
/*|                                                                     |*/
/*|%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%|*/
/*| dd/mm/yyyy | Programmer        | Modifications                      |*/ 
/*|=====================================================================|*/
/*| 17/05/2005 | JVillagrana       | Initial Version                    |*/
/*|---------------------------------------------------------------------|*/
/*************************************************************************/

#ifndef _PARTICLE_EMITTER_H
#define _PARTICLE_EMITTER_H

#include <windows.h>
#include <stdio.h>
#include <assert.h>
#include <glee/glee.h>
#include <gl/gl.h>
#include <gl/glu.h>

#include <vector>
using namespace std;

#include "../Common/MGLVector.h"
#include "../Math/helper_math.h"
#include "../Util/helper_defs.h"
#include "../Util/CColor.h"
#include "../Texture/textureManager.h"

#include "CParticle.h"

#define MAX_EMITTER_FORCES	3
#define LITERAL_LENGTH		40

/*$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
=====================================================================================
					Definition of the CParticle_Emitter Class
=====================================================================================
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$*/
class CParticle_Emitter
{
	private:
		// ===================
		// EMITTER PROPERTIES|
		// ===================
		char				m_ID[LITERAL_LENGTH];			// Emitter ID
		PSYS_EMITTERTYPE	m_type;							// Emitter Type
		MGL::Vector3		m_position;						// Emitter Position
		PARTICLE_REP		m_representation;				// Particle Representation 
		int					m_maxParticles;					// Max Particles for this Emitter
		int					m_numParticles;					// Current # of particles
		char				m_texture[LITERAL_LENGTH];		// Texture path for the particles 
		GLuint				m_textureID;					// Texture ID	
		bool				m_textureLoaded;				// Is Texture Loaded?
		bool				m_useTexture;					// Use texture?
		float				m_emissionRate;					// Emission Rate
		float				m_lastEmission;					// Last Particle Emission
		int					m_batchSize;					// Batch Size
		GLenum				m_srcBlend, m_dstBlend;			// Blending Modes
		char				m_vxShader[LITERAL_LENGTH];		// Vertex Shader
		char				m_pxShader[LITERAL_LENGTH];		// Pixel Shader
		MGL::Vector3		m_forces[MAX_EMITTER_FORCES];	// Local forces
		MGL::Vector3		m_forcesPos[MAX_EMITTER_FORCES];// Local forces
		bool				m_forcesAct[MAX_EMITTER_FORCES];// Local forces
		float				m_forcesDst[MAX_EMITTER_FORCES];// Max Dst of local forces
		int					m_numForces;					// Number of local forces
		float				m_minLife, m_maxLife;			// Min-Max Life
		float				m_minSize, m_maxSize;			// Min-Max Size
		bool				m_emitterON;					// Emitter On-Off
		bool				m_pointSpritesSupported;		// Point Sprites supported on card?
		int					m_numDead;						// Number of dead particles
		MGL::Vector3		m_lineEndPoint;					// Endpoint If emitter is a line
		MGL::Vector3		m_totalLocalForces;				// Total local forces
		MGL::Vector3		m_totalGlobalForces;			// Total global (system) forces
		CColour				m_particleInitialColor;			// Initial color of the particles

		// ==========
		// PARTICLES|
		// ==========
		STLparticleVec		m_particles;					// Particles
		STLparticleVec_it	m_particles_it;					// Particles Iterator

	public:

		CParticle_Emitter();
		~CParticle_Emitter();

		HRESULT	InitAll();
		VOID	DestroyAll();

		void	Update(float elapsedTime, float startTime);	
		void	Render();
		void	LoadTexture();
		void	InitializeParticle(CParticle *particle, float startTime);
		int		EmitParticles(float startTime);
		void	CalculateInitialPosition(MGL::Vector3 *pos, MGL::Vector3 *dir);
		void	Set_Force(int index, MGL::Vector3 force, MGL::Vector3 forcePos, float dst);
		void	Get_Force(int index, MGL::Vector3 *force, MGL::Vector3 *forcePos, float *dst);
		void	Fade_Particles_Size(float finalSize, float elapsedTime, float timeRange);
		void	Fade_Particles_Color(CColour finalColor, float elapsedTime, float timeRange);
		void	Fade_Particles_Weight(float finalWeight, float elapsedTime, float timeRange);

		// ACCESSOR FUNCTIONS
		// ==============================================================
		void	Set_ID(char *id)				{	memcpy(m_ID, id, LITERAL_LENGTH);		}
		void	Get_ID(char *id)				{	memcpy(id, m_ID, LITERAL_LENGTH);		}
		void	Set_Type(PSYS_EMITTERTYPE e)	{	m_type = e;								}
		PSYS_EMITTERTYPE	Get_Type()			{	return m_type;							}
		void	Set_Position(MGL::Vector3 pos)	{	m_position = pos;						}
		void	Get_Position(MGL::Vector3 *pos)	{	*pos = m_position;						}
		void	Set_ParticleRep(PARTICLE_REP p);
		PARTICLE_REP		Get_ParticleRep()	{	return m_representation;				}
		void	Set_MaxParticles(int m)			{	m_maxParticles = m;						}
		int		Get_MaxParticles()				{	return m_maxParticles;					}
		void	Set_Texture(char *tex)			{	memcpy(m_texture, tex, LITERAL_LENGTH);	
													m_useTexture = true; m_textureID = -1;
													m_textureLoaded = false;				}
		void	Get_Texture(char *tex)			{	memcpy(tex, m_texture, LITERAL_LENGTH);	}
		void	Set_EmissionRate(float er)		{	m_emissionRate = er;					}
		float	Get_EmissionRate()				{	return m_emissionRate;					}
		void	Set_BatchSize(int bs)			{	m_batchSize = bs;						}
		int		Get_BatchSize()					{	return m_batchSize;						}
		void	Set_Blends(GLenum s, GLenum d)	{	m_srcBlend = s;	m_dstBlend = d;			}
		void	Get_Blends(GLenum *s,GLenum *d)	{	*s = m_srcBlend; *d = m_dstBlend;		}
		void	Set_VxShader(char *vx)			{	memcpy(m_vxShader, vx, LITERAL_LENGTH);	}
		void	Get_VxShader(char *vx)			{	memcpy(vx, m_vxShader, LITERAL_LENGTH);	}
		void	Set_PxShader(char *px)			{	memcpy(m_pxShader, px, LITERAL_LENGTH);	}
		void	Get_PxShader(char *px)			{	memcpy(px, m_pxShader, LITERAL_LENGTH);	}
		void	Set_Life(float min, float max)	{	m_minLife = min; m_maxLife = max;		}
		void	Get_Life(float *min,float *max)	{	*min = m_minLife; *max = m_maxLife;		}
		void	Set_Size(float min, float max)	{	m_minSize = min; m_maxSize = max;		}
		void	Get_Size(float *min,float *max)	{	*min = m_minSize; *max = m_maxSize;		}
		void	Set_EmitterState(bool state)	{	m_emitterON = state;					}
		bool	IsEmitterActive()				{	return m_emitterON;						}
		void	AddGlobalForce(MGL::Vector3 force){	m_totalGlobalForces += force;			}
		void	Set_ParticleColor(CColour col)	{	m_particleInitialColor = col;			}
};

typedef vector<CParticle_Emitter*> STLemitterVec;
typedef STLemitterVec::iterator STLemitterVec_it;

#endif _PARTICLE_EMITTER_H


// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% END OF FILE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
