/*************************************************************************/
/*|=====================================================================|*/
/*| Filename: CParticle_Event.cpp                                       |*/
/*| Author: Joel Villagrana    (joel.villagrana@gmail.com)              |*/
/*|                                                                     |*/
/*|%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%|*/
/*| dd/mm/yyyy | Programmer        | Modifications                      |*/ 
/*|=====================================================================|*/
/*| 17/05/2005 | JVillagrana       | Initial Version                    |*/
/*|---------------------------------------------------------------------|*/
/*************************************************************************/

#include "CParticle_Event.h"
#include "CParticleSys_Scriptable.h"


// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
// ========================================================================
// IMPLEMENTATION OF CPARTICLE_EVENT CLASS 
// ========================================================================
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

// 
// Constructor
// 
CParticle_Event::CParticle_Event()
{
	InitAll(10);
}

// 
// Destructor
// 
CParticle_Event::~CParticle_Event()
{		
}

// 
// Init
// 
HRESULT CParticle_Event::InitAll(int numEmitters)
{
	m_numEmitters		= 0;
	m_numChanges		= 0;
	m_eventTime			= 0.0f;
	m_numChgProcessed	= 0;
	m_eventProcessed	= false;
	m_changes.reserve(numEmitters);
	m_changes_it = m_changes.begin();

	return S_OK;
}

// 
// Cleanup
// 
VOID CParticle_Event::DestroyAll()
{
	// Delete all changes
	for(m_changes_it = m_changes.begin(); m_changes_it != m_changes.end();)
	{
		delete (*m_changes_it);
		m_changes_it = m_changes.erase(m_changes_it);
	}
	m_numEmitters = 0;
	m_numChanges = 0;
}

// 
// Add a change (INT)
// 
int CParticle_Event::AddPropChange_Int(int eIndex, EVENT_CHANGE chg, 
			float startTime, float endTime, int *finalValues, int numValues)
{
	int changeIndex = -1;	
	CParticle_EventChange_T<int>   int_instantiation; //dummy instantiation
	CParticle_EventChange_T<int>   *pPropChange_Int=NULL;

	// CREATE NEW PROPERTY CHANGE
	pPropChange_Int = new CParticle_EventChange_T<int>(eIndex, chg, startTime, endTime);

	// MAKE SURE MEMORY WAS ALLOCATED
	assert(pPropChange_Int != NULL && "CParticle_Event::AddPropChange_Int");

	// ADD VALUES
	for(int i=0; i<numValues; i++)
		pPropChange_Int->AddVal(finalValues[i]);

	// ADD PROPERTY CHANGE TO THE VECTOR OF CHANGES OF THIS EVENT
	m_changes.push_back(pPropChange_Int);
	changeIndex = m_numChanges++;

	return changeIndex;
}

// 
// Add a change (FLOAT)
// 
int CParticle_Event::AddPropChange_Float(int eIndex, EVENT_CHANGE chg, 
			float startTime, float endTime, float *finalValues, int numValues)
{
	int changeIndex = -1;	
	CParticle_EventChange_T<float> float_instantiation; //dummy instantiation
	CParticle_EventChange_T<float> *pPropChange_Float=NULL;

	// CREATE NEW PROPERTY CHANGE
	pPropChange_Float = new CParticle_EventChange_T<float>(eIndex, chg, startTime, endTime);

	// MAKE SURE MEMORY WAS ALLOCATED
	assert(pPropChange_Float != NULL && "CParticle_Event::AddPropChange_Float");

	// ADD VALUES
	for(int i=0; i<numValues; i++)
		pPropChange_Float->AddVal(finalValues[i]);

	// ADD PROPERTY CHANGE TO THE VECTOR OF CHANGES OF THIS EVENT
	m_changes.push_back(pPropChange_Float);
	changeIndex = m_numChanges++;

	return changeIndex;
}

// 
// Add a change (ENUM)
// 
int CParticle_Event::AddPropChange_Enum(int eIndex, EVENT_CHANGE chg, 
			float startTime, float endTime, GLenum *finalValues, int numValues)
{
	int changeIndex = -1;	
	CParticle_EventChange_T<GLenum> enum_instantiation; //dummy instantiation
	CParticle_EventChange_T<GLenum>	*pPropChange_Enum=NULL;

	// CREATE NEW PROPERTY CHANGE
	pPropChange_Enum = new CParticle_EventChange_T<GLenum>(eIndex, chg, 
															startTime, endTime);
	// MAKE SURE MEMORY WAS ALLOCATED
	assert(pPropChange_Enum != NULL && "CParticle_Event::AddPropChange_Enum");

	// ADD VALUES
	for(int i=0; i<numValues; i++)
		pPropChange_Enum->AddVal(finalValues[i]);

	// ADD PROPERTY CHANGE TO THE VECTOR OF CHANGES OF THIS EVENT
	m_changes.push_back(pPropChange_Enum);
	changeIndex = m_numChanges++;

	return changeIndex;
}

// 
// Add a change (STRING)
// 
int CParticle_Event::AddPropChange_Str(int eIndex, EVENT_CHANGE chg, 
			float startTime, float endTime, char *finalValue, int numValues)
{
	int changeIndex = -1;		
	CParticle_EventChange_T<char *> char_instantiation; //dummy instantiation	
	CParticle_EventChange_T<char *> *pPropChange_Str=NULL;	
	char *fValue=NULL;

	// CREATE NEW PROPERTY CHANGE
	pPropChange_Str = new CParticle_EventChange_T<char *>(eIndex, chg, 
												startTime, endTime);

	// MAKE SURE MEMORY WAS ALLOCATED
	assert(pPropChange_Str != NULL && "CParticle_Event::AddPropChange_Str");

	// ADD VALUES (1 DEFAULT)
	fValue = new char[40];
	memset(fValue,0,40);
	memcpy(fValue,finalValue,40);
	//pPropChange_Str->AddVal(fValue);
	pPropChange_Str->AddVal(finalValue);
	delete [] fValue;

	// ADD PROPERTY CHANGE TO THE VECTOR OF CHANGES OF THIS EVENT
	m_changes.push_back(pPropChange_Str);
	changeIndex = m_numChanges++;

	return changeIndex;
}

// 
// Process changes
// 
bool CParticle_Event::ProcessIfNeeded(float sysStartTime, float currentTime,
																VOID *pSys)
{
	int eIndex=-1, changeFinished=0, fade=0, i=0;
	EVENT_CHANGE eventType;
	float fadeStart=0.0f, fadeEnd=0.0f, elapsedTime=0.0f;

	elapsedTime = currentTime - sysStartTime;

	// IF EVENT NOT PROCESSED YET AND TIME TO PROCESS...
	if(!m_eventProcessed && (elapsedTime >= m_eventTime))
	{	
		m_changes_it = m_changes.begin();

		// CHECK ALL EVENT CHANGES...
		while((m_changes_it != m_changes.end()) && !m_eventProcessed)
		{
			// IF CHANGE NOT PROCESSED
			if( !((*m_changes_it)->IsChangeProcessed()) )
			{				
				// GET CHANGE INFO...
				(*m_changes_it)->GetChangeInfo(&eIndex, elapsedTime, &eventType, 
									&changeFinished, &fade, &fadeStart, &fadeEnd);				

				if(eventType==SYS_FORCE) // NOT ASSOCIATED WITH AN EMITTER
				{}
				else // PROCESS					
					ProcessChange(eIndex, (elapsedTime-m_eventTime), 
							eventType, fade, fadeStart, fadeEnd, pSys);

				// IF FINISHED PROCESSING THIS CHANGE...
				if(changeFinished)
					m_numChgProcessed++;
			}
		
			if(m_numChgProcessed == m_numChanges)
				m_eventProcessed = true;

			m_changes_it++;
			i++;
		}
	}

	return true;
}

// 
// Process a change 
// 
void CParticle_Event::ProcessChange(int eIndex, float elapsedEventTime, 
		EVENT_CHANGE eventType, int fade, float fadeStart, float fadeEnd, VOID *pSys)
{
	int numValues=0, i=0, intValues[5];
	float floatValues[5], timeRange=0.0f;
	GLenum enumValues[5];
	bool boolVal = false;
	CColour newColor;
	CParticleSys_Scriptable *pParticleSys=NULL;
//	char newTex[40], debugStr[150];

	CParticle_EventChange_T<float>  *pPropChange_Float=NULL;	
	CParticle_EventChange_T<int>    *pPropChange_Int=NULL;	
	CParticle_EventChange_T<GLenum> *pPropChange_Enum=NULL;	
	CParticle_EventChange_T<char *> *pPropChange_Str=NULL;	

	pParticleSys = (CParticleSys_Scriptable *)pSys;
	assert(pParticleSys != NULL && "CParticle_Event::ProcessChange");

	// MAKE SURE WE HAVE A VALID EMITTER INDEX AND A VALID CHANGE ITERATOR
	assert(((*m_changes_it) != NULL) && "CParticle_Event::ProcessChange");	
	
	// SET ITERATOR AT THE BEGINNING
	(*m_changes_it)->SetReadValues();
	// NUM VALUES TO GET
	numValues = (*m_changes_it)->NumValues();

	timeRange = fadeEnd - fadeStart;

	switch(eventType)
	{
		case PA_SIZE:	
						// CAST...
						pPropChange_Float = (CParticle_EventChange_T<float> *)(*m_changes_it);
						// GET CHANGE DATA						
						floatValues[0] = pPropChange_Float->GetVal();
						// APPLY CHANGE
						pParticleSys->Fade_ParticleSize(eIndex, floatValues[0], elapsedEventTime, timeRange);
						break;

		case PA_COLOR:
						// CAST...
						pPropChange_Int = (CParticle_EventChange_T<int> *)(*m_changes_it);
						// GET CHANGE DATA
						for(i=0; i<numValues; i++)
							intValues[i] = pPropChange_Int->GetVal();
						newColor.Set_ColourRGBA((GLuint)intValues[0], (GLuint)intValues[1], 
												(GLuint)intValues[2], (GLuint)intValues[3]);
						// APPLY CHANGE
						pParticleSys->Fade_ParticleColor(eIndex, newColor, elapsedEventTime, timeRange);						
						break;

		case PA_WEIGHT:
						// CAST...
						pPropChange_Float = (CParticle_EventChange_T<float> *)(*m_changes_it);
						// GET CHANGE DATA						
						floatValues[0] = pPropChange_Float->GetVal();
						// APPLY CHANGE
						pParticleSys->Fade_ParticleWeight(eIndex, floatValues[0], elapsedEventTime, timeRange);
						break;

		case EM_POSITION:
						// CAST...
						pPropChange_Float = (CParticle_EventChange_T<float> *)(*m_changes_it);
						// GET CHANGE DATA
						for(i=0; i<numValues; i++)
							floatValues[i] = pPropChange_Float->GetVal();
						// APPLY CHANGE
						pParticleSys->Set_EmitterPos(eIndex,MGL::Vector3(floatValues[0], floatValues[1], floatValues[2]));
						break;
						
		case EM_EMISSIONRATE:
						// CAST...
						pPropChange_Float = (CParticle_EventChange_T<float> *)(*m_changes_it);
						// GET CHANGE DATA						
						floatValues[0] = pPropChange_Float->GetVal();
						// APPLY CHANGE
						pParticleSys->Set_EmitterEmissionRate(eIndex, floatValues[0]);
						break;

		case EM_BATCHSIZE:
						// CAST...
						pPropChange_Int = (CParticle_EventChange_T<int> *)(*m_changes_it);
						// GET CHANGE DATA						
						intValues[0] = pPropChange_Int->GetVal();
						// APPLY CHANGE
						pParticleSys->Set_EmitterBatchSize(eIndex, intValues[0]);
						break;

		case EM_BLENDING:
						// CAST...
						pPropChange_Enum = (CParticle_EventChange_T<GLenum> *)(*m_changes_it);
						// GET CHANGE DATA						
						enumValues[0] = pPropChange_Enum->GetVal();
						enumValues[1] = pPropChange_Enum->GetVal();
						// APPLY CHANGE
						pParticleSys->Set_EmitterBlends(eIndex, enumValues[0], enumValues[1]);
						break;

		case EM_STATE:
						// CAST...
						pPropChange_Int = (CParticle_EventChange_T<int> *)(*m_changes_it);
						// GET CHANGE DATA						
						intValues[0] = pPropChange_Int->GetVal();
						boolVal = (intValues[0]==1 ? true : false);
						// APPLY CHANGE
						pParticleSys->Set_EmitterState(eIndex, boolVal);
						break;

		case EM_FORCE:
						// CAST...
						pPropChange_Float = (CParticle_EventChange_T<float> *)(*m_changes_it);
						// GET CHANGE DATA						
						for(i=0; i<numValues; i++)
							floatValues[i] = pPropChange_Float->GetVal();
						// APPLY CHANGE
						pParticleSys->Set_EmitterForce(eIndex, (int)(floatValues[7]),
								MGL::Vector3(floatValues[0], floatValues[1], floatValues[2]), 
								MGL::Vector3(floatValues[3], floatValues[4], floatValues[5]),
								floatValues[6]);
						break;

		case EM_VERTEXSH:
						break;
		case EM_PIXELSH:
						break;
		case EM_TEXTURE:
						// CAST...
						//pPropChange_Str = (CParticle_EventChange_T<char *> *)(*m_changes_it);
						// GET CHANGE DATA						
						//for(i=0; i<numValues; i++)
						//	memcpy(newTex, (char *)(pPropChange_Str->GetVal()), sizeof(newTex));
						// APPLY CHANGE
						
						break;

	}// switch
	
}

// ------------------------- END OF FILE -----------------------------------
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%





