/*************************************************************************/
/*|=====================================================================|*/
/*| Filename: CParticle_Event.h                                         |*/
/*| Author: Joel Villagrana    (joel.villagrana@gmail.com)              |*/
/*|                                                                     |*/
/*|%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%|*/
/*| dd/mm/yyyy | Programmer        | Modifications                      |*/ 
/*|=====================================================================|*/
/*| 17/05/2005 | JVillagrana       | Initial Version                    |*/
/*|---------------------------------------------------------------------|*/
/*************************************************************************/

#ifndef _PARTICLE_EVENT_H
#define _PARTICLE_EVENT_H

#include <windows.h>
#include <stdio.h>
#include <assert.h>
#include <glee/glee.h>
#include <gl/gl.h>
#include <gl/glu.h>

#include <vector>
using namespace std;

#include "../Common/MGLVector.h"
//#include "../Math/helper_math.h"
#include "../Util/helper_defs.h"
#include "../Util/CColor.h"

#include "CParticle_Emitter.h"
#include "CParticle_EventChange.h"

/*$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
=====================================================================================
					Definition of the CParticle_Event Class
=====================================================================================
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$*/
class CParticle_Event
{
	private:
		TPL_ChangesVec		m_changes;			// Vector of changes
		TPL_ChangesVec_it	m_changes_it;		// Iterator for the changes
		float				m_eventTime;		// Time the event needs to be processed
		bool				m_eventProcessed;	// Has the event been processed?
		int					m_numEmitters;		// Number of emitters affected by this event
		int					m_numChanges;		// Number of property changes in this event
		int					m_numChgProcessed;	// Number of changes already processed
		
	public:

		CParticle_Event();		
		~CParticle_Event();						

		HRESULT	InitAll(int numEmitters);		
		VOID	DestroyAll();

		int		AddPropChange_Int(int eIndex, EVENT_CHANGE chg, float startTime,
								  float endTime, int *finalValues, int numValues);
		int		AddPropChange_Float(int eIndex, EVENT_CHANGE chg, float startTime,
								  float endTime, float *finalValues, int numValues);
		int		AddPropChange_Enum(int eIndex, EVENT_CHANGE chg, float startTime,
								  float endTime, GLenum *finalValues, int numValues);
		int		AddPropChange_Str(int eIndex, EVENT_CHANGE chg, float startTime,
								  float endTime, char *finalValues, int numValues);

		bool	ProcessIfNeeded(float sysStartTime, float elapsedTime, VOID *pSys);
		void	ProcessChange(int eIndex, float elapsedTime, EVENT_CHANGE eventType, int fade,
													float fadeStart, float fadeEnd, VOID *pSys);

		bool	AlreadyProcessed(){		return m_eventProcessed;		}
		void	Set_EventTime(float eTime){		m_eventTime = eTime;	}
};

typedef vector<CParticle_Event*> STLeventVec;
typedef STLeventVec::iterator STLeventVec_It;

#endif _PARTICLE_EVENT_H


// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% END OF FILE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
