/*************************************************************************/
/*|=====================================================================|*/
/*| Filename: CParticle_EventChange.h                                   |*/
/*| Author: Joel Villagrana    (joel.villagrana@gmail.com)              |*/
/*|                                                                     |*/
/*|%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%|*/
/*| dd/mm/yyyy | Programmer        | Modifications                      |*/ 
/*|=====================================================================|*/
/*| 17/05/2005 | JVillagrana       | Initial Version                    |*/
/*|---------------------------------------------------------------------|*/
/*************************************************************************/

#ifndef _PARTICLE_EVENTCHG_H
#define _PARTICLE_EVENTCHG_H

#include <windows.h>
#include <stdio.h>
#include <assert.h>
#include <glee/glee.h>
#include <gl/gl.h>
#include <gl/glu.h>

#include <vector>
#include <stdexcept>
using namespace std;

//#include "../Math/helper_math.h"
#include "../Util/helper_defs.h"
#include "../Util/CColor.h"

/*$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
=====================================================================================
				Definition of the CParticle_EventChange Base Class
(Need a base class because we'll be declaring a vector of only one type of event chg)
=====================================================================================
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$*/
class CParticle_EventChange
{
	public:
		CParticle_EventChange(){}
		virtual ~CParticle_EventChange(){}

		virtual void	GetChangeInfo(int *index, float elapsedTime, 
						EVENT_CHANGE *eventType, int *finished, int *fade,
						float *start, float *end) = 0;
		virtual	void	SetReadValues() = 0;
		virtual int		NumValues() const = 0;
		virtual	float	Get_StartTime() = 0;
		virtual	float	Get_EndTime() = 0;
		virtual	bool	IsChangeProcessed() = 0;	
};

/*$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
=====================================================================================
				Definition of the CParticle_EventChange_T Template Class
=====================================================================================
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$*/
template <class T>
class CParticle_EventChange_T : public CParticle_EventChange
{
	private:
		
		int					m_emitterIndex;			// Emitter Index
		EVENT_CHANGE		m_eventChange;			// Type of change
		float				m_changeStartTime,		// Start time
							m_changeEndTime;		// End time	
		bool				m_changeProcessed;		// Finished processing change?
		std::vector<T>		m_finalValues;			// Vector of values in this change
		vector<T>::iterator	m_iterator;				// Iterator

	public:
		
		CParticle_EventChange_T(){}
		CParticle_EventChange_T(int eIndex, EVENT_CHANGE chg, float startTime, float endTime);
		~CParticle_EventChange_T();

		void	GetChangeInfo(int *index, float elapsedTime, EVENT_CHANGE *eventType, 
								int *finished, int *fade,  float *start, float *end);

		bool	AddVal(T const&);
		T		GetVal();

		void	SetReadValues()		{	m_iterator = m_finalValues.begin();	}
		int		NumValues() const	{	return m_finalValues.size();		}
		float	Get_StartTime()		{	return m_changeStartTime;			}
		float	Get_EndTime()		{	return m_changeEndTime;				}
		bool	IsChangeProcessed()	{	return m_changeProcessed;			}
};

// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
// TYPEDEF FOR HAVING A VECTOR OF DIFFERENT TYPES OF CHANGES
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
typedef vector<CParticle_EventChange*>	TPL_ChangesVec;
typedef TPL_ChangesVec::iterator		TPL_ChangesVec_it;

// WEIRD, BUT TEMPLATES DIDN'T COMPILE OTHERWISE...
#include "CParticle_EventChange.cpp"

#endif _PARTICLE_EVENTCHG_H


// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% END OF FILE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%