/*************************************************************************/
/*|=====================================================================|*/
/*| Filename: CParticle_Sys.h                                           |*/
/*| Author: Joel Villagrana    (joel.villagrana@gmail.com)              |*/
/*|                                                                     |*/
/*|%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%|*/
/*| dd/mm/yyyy | Programmer        | Modifications                      |*/ 
/*|=====================================================================|*/
/*| 17/05/2005 | JVillagrana       | Initial Version                    |*/
/*|---------------------------------------------------------------------|*/
/*************************************************************************/

#ifndef _PARTICLESYS_H
#define _PARTICLESYS_H

#include <windows.h>
#include <stdio.h>
#include <assert.h>
#include <glee/glee.h>
#include <gl/gl.h>
#include <gl/glu.h>

#include <vector>
using namespace std;

#include "CParticle.h"
#include "CParticle_Emitter.h"

//#include "../Math/helper_math.h"
#include "../Common/MGLVector.h"
#include "../Util/helper_defs.h"
#include "../Util/CColor.h"

#define MAX_PARTICLES 100000
#define MAX_FORCES 3

/*$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
=====================================================================================
					Definition of the CParticle_Sys Class
=====================================================================================
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$*/
class CParticle_Sys
{
	protected:

		// PARTICLE SYSTEM PROPERTIES
		// =========================================================================
		PARTICLESYS_TYPE	m_sysType;				// The type of particle system
		BOOL				m_active;				// Is particle system active?

		STLemitterVec		m_emitters;				// Vector of emitters
		STLemitterVec_it	m_emitters_it;			// Iterator
		
		float				m_startTime;			// Start time
		int					m_maxParticles;			// Max particles		
		MGL::Vector3		m_forces[MAX_FORCES];	// Sys forces
		int					m_totalForces;			// Number of global forces

	public:

		CParticle_Sys();
		virtual ~CParticle_Sys(){};

		VOID	InitAll(PARTICLESYS_TYPE sType, int maxParticles);
		VOID	DestroySystem();
		VOID	UpdateSystem(GLfloat elapsedTime);
		VOID	RenderParticleSystem();
		VOID	AddEmitter(CParticle_Emitter *newEmitter);
		int		GetEmitterIndex(char *emitterID);
		VOID	SetSysForce(int forceIndex, float force[3]);
		VOID	SetStartTime();

		VOID	Set_MaxParticles(int max)	{	m_maxParticles = max;	}
		BOOL	IsSysActive()				{	return m_active;		}
		VOID	SetActive(BOOL a)			{	m_active = a;			}

		// Virtual functions
		// -----------------
		virtual VOID	DestroyAll() = 0;
		virtual	VOID	Update(GLfloat elapsedTime) = 0;
		virtual VOID	Render() = 0;
};

#endif _PARTICLESYS_H

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% END OF FILE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


