/*************************************************************************/
/*|=====================================================================|*/
/*| Filename: CParticleScript.cpp                                       |*/
/*| Author: Joel Villagrana    (joel.villagrana@gmail.com)              |*/
/*|                                                                     |*/
/*|%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%|*/
/*| dd/mm/yyyy | Programmer        | Modifications                      |*/
/*|=====================================================================|*/
/*| 11/07/2005 | JVillagrana       | Initial Version                    |*/
/*|---------------------------------------------------------------------|*/
/*| 04/08/2005 | JVillagrana   @1  | Added some optimization methods    |*/
/*|---------------------------------------------------------------------|*/
/*************************************************************************/

#include "CParticleScript.h"

extern float RangedRandom(float v1, float v2);
extern int RangedRandomInt(int v1, int v2);

// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
// ========================================================================
// IMPLEMENTATION OF CTokenNode CLASS 
// ========================================================================
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

// 
// Constructor
// 
CTokenNode::CTokenNode(char *tk, SCR_TOKEN_TYPE tt, int line)
{
	memcpy(m_token, tk, SCRIPT_TOKEN_LENGTH);
	m_type = tt;
	m_scriptLine = line;
}

// 
// Destructor
// 
CTokenNode::~CTokenNode()
{
}

// 
// Get Token Type as a String
// 
void CTokenNode::GetTokenTypeStr(char *str)
{
	switch(m_type)
	{
	case KEYWORD_PARTICLESYSTEM:	sprintf(str, "KEYWORD_PARTICLESYSTEM");		break;
	case KEYWORD_SYSTEMPROPERTIES:	sprintf(str, "KEYWORD_SYSTEMPROPERTIES");	break;
	case KEYWORD_EMITTER:			sprintf(str, "KEYWORD_EMITTER");			break;
	case KEYWORD_EVENTAT:			sprintf(str, "KEYWORD_EVENTAT");			break;
	case KEYWORD_SYSPROPCHG:		sprintf(str, "KEYWORD_SYSPROPCHG");			break;
	case KEYWORD_EMITTERPROPCHG:	sprintf(str, "KEYWORD_EMITTERPROPCHG");		break;
	case KEYWORD_RANDOM:			sprintf(str, "KEYWORD_RANDOM");				break;
	case KEYWORD_VECTOR:			sprintf(str, "KEYWORD_VECTOR");				break;
	case KEYWORD_COLOR:				sprintf(str, "KEYWORD_COLOR");				break;
	case KEYWORD_ATPOS:				sprintf(str, "KEYWORD_ATPOS");				break;
	case KEYWORD_DIST:				sprintf(str, "KEYWORD_DIST");				break;
	case CURLYBRACE_OPEN:			sprintf(str, "CURLYBRACE_OPEN");			break;
	case CURLYBRACE_CLOSE:			sprintf(str, "CURLYBRACE_CLOSE");			break;
	case BRACKET_OPEN:				sprintf(str, "BRACKET_OPEN");				break;
	case BRACKET_CLOSE:				sprintf(str, "BRACKET_CLOSE");				break;
	case PARENTHESIS_OPEN:			sprintf(str, "PARENTHESIS_OPEN");			break;
	case PARENTHESIS_CLOSE:			sprintf(str, "PARENTHESIS_CLOSE");			break;
	case BLENDINGMODE_ID:			sprintf(str, "BLENDINGMODE_ID");			break;
	case EMITTERTYPE_ID:			sprintf(str, "EMITTERTYPE_ID");				break;
	case PARTICLEREP_ID:			sprintf(str, "PARTICLEREP_ID");				break;
	case COMMA:						sprintf(str, "COMMA");						break;
	case IDENTIFIER:				sprintf(str, "IDENTIFIER");					break;
	case LITERAL:					sprintf(str, "LITERAL");					break;
	case NUMBER:					sprintf(str, "NUMBER");						break;
	case SEMI_COLON:				sprintf(str, "SEMI_COLON");					break;
	case ASSIGNMENT_OPERATOR:		sprintf(str, "ASSIGNMENT_OPERATOR");		break;
	case NONE:						sprintf(str, "NONE");						break;
	case KEYWORD_SYSP_MAXPARTICLES:	sprintf(str, "KEYWORD_SYSP_MAXPARTICLES");	break;
	case KEYWORD_SYSP_FORCE:		sprintf(str, "KEYWORD_SYSP_FORCE");			break;
	case KEYWORD_EM_POSITION:		sprintf(str, "KEYWORD_EM_POSITION");		break;
	case KEYWORD_EM_TYPE:			sprintf(str, "KEYWORD_EM_TYPE");			break;
	case KEYWORD_EM_PARTICLEREP:	sprintf(str, "KEYWORD_EM_PARTICLEREP");		break;
	case KEYWORD_EM_TEXTURE:		sprintf(str, "KEYWORD_EM_TEXTURE");			break;
	case KEYWORD_EM_EMISSIONRATE:	sprintf(str, "KEYWORD_EM_EMISSIONRATE");	break;
	case KEYWORD_EM_BATCHSIZERANGE:	sprintf(str, "KEYWORD_EM_BATCHSIZERANGE");	break;
	case KEYWORD_EM_BLENDINGMODES:	sprintf(str, "KEYWORD_EM_BLENDINGMODES");	break;
	case KEYWORD_EM_VERTEXSHADER:	sprintf(str, "KEYWORD_EM_VERTEXSHADER");	break;
	case KEYWORD_EM_PIXELSHADER:	sprintf(str, "KEYWORD_EM_PIXELSHADER");		break;
	case KEYWORD_EM_FORCE:			sprintf(str, "KEYWORD_EM_FORCE");			break;
	case KEYWORD_EM_LIFERANGE:		sprintf(str, "KEYWORD_EM_LIFERANGE");		break;
	case KEYWORD_EM_SIZERANGE:		sprintf(str, "KEYWORD_EM_SIZERANGE");		break;
	case KEYWORD_EM_STATE:			sprintf(str, "KEYWORD_EM_STATE");			break;
	case KEYWORD_PARTICLE_COLOR:	sprintf(str, "KEYWORD_PARTICLE_COLOR");		break;
	case KEYWORD_PARTICLE_SIZE:		sprintf(str, "KEYWORD_PARTICLE_SIZE");		break;
	case KEYWORD_PARTICLE_WEIGHT:	sprintf(str, "KEYWORD_PARTICLE_WEIGHT");	break;
	case KEYWORD_PARTICLE_FADE:		sprintf(str, "KEYWORD_PARTICLE_FADE");		break;
	default: sprintf(str, "<NOT FOUND>"); break;
	}	
}

// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
// ========================================================================
// IMPLEMENTATION OF CParticleScript CLASS 
// ========================================================================
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

// 
// Constructor
// 
CParticleScript::CParticleScript()
{
	InitAll();
}

// 
// Destructor
// 
CParticleScript::~CParticleScript()
{	
}

// 
// Initialize everything
// 
void CParticleScript::InitAll()
{	
	m_state				= STATE_NONE;
	m_lineNumber		= 0;
	m_numErrors			= 0;
	m_currError			= 0;

	memset(m_filename,0,sizeof(m_filename));
	memset(m_errorString,0,sizeof(m_errorString));
	memset(m_lex,0,sizeof(m_lex));
	memset(m_token,0,sizeof(m_token));
	memset(m_line,0,sizeof(m_line));

	memset(m_keywords,0,sizeof(m_keywords));
	
	memcpy(m_keywords[0],  "particlesystem",	strlen("particlesystem"));
	memcpy(m_keywords[1],  "systemproperties",	strlen("systemproperties"));
	memcpy(m_keywords[2],  "emitter",			strlen("emitter"));
	memcpy(m_keywords[3],  "event_at",			strlen("event_at"));
	memcpy(m_keywords[4],  "sysprop_changes",	strlen("sysprop_changes"));
	memcpy(m_keywords[5],  "for_emitter",       strlen("for_emitter"));
	memcpy(m_keywords[6],  "randomrange",		strlen("randomrange"));
	memcpy(m_keywords[7],  "vector",			strlen("vector"));
	memcpy(m_keywords[8],  "rgba",				strlen("rgba"));
	memcpy(m_keywords[9],  "at_pos",			strlen("at_pos"));
	memcpy(m_keywords[10], "dist",				strlen("dist"));
	// system keywords
	memcpy(m_keywords[11], "maxparticles",		strlen("maxparticles"));
	memcpy(m_keywords[12], "force",				strlen("force"));
	// emitter keywords
	memcpy(m_keywords[13], "emitterpos",		strlen("emitterpos"));
	memcpy(m_keywords[14], "emittertype",       strlen("emittertype"));
	memcpy(m_keywords[15], "particlerep",		strlen("particlerep"));
	memcpy(m_keywords[16], "texture",			strlen("texture"));
	memcpy(m_keywords[17], "emissionrate",		strlen("emissionrate"));
	memcpy(m_keywords[18], "batchsize",			strlen("batchsize"));
	memcpy(m_keywords[19], "blendingmodes",		strlen("blendingmodes"));
	memcpy(m_keywords[20], "vertexshader",		strlen("vertexshader"));
	memcpy(m_keywords[21], "pixelshader",		strlen("pixelshader"));
	memcpy(m_keywords[22], "liferange",			strlen("liferange"));
	memcpy(m_keywords[23], "sizerange",			strlen("sizerange"));
	memcpy(m_keywords[24], "emitter_on",		strlen("emitter_on"));
	memcpy(m_keywords[25], "emitter_off",		strlen("emitter_off"));
	// particle keywords
	memcpy(m_keywords[26], "particlecolor",		strlen("particlecolor"));
	memcpy(m_keywords[27], "particlesize",		strlen("particlesize"));
	memcpy(m_keywords[28], "particleweight",	strlen("particleweight"));
	memcpy(m_keywords[29], "fade_from_now_to",  strlen("fade_from_now_to"));

	m_tokenList.reserve(400);
	m_errorsList.reserve(100);
	m_scriptLines.reserve(200);
	m_emitterIDs.reserve(20);
}

// 
// Clean up
// 
void CParticleScript::DestroyAll()
{
	// Clean up lists
	STL_strIter iterator;
	
	for(m_tokenIterator = m_tokenList.begin(); m_tokenIterator != m_tokenList.end();)
	{
		delete (*m_tokenIterator);
		m_tokenIterator = m_tokenList.erase(m_tokenIterator);
	}

	for(iterator = m_scriptLines.begin(); iterator != m_scriptLines.end();)
	{
		delete [] (*iterator);
		iterator = m_scriptLines.erase(iterator);
	}

	for(iterator = m_errorsList.begin(); iterator != m_errorsList.end();)
	{
		delete [] (*iterator);
		iterator = m_errorsList.erase(iterator);
	}

	for(iterator = m_emitterIDs.begin(); iterator != m_emitterIDs.end();)
	{
		delete [] (*iterator);
		iterator = m_emitterIDs.erase(iterator);
	}

	m_state		= STATE_NONE;
	m_lineNumber= 0;
	m_numErrors	= 0;
	m_currError	= 0;
}

// 
// Add Script Line
// (method called from dialog box)
// 
bool CParticleScript::AddScriptLine(char *line)
{
	bool lineAdded = true;
	char *newScriptLine=NULL;

	if((strlen(line)>0) && (line[0]!='\n'))
	{
		newScriptLine = new char[SCRIPT_LINE_LENGTH];	
		memset(newScriptLine, 0, SCRIPT_LINE_LENGTH);
		memcpy(newScriptLine, line, strlen(line));
		m_scriptLines.push_back(newScriptLine);
	}
	else
		lineAdded = false;

	return lineAdded;
}

// 
// Checks if a char is valid
// 
bool CParticleScript::IsValidChar(char c)
{
	bool isValid = false;

	if((c=='_')||(isalpha(c))||(isdigit(c)))
		isValid = true;

	return isValid;
}

// 
// Checks if a char is alphabetic
// 
bool CParticleScript::IsAlpha(char c)
{
	if(isalpha(c))
		return true;
	else
		return false;
}

// 
// Checks if this is a keyword
// 
bool CParticleScript::IsKeyword(char *word)
{
	bool	keywordFound = false;
	char	word_lowercase[SCRIPT_TOKEN_LENGTH];
	int		i=0, j=0;
	
	memset(word_lowercase,0,sizeof(word_lowercase));
	memcpy(word_lowercase, word, SCRIPT_TOKEN_LENGTH);

	j = strlen(word_lowercase);
	for(i=0; i<j; i++)
		word_lowercase[i] = tolower(word_lowercase[i]);

	i = 0;
	while( (i<NUM_KEYWORDS)&&(!keywordFound) )
	{
		if(0 == memcmp(m_keywords[i], word_lowercase, SCRIPT_TOKEN_LENGTH))
			keywordFound = true;
		i++;
	}

	return keywordFound;
}

// 
// Checks if this is an OpenGL blending mode
// 
bool CParticleScript::IsBlendingMode(char *word, GLenum *blendEnum)
{
	char word_uppercase[SCRIPT_TOKEN_LENGTH];
	int  i=0, j=0;
	bool isBlend=false;

	memset(word_uppercase,0,sizeof(word_uppercase));
	memcpy(word_uppercase, word, SCRIPT_TOKEN_LENGTH);

	j = strlen(word_uppercase);
	for(i=0; i<j; i++)
		word_uppercase[i] = toupper(word_uppercase[i]);

	if(0==memcmp(word, "GL_ZERO", j))
	{ *blendEnum = GL_ZERO;	isBlend = true;	}
	else if(0==memcmp(word, "GL_ONE", j))
	{ *blendEnum = GL_ONE;	isBlend = true;	}
	else if(0==memcmp(word, "GL_DST_COLOR", j))
	{ *blendEnum = GL_DST_COLOR;	isBlend = true;	}
	else if(0==memcmp(word, "GL_SRC_COLOR", j))
	{ *blendEnum = GL_SRC_COLOR;	isBlend = true;	}
	else if(0==memcmp(word, "GL_ONE_MINUS_DST_COLOR", j))
	{ *blendEnum = GL_ONE_MINUS_DST_COLOR;	isBlend = true;	}
	else if(0==memcmp(word, "GL_ONE_MINUS_SRC_COLOR", j))
	{ *blendEnum = GL_ONE_MINUS_SRC_COLOR;	isBlend = true;	}
	else if(0==memcmp(word, "GL_SRC_ALPHA", j))
	{ *blendEnum = GL_SRC_ALPHA;	isBlend = true;	}
	else if(0==memcmp(word, "GL_ONE_MINUS_SRC_ALPHA", j))
	{ *blendEnum = GL_ONE_MINUS_SRC_ALPHA;	isBlend = true;	}
	else if(0==memcmp(word, "GL_DST_ALPHA", j))
	{ *blendEnum = GL_DST_ALPHA;	isBlend = true;	}
	else if(0==memcmp(word, "GL_ONE_MINUS_DST_ALPHA", j))
	{ *blendEnum = GL_ONE_MINUS_DST_ALPHA;	isBlend = true;	}
	else if(0==memcmp(word, "GL_SRC_ALPHA_SATURATE", j))
	{ *blendEnum = GL_SRC_ALPHA_SATURATE;	isBlend = true;	}
//	else if(0==memcmp(word, "GL_CONSTANT_COLOR", j))
//	{ *blendEnum = GL_CONSTANT_COLOR;	isBlend = true;	}
//	else if(0==memcmp(word, "GL_ONE_MINUS_CONSTANT_COLOR", j))
//	{ *blendEnum = GL_ONE_MINUS_CONSTANT_COLOR;	isBlend = true;	}
//	else if(0==memcmp(word, "GL_CONSTANT_ALPHA", j))
//	{ *blendEnum = GL_CONSTANT_ALPHA;	isBlend = true;	}
//	else if(0==memcmp(word, "GL_ONE_MINUS_CONSTANT_ALPHA", j))
//	{ *blendEnum = GL_ONE_MINUS_CONSTANT_ALPHA;	isBlend = true;	}

	return isBlend;
}

// 
// Checks if this is an emitter type
// 
bool CParticleScript::IsEmitterType(char *word)
{
	char word_uppercase[SCRIPT_TOKEN_LENGTH];
	int  i=0, j=0;

	memset(word_uppercase,0,sizeof(word_uppercase));
	memcpy(word_uppercase, word, SCRIPT_TOKEN_LENGTH);

	j = strlen(word_uppercase);
	for(i=0; i<j; i++)
		word_uppercase[i] = toupper(word_uppercase[i]);

	if(0==memcmp(word, "E_LINE", j))
		return true;
	else if(0==memcmp(word, "E_POINT", j))
		return true;
	else if(0==memcmp(word, "E_SPHERE", j))
		return true;
	else if(0==memcmp(word, "E_BEAM", j))
		return true;
	else if(0==memcmp(word, "E_RING", j))
		return true;

	return false;
}

// 
// Checks if this is a particle representation
// 
bool CParticleScript::IsParticleRep(char *word)
{
	char word_uppercase[SCRIPT_TOKEN_LENGTH];
	int  i=0, j=0;

	memset(word_uppercase,0,sizeof(word_uppercase));
	memcpy(word_uppercase, word, SCRIPT_TOKEN_LENGTH);

	j = strlen(word_uppercase);
	for(i=0; i<j; i++)
		word_uppercase[i] = toupper(word_uppercase[i]);

	if(0==memcmp(word, "R_POINTSPRITES", j))
		return true;
	else if(0==memcmp(word, "R_POINTS", j))
		return true;
	else if(0==memcmp(word, "R_QUADS", j))
		return true;
	else if(0==memcmp(word, "R_LINES", j))
		return true;

	return false;
}

// 
// Returns the type of keyword
// 
SCR_TOKEN_TYPE CParticleScript::KeywordType(char *word)
{
	for(int i=0; i<strlen(word); i++)
		word[i] = tolower(word[i]);

	if(0==memcmp(word, "particlesystem", strlen(word)))
		return KEYWORD_PARTICLESYSTEM;
	else if(0==memcmp(word, "systemproperties", strlen(word)))
		return KEYWORD_SYSTEMPROPERTIES;
	else if(0==memcmp(word, "emitter", strlen(word)))
		return KEYWORD_EMITTER;
	else if(0==memcmp(word, "event_at", strlen(word)))
		return KEYWORD_EVENTAT;
	else if(0==memcmp(word, "sysprop_changes", strlen(word)))
		return KEYWORD_SYSPROPCHG;
	else if(0==memcmp(word, "for_emitter", strlen(word)))
		return KEYWORD_EMITTERPROPCHG;
	else if(0==memcmp(word, "randomrange", strlen(word)))
		return KEYWORD_RANDOM;
	else if(0==memcmp(word, "vector", strlen(word)))
		return KEYWORD_VECTOR;
	else if(0==memcmp(word,"rgba", strlen(word)))
		return KEYWORD_COLOR;
	else if(0==memcmp(word,"at_pos", strlen(word)))
		return KEYWORD_ATPOS;
	else if(0==memcmp(word,"dist", strlen(word)))
		return KEYWORD_DIST;
	// PARTICLE SYSTEM KEYWORDS
	else if(0==memcmp(word, "maxparticles", strlen(word)))
		return KEYWORD_SYSP_MAXPARTICLES;
	else if(0==memcmp(word, "force", strlen(word)))
		return KEYWORD_SYSP_FORCE;
	// EMITTER KEYWORDS
	else if(0==memcmp(word, "emitterpos", strlen(word)))
		return KEYWORD_EM_POSITION;
	else if(0==memcmp(word, "emittertype", strlen(word)))
		return KEYWORD_EM_TYPE;
	else if(0==memcmp(word, "particlerep", strlen(word)))
		return KEYWORD_EM_PARTICLEREP;
	else if(0==memcmp(word, "texture", strlen(word)))
		return KEYWORD_EM_TEXTURE;
	else if(0==memcmp(word, "emissionrate", strlen(word)))
		return KEYWORD_EM_EMISSIONRATE;
	else if(0==memcmp(word, "batchsize", strlen(word)))
		return KEYWORD_EM_BATCHSIZERANGE;
	else if(0==memcmp(word, "blendingmodes", strlen(word)))
		return KEYWORD_EM_BLENDINGMODES;
	else if(0==memcmp(word, "vertexshader", strlen(word)))
		return KEYWORD_EM_VERTEXSHADER;
	else if(0==memcmp(word, "pixelshader", strlen(word)))
		return KEYWORD_EM_PIXELSHADER;
	else if(0==memcmp(word, "liferange", strlen(word)))
		return KEYWORD_EM_LIFERANGE;
	else if(0==memcmp(word, "sizerange", strlen(word)))
		return KEYWORD_EM_SIZERANGE;
	else if(0==memcmp(word, "emitter_on", strlen(word)))
		return KEYWORD_EM_STATE;
	else if(0==memcmp(word, "emitter_off", strlen(word)))
		return KEYWORD_EM_STATE;
	// PARTICLE KEYWORDS
	else if(0==memcmp(word, "particlecolor", strlen(word)))
		return KEYWORD_PARTICLE_COLOR;
	else if(0==memcmp(word, "particlesize", strlen(word)))
		return KEYWORD_PARTICLE_SIZE;
	else if(0==memcmp(word, "particleweight", strlen(word)))
		return KEYWORD_PARTICLE_WEIGHT;
	else if(0==memcmp(word, "fade_from_now_to", strlen(word)))
		return KEYWORD_PARTICLE_FADE;
	else
		return NONE;
}

// 
// Checks if this character should be trated as space
// (space, tab, new line...)
// 
bool CParticleScript::IsSpace(char c)
{
	bool isSpace = false;

	if((c==' ')||(c=='\t')||(c=='\n')||(c=='\r'))
		isSpace = true;

	return isSpace;
}		

// 
// Adds a new error to the list of errors
// 
void CParticleScript::AddError(char *error)
{
	char *newError = new char[SCRIPT_LINE_LENGTH];

	ClearLastError();
	memcpy(m_errorString, error, strlen(error));
	memset(newError, 0, SCRIPT_LINE_LENGTH);
	memcpy(newError, error, strlen(error));
	m_errorsList.push_back(newError);
	m_numErrors++;
}

// 
// Get all script lines from file
// 
bool CParticleScript::GetScriptFromFile(char *filename)
{
	bool	scriptLoaded = true;
	FILE	*pInputFile = NULL;
	char	c[1], oneLine[SCRIPT_LINE_LENGTH], error[SCRIPT_LINE_LENGTH];
	int		numRead=0, i=0;

	memset(error,0,sizeof(error));

	if((pInputFile = fopen(filename, "r")) == NULL)
	{
		scriptLoaded = false;
		sprintf(error,"Error: Cannot Open Input Script File");
		AddError(error);
	}
	else
	{
		numRead = fread(c, sizeof(char), 1, pInputFile);
		while(numRead>0)
		{
			memset(oneLine,0,sizeof(oneLine));
			i=0;

			// GET ALL LINES
			while((numRead>0) && (c[0]!='\n') && (c[0]!='\r') && (i<(SCRIPT_LINE_LENGTH-2)))
			{
				memcpy(&oneLine[i], c, 1);
				i++;
				numRead = fread(c, sizeof(char), 1, pInputFile);
			}

			if((numRead>0) && (c[0]!='\n') && (c[0]!='\r') && (i==(SCRIPT_LINE_LENGTH-1)))
			{
				scriptLoaded = false;
				sprintf(error,"Error: Script Line too long");
				AddError(error);
			}

			if(i<(SCRIPT_LINE_LENGTH-1))
			{
				oneLine[i++]='\n';
				oneLine[i]='\0';
				AddScriptLine(oneLine);
			}
		}

		fclose(pInputFile);
	}	

	return scriptLoaded;
}

// 
// Process script lines
// 
bool CParticleScript::ProcessScriptLines()
{
	bool linesProcessed = true;

	m_scriptIterator = m_scriptLines.begin();
	m_lineNumber = 1;
	while(m_scriptIterator != m_scriptLines.end())
	{
		TokenizeScriptLine();
		m_lineNumber++;
		m_scriptIterator++;
	}

	// IF ERRORS OCURRED
	if(m_errorsList.size() > 0)
		linesProcessed = false;

	return linesProcessed;
}

// 
// Process one script line
// AN IMPORTANT FUNCTION, THIS IS WHERE OUR FSM STATE DESIGN IS IMPLEMENTED)
// THE FSM WILL RECOGNIZE:
// IDENTIFIERS, KEYWORDS, NUMBERS, COMMENTS, LITERALS, SPACES...
// 
bool CParticleScript::TokenizeScriptLine()
{	
	bool	processed = true;	
	char	line[SCRIPT_LINE_LENGTH], error[SCRIPT_LINE_LENGTH], c='$';
	char	nToken[SCRIPT_TOKEN_LENGTH];
	int		i=-1, j=0;
	SCR_TOKEN_TYPE tokenType = NONE;

	memset(error,0,sizeof(error));
	memset(line,0,sizeof(line));
	memset(nToken,0,sizeof(nToken));
	m_state = STATE_NONE;

	memcpy(line, (*m_scriptIterator), sizeof(line));
	i++;
	// WHILE WE HAVE CHARS TO READ IN THE LINE
	while(c!='\n')
	{
		// GET NEXT CHARACTER
		c = line[i++];
		memset(error,0,sizeof(error));

		// CHECK WHICH STATE WE'RE IN
		switch(m_state)
		{
			// IF CURRENT STATUS IS 'NONE' THEN...
			case STATE_NONE:	// --------------------------------------
								// IF CHAR IS 'A..Z' OR '_' WE WILL READ 
								// AN IDENTIFIER OR A KEYWORD
								// --------------------------------------
								if((IsAlpha(c)) || (c=='_'))
								{
									nToken[j++] = c;
									m_state = STATE_IDENTIFIER;
								}
								// --------------------------------------
								// IF CHAR IS '0..9' THEN WE WILL READ
								// A NUMBER
								// --------------------------------------
								else if(isdigit(c))
								{
									nToken[j++] = c;
									m_state = STATE_NUMBER;
								}
								// --------------------------------------
								// IF CHAR IS '-' THEN WE WILL READ
								// A NUMBER
								// --------------------------------------
								else if(c=='-')
								{
									nToken[j++] = c;
									m_state = STATE_NUMBERSIGN;
								}
								// --------------------------------------
								// IF CHAR IS '=' THEN WE ARE ASSIGNING
								// A VALUE TO A PROPERTY
								// NO STATE CHANGE OCCURS HERE, THIS
								// TOKEN WILL BE ADDED RIGHT AWAY
								// --------------------------------------
								else if(c=='=')
								{
									nToken[j++] = c;
									tokenType = ASSIGNMENT_OPERATOR;
								}
								// --------------------------------------
								// IF THIS CHAR IS A '/' THEN WE ARE
								// STARTING A COMMENT
								// --------------------------------------
								else if(c=='/')
								{
									nToken[j++] = c;
									m_state = STATE_COMMENT;
								}
								// --------------------------------------
								// IF THIS CHAR IS A '"' THEN WE ARE
								// GOING TO READ A LITERAL
								// --------------------------------------
								else if(c=='\"')
								{
									// DON'T SAVE THE '"', ONLY CHANGE
									// STATE
									m_state = STATE_LITERAL;
								}
								// --------------------------------------
								// IF THIS CHAR IS A '{' SAVE TOKEN ONLY
								// NO STATE CHANGE OCCURS HERE 
								// --------------------------------------
								else if(c=='{')
								{
									nToken[j++] = c;
									tokenType = CURLYBRACE_OPEN;
								}
								// --------------------------------------
								// IF THIS CHAR IS A '}' SAVE TOKEN ONLY
								// NO STATE CHANGE OCCURS HERE EITHER
								// --------------------------------------
								else if(c=='}')
								{
									nToken[j++] = c;
									tokenType = CURLYBRACE_CLOSE;
								}
								// --------------------------------------
								// IF THIS CHAR IS A '[' SAVE TOKEN ONLY
								// NO STATE CHANGE OCCURS HERE 
								// --------------------------------------
								else if(c=='[')
								{
									nToken[j++] = c;
									tokenType = BRACKET_OPEN;
								}
								// --------------------------------------
								// IF THIS CHAR IS A ']' SAVE TOKEN ONLY
								// NO STATE CHANGE OCCURS HERE EITHER
								// --------------------------------------
								else if(c==']')
								{
									nToken[j++] = c;
									tokenType = BRACKET_CLOSE;
								}
								// --------------------------------------
								// IF THIS CHAR IS A '(' SAVE TOKEN ONLY
								// NO STATE CHANGE OCCURS HERE 
								// --------------------------------------
								else if(c=='(')
								{
									nToken[j++] = c;
									tokenType = PARENTHESIS_OPEN;
								}
								// --------------------------------------
								// IF THIS CHAR IS A ')' SAVE TOKEN ONLY
								// NO STATE CHANGE OCCURS HERE EITHER
								// --------------------------------------
								else if(c==')')
								{
									nToken[j++] = c;
									tokenType = PARENTHESIS_CLOSE;
								}
								// --------------------------------------
								// IF THIS CHAR IS A ',' SAVE TOKEN ONLY
								// NO STATE CHANGE OCCURS HERE EITHER
								// --------------------------------------
								else if(c==',')
								{
									nToken[j++] = c;
									tokenType = COMMA;
								}
								// --------------------------------------
								// IF THIS CHAR IS A ';' SAVE TOKEN ONLY
								// NO STATE CHANGE OCCURS HERE EITHER
								// --------------------------------------
								else if(c==';')
								{
									nToken[j++] = c;
									tokenType = SEMI_COLON;
								}
								// --------------------------------------
								// IF THIS CHAR IS A SPACE (SPACE, TAB,
								// LINE FEED...) DO NOTHING...
								// --------------------------------------
								else if(IsSpace(c))
								{
								}
								// --------------------------------------
								// IF THIS CHAR IS INVALID, CHECK FIRST 
								// IF IT'S '\n' (FINISHED READING LINE)
								// BEFORE SENDING AN ERROR
								// --------------------------------------
								else if(c!='\n')
								{
									sprintf(error, "Error: Invalid char %c at line %d", c, m_lineNumber);
									AddError(error);
									MessageBox(NULL, error, "ERROR", MB_OK|MB_ICONINFORMATION);
								}
								
								break;

			// IF WE'RE READING AN IDENTIFIER OR A KEYWORD...
			case STATE_IDENTIFIER:
								// --------------------------------------
								// IF WE HAVE A VALID CHAR, ADD IT TO
								// THE BUFFER, OTHERWISE CHECK IF THIS
								// IS A KEYWORD
								// --------------------------------------
								if(IsValidChar(c))
									nToken[j++] = c;
								// --------------------------------------
								// ELSE, THIS CHAR WE'VE JUST READ IS NOT
								// A VALID CHAR, SO WE HAD FINISHED
								// READING THE IDENTIFIER ALREADY.., 
								// SET STATE AS STATE_NONE AGAIN
								// AND DECREASE 'i' SO STATE_NONE CAN
								// READ THIS CHAR AGAIN
								// --------------------------------------
								else
								{
									GLenum blendEnum;
									i--;
									nToken[j] = '\0';
									m_state = STATE_NONE;
									tokenType = IDENTIFIER;
									if(IsKeyword(nToken))
										tokenType = KeywordType(nToken);
									else if(IsBlendingMode(nToken, &blendEnum))
										tokenType = BLENDINGMODE_ID;
									else if(IsEmitterType(nToken))
										tokenType = EMITTERTYPE_ID;
									else if(IsParticleRep(nToken))
										tokenType = PARTICLEREP_ID;
								}

								break;

			// IF WE'RE READING A NUMBER
			case STATE_NUMBER:	// --------------------------------------
								// IF WE HAVE A VALID NUMBER, ADD IT TO
								// THE BUFFER
								// --------------------------------------
								if(isdigit(c))
									nToken[j++] = c;
								// --------------------------------------
								// BUF IF WE HAVE A '.' THEN CHANGE STATE
								// (WE NEED ANOTHER STATE AFTER WE READ
								// THE DOT TO MAKE SURE WE DO NOT ALLOW
								// MULTIPLE DOTS IN A NUMBER)
								// --------------------------------------
								else if(c=='.')
								{
									nToken[j++] = c;
									m_state = STATE_NUMBERDOT;
								}
								// --------------------------------------
								// THIS CHAR WE'VE JUST READ IS NOT
								// A VALID DIGIT, SO WE HAD FINISHED
								// READING THE NUMBER ALREADY.., 
								// SET STATE AS STATE_NONE AGAIN
								// AND DECREASE 'i' SO STATE_NONE CAN
								// READ THIS CHAR AGAIN
								// --------------------------------------
								else
								{
									i--;
									m_state = STATE_NONE;
									tokenType = NUMBER;
								}

								break;

			// IF WE FOUND A MINUS SIGN, WERE READING A NUMBER...
			case STATE_NUMBERSIGN:
								// --------------------------------------
								// IF WE HAVE A VALID DIGIT, ADD IT TO
								// THE BUFFER AND CHANGE STATE TO 
								// STATE_NUMBER BECAUSE WE NEED TO SEND 
								// AN ERROR IF THE FIRST CHARACTER AFTER 
								// THE SIGN IS NOT A DIGIT AND WE WOULDN'T 
								// KNOW HOW MANY DIGITS AFTER THE SIGN WE
								// HAVE READ ALREADY IF WE STAY IN THIS
								// STATE
								// --------------------------------------
								if(isdigit(c))
								{
									nToken[j++] = c;
									m_state = STATE_NUMBER;
								}
								// --------------------------------------
								// ELSE, SEND AN ERROR AND RESET STATE
								// TO STATE_NONE
								// AND DECREASE 'i' SO STATE_NONE CAN
								// READ THIS CHAR AGAIN
								// --------------------------------------
								else
								{
									i--;
									m_state = STATE_NONE;
									//tokenType = NUMBER;
									sprintf(error, "Error: Digit expected instead of %c at line %d",
											c, m_lineNumber);
									AddError(error);
								}

								break;

			// IF WE WERE READING A NUMBER AND WE FOUND A DOT
			case STATE_NUMBERDOT:
								// --------------------------------------
								// IF WE HAVE A VALID DIGIT, ADD IT TO
								// THE BUFFER AND CHANGE STATE TO A
								// STATE THAT ACCEPTS ONLY NUMBERS
								// BECAUSE WE NEED TO SEND AN ERROR
								// IF THE FIRST CHARACTER AFTER THE DOT
								// IS NOT A NUMBER AND WE WOULDN'T KNOW
								// HOW MANY DIGITS AFTER THE DOT WE
								// HAVE READ ALREADY IF WE STAY IN THIS
								// STATE
								// --------------------------------------
								if(isdigit(c))
								{
									nToken[j++] = c;
									m_state = STATE_DIGITONLY;
								}
								// --------------------------------------
								// ELSE, SEND AN ERROR AND RESET STATE
								// TO STATE_NONE
								// AND DECREASE 'i' SO STATE_NONE CAN
								// READ THIS CHAR AGAIN
								// --------------------------------------
								else
								{
									i--;
									m_state = STATE_NONE;
									//tokenType = NUMBER;
									sprintf(error, "Error: Digit expected instead of %c at line %d",
											c, m_lineNumber);
									AddError(error);
								}

								break;

			// IF WE'RE READING DIGITS ONLY
			case STATE_DIGITONLY:
								// --------------------------------------
								// IF WE HAVE A VALID DIGIT, ADD IT TO 
								// THE BUFFER
								// --------------------------------------
								if(isdigit(c))
									nToken[j++] = c;
								// --------------------------------------
								// THIS CHAR WE'VE JUST READ IS NOT
								// A VALID DIGIT, SO WE HAD FINISHED
								// READING THE NUMBER ALREADY.., 
								// SET STATE AS STATE_NONE AGAIN
								// AND DECREASE 'i' SO STATE_NONE CAN
								// READ THIS CHAR AGAIN
								// --------------------------------------
								else
								{
									i--;
									m_state = STATE_NONE;
									tokenType = NUMBER;
								}

								break;

			// IF WE'RE READING A COMMENT
			case STATE_COMMENT:
								// --------------------------------------
								// WE'VE JUST SET THIS STATE WHEN WE
								// FOUND THE FIRST '/', SO THIS NEXT
								// CHAR SHOULD BE '/' AS WELL
								// (WE DEFINED A COMMENT TO START WITH //)
								// ONCE WE KNOW THIS IS A COMMENT, WE CAN
								// SAFELY DISCARD THIS WHOLE LINE...
								// (WE WON'T SAVE COMMENTS AS TOKENS)
								// --------------------------------------
								if(c=='/')
								{
									c='\n';
								}
								// --------------------------------------
								// THIS CHAR WE'VE JUST READ IS NOT '/'
								// SO IT'S NOT A VALID COMMENT...
								// SET STATE AS STATE_NONE AND DECREASE
								// 'i' SO STATE_NONE CAN READ THIS CHAR
								// AGAIN
								// --------------------------------------
								else
								{
									i--;
									m_state = STATE_NONE;
									sprintf(error, "Error: '/' expected instead of %c at line %d",
											c, m_lineNumber);
									AddError(error);
								}

								break;

			// IF WE'RE READING A LITERAL
			case STATE_LITERAL:
								// --------------------------------------
								// IF WE HAVE ANOTHER '"' THEN THIS ENDS
								// THE LITERAL
								// --------------------------------------
								if(c=='\"')
								{
									m_state = STATE_NONE;
									tokenType = LITERAL;
								}
								// --------------------------------------
								// BUT IF WE REACH THE END OF THE LINE
								// AND WE HAVEN'T FOUND A '"' THEN WE
								// SHOULD SEND AN ERROR
								// --------------------------------------
								else if((c=='\n')||(c=='\r'))
								{
									m_state = STATE_NONE;
									tokenType = LITERAL;
									sprintf(error, "Error: String not closed at line %d",
											m_lineNumber);
									AddError(error);
								}
								// --------------------------------------
								// ELSE, ADD CHAR
								// --------------------------------------
								else if(IsValidChar(c) || (c=='\\') || 
										(c=='.') || (c=='/'))
								{
									nToken[j++] = c;
								}

								break;
		} // END SWITCH

		// IF WE HAVE A TOKEN IN OUR BUFFER, ADD IT TO THE TOKENS LIST
		if(tokenType!=NONE)
		{
			nToken[j] = '\0';
			processed = AddToken(nToken, tokenType);			
			memset(nToken,0,sizeof(nToken));
			j = 0;	
			tokenType = NONE;
		}

	} // END c NOT '\n'

	return processed;
}

// 
// Add a token to the list
// 
bool CParticleScript::AddToken(char *token, SCR_TOKEN_TYPE type)
{
	bool added = true;
	CTokenNode *newToken = new CTokenNode(token, type, m_lineNumber);
	assert(newToken != NULL && "AddToken");
	if(newToken!=NULL)
	{
		m_tokenList.push_back(newToken);
	}
	else
		added = false;

	return added;
}

// 
// Analyze Script
// 
bool CParticleScript::ParseScript()
{
	bool	scriptOk=true, continueParsing=true;
	char	error[SCRIPT_LINE_LENGTH];
	
	memset(error,0,sizeof(error));

	// PROCESS ALL TOKENS
	do{
		if(m_tokenList.size()==0)
		{
			sprintf(error,"Unexpected end of script");
			AddError(error);
			break;
		}

		m_tokenIterator = m_tokenList.begin();
		if(m_tokenIterator == m_tokenList.end())
			break;

		// FIRST TOKEN SHOULD BE PARTICLESYSTEM KEYWORD
		if(!(CheckToken(KEYWORD_PARTICLESYSTEM, "ParticleSystem Keyword")))
			scriptOk = false;
		if(!(continueParsing = GetNextToken()))
			break;

		// This token is the script name
		if(!(CheckToken(IDENTIFIER, "Particle System Identifier")))
			scriptOk = false;
		if(!(continueParsing = GetNextToken()))
			break;

		// This token should be '{'
		if(!(CheckToken(CURLYBRACE_OPEN, "{")))
			scriptOk = false;
		if(!(continueParsing = GetNextToken()))
			break;

		// This token should be 'SystemProperties'
		if(CheckToken(KEYWORD_SYSTEMPROPERTIES, "SystemProperties"))
		{
			if(!(continueParsing = ParseSystemProperties()))
			{
				sprintf(error,"Error, final } expected");
				AddError(error);
				break;
			}
		}
		else
		{
			scriptOk = false;
			if(!(continueParsing = GetNextToken()))
				break;
		}

		// This token should be 'Emitter'
		if(CheckToken(KEYWORD_EMITTER, "Emitter"))
		{
			if(!(continueParsing = ParseEmitters()))
			{
				sprintf(error,"Error, final } expected");
				AddError(error);
				break;
			}
		}
		else
		{
			scriptOk = false;
			if(!(continueParsing = GetNextToken()))
				break;
		}

		if((*m_tokenIterator)->GetTokenType() == KEYWORD_EVENTAT)
		{
			if(!(continueParsing = ParseEvents()))
			{
				sprintf(error,"Error, final } expected");
				AddError(error);
				break;
			}
		}

		CheckToken(CURLYBRACE_CLOSE, "}");

	}while(0);
	// ----------------------------------
	// I KNOW THE DO{}WHILE(0); DOESN'T
	// LOOK VERY GOOD BUT IT ALLOWS ME
	// TO SKIP PROCESSING WITHOUT 
	// CREATING A LOT OF NESTED if's
	// ----------------------------------

	// IF ERRORS OCURRED IN OTHER METHODS
	if(m_numErrors > 0)
		scriptOk = false;

	// IF WE RAN OUT OF TOKENS
	if(!continueParsing)
	{
		sprintf(error,"Unexpected end of script");
		AddError(error);
	}

	return scriptOk;
}

// 
// Process System Properties
// 
bool CParticleScript::ParseSystemProperties()
{
	bool	continueParsing = true;
	char	error[SCRIPT_LINE_LENGTH];
	int		sysPropProcessed=1;

	memset(error,0,sizeof(error));
	ClearLastError();
	
	do{
		// MAKE SURE WE HAVE MORE TOKENS
		// WE SHOULD HAVE AT LEAST {}
		if(!(continueParsing = GetNextToken()))
			break;

		// FIRST TOKEN AFTER SYSTEMPROPERTIES SHOULD BE {
		CheckToken(CURLYBRACE_OPEN, "{");

		// TOKEN WAS {, GET NEXT TOKEN AND SEE IF IT'S A SYSTEM PROPERTY
		if(!(continueParsing = GetNextToken()))
			break;

		while((sysPropProcessed)&&(continueParsing))
		{
			sysPropProcessed = 0;

			if(!continueParsing)break;
			if((*m_tokenIterator)->GetTokenType() == KEYWORD_SYSP_MAXPARTICLES)
			{	continueParsing = Parse_SYSP_MaxParticles(); sysPropProcessed = 1;	}
			
			if(!continueParsing)break;
			if((*m_tokenIterator)->GetTokenType() == KEYWORD_SYSP_FORCE)
			{	continueParsing = Parse_SYSP_Force();	sysPropProcessed = 1;	}		
		}

		if(!continueParsing)
			break;

		// CHECK IF LAST TOKEN IS '}'
		CheckToken(CURLYBRACE_CLOSE, "}");

	}while(0);

	// IF WE RAN OUT OF TOKENS
	if(!continueParsing)
	{
		sprintf(error,"Unexpected end of script");
		AddError(error);
	}
	else
	{
		// PREPARE FOR NEXT TOKEN
		continueParsing = GetNextToken();
	}

	return continueParsing;
}

// 
// Process Emitter
// 
bool CParticleScript::ParseEmitters()
{
	bool	continueParsing = true;
	char	error[SCRIPT_LINE_LENGTH];
	int		emitterProp=1;

	memset(error,0,sizeof(error));
	ClearLastError();
	
	// WHILE THERE ARE EMITTERS TO PROCESS
	while( ((*m_tokenIterator)->GetTokenType() == KEYWORD_EMITTER) && (continueParsing))
	{
		emitterProp = 1;

		do{
			// MAKE SURE WE HAVE MORE TOKENS
			if(!(continueParsing = GetNextToken()))
				break;

			// FIRST TOKEN AFTER EMITTER SHOULD BE AN IDENTIFIER
			if((CheckToken(IDENTIFIER, "Identifier")))
			{
				// SAVE IDENTIFIER, WE NEED TO CHECK IF
				// IDENTIFIERS ARE DEFINED WHEN PARSING
				// THE EVENTS SECTION
				SaveIdentifier();
			}

			if(!(continueParsing = GetNextToken()))
				break;

			// SHOULD BE {
			CheckToken(CURLYBRACE_OPEN, "{");

			// TOKEN WAS {, GET NEXT TOKEN AND SEE IF IT'S AN EMITTER PROP
			if(!(continueParsing = GetNextToken()))
				break;

			while((emitterProp)&&(continueParsing))
			{
				emitterProp = 0;

				if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_POSITION)
				{	continueParsing = Parse_EM_Position();	emitterProp = 1;	}

				if(!continueParsing)break;
				if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_TYPE)
				{	continueParsing = Parse_EM_Type();	emitterProp = 1;	}

				if(!continueParsing)break;
				if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_PARTICLEREP)
				{	continueParsing = Parse_EM_ParticleRep();	emitterProp = 1;	}
		
				if(!continueParsing)break;
				if((*m_tokenIterator)->GetTokenType() == KEYWORD_SYSP_MAXPARTICLES)
				{	continueParsing = Parse_SYSP_MaxParticles(); emitterProp = 1;	}
			
				if(!continueParsing)break;
				if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_TEXTURE)
				{	continueParsing = Parse_EM_Texture();	emitterProp = 1;	}
		
				if(!continueParsing)break;
				if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_EMISSIONRATE)
				{	continueParsing = Parse_EM_EmissionRate();	emitterProp = 1;	}
		
				if(!continueParsing)break;
				if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_BATCHSIZERANGE)
				{	continueParsing = Parse_EM_BatchSize();	emitterProp = 1;	}
		
				if(!continueParsing)break;
				if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_BLENDINGMODES)
				{	continueParsing = Parse_EM_BlendingModes();	emitterProp = 1;	}
		
				if(!continueParsing)break;
				if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_VERTEXSHADER)
				{	continueParsing = Parse_EM_VertexShader();	emitterProp = 1;	}
		
				if(!continueParsing)break;
				if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_PIXELSHADER)
				{	continueParsing = Parse_EM_PixelShader();	emitterProp = 1;	}
		
				if(!continueParsing)break;
				if((*m_tokenIterator)->GetTokenType() == KEYWORD_SYSP_FORCE)
				{	continueParsing = Parse_EM_Force();	emitterProp = 1;	}

				if(!continueParsing)break;
				if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_LIFERANGE)
				{	continueParsing = Parse_EM_LifeRange();	emitterProp = 1;	}
		
				if(!continueParsing)break;
				if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_SIZERANGE)
				{	continueParsing = Parse_EM_SizeRange();	emitterProp = 1;	}
		
				if(!continueParsing)break;
				if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_STATE)
				{	continueParsing = Parse_EM_EmitterState();	emitterProp = 1;	}

				if(!continueParsing)break;
				if((*m_tokenIterator)->GetTokenType() == KEYWORD_PARTICLE_COLOR)
				{	continueParsing = Parse_Particle_Color();	emitterProp=1;		}
			}

			if(!continueParsing)
				break;

			// CHECK IF LAST TOKEN IS '}'
			CheckToken(CURLYBRACE_CLOSE, "}");

		}while(0);

		// PREPARE FOR NEXT TOKEN
		if(continueParsing)
			continueParsing = GetNextToken();
	}

	// IF WE RAN OUT OF TOKENS
	if(!continueParsing)
	{
		sprintf(error,"Unexpected end of script");
		AddError(error);
	}

	return continueParsing;
}

// 
// Process Events
// 
bool CParticleScript::ParseEvents()
{
	bool	continueParsing = true;
	char	error[SCRIPT_LINE_LENGTH];
	int		chgProcessed=1;

	memset(error,0,sizeof(error));
	ClearLastError();

	// WHILE THERE ARE EMITTERS TO PROCESS
	while( ((*m_tokenIterator)->GetTokenType() == KEYWORD_EVENTAT) && (continueParsing))
	{
		chgProcessed = 1;

		do{
			// FIRST TOKEN AFTER EVENT_AT SHOULD BE A NUMBER
			if(!(continueParsing = ParseNumber()))
				break;

			// FIRST TOKEN AFTER THE NUMBER SHOULD BE {
			CheckToken(CURLYBRACE_OPEN, "{");

			if(!(continueParsing = GetNextToken()))
				break;

//			CheckToken(CURLYBRACE_OPEN, "{");

//			if(!(continueParsing = GetNextToken()))
//				break;

			while((chgProcessed)&&(continueParsing))
			{
				chgProcessed = 0;
	
				if((*m_tokenIterator)->GetTokenType() == KEYWORD_SYSPROPCHG)
				{	continueParsing = ParseSystemPropChg();	chgProcessed = 1;	}
		
				if(!continueParsing)break;
				if((*m_tokenIterator)->GetTokenType() == KEYWORD_EMITTERPROPCHG)
				{	continueParsing = ParseEmitterPropChg(); chgProcessed = 1;	}
			}

			if(!continueParsing)
				break;

			// CHECK IF LAST TOKEN IS '}'
			CheckToken(CURLYBRACE_CLOSE, "}");

			if(!(continueParsing = GetNextToken()))
				break;

		}while(0);
	}

	// IF WE RAN OUT OF TOKENS
	if(!continueParsing)
	{
		sprintf(error,"Unexpected end of script");
		AddError(error);
	}
//	else
//	{
		// PREPARE FOR NEXT TOKEN
//		continueParsing = GetNextToken();
//	}

	return continueParsing;
}

// 
// 
// 
bool CParticleScript::ParseSystemPropChg()
{
	bool	continueParsing = true;
	char	error[SCRIPT_LINE_LENGTH];
	int		sysChange=1, count=0;

	memset(error,0,sizeof(error));
	ClearLastError();

	do{
		// MAKE SURE WE HAVE MORE TOKENS
		if(!(continueParsing = GetNextToken()))
			break;

		// CHECK IF NEXT TOKEN IS '{'
		CheckToken(CURLYBRACE_OPEN, "{");
		
		if(!(continueParsing = GetNextToken()))
			break;

		while((sysChange)&&(continueParsing))
		{
			sysChange = 0;

			if((*m_tokenIterator)->GetTokenType() == KEYWORD_SYSP_FORCE)
			{	continueParsing = Parse_SYSP_Force(); sysChange = 1; count++; }
		}
		if(count==0)
		{
			sprintf(error,"Error, No system property that can be changed was found");
			AddError(error);					
		}

		if(!continueParsing)
			break;

		// CHECK IF LAST TOKEN IS '}'
		CheckToken(CURLYBRACE_CLOSE, "}");

	}while(0);

	// IF WE RAN OUT OF TOKENS
	if(!continueParsing)
	{
		sprintf(error,"Unexpected end of script");
		AddError(error);
	}
	else
	{
		// PREPARE FOR NEXT TOKEN
		continueParsing = GetNextToken();
	}

	return continueParsing;
}

// 
// 
// 
bool CParticleScript::ParseEmitterPropChg()
{
	bool	continueParsing = true;
	char	error[SCRIPT_LINE_LENGTH];
	int		eventChange=1;

	memset(error,0,sizeof(error));
	ClearLastError();

	do{
		// MAKE SURE WE HAVE MORE TOKENS
		if(!(continueParsing = GetNextToken()))
			break;

		// NEXT TOKEN SHOULD BE AN IDENTIFIER
		if(CheckToken(IDENTIFIER, "Identifier"))
		{
			// CHECK THIS IS A VALID IDENTIFIER
			// 
			if(!(IdentifierDefined()))
			{
				sprintf(error,"Undefined Identifier found");
				AddError(error);
			}
		}
		
		if(!(continueParsing = GetNextToken()))
			break;

		// NEXT TOKEN SHOULD BE '{'
		CheckToken(CURLYBRACE_OPEN, "{");

		if(!(continueParsing = GetNextToken()))
			break;

		while((eventChange)&&(continueParsing))
		{
			eventChange = 0;

			// CAN ONLY CHANGE THESE EMITTER PROPERTIES IN AN EVENT
			// ====================================================
			if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_POSITION)
			{	continueParsing = Parse_EM_Position();	eventChange=1;	}

			if(!continueParsing)break;
			if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_TEXTURE)
			{	continueParsing = Parse_EM_Texture();	eventChange=1;	}

			if(!continueParsing)break;
			if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_EMISSIONRATE)
			{	continueParsing = Parse_EM_EmissionRate();	eventChange=1;	}

			if(!continueParsing)break;
			if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_BATCHSIZERANGE)
			{	continueParsing = Parse_EM_BatchSize();	eventChange=1;	}

			if(!continueParsing)break;
			if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_BLENDINGMODES)
			{	continueParsing = Parse_EM_BlendingModes();	eventChange=1;	}

			if(!continueParsing)break;
			if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_VERTEXSHADER)
			{	continueParsing = Parse_EM_VertexShader();	eventChange=1;	}
		
			if(!continueParsing)break;
			if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_PIXELSHADER)
			{	continueParsing = Parse_EM_PixelShader();	eventChange=1;	}

			if(!continueParsing)break;
			if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_STATE)
			{	continueParsing = Parse_EM_EmitterState();	eventChange=1;	}

			if(!continueParsing)break;
			if((*m_tokenIterator)->GetTokenType() == KEYWORD_SYSP_FORCE)
			{	continueParsing = Parse_EM_Force();	eventChange=1;	}

			// PARTICLE PROPERTIES THAT CAN CHANGE
			// ===================================
			if(!continueParsing)break;
			if((*m_tokenIterator)->GetTokenType() == KEYWORD_PARTICLE_COLOR)
			{	continueParsing = Parse_Particle_Color();	eventChange=1;	}

			if(!continueParsing)break;
			if((*m_tokenIterator)->GetTokenType() == KEYWORD_PARTICLE_SIZE)
			{	continueParsing = Parse_Particle_Size();	eventChange=1;	}

			if(!continueParsing)break;
			if((*m_tokenIterator)->GetTokenType() == KEYWORD_PARTICLE_WEIGHT)
			{	continueParsing = Parse_Particle_Weight();	eventChange=1;	}

			if(!continueParsing)break;
			if((*m_tokenIterator)->GetTokenType() == KEYWORD_PARTICLE_FADE)
			{	continueParsing = Parse_Particle_Fade();	eventChange=1;	}
		}

		if(!continueParsing)
			break;

		// CHECK IF LAST TOKEN IS '}'
		CheckToken(CURLYBRACE_CLOSE, "}");

	}while(0);

	// IF WE RAN OUT OF TOKENS
	if(!continueParsing)
	{
		sprintf(error,"Unexpected end of script");
		AddError(error);
	}
	else
	{
		// PREPARE FOR NEXT TOKEN
		continueParsing = GetNextToken();
	}

	return continueParsing;
}

// ************************************************************************
// ========================================================================
// ************************************************************************

// 
// 
// 
bool CParticleScript::Parse_SYSP_MaxParticles()
{
	// @1 REMOVED CODE AND CALL THE NEXT METHOD INSTEAD
	return ParseAssignmentNum();
}

// 
// 
// 
bool CParticleScript::Parse_SYSP_Force()
{
	bool	continueParsing = true;
	char	error[SCRIPT_LINE_LENGTH];

	memset(error,0,sizeof(error));
	ClearLastError();

	do{
		// MAKE SURE WE HAVE MORE TOKENS
		if(!(continueParsing = GetNextToken()))
			break;

		// FIRST TOKEN AFTER FORCE SHOULD BE '['
		CheckToken(BRACKET_OPEN, "[");

		// TOKEN WAS '[', CHECK IF WE HAVE A NUMBER NEXT
		if(!(continueParsing = ParseNumber()))
			break;

		// TOKEN AFTER THE NUMBER SHOULD BE ']'
		CheckToken(BRACKET_CLOSE, "]");

		if(!(continueParsing = GetNextToken()))
			break;

		// TOKEN AFTER THE NUMBER SHOULD BE ']'
		CheckToken(ASSIGNMENT_OPERATOR, "=");

		// CHECK IF WE HAVE A VECTOR NEXT
		if(!(continueParsing = ParseVector()))
			break;

		// AFTER THE VECTOR WE SHOULD HAVE A ';'
		CheckToken(SEMI_COLON,";");
	}while(0);

	// IF WE RAN OUT OF TOKENS
	if(!continueParsing)
	{
		sprintf(error,"Unexpected end of script");
		AddError(error);
	}
	else
	{
		// PREPARE FOR NEXT TOKEN
		continueParsing = GetNextToken();
	}

	return continueParsing;
}

// ************************************************************************
// ========================================================================
// ************************************************************************

// 
// 
// 
bool CParticleScript::Parse_EM_Position()
{
	// @1 REMOVED CODE AND CALL NEXT METHOD INSTEAD
	return ParseAssignmentVec();
}

// 
// 
// 
bool CParticleScript::Parse_EM_Type()
{
	bool	continueParsing = true;
	char	error[SCRIPT_LINE_LENGTH];

	memset(error,0,sizeof(error));
	ClearLastError();

	do{
		// MAKE SURE WE HAVE MORE TOKENS
		if(!(continueParsing = GetNextToken()))
			break;

		// FIRST TOKEN AFTER EMITTERTYPE SHOULD BE '='
		CheckToken(ASSIGNMENT_OPERATOR, "=");

		if(!(continueParsing = GetNextToken()))
			break;

		// CHECK IF WE HAVE AN EMITTER TYPE NEXT
		CheckToken(EMITTERTYPE_ID, "Emitter Type");

		if(!(continueParsing = GetNextToken()))
			break;

		// AFTER THE TYPE WE SHOULD HAVE A ';'
		CheckToken(SEMI_COLON, ";");
	}while(0);

	// IF WE RAN OUT OF TOKENS
	if(!continueParsing)
	{
		sprintf(error,"Unexpected end of script");
		AddError(error);
	}
	else
	{
		// PREPARE FOR NEXT TOKEN
		continueParsing = GetNextToken();
	}

	return continueParsing;
}

// 
// 
// 
bool CParticleScript::Parse_EM_ParticleRep()
{
	bool	continueParsing = true;
	char	error[SCRIPT_LINE_LENGTH];

	memset(error,0,sizeof(error));
	ClearLastError();

	do{
		// MAKE SURE WE HAVE MORE TOKENS
		if(!(continueParsing = GetNextToken()))
			break;

		// FIRST TOKEN AFTER PARTICLEREP SHOULD BE '='
		CheckToken(ASSIGNMENT_OPERATOR, "=");

		if(!(continueParsing = GetNextToken()))
			break;

		// CHECK IF WE HAVE A PARTICLE REP NEXT
		CheckToken(PARTICLEREP_ID, "Particle Representation");

		if(!(continueParsing = GetNextToken()))
			break;

		// WE SHOULD HAVE A ';'
		CheckToken(SEMI_COLON, ";");
	}while(0);

	// IF WE RAN OUT OF TOKENS
	if(!continueParsing)
	{
		sprintf(error,"Unexpected end of script");
		AddError(error);
	}
	else
	{
		// PREPARE FOR NEXT TOKEN
		continueParsing = GetNextToken();
	}

	return continueParsing;
}

// 
// 
// 
bool CParticleScript::Parse_EM_Texture()
{
	// @1 REMOVED CODE AND CALL NEXT METHOD INSTEAD
	return ParseAssignmentLit();
}

// 
// 
// 
bool CParticleScript::Parse_EM_EmissionRate()
{
	// @1 REMOVED CODE AND CALL NEXT METHOD INSTEAD
	return ParseAssignmentNum();
}

// 
// 
// 
bool CParticleScript::Parse_EM_BatchSize()
{
	// @1 REMOVED CODE AND CALL NEXT METHOD INSTEAD
	return ParseAssignmentNum();
}

// 
// 
// 
bool CParticleScript::Parse_EM_BlendingModes()
{
	bool	continueParsing = true;
	char	error[SCRIPT_LINE_LENGTH];

	memset(error,0,sizeof(error));
	ClearLastError();

	do{
		// MAKE SURE WE HAVE MORE TOKENS
		if(!(continueParsing = GetNextToken()))
			break;

		// FIRST TOKEN AFTER BLENDINGMODES SHOULD BE '='
		CheckToken(ASSIGNMENT_OPERATOR, "=");
		if(!(continueParsing = GetNextToken()))
			break;

		// AFTER '=' WE SHOULD HAVE A (
		CheckToken(PARENTHESIS_OPEN, "(");
		if(!(continueParsing = GetNextToken()))
			break;

		// AFTER '(' WE SHOULD HAVE AN BLENDING MODE
		CheckToken(BLENDINGMODE_ID, "Blending Mode Identifier");
		if(!(continueParsing = GetNextToken()))
			break;

		// AFTER BLENDING MODE HAVE A ','
		CheckToken(COMMA, ",");
		if(!(continueParsing = GetNextToken()))
			break;

		// AFTER ',' WE SHOULD HAVE AN BLENDING MODE
		CheckToken(BLENDINGMODE_ID, "Blending Mode Identifier");
		if(!(continueParsing = GetNextToken()))
			break;

		// AFTER BLENDING MODE HAVE A ')'
		CheckToken(PARENTHESIS_CLOSE, ")");
		if(!(continueParsing = GetNextToken()))
			break;

		// AFTER THE BLENDING MODE WE SHOULD HAVE A ';'
		CheckToken(SEMI_COLON, ";");
	}while(0);

	// IF WE RAN OUT OF TOKENS
	if(!continueParsing)
	{
		sprintf(error,"Unexpected end of script");
		AddError(error);
	}
	else
	{
		// PREPARE FOR NEXT TOKEN
		continueParsing = GetNextToken();
	}

	return continueParsing;
}

// 
// 
// 
bool CParticleScript::Parse_EM_VertexShader()
{
	// @1 REMOVED CODE AND CALL NEXT METHOD INSTEAD
	return ParseAssignmentLit();
}

// 
// 
// 
bool CParticleScript::Parse_EM_PixelShader()
{
	// @1 REMOVED CODE AND CALL NEXT METHOD INSTEAD
	return ParseAssignmentLit();
}

// 
// 
// 
bool CParticleScript::Parse_EM_Force()
{
	bool	continueParsing = true;
	char	error[SCRIPT_LINE_LENGTH];

	memset(error,0,sizeof(error));
	ClearLastError();

	do{
		// MAKE SURE WE HAVE MORE TOKENS
		if(!(continueParsing = GetNextToken()))
			break;

		// FIRST TOKEN AFTER FORCE SHOULD BE '['
		CheckToken(BRACKET_OPEN, "[");

		// TOKEN WAS '[', CHECK IF WE HAVE A NUMBER NEXT
		if(!(continueParsing = ParseNumber()))
			break;

		// TOKEN AFTER THE NUMBER SHOULD BE ']'
		CheckToken(BRACKET_CLOSE, "]");

		if(!(continueParsing = GetNextToken()))
			break;

		// TOKEN AFTER THE ] SHOULD BE '='
		CheckToken(ASSIGNMENT_OPERATOR, "=");

		// CHECK IF WE HAVE A VECTOR NEXT
		if(!(continueParsing = ParseVector()))
			break;

		// WE SHOULD HAVE THE POSITION OF THE FORCE NEXT
		CheckToken(KEYWORD_ATPOS, "at_pos");
		if(!(continueParsing = ParseVector()))
			break;

		// WE SHOULD HAVE THE DISTANCE OF MAX EFFECT
		CheckToken(KEYWORD_DIST, "dist");
		if(!(continueParsing = ParseNumber()))
			break;

		// AFTER THE VECTOR WE SHOULD HAVE A ';'
		CheckToken(SEMI_COLON,";");
	}while(0);

	// IF WE RAN OUT OF TOKENS
	if(!continueParsing)
	{
		sprintf(error,"Unexpected end of script");
		AddError(error);
	}
	else
	{
		// PREPARE FOR NEXT TOKEN
		continueParsing = GetNextToken();
	}

	return continueParsing;
}

// 
// 
// 
bool CParticleScript::Parse_EM_LifeRange()
{
	bool	continueParsing = true;
	char	error[SCRIPT_LINE_LENGTH];

	memset(error,0,sizeof(error));
	ClearLastError();

	do{
		// MAKE SURE WE HAVE MORE TOKENS
		if(!(continueParsing = GetNextToken()))
			break;

		// FIRST TOKEN AFTER LIFERANGE SHOULD BE '='
		CheckToken(ASSIGNMENT_OPERATOR, "=");

		if(!(continueParsing = GetNextToken()))
			break;

		// NEXT TOKEN SHOULD BE '('
		CheckToken(PARENTHESIS_OPEN, "(");

		// CHECK IF WE HAVE A NUMBER NEXT
		if(!(continueParsing = ParseNumber()))
			break;

		// NEXT TOKEN SHOULD BE ','
		CheckToken(COMMA, ",");
		
		// CHECK IF WE HAVE A NUMBER NEXT
		if(!(continueParsing = ParseNumber()))
			break;

		// NEXT TOKEN SHOULD BE ')'
		CheckToken(PARENTHESIS_CLOSE, ")");

		if(!(continueParsing = GetNextToken()))
			break;

		// WE SHOULD HAVE A ';'
		CheckToken(SEMI_COLON, ";");
	}while(0);

	// IF WE RAN OUT OF TOKENS
	if(!continueParsing)
	{
		sprintf(error,"Unexpected end of script");
		AddError(error);
	}
	else
	{
		// PREPARE FOR NEXT TOKEN
		continueParsing = GetNextToken();
	}

	return continueParsing;
}

// 
// 
// 
bool CParticleScript::Parse_EM_SizeRange()
{
	// THIS HAS EXACTLY THE SAME SYNTAX TO CHECK AS Parse_EM_LifeRange
	// SO CALL THAT METHOD INSTEAD OF DUPLICATING THE CODE HERE
	return Parse_EM_LifeRange();	
}

// 
// 
// 
bool CParticleScript::Parse_EM_EmitterState()
{	
	bool continueParsing = true;
	char error[SCRIPT_LINE_LENGTH];

	memset(error,0,sizeof(error));
	ClearLastError();

	do{
		// MAKE SURE WE HAVE MORE TOKENS
		if(!(continueParsing = GetNextToken()))
			break;

		// AFTER EMITTERON/EMITTEROFF WE SHOULD HAVE A ';'
		CheckToken(SEMI_COLON,";");
	}while(0);

	// IF WE RAN OUT OF TOKENS
	if(!continueParsing)
	{
		sprintf(error,"Unexpected end of script");
		AddError(error);
	}
	else
	{
		// PREPARE FOR NEXT TOKEN
		continueParsing = GetNextToken();
	}

	return continueParsing;
}

// 
// 
// 
bool CParticleScript::Parse_Particle_Color()
{
	bool	continueParsing = true;
	char	error[SCRIPT_LINE_LENGTH];

	memset(error,0,sizeof(error));
	ClearLastError();

	do{
		// MAKE SURE WE HAVE MORE TOKENS
		if(!(continueParsing = GetNextToken()))
			break;

		// TOKEN AFTER THE NUMBER SHOULD BE '='
		CheckToken(ASSIGNMENT_OPERATOR, "=");

		// CHECK IF WE HAVE A COLOR NEXT
		if(!(continueParsing = ParseColor()))
			break;

		// AFTER THE VECTOR WE SHOULD HAVE A ';'
		CheckToken(SEMI_COLON,";");
	}while(0);

	// IF WE RAN OUT OF TOKENS
	if(!continueParsing)
	{
		sprintf(error,"Unexpected end of script");
		AddError(error);
	}
	else
	{
		// PREPARE FOR NEXT TOKEN
		continueParsing = GetNextToken();
	}

	return continueParsing;
}

// 
// 
// 
bool CParticleScript::Parse_Particle_Size()
{
	// @1 REMOVE CODE AND CALL NEXT METHOD INSTEAD
	return ParseAssignmentNum();
}

// 
// 
// 
bool CParticleScript::Parse_Particle_Weight()
{
	// @1 REMOVE CODE AND CALL NEXT METHOD INSTEAD
	return ParseAssignmentNum();
}

// 
// 
// 
bool CParticleScript::Parse_Particle_Fade()
{
	bool	continueParsing = true;
	char	error[SCRIPT_LINE_LENGTH];

	memset(error,0,sizeof(error));
	ClearLastError();

	do{
		// CHECK IF WE HAVE A NUMBER NEXT
		if(!(continueParsing = ParseNumber()))
			break;

		// PARTICLE PROPERTIES THAT CAN CHANGE
		// ===================================
		if((*m_tokenIterator)->GetTokenType() == KEYWORD_PARTICLE_COLOR)
			continueParsing = Parse_Particle_Color();
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_PARTICLE_SIZE)
			continueParsing = Parse_Particle_Size();
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_PARTICLE_WEIGHT)
			continueParsing = Parse_Particle_Weight();	

	}while(0);

	// IF WE RAN OUT OF TOKENS
	if(!continueParsing)
	{
		sprintf(error,"Unexpected end of script");
		AddError(error);
	}

	return continueParsing;
}

// 
// Parses a number
// 
bool CParticleScript::ParseNumber()
{
	bool	continueParsing = true;
	char	error[SCRIPT_LINE_LENGTH];

	memset(error,0,sizeof(error));
	ClearLastError();

	do{
		// MAKE SURE WE HAVE MORE TOKENS
		continueParsing = GetNextToken();
		if(!continueParsing)
			break;

		// WE SHOULD HAVE A NUMBER OR RANDOM(NUMBER, NUMBER)
		if((*m_tokenIterator)->GetTokenType() != NUMBER)
		{
			if((*m_tokenIterator)->GetTokenType() != KEYWORD_RANDOM)
				LogExpectedToken("Number or RandomRange(Number, Number)", true);
			else
			{
				continueParsing = GetNextToken();
				if(!continueParsing)
					break;
			
				if((*m_tokenIterator)->GetTokenType() != PARENTHESIS_OPEN)
					LogExpectedToken("(", true);

				continueParsing = GetNextToken();
				if(!continueParsing)
					break;

				if((*m_tokenIterator)->GetTokenType() != NUMBER)
					LogExpectedToken("Number", true);

				continueParsing = GetNextToken();
				if(!continueParsing)
					break;

				if((*m_tokenIterator)->GetTokenType() != COMMA)
					LogExpectedToken(",", true);

				continueParsing = GetNextToken();
				if(!continueParsing)
					break;

				if((*m_tokenIterator)->GetTokenType() != NUMBER)
					LogExpectedToken("Number", true);

				continueParsing = GetNextToken();
				if(!continueParsing)
					break;

				if((*m_tokenIterator)->GetTokenType() != PARENTHESIS_CLOSE)
					LogExpectedToken(")", true);
			}
		}
	}while(0);

	// IF WE RAN OUT OF TOKENS
	if(!continueParsing)
	{
		sprintf(error,"Unexpected end of script");
		AddError(error);
	}
	else
	{
		// PREPARE FOR NEXT TOKEN
		continueParsing = GetNextToken();
	}

	return continueParsing;
}

// 
// Parses a vector
// 
bool CParticleScript::ParseVector()
{
	bool	continueParsing = true;
	char	error[SCRIPT_LINE_LENGTH];
	int		i=0;

	memset(error,0,sizeof(error));
	ClearLastError();

	do{
		// MAKE SURE WE HAVE MORE TOKENS
		if(!(continueParsing = GetNextToken()))
			break;

		// TOKEN SHOULD BE 'VECTOR'
		CheckToken(KEYWORD_VECTOR, "Vector");
		if(!(continueParsing = GetNextToken()))
			break;

		// TOKEN SHOULD BE '('
		CheckToken(PARENTHESIS_OPEN, "(");

		while(i<3 && continueParsing)
		{
			if(continueParsing = ParseNumber())
			{
				if(i<2)
					CheckToken(COMMA, ",");
			}
			i++;
		}
		if(!continueParsing)
			break;

		// TOKEN SHOULD BE ')'
		CheckToken(PARENTHESIS_CLOSE, ")");
	}while(0);

	// IF WE RAN OUT OF TOKENS
	if(!continueParsing)
	{
		sprintf(error,"Unexpected end of script");
		AddError(error);
	}
	else
	{
		// PREPARE FOR NEXT TOKEN
		continueParsing = GetNextToken();
	}

	return continueParsing;
}

// 
// Parses a color
// 
bool CParticleScript::ParseColor()
{
	bool	continueParsing = true;
	char	error[SCRIPT_LINE_LENGTH];
	int		i=0;

	memset(error,0,sizeof(error));
	ClearLastError();

	do{
		// MAKE SURE WE HAVE MORE TOKENS
		if(!(continueParsing = GetNextToken()))
			break;

		// TOKEN SHOULD BE 'RGBA'
		CheckToken(KEYWORD_COLOR, "RGBA");
		if(!(continueParsing = GetNextToken()))
			break;

		// TOKEN SHOULD BE '('
		CheckToken(PARENTHESIS_OPEN, "(");

		while(i<4 && continueParsing)
		{
			if(continueParsing = ParseNumber())
			{
				if(i<3)
					CheckToken(COMMA, ",");
			}
			i++;
		}
		if(!continueParsing)
			break;

		// TOKEN SHOULD BE ')'
		CheckToken(PARENTHESIS_CLOSE, ")");
	}while(0);

	// IF WE RAN OUT OF TOKENS
	if(!continueParsing)
	{
		sprintf(error,"Unexpected end of script");
		AddError(error);
	}
	else
	{
		// PREPARE FOR NEXT TOKEN
		continueParsing = GetNextToken();
	}

	return continueParsing;
}

// 
// Checks if there are more tokens in the list
// 
bool CParticleScript::GetNextToken()
{		
	if(m_tokenIterator != m_tokenList.end())
	{
		m_tokenIterator++;
		if(m_tokenIterator != m_tokenList.end())
			return true;
		else
			return false;
	}		
	else
		return false;
}

// 
// Get previous token
// 
bool CParticleScript::GetPreviousToken()
{
	if(m_tokenIterator != m_tokenList.begin())
	{
		m_tokenIterator--;
		return true;
	}
	else
		return false;
}

// 
// Log Expected Token
// 
void CParticleScript::LogExpectedToken(char *eToken, bool goBack)
{
	int		errorLine = (*m_tokenIterator)->GetScrLine();
	char	error[SCRIPT_LINE_LENGTH];

	sprintf(error,"Error, %s expected at line %d", eToken, errorLine);
	AddError(error);
	// ---------------------------------------------------------------
	// (ALMOST) WHENEVER WE GET SOMETHING WE DON'T EXPECT, GO BACK ONE 
	// TOKEN SO THIS TOKEN CAN BE READ AGAIN AND CHECKED
	// so for example if you input this and you forget to define the:
	// script name and a semi colon in the emitterPos sentence
	// ParticleSystem
	// {
	//    SystemProperties
	//    {
	//      emitterPos = 50
	//    }  
	// }
	// the script will only log two errors: script name missing
	// and semi-colon missing, instead of many errors because everything
	// is out of place... if we didn't go back one token it would
	// expect 'SystemProperties' to be '{' and so on...
	// ---------------------------------------------------------------
	// THERE ARE SOME EXCEPTIONS... IE, WE CAN'T GO BACK IF WE ARE
	// ON THE FIRST TOKEN...
	// ---------------------------------------------------------------
	if(goBack)
		GetPreviousToken();
}

// 
// Check Current Token
// If it doesn't match what is expected then log an error
// 
bool CParticleScript::CheckToken(SCR_TOKEN_TYPE tkExpected, char *expected)
{
	bool matches = true, getPreviousToken = true;
	SCR_TOKEN_TYPE currentToken = (*m_tokenIterator)->GetTokenType();

	if(currentToken != tkExpected)
	{		
		if(currentToken == KEYWORD_PARTICLESYSTEM)
			getPreviousToken = false;
		LogExpectedToken(expected, getPreviousToken);
		matches = false;
	}

	return matches;
}

// 
// Saves an identifier
// 
void CParticleScript::SaveIdentifier()
{	
	char *newEmitterID=NULL;

	newEmitterID = new char[SCRIPT_TOKEN_LENGTH];
	memset(newEmitterID, 0, SCRIPT_TOKEN_LENGTH);
	(*m_tokenIterator)->GetToken(newEmitterID);
	m_emitterIDs.push_back(newEmitterID);
}

// 
// Checks if an identifier is defined
// 
bool CParticleScript::IdentifierDefined()
{
	bool found=false;
	char currIdentifier[SCRIPT_TOKEN_LENGTH];
	STL_strIter iterator;

	(*m_tokenIterator)->GetToken(currIdentifier);

	iterator = m_emitterIDs.begin();
	while((iterator != m_emitterIDs.end()) && !found)	
	{
		if(0 == memcmp((*iterator), currIdentifier, SCRIPT_TOKEN_LENGTH) )
			found = true;		
		
		++iterator;
	}

	return found;
}

// ******************************* @1A BEGINS *****************************

// 
// Parses the assignment of a number
// 
bool CParticleScript::ParseAssignmentNum()
{
	bool	continueParsing = true;
	char	error[SCRIPT_LINE_LENGTH];

	memset(error,0,sizeof(error));
	ClearLastError();

	do{
		// MAKE SURE WE HAVE MORE TOKENS
		if(!(continueParsing = GetNextToken()))
			break;

		// NEXT TOKEN SHOULD BE '='
		CheckToken(ASSIGNMENT_OPERATOR, "=");

		// TOKEN WAS '=', CHECK IF WE HAVE A NUMBER NEXT
		if(!(continueParsing = ParseNumber()))
			break;

		// AFTER THE NUMBER WE SHOULD HAVE A ';'
		CheckToken(SEMI_COLON, ";");
	}while(0);

	// IF WE RAN OUT OF TOKENS
	if(!continueParsing)
	{
		sprintf(error,"Unexpected end of script");
		AddError(error);
	}
	else
	{
		// PREPARE FOR NEXT TOKEN
		continueParsing = GetNextToken();
	}

	return continueParsing;
}

// 
// Parses the assignment of a literal
// 
bool CParticleScript::ParseAssignmentLit()
{
	bool	continueParsing = true;
	char	error[SCRIPT_LINE_LENGTH];

	memset(error,0,sizeof(error));
	ClearLastError();

	do{
		// MAKE SURE WE HAVE MORE TOKENS
		if(!(continueParsing = GetNextToken()))
			break;

		// NEXT TOKEN SHOULD BE '='
		CheckToken(ASSIGNMENT_OPERATOR, "=");

		if(!(continueParsing = GetNextToken()))
			break;

		// CHECK IF WE HAVE A LITERAL NEXT
		CheckToken(LITERAL, "Literal");

		if(!(continueParsing = GetNextToken()))
			break;

		// WE SHOULD HAVE A ';'
		CheckToken(SEMI_COLON, ";");
	}while(0);

	// IF WE RAN OUT OF TOKENS
	if(!continueParsing)
	{
		sprintf(error,"Unexpected end of script");
		AddError(error);
	}
	else
	{
		// PREPARE FOR NEXT TOKEN
		continueParsing = GetNextToken();
	}

	return continueParsing;
}

// 
// Parses the assignment of a vector
// 
bool CParticleScript::ParseAssignmentVec()
{
	bool	continueParsing = true;
	char	error[SCRIPT_LINE_LENGTH];

	memset(error,0,sizeof(error));
	ClearLastError();

	do{
		// MAKE SURE WE HAVE MORE TOKENS
		if(!(continueParsing = GetNextToken()))
			break;

		// NEXT TOKEN SHOULD BE '='
		CheckToken(ASSIGNMENT_OPERATOR, "=");

		// CHECK IF WE HAVE A VECTOR NEXT
		if(!(continueParsing = ParseVector()))
			break;

		// AFTER THE VECTOR WE SHOULD HAVE A ';'
		CheckToken(SEMI_COLON, ";");
	}while(0);

	// IF WE RAN OUT OF TOKENS
	if(!continueParsing)
	{
		sprintf(error,"Unexpected end of script");
		AddError(error);
	}
	else
	{
		// PREPARE FOR NEXT TOKEN
		continueParsing = GetNextToken();
	}

	return continueParsing;
}

// ******************************** @1A ENDS ******************************

// 
// Tmp method to output all tokens to file
// *Method created to make debugging easier when parsing*
// 
void CParticleScript::OutputTokensToFile(FILE *pFile)
{	
	assert(pFile != NULL && "OutputTokensToFile");
	char outToken[SCRIPT_TOKEN_LENGTH];
	char outTokenType[30];
	char outLine[SCRIPT_TOKEN_LENGTH + 35];
	int  offset=0;
	
	for(m_tokenIterator =  m_tokenList.begin();
		m_tokenIterator != m_tokenList.end();
		++m_tokenIterator)
	{
		offset = 0;
		memset(outToken, 0, sizeof(outToken));
		memset(outTokenType, 0, sizeof(outTokenType));

		(*m_tokenIterator)->GetToken(outToken);
		(*m_tokenIterator)->GetTokenTypeStr(outTokenType);
		
		memset(outLine, 0, sizeof(outLine));
		memcpy(outLine, outTokenType, sizeof(outTokenType));
		offset += sizeof(outTokenType);
		memcpy(outLine+offset, " - ", 3);
		offset += 3;
		memcpy(outLine+offset, outToken, sizeof(outToken));
		offset += sizeof(outToken);
		memcpy(outLine+offset, "\n", 1);

		int nwritten = fwrite(outLine, sizeof(char), sizeof(outLine)-1, pFile ); 
		assert(nwritten > 0);
	}
}

// 
// 
bool CParticleScript::HasMoreErrors(char *error)
{
	if(m_currError < m_numErrors)
	{
		memset(error,0,SCRIPT_LINE_LENGTH);
		memcpy(error, m_errorsList[m_currError], SCRIPT_LINE_LENGTH);
		m_currError++;	
		return true;
	}
	else return false;
}

// 
// SET TOKEN ITERATOR TO THE SYSTEM PROPERTIES
// 
bool CParticleScript::SetIterator_SystemProperties()
{	
	SCR_TOKEN_TYPE token;
	bool found = false, moreTokens = true;

	m_tokenIterator = m_tokenList.begin();
	while((m_tokenIterator != m_tokenList.end()) && !found && moreTokens)
	{
		token = (*m_tokenIterator)->GetTokenType();
		if(token == KEYWORD_SYSTEMPROPERTIES)
			found=true;

		moreTokens = GetNextToken();		
	}

	return found;
}

// 
// SET TOKEN ITERATOR TO THE EMITTERS SECTION
// 
bool CParticleScript::SetIterator_Emitters()
{	
	SCR_TOKEN_TYPE token;
	bool found = false, moreTokens = true;

	m_tokenIterator = m_tokenList.begin();
	while((m_tokenIterator != m_tokenList.end()) && !found && moreTokens)
	{
		token = (*m_tokenIterator)->GetTokenType();
		if(token == KEYWORD_EMITTER)
			found=true;
		else
			moreTokens = GetNextToken();
	}

	return found;
}

// 
// SET TOKEN ITERATOR TO THE EVENTS SECTION
// 
bool CParticleScript::SetIterator_Events()
{	
	SCR_TOKEN_TYPE token;
	bool found = false, moreTokens = true;

	m_tokenIterator = m_tokenList.begin();
	while((m_tokenIterator != m_tokenList.end()) && !found && moreTokens)
	{
		token = (*m_tokenIterator)->GetTokenType();
		if(token == KEYWORD_EVENTAT)
			found = true;

		moreTokens = GetNextToken();
	}

	return found;
}

// 
// SCRIPT HAS MORE SYSTEM PROPERTIES?
// 
bool CParticleScript::MoreSystemProperties()
{
	bool found=false, endOfSection=false, moreTokens=true, emitterChgFound=false;

	// THIS METHOD INCREMENTS THE TOKEN ITERATOR UNTIL ANOTHER
	// SYSTEM PROPERTY IS FOUND OR STOPS AND RETURNS FALSE IF
	// A CLOSING } IS FOUND (END OF SECTION)

	while(!endOfSection && !found && moreTokens && !emitterChgFound)
	{						
		if((*m_tokenIterator)->GetTokenType() == KEYWORD_SYSP_MAXPARTICLES)
			found = true;
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_SYSP_FORCE)
			found = true;
		else if((*m_tokenIterator)->GetTokenType() == CURLYBRACE_CLOSE)
			endOfSection = true;
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_EMITTERPROPCHG)
			emitterChgFound = true;
		else
		{
			// IF NO PROP FOUND, GET NEXT TOKEN
			moreTokens = GetNextToken();
		}
	}

	return found;
}

// 
// SCRIPT HAS MORE EMITTERS?
// 
bool CParticleScript::MoreEmitters()
{
	SCR_TOKEN_TYPE token;
	bool found = false, moreTokens = true, eventsFound = false;

	while(!found && moreTokens && !eventsFound)
	{
		token = (*m_tokenIterator)->GetTokenType();
		if(token == KEYWORD_EMITTER)
			found=true;
		else if(token == KEYWORD_EVENTAT)
			eventsFound = true;
		else
			moreTokens = GetNextToken();		
	}

	return found;
}

// 
// SCRIPT HAS MORE EMITTER PROPERTIES?
// 
bool CParticleScript::MoreEmitterProperties()
{
	bool found=false, endOfSection=false, moreTokens=true;

	while(!endOfSection && !found && moreTokens)
	{		
		if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_POSITION)
			found = true;	
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_TYPE)
			found = true;		
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_PARTICLEREP)
			found = true;		
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_SYSP_MAXPARTICLES)
			found = true;			
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_TEXTURE)
			found = true;
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_EMISSIONRATE)
			found = true;
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_BATCHSIZERANGE)
			found = true;
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_BLENDINGMODES)
			found = true;
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_VERTEXSHADER)
			found = true;		
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_PIXELSHADER)
			found = true;
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_SYSP_FORCE)
			found = true;
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_LIFERANGE)
			found = true;
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_SIZERANGE)
			found = true;
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_STATE)
			found = true;		
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_PARTICLE_COLOR) //
			found = true;													  //
		else if((*m_tokenIterator)->GetTokenType() == CURLYBRACE_CLOSE)
			endOfSection = true;
		else
		{
			// IF NO PROP FOUND, GET NEXT TOKEN
			moreTokens = GetNextToken();
		}

		// BE READY TO PROCESS MORE EMITTERS
		if(endOfSection)
			moreTokens = GetNextToken();
	}

	return found;
}

// 
// SCRIPT HAS MORE EVENTS?
// IF TOKEN IS NOT EVENT_AT, NO MORE EVENTS TO BE PROCESSED
// 
bool CParticleScript::MoreEvents()
{
	bool found=false, moreTokens=true;
	SCR_TOKEN_TYPE token;

	if(m_tokenIterator != m_tokenList.end())
	{
		token = (*m_tokenIterator)->GetTokenType();

		if(token==CURLYBRACE_CLOSE)
			moreTokens = GetNextToken();
	
		if(moreTokens)
		{
			token = (*m_tokenIterator)->GetTokenType();
			if(token == KEYWORD_EVENTAT)
				found = true;
		}
	}

	return found;
}

// 
// SCRIPT HAS MORE EMITTER CHANGES?
// IF TOKEN IS NOT FOR_EMITTER, NO MORE CHANGES TO BE PROCESSED FOR AN EMITTER
// 
bool CParticleScript::MoreEmitterChanges(char *emitterID)
{
	bool found=false, moreTokens=true;
	SCR_TOKEN_TYPE token;
	char value[SCRIPT_TOKEN_LENGTH];

	token = (*m_tokenIterator)->GetTokenType();

	// IF JUST FINISHED READING SYSPROP CHANGES WE NEED THE NEXT TOKEN...
	if(token == CURLYBRACE_CLOSE)
	{
		moreTokens = GetNextToken();
		if(moreTokens)
			token = (*m_tokenIterator)->GetTokenType();
	}

	if(moreTokens && (token == KEYWORD_EMITTERPROPCHG))
		found = true;

	if(found)
	{
		if(GetNextToken())
		{
			(*m_tokenIterator)->GetToken(value);
			value[SCRIPT_TOKEN_LENGTH-1]='\0';
			memcpy(emitterID,value,SCRIPT_TOKEN_LENGTH-1);
		}
	}	

	return found;
}

// 
// SCRIPT HAS MORE PARTICLE EMITTER OR PROPERTIES CHANGES?
// (BASICALLY, A COPY OF MOREEMITTERPROPERTIES)
// 
bool CParticleScript::More_EmitterORParticle_Changes()
{
	bool found=false, endOfSection=false, moreTokens=true;

	while(!endOfSection && !found && moreTokens)
	{		
		// EMITTER CHANGES
		if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_POSITION)
			found = true;	
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_TEXTURE)
			found = true;
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_EMISSIONRATE)
			found = true;
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_BATCHSIZERANGE)
			found = true;
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_BLENDINGMODES)
			found = true;
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_VERTEXSHADER)
			found = true;		
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_PIXELSHADER)
			found = true;
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_SYSP_FORCE)
			found = true;
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_STATE)
			found = true;	
		// PARTICLE CHANGES
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_PARTICLE_COLOR)
			found = true;
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_PARTICLE_SIZE)
			found = true;
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_PARTICLE_WEIGHT)
			found = true;
		else if((*m_tokenIterator)->GetTokenType() == KEYWORD_PARTICLE_FADE)
			found = true;
		else if((*m_tokenIterator)->GetTokenType() == CURLYBRACE_CLOSE)
			endOfSection = true;
		else
		{
			// IF NO PROP FOUND, GET NEXT TOKEN
			moreTokens = GetNextToken();
		}

		// BE READY TO PROCESS MORE EMITTERS
		if(endOfSection)
			moreTokens = GetNextToken();
	}

	return found;
}

// 
// GET CURRENT TOKEN TYPE
// 
SCR_TOKEN_TYPE CParticleScript::Get_PropertyType()
{
	assert(m_tokenIterator!= m_tokenList.end() && "CParticleScript::Get_PropertyType");
	return (*m_tokenIterator)->GetTokenType();
}

// 
// GET VALUES
// 
bool CParticleScript::Get_IntValues(int *intVar, int numIntsToGet)
{
	bool valuesFound=false, moreTokens=true;
	int  copied=0, tempInt_1=0, tempInt_2=0;
	char value[SCRIPT_TOKEN_LENGTH];

	assert((intVar!=NULL) && (numIntsToGet>0) && "CParticleScript::Get_IntValues");

	while((copied<numIntsToGet) && moreTokens)
	{
		memset(value,0,sizeof(value));

		if((*m_tokenIterator)->GetTokenType() == KEYWORD_RANDOM)
		{
			if(!(moreTokens=GetNextToken()))
				break;
			if(!(moreTokens=GetNextToken()))
				break;

			(*m_tokenIterator)->GetToken(value);
			value[SCRIPT_TOKEN_LENGTH-1]='\0';
			tempInt_1 = atoi(value);

			if(!(moreTokens=GetNextToken()))
				break;
			if(!(moreTokens=GetNextToken()))
				break;

			(*m_tokenIterator)->GetToken(value);
			value[SCRIPT_TOKEN_LENGTH-1]='\0';
			tempInt_2 = atoi(value);

			if(tempInt_1 <= tempInt_2)
				intVar[copied++] = RangedRandomInt(tempInt_1, tempInt_2);
			else
				intVar[copied++] = RangedRandomInt(tempInt_2, tempInt_1);
		}
		else if((*m_tokenIterator)->GetTokenType() == NUMBER)
		{
			(*m_tokenIterator)->GetToken(value);
			value[SCRIPT_TOKEN_LENGTH-1]='\0';
			intVar[copied++]=atoi(value);
		}				
		
		moreTokens = GetNextToken();
	}

	if(copied==numIntsToGet)
		valuesFound = true;

	return valuesFound;
}

// 
// GET VALUES
// 
bool CParticleScript::Get_FloatValues(float *floatVar, int numFloatsToGet)
{
	bool  valuesFound=false, moreTokens=true;
	int   copied=0;
	float tempFloat_1=0.0f, tempFloat_2=0.0f;
	char  value[SCRIPT_TOKEN_LENGTH];

	assert((floatVar!=NULL) && (numFloatsToGet>0) && "CParticleScript::Get_FloatValues");

	while((copied<numFloatsToGet) && moreTokens)
	{
		memset(value,0,sizeof(value));

		if((*m_tokenIterator)->GetTokenType() == KEYWORD_RANDOM)
		{
			if(!(moreTokens=GetNextToken()))
				break;
			if(!(moreTokens=GetNextToken()))
				break;
			
			(*m_tokenIterator)->GetToken(value);
			value[SCRIPT_TOKEN_LENGTH-1]='\0';
			tempFloat_1 = atof(value);

			if(!(moreTokens=GetNextToken()))
				break;
			if(!(moreTokens=GetNextToken()))
				break;

			(*m_tokenIterator)->GetToken(value);
			value[SCRIPT_TOKEN_LENGTH-1]='\0';
			tempFloat_2 = atof(value);

			if(tempFloat_1 <= tempFloat_2)
				floatVar[copied++] = RangedRandom(tempFloat_1, tempFloat_2);
			else
				floatVar[copied++] = RangedRandom(tempFloat_2, tempFloat_1);
		}
		else if((*m_tokenIterator)->GetTokenType() == NUMBER)
		{
			(*m_tokenIterator)->GetToken(value);
			value[SCRIPT_TOKEN_LENGTH-1]='\0';
			floatVar[copied++]=atof(value);
		}				
		
		moreTokens = GetNextToken();
	}

	if(copied==numFloatsToGet)
		valuesFound = true;

	return valuesFound;
}

// 
// GET VALUES
// 
bool CParticleScript::Get_StringValues(char *literal, int numStringsToGet)
{
	bool  valuesFound=false, moreTokens=true;
	int   copied=0;	
	char  value[SCRIPT_TOKEN_LENGTH];	

	assert((literal!=NULL) && (numStringsToGet>0) && "CParticleScript::Get_StringValues");

	while((copied<numStringsToGet) && moreTokens)
	{
		memset(value,0,sizeof(value));

		if((*m_tokenIterator)->GetTokenType() == LITERAL)
		{
			(*m_tokenIterator)->GetToken(value);
			value[SCRIPT_TOKEN_LENGTH-1]='\0';
			
			memcpy(literal,value,strlen(value));
			copied++;
		}
		moreTokens = GetNextToken();
	}

	if(copied==numStringsToGet)
		valuesFound = true;

	return valuesFound;	
}

// 
// GET VALUES
// 
bool CParticleScript::Get_Identifier(char *identifier, int numIDsToGet)
{
	bool  valuesFound=false, moreTokens=true;
	int   copied=0;	
	char  value[SCRIPT_TOKEN_LENGTH];	

	assert((identifier!=NULL) && (numIDsToGet>0) && "CParticleScript::Get_Identifier");

	while((copied<numIDsToGet) && moreTokens)
	{
		memset(value,0,sizeof(value));

		if((*m_tokenIterator)->GetTokenType() == IDENTIFIER)
		{
			(*m_tokenIterator)->GetToken(value);
			value[SCRIPT_TOKEN_LENGTH-1]='\0';
			
			memcpy(identifier,value,strlen(value));
			copied++;
		}
		moreTokens = GetNextToken();
	}

	if(copied==numIDsToGet)
		valuesFound = true;

	return valuesFound;	
}

// 
// GET VALUES
// 
bool CParticleScript::Get_TypeValues(PSYS_EMITTERTYPE *emitterType, int numTypesToGet)
{
	bool  valuesFound=false, moreTokens=true;
	int   copied=0;	
	char  value[SCRIPT_TOKEN_LENGTH];	

	assert((emitterType!=NULL) && (numTypesToGet>0) && "CParticleScript::Get_TypeValues");

	while((copied<numTypesToGet) && moreTokens)
	{
		memset(value,0,sizeof(value));

		if((*m_tokenIterator)->GetTokenType() == EMITTERTYPE_ID)
		{
			(*m_tokenIterator)->GetToken(value);
			value[SCRIPT_TOKEN_LENGTH-1]='\0';

			if(0==memcmp(value, "E_LINE", 6))
				emitterType[copied++] = E_LINE;
			else if(0==memcmp(value, "E_POINT", 7))
				emitterType[copied++] = E_POINT;
			else if(0==memcmp(value, "E_SPHERE", 8))
				emitterType[copied++] = E_SPHERE;
			else if(0==memcmp(value, "E_BEAM", 6))
				emitterType[copied++] = E_BEAM;
			else if(0==memcmp(value, "E_RING", 6))
				emitterType[copied++] = E_RING;

		}
		moreTokens = GetNextToken();
	}

	if(copied==numTypesToGet)
		valuesFound = true;

	return valuesFound;	
}

// 
// GET VALUES
// 
bool CParticleScript::Get_RepValues(PARTICLE_REP *particleRep, int numRepToGet)
{
	bool  valuesFound=false, moreTokens=true;
	int   copied=0;	
	char  value[SCRIPT_TOKEN_LENGTH];	

	assert((particleRep!=NULL) && (numRepToGet>0) && "CParticleScript::Get_RepValues");

	while((copied<numRepToGet) && moreTokens)
	{
		memset(value,0,sizeof(value));

		if((*m_tokenIterator)->GetTokenType() == PARTICLEREP_ID)
		{
			(*m_tokenIterator)->GetToken(value);
			value[SCRIPT_TOKEN_LENGTH-1]='\0';

			if(0==memcmp(value, "R_POINTSPRITES", 14))
				particleRep[copied++] = R_POINT_SPRITES;
			else if(0==memcmp(value, "R_POINTS", 8))
				particleRep[copied++] = R_POINTS;
			else if(0==memcmp(value, "R_QUADS", 7))
				particleRep[copied++] = R_QUADS;
			else if(0==memcmp(value, "R_LINES", 7))
				particleRep[copied++] = R_LINES;
		}
		moreTokens = GetNextToken();
	}

	if(copied==numRepToGet)
		valuesFound = true;

	return valuesFound;	
}

// 
// GET VALUES
// 
bool CParticleScript::Get_BlendValues(GLenum *blends, int numBlendsToGet)
{
	bool  valuesFound=false, moreTokens=true;
	int   copied=0;	
	char  value[SCRIPT_TOKEN_LENGTH];
	GLenum blendMode;

	assert((blends!=NULL) && (numBlendsToGet>0) && "CParticleScript::Get_BlendValues");

	while((copied<numBlendsToGet) && moreTokens)
	{
		memset(value,0,sizeof(value));

		if((*m_tokenIterator)->GetTokenType() == BLENDINGMODE_ID)
		{
			(*m_tokenIterator)->GetToken(value);
			value[SCRIPT_TOKEN_LENGTH-1]='\0';

			if(IsBlendingMode(value, &blendMode))
				blends[copied++] = blendMode;
		}
		moreTokens = GetNextToken();
	}

	if(copied==numBlendsToGet)
		valuesFound = true;

	return valuesFound;	
}

// 
// GET VALUES
// 
bool CParticleScript::Get_StateValues(int *intVar, int numIntsToGet)
{
	bool  valuesFound=false, moreTokens=true;
	int   copied=0;	
	char  value[SCRIPT_TOKEN_LENGTH];

	assert((intVar!=NULL) && (numIntsToGet>0) && "CParticleScript::Get_StateValues");

	while((copied<numIntsToGet) && moreTokens)
	{
		memset(value,0,sizeof(value));

		if((*m_tokenIterator)->GetTokenType() == KEYWORD_EM_STATE)
		{
			(*m_tokenIterator)->GetToken(value);
			value[SCRIPT_TOKEN_LENGTH-1]='\0';
			if(0==memcmp(value,"emitter_on",10))
				intVar[copied++]=1;
			else if(0==memcmp(value,"emitter_off",11))
				intVar[copied++]=0;			
		}
		moreTokens = GetNextToken();
	}

	if(copied==numIntsToGet)
		valuesFound = true;

	return valuesFound;
}

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%% END OF FILE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%





