/*************************************************************************/
/*|=====================================================================|*/
/*| Filename: textureManager.cpp                                        |*/
/*| Author: Joel Villagrana    (joel.villagrana@gmail.com)              |*/
/*|                                                                     |*/
/*| Classes Implemented:                                                |*/
/*| CTextureManager			                                            |*/
/*|                                                                     |*/
/*|%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%|*/
/*| dd/mm/yy   | Programmer        | Modifications                      |*/ 
/*|=====================================================================|*/
/*| 11/03/04   | JVillagrana       | Initial Version                    |*/
/*| 12/03/04   | JVillagrana   @1  | Add support to compressed tga files|*/
/*|---------------------------------------------------------------------|*/
/*| 10/07/05   | JVillagrana   @2  | Add filter as param                |*/
/*|---------------------------------------------------------------------|*/
/*| 01/10/05   | JVillagrana       | -Modified to use TGA loader from   |*/
/*|            |               @3  |  Dave                              |*/
/*************************************************************************/

#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <gl/glaux.h>

#include "textureManager.h"

#define GL_CLAMP_TO_EDGE	0x812F

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%###################################################################################%
%#																				   #%	
%#		IMPLEMENTATION OF THE CTextureManager CLASS METHODS						   #%
%#																				   #%
####################################################################################%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/


// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
// ==================================================================================
// CONSTRUCTOR 
// ==================================================================================
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
CTextureManager::CTextureManager()
{
	InitAll();
}

// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
// ==================================================================================
// DESTRUCTOR 
// ==================================================================================
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
CTextureManager::~CTextureManager()
{
}

// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
// ==================================================================================
// CLEAN UP, DELETE TEXTURES USED
// ==================================================================================
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
void CTextureManager::DestroyAll()
{
	m_textureFile	 = NULL;
	
	for(int i=0; i<MAX_TEXTURES; ++i)
		glDeleteTextures(1, &m_textures[i]); //@2A
	memset(m_textures,0,sizeof(m_textures)); //@2A

	m_texturesLoaded	= 1;
	m_currentOffset		= 0;
}

// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
// ==================================================================================
// INITIALIZE ALL 
// ==================================================================================
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
HRESULT CTextureManager::InitAll()
{
	m_textureFile		= NULL;
//	m_imageDataTGA		= NULL;	@3D
	m_targaLoader.Release();	//@3A
	m_imageDataBMP[0]	= NULL;
	m_currentOffset		= 0;
	m_texturesLoaded	= 1;
	memset(m_textures,0,sizeof(m_textures)); //@2A

	return S_OK;
}

// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
// ==================================================================================
// CREATE A TEXTURE FROM A TGA FILE
// THIS METHOD WILL CALL THE PROPER METHOD TO LOAD A COMPRESSED OR UNCOMPRESSED TGA
// ==================================================================================
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
HRESULT CTextureManager::LoadTGATexture( char filename[], int offset, int filter )
{
	// MAKE SURE FILENAME IS VALID
	assert( filename!=NULL && "CTextureManager::LoadTGATexture");

	/* *************** @3D ******************************
	// UNCOMPRESSED TGA HEADER
	GLubyte		UTGAheader[12]={0,0,2,0,0,0,0,0,0,0,0,0};
	// COMPRESSED TGA HEADER	
	GLubyte		CTGAheader[12]={0,0,10,0,0,0,0,0,0,0,0,0};	
	// USED TO COMPARE TGA HEADER
	GLubyte		TGAcompare[12];							
	// FIRST 6 USEFUL BYTES FROM THE HEADER
	GLubyte		header[6];				

	// OPEN THE TGA FILE AND CHECK THE HEADER
	// OPEN THE TGA FILE FOR BINARY READING
	m_textureFile = NULL;
	m_textureFile = fopen(filename, "rb");					
	
	// IF THE FILE DOESN'T EXISTS OR THE HEADER DOESN'T MATCH A TGA HEADER
	if(	m_textureFile==NULL														 ||	
		fread(TGAcompare,1,sizeof(TGAcompare),m_textureFile)!=sizeof(TGAcompare) ||	
		fread(header,1,sizeof(header),m_textureFile)!=sizeof(header))	
	{
		if (m_textureFile!=NULL)fclose(m_textureFile);
		return E_FAIL;										
	}
	****************** @3D **************************** */

	// SAVE OFFSET
	m_currentOffset = offset;

	if(m_targaLoader.Load(filename))
	{
		GLuint type;
		if(m_targaLoader.GetImageFormat() == 0)
			type = GL_RGB;
		else if(m_targaLoader.GetImageFormat() == 1)
			type = GL_RGBA;
	
		GenerateOpenGLTextureFromTGAData( type, filter );

		// INCREASE THE COUNTER OF TOTAL TEXTURES LOADED
		m_texturesLoaded++;
	}

	/* ******************* @3D BEGINS ***************************
	// CHANGED THE REST OF THE METHOD TO 2 SEPARATE METHODS, ONE FOR
	// COMPRESSED AND OTHER FOR UNCOMPRESSED TGA FILES

	// IT'S AN COMPRESSED TGA
	if(memcmp(UTGAheader, TGAcompare, sizeof(UTGAheader)) == 0)			
	{																	
		if(FAILED(LoadUncompressedTGATexture(header, filter)))
			return E_FAIL;
	}
	// IT'S A COMPRESSED TGA
	else if(memcmp(CTGAheader, TGAcompare, sizeof(CTGAheader)) == 0)	
	{																	
		if(FAILED(LoadCompressedTGATexture(header, filter)))
			return E_FAIL;
	}
	else
	{
		assert("Unsupported TGA header, Error loading TGA texture");
		if (m_textureFile!=NULL)fclose(m_textureFile);		
		return E_FAIL;
	}
	********************* @3D ENDS ***************************** */

	return S_OK;
}

/* ******************************** @2D CODE BEGINS ********************************
// ******************************** @1A CODE BEGINS ********************************
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
// ==================================================================================
// CREATE A TEXTURE FROM AN UNCOMPRESSED TGA FILE
// ==================================================================================
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
HRESULT CTextureManager::LoadUncompressedTGATexture( GLubyte tgaheader[],
																		int filter)
{
	// HOLDS NUMBER OF BYTES PER PIXEL USED IN THE TGA FILE
	GLuint		bytesPerPixel;									
	// USED TO STORE THE IMAGE SIZE WHEN SETTING ASIDE RAM
	GLuint		imageSize;										
	// SET THE DEFAULT GLMODE TO RBGA (32 BPP)
	GLuint		type = GL_RGBA;								
	// BITS PER PIXEL
	GLuint		txBpp;	
	// IMAGE WIDTH
	GLuint		txWidth;	
	// IMAGE HEIGHT
	GLuint		txHeight;		
	// TEMP TO SWAP BYTES
	GLuint		temp;									


	// DETERMINE THE TGA WIDTH	(highbyte*256+lowbyte)
	txWidth  = tgaheader[1] * 256 + tgaheader[0];	
	// DETERMINE THE TGA HEIGHT	(highbyte*256+lowbyte)
	txHeight = tgaheader[3] * 256 + tgaheader[2];	
	// DETERMINE THE BITS PER PIXEL (24 OR 32)
	txBpp	 = tgaheader[4];
    
	// CHECK IF THE WIDTH OR THE HEIGHT IS LESS THAN OR EQUAL TO 0
	// OR IF THE TGA IS NOT EITHER 24 OR 32 BITS
 	if(	txWidth<=0 || txHeight<=0 || (txBpp!=24 && txBpp!=32))
	{
		assert("TGA SIZE OR BPP ARE INVALID");
		fclose(m_textureFile);
		return E_FAIL;
	}

	// DIVIDE BY 8 TO GET THE BYTES PER PIXEL
	bytesPerPixel	= txBpp/8;				
	// CALCULATE THE MEMORY REQUIRED FOR THE TGA DATA
	imageSize		= txWidth*txHeight*bytesPerPixel;
	// SET THE TYPE TO RGB OR RGBA
	if(txBpp==24)type=GL_RGB;										
	else type=GL_RGBA;


	// READ THE TGA DATA

	// RESERVE MEMORY TO HOLD THE TGA DATA
	m_imageDataTGA=(GLubyte *)malloc(imageSize);				
	assert(m_imageDataTGA != NULL);

	// CHECK IF THE STORAGE MEMORY EXISTS AND THE IMAGE SIZE MATCHES THE MEMORY RESERVED
	if(m_imageDataTGA==NULL ||									
	   fread(m_imageDataTGA, 1, imageSize, m_textureFile)!=imageSize)
	{
		assert("COULD NOT READ CONTENTS OF TGA IN DYNAMICALLY ALLOCATED MEMORY");
		if(m_imageDataTGA!=NULL)free(m_imageDataTGA);
		fclose(m_textureFile);							
		return E_FAIL;								
	}

	// LOOP THROUGH THE IMAGE DATA AND SWAP THE 1ST AND 3RD BYTES (RED AND BLUE)
	for(GLuint i=0; i<int(imageSize); i+=bytesPerPixel)			
	{															
		temp=m_imageDataTGA[i];									
		m_imageDataTGA[i]=m_imageDataTGA[i+2];					
		m_imageDataTGA[i+2]=temp;								
	}

	// CLOSE .TGA FILE
	fclose(m_textureFile);								

	// GENERATE THE OPENGL TEXTURE
	GenerateOpenGLTextureFromTGAData( type, txWidth, txHeight, filter );

	// INCREASE THE COUNTER OF TOTAL TEXTURES LOADED
	m_texturesLoaded++;

	// DELETE THE TGA IMAGE DATA THAT WAS READ
	FreeTGAImageData();

	return S_OK;								
}


// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
// ==================================================================================
// CREATE A TEXTURE FROM A COMPRESSED TGA FILE
// ==================================================================================
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
HRESULT CTextureManager::LoadCompressedTGATexture( GLubyte tgaheader[],
																		int filter)
{
	// HOLDS NUMBER OF BYTES PER PIXEL USED IN THE TGA FILE
	GLuint		bytesPerPixel;									
	// USED TO STORE THE IMAGE SIZE WHEN SETTING ASIDE RAM
	GLuint		imageSize;										
	// SET THE DEFAULT GLMODE TO RBGA (32 BPP)
	GLuint		type = GL_RGBA;								
	// BITS PER PIXEL
	GLuint		txBpp;	
	// IMAGE WIDTH
	GLuint		txWidth;	
	// IMAGE HEIGHT
	GLuint		txHeight;		
	// PIXEL COUNT
	GLuint		pixelCount;
	// CURRENT PIXEL
	GLuint		currentPixel;
	// CURRENT BYTE
	GLuint		currentByte;
	// COLOUR BUFFER
	GLubyte		*colourBuffer;


	// DETERMINE THE TGA WIDTH	(highbyte*256+lowbyte)
	txWidth  = tgaheader[1] * 256 + tgaheader[0];	
	// DETERMINE THE TGA HEIGHT	(highbyte*256+lowbyte)
	txHeight = tgaheader[3] * 256 + tgaheader[2];	
	// DETERMINE THE BITS PER PIXEL (24 OR 32)
	txBpp	 = tgaheader[4];
    
	// CHECK IF THE WIDTH OR THE HEIGHT IS LESS THAN OR EQUAL TO 0
	// OR IF THE TGA IS NOT EITHER 24 OR 32 BITS
 	if(	txWidth<=0 || txHeight<=0 || (txBpp!=24 && txBpp!=32))
	{
		assert("TGA SIZE OR BPP ARE INVALID");
		fclose(m_textureFile);
		return E_FAIL;
	}

	// DIVIDE BY 8 TO GET THE BYTES PER PIXEL
	bytesPerPixel	= txBpp/8;				
	// CALCULATE THE MEMORY REQUIRED FOR THE TGA DATA
	imageSize		= txWidth*txHeight*bytesPerPixel;
	// SET THE TYPE TO RGB OR RGBA
	if(txBpp==24)type=GL_RGB;										
	else type=GL_RGBA;


	// READ THE TGA DATA

	// RESERVE MEMORY TO HOLD THE TGA DATA
	m_imageDataTGA=(GLubyte *)malloc(imageSize);				
	assert(m_imageDataTGA != NULL);

	// NUMBER OF PIXELS IN THE IMAGE
	pixelCount		= txWidth * txHeight;
	currentPixel	= 0;		
	currentByte		= 0;		
	colourBuffer	= (GLubyte *)malloc(bytesPerPixel);
	do
	{
		GLubyte chunkheader = 0;											// Storage for "chunk" header
		if(fread(&chunkheader, sizeof(GLubyte), 1,  m_textureFile) == 0)	// Read in the 1 byte header
		{
			assert ("Error loading compressed tga");
			if( m_textureFile != NULL){										// If file is open
				fclose(m_textureFile);										// Close file
			}
			if(colourBuffer != NULL){										// See if colorbuffer has data in it
				free(colourBuffer);											// If so, delete it
			}
			if(m_imageDataTGA != NULL){										// If there is stored image data
				free(m_imageDataTGA);										// Delete image data
			}
			return E_FAIL;													// Return failed
		}
		if(chunkheader < 128)												// If the ehader is < 128, it means the that is the number of RAW color packets minus 1
		{																	// that follow the header
			chunkheader++;													// add 1 to get number of following color values
			for(short counter = 0; counter < chunkheader; counter++)		// Read RAW color values
			{
				if(fread(colourBuffer, 1, bytesPerPixel, m_textureFile) != bytesPerPixel) // Try to read 1 pixel
				{
					assert ("Error loading compressed tga");
					if(m_textureFile != NULL){											// See if file is open
						fclose(m_textureFile);											// If so, close file
					}
					if(colourBuffer != NULL){											// See if colorbuffer has data in it
						free(colourBuffer);												// If so, delete it
					}
					if(m_imageDataTGA != NULL){											// See if there is stored Image data
						free(m_imageDataTGA);											// If so, delete it too
					}
					return E_FAIL;														// Return failed
				}
																						// write to memory
				m_imageDataTGA[currentByte		] = colourBuffer[2];				    // Flip R and B vcolor values around in the process 
				m_imageDataTGA[currentByte + 1	] = colourBuffer[1];
				m_imageDataTGA[currentByte + 2	] = colourBuffer[0];
				if(bytesPerPixel == 4)													// if its a 32 bpp image
				{
					m_imageDataTGA[currentByte + 3] = colourBuffer[3];					// copy the 4th byte
				}
				currentByte += bytesPerPixel;											// Increase thecurrent byte by the number of bytes per pixel
				currentPixel++;															// Increase current pixel by 1
				if(currentPixel > pixelCount)											// Make sure we havent read too many pixels
				{
					assert("Error loading compressed tga");
					if(m_textureFile != NULL){											// If there is a file open
						fclose(m_textureFile);											// Close file
					}	
					if(colourBuffer != NULL){											// If there is data in colorbuffer
						free(colourBuffer);												// Delete it
					}
					if(m_imageDataTGA != NULL){											// If there is Image data
						free(m_imageDataTGA);											// delete it
					}
					return E_FAIL;														// Return failed
				}
			}
		}
		else																			// chunkheader > 128 RLE data, next color reapeated chunkheader - 127 times
		{
			chunkheader -= 127;															// Subteact 127 to get rid of the ID bit
			if(fread(colourBuffer, 1, bytesPerPixel, m_textureFile) != bytesPerPixel)		// Attempt to read following color values
			{	
				assert("Error loading tga texture");
				if(m_textureFile != NULL){												// If thereis a file open
					fclose(m_textureFile);												// Close it
				}
				if(colourBuffer != NULL){												// If there is data in the colorbuffer
					free(colourBuffer);													// delete it
				}
				if(m_imageDataTGA != NULL){												// If thereis image data
					free(m_imageDataTGA);												// delete it
				}
				return E_FAIL;															// return failed
			}
			for(short counter = 0; counter < chunkheader; counter++)					// copy the color into the image data as many times as dictated 
			{																			// by the header
				m_imageDataTGA[currentByte		] = colourBuffer[2];					// switch R and B bytes areound while copying
				m_imageDataTGA[currentByte + 1	] = colourBuffer[1];
				m_imageDataTGA[currentByte + 2	] = colourBuffer[0];
				if(bytesPerPixel == 4)												// If TGA images is 32 bpp
				{
					m_imageDataTGA[currentByte + 3] = colourBuffer[3];				// Copy 4th byte
				}
				currentByte += bytesPerPixel;										// Increase current byte by the number of bytes per pixel
				currentPixel++;															// Increase pixel count by 1
				if(currentPixel > pixelCount)											// Make sure we havent written too many pixels
				{
					assert("Error loading tga texture");
					if(m_textureFile != NULL){											// If there is a file open
						fclose(m_textureFile);											// Close file
					}	
					if(colourBuffer != NULL){											// If there is data in colorbuffer
						free(colourBuffer);												// Delete it
					}
					if(m_imageDataTGA != NULL){											// If there is Image data
						free(m_imageDataTGA);											// delete it
					}
					return E_FAIL;														// Return failed
				}
			}
		}
	}
	while(currentPixel < pixelCount);													// Loop while there are still pixels left


	// CLOSE .TGA FILE
	fclose(m_textureFile);								

	// FREE COLOUR BUFFER
	if(colourBuffer != NULL)
		free(colourBuffer);

	// GENERATE THE OPENGL TEXTURE
	GenerateOpenGLTextureFromTGAData( type, txWidth, txHeight, filter );

	// INCREASE THE COUNTER OF TOTAL TEXTURES LOADED
	m_texturesLoaded++;

	// DELETE THE TGA IMAGE DATA THAT WAS READ
	FreeTGAImageData();

	return S_OK;								
}
************************************* @2D CODE ENDS ******************************* */

// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
// ==================================================================================
// THIS FUNCTION TAKES THE DATA READ FROM THE TGA FILE AND CREATES AN OPENGL TEXTURE
// ==================================================================================
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
VOID  CTextureManager::GenerateOpenGLTextureFromTGAData(GLuint type, 
										/*GLuint txWidth, GLuint txHeight, @3D*/ int filter)
{
	// GENERATE TEXTURES

	// SET THE ALIGNMENT REQUIREMENTS FOR THE START OF EACH PIXEL ROW IN MEMORY
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

	// GENERATE OPENGL TEXTURE IDS
	glGenTextures(1, &m_textures[m_currentOffset]);	//@2A				
	assert(m_textures[m_currentOffset]!=0 && "GenerateOpenGLTextureFromTGAData()"); //@2A

	// GENERATE THE TEXTURE -- @2A
	glBindTexture(  GL_TEXTURE_2D, m_textures[m_currentOffset]);	
	glTexEnvf(		GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE, GL_MODULATE); 
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, filter); //@2C -- changed from 'f' to 'i'
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, filter); //@2C -- changed from 'f' to 'i'
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT); 
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT); 

	// @3C - Changed:
	// 'm_imageDataTGA' for 'm_targaLoader.GetImage()'
	// 'txWidth' for 'm_targaLoader.GetWidth()'
	// 'txHeight' for 'm_targaLoader.GetHeight()'
	
	glTexImage2D(	GL_TEXTURE_2D, 0, type, m_targaLoader.GetWidth(), m_targaLoader.GetHeight(), 
											0, type, GL_UNSIGNED_BYTE, m_targaLoader.GetImage());
}
// *************************************** @1A CODE ENDS ****************************

// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
// ==================================================================================
// FREE THE BMP IMAGE DATA THAT WAS READ
// ==================================================================================
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
HRESULT CTextureManager::FreeBMPImageData()
{
	if(m_imageDataBMP[0])
	{
		if(m_imageDataBMP[0]->data)
		{
			free(m_imageDataBMP[0]->data);
		}
		free(m_imageDataBMP[0]);
	}

	return S_OK;
}

// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
// ==================================================================================
// FREE THE TGA IMAGE DATA THAT WAS READ
// ==================================================================================
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
/* ************************** @3D BEGINS *******************************************
HRESULT CTextureManager::FreeTGAImageData()
{
	if(m_imageDataTGA!=NULL)free(m_imageDataTGA);
	return S_OK;
}
***************************** @3D ENDS ******************************************* */

// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
// ==================================================================================
// CREATE A TEXTURE FROM A BMP FILE
// ==================================================================================
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
HRESULT CTextureManager::LoadBMPTexture( char filename[], int offset, int filter)
{
	memset(m_imageDataBMP,0,sizeof(void *)*1);

	assert( filename!=NULL && "CTextureManager::LoadBMPTexture" );

	m_textureFile = NULL;
	m_textureFile = fopen(filename, "r");
	if(!m_textureFile)return E_FAIL;
	else fclose(m_textureFile);

	// LOAD THE IMAGE DATA
	if(m_imageDataBMP[0]= auxDIBImageLoad(filename))
	{
		// SET THE ALIGNMENT REQUIREMENTS FOR THE START OF EACH PIXEL ROW IN MEMORY
		glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	
		// GENERATE 3 TEXTURES
		//glGenTextures(3, &m_textures[offset][0]); @2D
		glGenTextures(1, &m_textures[offset]); //@2A
		
		// CREATE THE TEXTURE -- @2A
		glBindTexture( GL_TEXTURE_2D, m_textures[offset] );
		glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE); 
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,filter);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,filter); 
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT); 
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT); 
		glTexImage2D( GL_TEXTURE_2D, 0, 3, m_imageDataBMP[0]->sizeX, 
					  m_imageDataBMP[0]->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE,
					  m_imageDataBMP[0]->data);

		// DELETE TEMPORARY IMAGE DATA
		FreeBMPImageData();

		m_texturesLoaded++;

		return S_OK;
	}
	else return E_FAIL;	
}

// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
// ==================================================================================
// RECEIVES A FILENAME AND CHECKS WHETHER IT'S A BMP OR TGA AND CALLS THE
// APPROPIATE METHOD TO CREATE THE TEXTURE
// ==================================================================================
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
GLuint CTextureManager::LoadTexture( char filename[], int filter )
{
	char c='\0', type[4]="";
	int i=0;

	assert((m_texturesLoaded+1)<=50 && "Too many textures loaded");

	c = filename[i];
	while((c!='.')&&(c!='?')&&(c!='\n'))
	{
		i++;
		c=filename[i];
	}
	if(c!='.')return E_FAIL;

	i++;
	type[0]=filename[i];
	type[1]=filename[i+1];
	type[2]=filename[i+2];
	type[3]='\0';	

	// IS IT A BMP ?
	if(!strcmp(type,"bmp"))
	{
		if(FAILED(LoadBMPTexture(filename, m_texturesLoaded, filter)))
		{
			MessageBox(NULL,filename,"Cannot load BMP texture",MB_OK|MB_ICONSTOP);			
			return 0;
		}
	}// IS IT A TGA ?
	else if(!strcmp(type, "tga"))
	{
		if(FAILED(LoadTGATexture(filename, m_texturesLoaded, filter)))
		{
			MessageBox(NULL,filename,"Cannot load TGA texture",MB_OK|MB_ICONSTOP);			
			return 0;
		}
	}

	return (m_texturesLoaded-1);
}

// -------------------------------- END OF FILE -----------------------------------
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




