/*************************************************************************/
/*|=====================================================================|*/
/*| Filename: textureManager.h                                          |*/
/*| Author: Joel Villagrana    (joel.villagrana@gmail.com)              |*/
/*|                                                                     |*/
/*| Classes Defined:                                                    |*/
/*| CTextureManager				                                        |*/
/*|                                                                     |*/
/*|%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%|*/
/*| dd/mm/yy   | Programmer        | Modifications                      |*/ 
/*|=====================================================================|*/
/*| 12/03/04   | JVillagrana       | Initial Version                    |*/
/*|---------------------------------------------------------------------|*/
/*| 10/07/05   | JVillagrana   @1  | -Add filter as param               |*/
/*|---------------------------------------------------------------------|*/
/*| 01/10/05   | JVillagrana       | -Modified to use TGA loader from   |*/
/*|            |               @2  |  Dave                              |*/
/*************************************************************************/

#ifndef _TEXTUREMANAGER_H
#define _TEXTUREMANAGER_H

#include <stdio.h>
#include <assert.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <gl/glaux.h>

#include "../Common/MGLTargaImage.h"

#define MAX_TEXTURES 50

/* $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
================================================================================
			Definition of the C3DEngine_TextureManager Class
				 (Supports loading BMP and TGA files)
================================================================================
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ */
class CTextureManager
{
private:

	static CTextureManager	*m_pGTextureManager;

	FILE			*m_textureFile;

//	GLubyte			*m_imageDataTGA;		// Image Data TGA (Up To 32 Bits) @2D
	MGL::TargaImage	m_targaLoader;			// Targa Loader

	AUX_RGBImageRec	*m_imageDataBMP[1];		// Image Data BMP

	GLuint			m_textures[MAX_TEXTURES];	

	GLuint			m_texturesLoaded;		// Number of textures loaded
	int				m_currentOffset;

public:

	CTextureManager();
	~CTextureManager();

	HRESULT InitAll();
	void	DestroyAll();
	
	GLuint	LoadTexture( char filename[], int filter );

	HRESULT LoadBMPTexture( char filename[], int offset ,int filter);
	HRESULT	LoadTGATexture( char filename[], int offset ,int filter);
//	HRESULT	LoadUncompressedTGATexture( GLubyte tgaheader[], int filter);	//@2D
//	HRESULT	LoadCompressedTGATexture(   GLubyte tgaheader[], int filter);	//@2D
	VOID	GenerateOpenGLTextureFromTGAData(GLuint type, /*GLuint txWidth, 
												GLuint txHeight, @3D*/ int filter);

	HRESULT FreeBMPImageData();
//	HRESULT FreeTGAImageData(); @2D

	GLuint	Get_texture(int id){	return	m_textures[id];							} // @1A
	GLuint	Get_texturesLoaded(){	return	m_texturesLoaded;						}

	static CTextureManager* GetTextureManager(){ return  m_pGTextureManager;		}
	static void SetTextureManager(CTextureManager* tm){  m_pGTextureManager = tm;	}
};
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ 


inline CTextureManager* _TextureManager()
{
	assert(CTextureManager::GetTextureManager() && "Error generated in class definition file, CTextureManager* _TextureManager()");
	return CTextureManager::GetTextureManager();
}
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ 



#endif _TEXTUREMANAGER_H


// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% END OF FILE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


