/*************************************************************************
MGLApp.h

Main application class.

Author   :   Dave Astle

Written for More OpenGL Game Programming
*************************************************************************/

#ifndef MGLAPP_H_INCLUDED
#define MGLAPP_H_INCLUDED

#ifdef WIN32
  #define WIN32_LEAN_AND_MEAN
  #define WIN32_EXTRA_LEAN
  #include <windows.h>
#endif

#include <gl/glee.h>
#include <gl/glu.h>
#include "prim.h"

// values used to set up the window and rendering context
const char MGL_APP_NAME[]	= "MOGLGP - Chapter 7 - Shadow Mapping";

const int MGL_WINDOW_WIDTH = 1024;
const int MGL_WINDOW_HEIGHT = 768;

const int MGL_COLOR_BITS = 32;
const int MGL_DEPTH_BITS = 24;
const int MGL_STENCIL_BITS = 0;

enum
{
  MGL_FULLSCREEN_FALSE = 0,
  MGL_FULLSCREEN_TRUE = 1,
  MGL_FULLSCREEN_CHOOSE = 2
};

const int MGL_FULLSCREEN_MODE = MGL_FULLSCREEN_FALSE;


class MGLApp
{
public:
	MGLApp();
	virtual ~MGLApp();

	bool Init();
	bool Shutdown();
	void SetupProjection( int width, int height );
	void Update( float dt );
	void Render();

protected:
  void GenerateShadowMap();
  void RenderObjects(bool isShadowPass);

private:
  GLuint m_shadowMap;
  Primitive* m_ground;
  GLUquadricObj* m_quadric;
  GLfloat m_bounce;

  GLint m_windowHeight, m_windowWidth;
};

#endif // MGLAPP_H_INCLUDED