/*************************************************************************
MGLApp.h

Main application class.

Author   :   Dave Astle (framework) & Angus Dorbie (water render etc.)

Written for More OpenGL Game Programming
*************************************************************************/

#ifndef MGLAPP_H_INCLUDED
#define MGLAPP_H_INCLUDED

#ifdef WIN32
  #define WIN32_LEAN_AND_MEAN
  #define WIN32_EXTRA_LEAN
  #include <windows.h>
#endif

#include <glee.h>
#include <gl/glu.h>
#include <MGLVector.h>
#include "fontprint.h"

// values used to set up the window and rendering context
const char MGL_APP_NAME[]	= "MOGLGP - Chapter 8 - Water; Using Fresnel Coefficients for Refraction and Reflection";

const int MGL_WINDOW_WIDTH = 800;
const int MGL_WINDOW_HEIGHT = 600;

const int MGL_COLOR_BITS = 32;
const int MGL_DEPTH_BITS = 32;
const int MGL_STENCIL_BITS = 0;

const bool MGL_FORCE_FULLSCREEN = false;

class gmesh {
public:
	gmesh(int x, int y, float xpos, float ypos, float scale, MGL::Vector3 color, GLuint tex);
	~gmesh();
	GLfloat *w_verts;
	GLfloat *w_normals;
	GLfloat *w_tcoord;
	GLfloat *w_color;
	GLfloat *w_color_p;
	GLfloat *w_tex;
	GLfloat *w_fog;
	GLuint tex_handle;
	GLshort *w_indices;
	int w_xsize;
	int w_ysize;

	void CalculateWave(MGL::Vector3 eye_pos, float level, float amplitude, float phase, bool mode_polarize, float *color);
	void CalculateFog(MGL::Vector3 eye_pos, bool alpha_fog, float turbidity, float *color);
	void Render(bool polarized_region, bool mode_polarize, bool mode_hedgehog, bool mode_points, bool transparent, bool zfillfirst, bool lit, bool tex, bool fog, bool vfog);
};

class MGLApp
{
public:
	MGLApp();
	virtual ~MGLApp();

	bool Init();
	bool Shutdown();
	void SetupProjection( int width, int height );
	void Clear(bool);
	void Update( double dt );
	void Render_sky(void);
	void Render(bool);
	void GenerateMeshes();
	void UpdateMesh();

	char message_str[256];

	int app_width;
	int app_height;

private:


	gmesh *water;
	gmesh *ground;
	gmesh *ground_fog;

	fontprint *text_display;

	bool mode_polarize;
	bool mode_hedgehog;
	bool mode_points;
	bool mode_use_vertex_fog;
	

	float wave_amplitude;
	float turbidity;

	float eye_pitch;
	float eye_heading;
	GLuint uw_tex;
	MGL::Vector3 eye_pos;
	MGL::Vector3 sky_color;
	MGL::Vector3 clear_color;
};

#endif // MGLAPP_H_INCLUDED