/*************************************************************************
MGLApp.h

Main application class.

Author   :   Dave Astle / Baldur Karlsson

Written for More OpenGL Game Programming
*************************************************************************/

#ifndef MGLAPP_H_INCLUDED
#define MGLAPP_H_INCLUDED

#ifdef WIN32
  #define WIN32_LEAN_AND_MEAN
  #define WIN32_EXTRA_LEAN
  #include <windows.h>
#endif

#include <GL/GLee.h>
#include <GL/glu.h>

// values used to set up the window and rendering context
const char MGL_APP_NAME[]	= "MOGLGP - Chapter 6 - Procedural Generation of Trees";

const int MGL_WINDOW_WIDTH = 800;
const int MGL_WINDOW_HEIGHT = 600;

const int MGL_COLOR_BITS = 32;
const int MGL_DEPTH_BITS = 24;
const int MGL_STENCIL_BITS = 0;

const bool MGL_FORCE_FULLSCREEN = false;

// stl vector array class
#include <vector>
using namespace::std;

#include "MGLMatrix.h"

#include "MGLVector.h"
#include "MGLTargaImage.h"
#include "MGLTree.h"
#include "MGLNoise.h"

#include "MGLKeys.h"

class MGLApp
{
public:
	MGLApp();
	virtual ~MGLApp();

	bool Init();
	bool Shutdown();
	void SetupProjection( int width, int height );
	void Update( double dt );
	void Render();

	void KeyUpdate();
private:
	int keyarray[512];
	int keyarraysize, mousex, mousey;

	bool GetKey(int key);
	void MoveMouse(int x, int y);

	void DrawSky();
	void DrawGround();

	MGL::Vector3 pos, rot;
	GLuint grass;
	MGLNoise noise;
	
	int lastx, lasty;
};

const int TREE_GRID_SIZE = 30;

struct MGLTree
{
    TreeGeometry *tree;
    MGL::Vector3 pos;
    bool enabled;
    static MGL::Matrix4 modelView;

    static int SortFunc(const void *a, const void *b);
};

#endif // MGLAPP_H_INCLUDED
