#include "MGLNoise.h"

float MGLNoise::Noise2D(int x, int y)
{
    int n = x + y * 57;
    n = (n<<13) ^ n;
    return float(1.0f - ((n * (n * n * 15731 + 789221) + 1376312589) & 0x7fffffff) / 1073741824.0);
}

float MGLNoise::SmoothedNoise2D(int x, int y)
{
    return Noise2D(x, y)/4.0f +
	(Noise2D(x+1, y) + Noise2D(x-1, y) + Noise2D(x, y+1) + Noise2D(x, y-1))/8.0f +
	(Noise2D(x+1, y+1) + Noise2D(x+1, y-1) + Noise2D(x-1, y+1) + Noise2D(x-1, y-1))/16.0f;
}

float MGLNoise::Interpolate1D(float a, float b, float x)
{
    float f = (1.0f - (float)cosf(x * PI)) * 0.5f;

    return a * (1.0f - f) + b * f;
}

float MGLNoise::InterpolatedNoise2D(float x, float y)
{
    float v1 = SmoothedNoise2D(int(x),   int(y));
    float v2 = SmoothedNoise2D(int(x)+1, int(y));
    float v3 = SmoothedNoise2D(int(x),   int(y)+1);
    float v4 = SmoothedNoise2D(int(x)+1, int(y)+1);

    float i1 = Interpolate1D(v1, v2, x-int(x));
    float i2 = Interpolate1D(v3, v4, x-int(x));

    return Interpolate1D(i1, i2, y-int(y));
}

void MGLNoise::Generate()
{
    int numoct = 3;
    float freq = 8.0f;
    float amp = 1.5f;
    for(int i=0; i < numoct; i++)
    {
	for(int j=0; j < 256*256; j++)
	{
	    float x = amp*(InterpolatedNoise2D(float(j%256)/256.0f*freq, float(j/256)/256.0f*freq));
	    noise[j] += x;
	}
	freq *= 2.0f;amp /= 2.0f;
    }

    for(int j=0; j < 256*256; j++)
    {
	noise[j] += 1.0f;
	noise[j] *= 0.5f;

	if(noise[j] < 0.4f)
	    noise[j] = 0.0f;
	else
	    noise[j] = 1.0f;
    }
}

