/*************************************************************************
MGLSDL.h

Setup code for a basic SDL application.

Author   :   Dave Astle

Written for More OpenGL Game Programming
*************************************************************************/

#include "SDL/SDL.h"
#include "MGLApp.h"
#include <stdlib.h>

// KEY CONSTANTS

const int KEY_BACKSPACE = SDLK_BACKSPACE;
const int KEY_TAB = SDLK_TAB;
const int KEY_RETURN = SDLK_RETURN;
const int KEY_ESCAPE = SDLK_ESCAPE;
const int KEY_SPACE = SDLK_SPACE;
const int KEY_DELETE = SDLK_DELETE;
const int KEY_NUMPAD0 = SDLK_KP0;
const int KEY_NUMPAD1 = SDLK_KP1;
const int KEY_NUMPAD2 = SDLK_KP2;
const int KEY_NUMPAD3 = SDLK_KP3;
const int KEY_NUMPAD4 = SDLK_KP4;
const int KEY_NUMPAD5 = SDLK_KP5;
const int KEY_NUMPAD6 = SDLK_KP6;
const int KEY_NUMPAD7 = SDLK_KP7;
const int KEY_NUMPAD8 = SDLK_KP8;
const int KEY_NUMPAD9 = SDLK_KP9;
const int KEY_NUMPAD_PERIOD = SDLK_KP_PERIOD;
const int KEY_NUMPAD_DIVIDE = SDLK_KP_DIVIDE;
const int KEY_NUMPAD_MULTIPLY = SDLK_KP_MULTIPLY;
const int KEY_NUMPAD_MINUS = SDLK_KP_MINUS;
const int KEY_NUMPAD_PLUS = SDLK_KP_PLUS;
const int KEY_NUMPAD_ENTER = SDLK_KP_ENTER;
const int KEY_ARROW_UP = SDLK_UP;
const int KEY_ARROW_DOWN = SDLK_DOWN;
const int KEY_ARROW_RIGHT = SDLK_RIGHT;
const int KEY_ARROW_LEFT = SDLK_LEFT;
const int KEY_HOME = SDLK_HOME;
const int KEY_END = SDLK_END;
const int KEY_INSERT = SDLK_INSERT;
const int KEY_PAGEUP = SDLK_PAGEUP;
const int KEY_PAGEDOWN = SDLK_PAGEDOWN;
const int KEY_F1 = SDLK_F1;
const int KEY_F2 = SDLK_F2;
const int KEY_F3 = SDLK_F3;
const int KEY_F4 = SDLK_F4;
const int KEY_F5 = SDLK_F5;
const int KEY_F6 = SDLK_F6;
const int KEY_F7 = SDLK_F7;
const int KEY_F8 = SDLK_F8;
const int KEY_F9 = SDLK_F9;
const int KEY_F10 = SDLK_F10;
const int KEY_F11 = SDLK_F11;
const int KEY_F12 = SDLK_F12;
// END KEY CONSTANTS

void SetGLAttributes()
{
  SDL_GL_SetAttribute(SDL_GL_BUFFER_SIZE, MGL_COLOR_BITS);
  SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, MGL_DEPTH_BITS);
  SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE, MGL_STENCIL_BITS);
  SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
  SDL_GL_SetAttribute(SDL_GL_ALPHA_SIZE, 8);
//  SDL_GL_SetAttribute(SDL_GL_MULTISAMPLEBUFFERS, 1);
//  SDL_GL_SetAttribute(SDL_GL_MULTISAMPLESAMPLES, 4);
}

int main(int argc, char* argv[])
{
    SDL_Init(SDL_INIT_VIDEO);// | SDL_INIT_TIMER);
  atexit(SDL_Quit);

  SDL_WM_SetCaption(MGL_APP_NAME, MGL_APP_NAME);

  SetGLAttributes();

  Uint32 displayFlags = SDL_OPENGL;

  if (MGL_FORCE_FULLSCREEN)
  {
    displayFlags |= SDL_FULLSCREEN;
  }
  else
  {
    displayFlags |= SDL_RESIZABLE;
  }

  SDL_Surface* pDisplay = SDL_SetVideoMode(MGL_WINDOW_WIDTH, MGL_WINDOW_HEIGHT, MGL_COLOR_BITS, displayFlags);

  if (pDisplay == NULL)
  {
    fprintf(stderr, "Couldn't set GL mode: %s\n", SDL_GetError());
    exit(0);
  }

  MGLApp* pApp = new MGLApp();
  pApp->Init();
  pApp->SetupProjection(MGL_WINDOW_WIDTH, MGL_WINDOW_HEIGHT);

  SDL_ShowCursor(0);

  for (;;)
  {
    SDL_Event event;
    Uint32 lastTime = SDL_GetTicks(), currTime;

    pApp->KeyUpdate();

    if (SDL_PollEvent(&event) == 0)
    {
      currTime = SDL_GetTicks();
      if(currTime-lastTime == 0)lastTime--;
      pApp->Update((currTime - lastTime)/1000.0f);
      lastTime = currTime;
      pApp->Render();
      SDL_GL_SwapBuffers();
      SDL_GL_SwapBuffers();
    }
    else
    {
      if (SDL_QUIT == event.type ||
          SDL_KEYDOWN == event.type && SDLK_ESCAPE == event.key.keysym.sym)
        break;
    }
  }
  pApp->Shutdown();
  delete pApp;

  return 0;
}

void MGLApp::MoveMouse(int x, int y)
{
    SDL_WarpMouse(x, y);
}

void MGLApp::KeyUpdate()
{
    SDL_PumpEvents();
    Uint8 *temp;
    temp = SDL_GetKeyState(&keyarraysize);
    for(int i=0; i < keyarraysize; i++)
	keyarray[i] = temp[i];
    SDL_GetMouseState(&mousex, &mousey);
}

bool MGLApp::GetKey(int key)
{
    if(key < keyarraysize && key > 0)
	return keyarray[key]==1;
    return false;
}
