#include "MGLTree.h"

MGL::Matrix4 TreeGeometry::modelview;

bool TreeGeometry::SetupTex(const char *barkname, const char *leafname, int spritenum)
{
    MGL::TargaImage image;

    if(!image.Load("bark.tga"))
    {
	fprintf(stderr, "Error loading bark.tga");
	return false;
    }

    glGenTextures(1, &bark);
    glBindTexture(GL_TEXTURE_2D, bark);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,
		 image.GetWidth(), image.GetHeight(),
		 0, GL_RGBA, GL_UNSIGNED_BYTE,
		 image.GetImage());
    
    if(!image.Load("leaf.tga"))
    {
	fprintf(stderr, "Error loading leaf.tga");
	return false;
    }

    glGenTextures(1, &leaf);
    glBindTexture(GL_TEXTURE_2D, leaf);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,
		 image.GetWidth(), image.GetHeight(),
		 0, GL_RGBA, GL_UNSIGNED_BYTE,
		 image.GetImage());

    numsprites = spritenum;
    sprites = new GLuint[numsprites];
    return true;
}

void TreeGeometry::Generate(int maxlevel,
		    float anglemod,
		    float twist,
		    float length, float lengthmod,
		    float anglerand, float BranchWidth, float leafsize)
{
    Maxlevel = maxlevel;
    Length = length;
    LengthMod = lengthmod;
    AngleMod = anglemod;
    AngleRand = anglerand;
    Twist = twist;
    branchwidth = BranchWidth;
    LeafSize = leafsize;
    
    glCullFace(GL_BACK);
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    treelist = glGenLists(1);

    Branch *trunk = new Branch;
    lines.clear();
    leaves.clear();
    trunk->start = 0.0f;
    MGL::Matrix4 m;
    TreeRecurse(trunk, m,
		0, Maxlevel,
		0.0f, AngleMod,
		Twist,
		Length, LengthMod,
		AngleRand);
    //glPushMatrix();

    delete[] leavesort;
    leavesort = new Leaf[leaves.size()];
    for(int i=0; i < (signed)leaves.size(); i++)
	leavesort[i] = leaves[i];

    delete trunk;

    glNewList(treelist, GL_COMPILE);
    Render3D();
    glEndList();
    
    glGetIntegerv(GL_VIEWPORT, view);

    glPushAttrib(GL_VIEWPORT_BIT);
    glViewport(0, 0, 512, 512);
    
    for(int i=0; i < numsprites; i++)
	RenderSprite(i);

    glPopAttrib();
}

void TreeGeometry::TreeRecurse(Branch *b, MGL::Matrix4 m,
		 int level, int maxlevel,
		 float angle, float anglemod,
		 float twist,
		 float length, float lengthmod,
		 float anglerand)
{
    if(level >= maxlevel)
	return;

    b->length = length;
    b->theta = angle;
    b->alpha = twist;

    MGL::Matrix4 mnew;
    mnew.identity();
    mnew.translate(0.0f, b->start, 0.0f);
    mnew.rotate(0.0f, DEG2RAD(b->alpha), 0.0f);	
    mnew.rotate(0.0f, 0.0f, DEG2RAD(b->theta));

    b->m = m = m * mnew;

    Line l;
    if(level > 2)
    {
	l.s = b->m * MGL::Vector3(0.0f, 0.0f, 0.0f);
	l.e = b->m * MGL::Vector3(0.0f, b->length, 0.0f);
	l.l = level;
	l.len = b->length;
	l.segs = 1;
	lines.push_back(l);
    }
    else
    {
	l.s = b->m * MGL::Vector3(0.0f, 0.0f, 0.0f);
	l.e = b->m * MGL::Vector3(0.0f, b->length, 0.0f);
	l.l = level;
	l.len = b->length;
	l.segs = 3;
	l.sgs[0].x = RAND(-0.1f, 0.1f);
	l.sgs[0].y = RAND(-0.1f, 0.1f);
	l.sgs[0].z = RAND(-0.1f, 0.1f);

	l.sgs[1].x = RAND(-0.1f, 0.1f);
	l.sgs[1].y = RAND(-0.1f, 0.1f);
	l.sgs[1].z = RAND(-0.1f, 0.1f);
	lines.push_back(l);
    }

    Leaf lf, lf2;
    lf.pos = l.s + (l.e-l.s)*RAND(0.3f, 1.0f);
    lf2.pos = l.s + (l.e-l.s)*RAND(0.3f, 1.0f);

    lf.dir = MGL::Vector3(RAND(-1.0f, 1.0f), RAND(-1.0f, 1.0f), RAND(-1.0f, 1.0f))*0.5f+MGL::Vector3(0.0f, -0.5f, 0.0f);lf.bb = false;
    lf2.dir = MGL::Vector3(RAND(-1.0f, 1.0f), RAND(-1.0f, 1.0f), RAND(-1.0f, 1.0f))*0.5f+MGL::Vector3(0.0f, -0.5f, 0.0f);lf2.bb = false;

    lf.dir.Normalize();
    lf2.dir.Normalize();

    if(level > 1 && level > maxlevel-3)
    {
	leaves.push_back(lf);
	leaves.push_back(lf2);
    }

    if(level < maxlevel-1)
    {
	b->middle = new Branch;
	b->middle->start = b->length;
	TreeRecurse(b->middle, m,
		    level+1, maxlevel,
		    0.0f + RAND(-anglerand, anglerand), anglemod,
		    twist + RAND(-anglerand, anglerand),
		    length*lengthmod, lengthmod,
		    anglerand);

	b->left = new Branch;
	    b->left->start = b->length * RAND(0.5f, 1.0f);
	TreeRecurse(b->left, m, level+1, maxlevel,
		    anglemod + RAND(-anglerand, anglerand), anglemod,
		    twist + RAND(-anglerand, anglerand),
		    length*lengthmod, lengthmod,
		    anglerand);

	b->right = new Branch;
	    b->right->start = b->length * RAND(0.5f, 1.0f);

	TreeRecurse(b->right, m, level+1, maxlevel,
		    -anglemod + RAND(-anglerand, anglerand), anglemod,
		    twist + RAND(-anglerand, anglerand),
		    length*lengthmod, lengthmod,
		    anglerand);
    }
}

int TreeGeometry::LeafSortFunc(const void *a, const void *b)
{
    MGL::Vector3 one = modelview * (*(Leaf *)a).pos;
    MGL::Vector3 two = modelview * (*(Leaf *)b).pos;

    if(one.z < two.z)
	return -1;

    if(one.z > two.z)
	return 1;

    return 0;
}

void TreeGeometry::RenderSprite(int sprite)
{
    glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glLoadIdentity();
    
    glTranslatef(0.0f, -2.71f, -6.4f);
    glRotatef(-10.0f, 1.0f, 0.0f, 0.0f);
    glRotatef(sprite * (360.0f/numsprites), 0.0f, 1.0f, 0.0f);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    double model[16], proj[16];
    glGetDoublev(GL_MODELVIEW_MATRIX, model);
    glGetDoublev(GL_PROJECTION_MATRIX, proj);

    glEnable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);

    Render3D();

    int size = 512;

    glGenTextures(1, &sprites[sprite]);
    glBindTexture(GL_TEXTURE_2D, sprites[sprite]);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);//_MIPMAP_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

    glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 0, 0, size, size, 0);
    //glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0, size, size);
}

void TreeGeometry::DrawBranch(Line l, float bottomrad, float toprad, float segs)
{
    for(int i=0; i < l.segs; i++)
    {
	MGL::Vector3 dir = l.e-l.s; 
	
	MGL::Vector3 end = l.s;
	MGL::Vector3 start = l.s;
	start += dir * float(i)/float(l.segs);
	end += dir * float(i+1)/float(l.segs);

	if(i > 0)
	    start += l.sgs[i-1];

	if(i+1 > 0)
	    end += l.sgs[i];

	MGL::Vector3 dirnorm = dir;dirnorm.Normalize();
	MGL::Vector3 right = dir.CrossProduct(MGL::Vector3(0.0f, 1.0f, 0.0f));
	if(dirnorm == MGL::Vector3(0.0f, 1.0f, 0.0f))
	    right = MGL::Vector3(1.0f, 0.0f, 0.0f);
	MGL::Vector3 forward = dir.CrossProduct(right);

	forward.Normalize();
	right.Normalize();

	glBegin(GL_TRIANGLE_STRIP);
	float lim = 2*PI, inc = 2*PI/segs;
	float botrad = bottomrad * (1.0f-float(i)/float(l.segs)) + toprad * (float(i)/float(l.segs));
	float tprad = bottomrad * (1.0f-float(i+1)/float(l.segs)) + toprad * (float(i+1)/float(l.segs));
	for(float r=0.0f; r <= lim; r += inc)
	{
	    MGL::Vector3 pnt = start + right * cosf(r) * botrad + forward * sinf(r) * botrad;
	    glTexCoord2f(r/lim, 0.0f);
	    glVertex3f(pnt.x, pnt.y, pnt.z);
	    
	    pnt = end + (right * cosf(r) * tprad + forward * sinf(r) * tprad);
	    glTexCoord2f(r/lim, 1.0f);
	    glVertex3f(pnt.x, pnt.y, pnt.z);
	}
	glEnd();
    }
}

void TreeGeometry::Render3D()
{
    glBindTexture(GL_TEXTURE_2D, bark);
    for(int i=0; i < (signed)lines.size(); i++)
    {
	float rad = lines[i].len*branchwidth/float(lines[i].l);
	float nextrad = lines[i].len*LengthMod*branchwidth/float(lines[i].l+1);
	    
	if(lines[i].l == 0)
	    rad = lines[i].len*branchwidth;
	    
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	    
	glPushMatrix();
	DrawBranch(lines[i], rad, nextrad, max(5.0f-float(lines[i].l), 3.0f));
	glPopMatrix();
    }

    modelview.GetModelView();
    qsort(leavesort, leaves.size(), sizeof(Leaf), LeafSortFunc);
	
    glBindTexture(GL_TEXTURE_2D, leaf);
    for(int i=0; i < (signed)leaves.size(); i++)
    {
	MGL::Vector3 pos = leavesort[i].pos;
	MGL::Vector3 dir = leavesort[i].dir;
	    
	MGL::Vector3 dirnorm = dir;dirnorm.Normalize();
	MGL::Vector3 right = dir.CrossProduct(MGL::Vector3(0.0f, 1.0f, 0.0f));
	if(dirnorm == MGL::Vector3(0.0f, 1.0f, 0.0f)
	   || dirnorm == MGL::Vector3(0.0f, -1.0f, 0.0f))
	    right = MGL::Vector3(1.0f, 0.0f, 0.0f);
	MGL::Vector3 up = dir.CrossProduct(right);
	    
	up.Normalize();
	right.Normalize();
	    
	float size=Length/7.0f*LeafSize;
	    
	glPushMatrix();

	MGL::Vector3 look = (modelview * MGL::Vector3(0.0f, 0.0f, 1.0f)) -
	    (modelview * MGL::Vector3(0.0f, 0.0f, 0.0f));
	look.y = 0.0f;
	look.Normalize();
	float x = (look.DotProduct(MGL::Vector3(0.0f, 0.0f, 1.0f)));
	MGL::Vector3 rot;

	if(look.x > 0.0f)
	    rot.y = RAD2DEG(acosf(x));
	else
	    rot.y = 360.0f-RAD2DEG(acosf(x));
	
	glTranslatef(pos.x, pos.y, pos.z);
	if(leavesort[i].bb)
	    glRotatef(rot.y, 0.0f, 1.0f, 0.0f);
	
	glBegin(GL_QUADS);
	glTexCoord2f(0.0f, 0.0f);
	glVertex3f((dir*-size + right*-size).x, (dir*-size + right*-size).y, (dir*-size + right*-size).z);

	glTexCoord2f(1.0f, 0.0f);
	glVertex3f((dir*-size + right*size).x, (dir*-size + right*size).y, (dir*-size + right*size).z);

	glTexCoord2f(1.0f, 1.0f);
	glVertex3f((dir*size + right*size).x, (dir*size + right*size).y, (dir*size + right*size).z);

	glTexCoord2f(0.0f, 1.0f);
	glVertex3f((dir*size + right*-size).x, (dir*size + right*-size).y, (dir*size + right*-size).z);
	glEnd();

	glPopMatrix();
    }
}

void TreeGeometry::Draw()
{
    float threshold = 100.0f;

    modelview.GetModelView();
    if(modelview[12]*modelview[12]+
       modelview[13]*modelview[13]+
       modelview[14]*modelview[14]  > threshold)
    {
	glLineWidth(4.0f);

	MGL::Vector3 look = (modelview * MGL::Vector3(0.0f, 0.0f, 1.0f)) -
	    (modelview * MGL::Vector3(0.0f, 0.0f, 0.0f));
	look.y = 0.0f;
	look.Normalize();
	float x = (look.DotProduct(MGL::Vector3(0.0f, 0.0f, 1.0f)));
	MGL::Vector3 rot;

	if(look.x > 0.0f)
	    rot.y = RAD2DEG(acosf(x));
	else
	    rot.y = 360.0f-RAD2DEG(acosf(x));

	if(rot.y+(180.0f/numsprites) >= 360.0f)
	    rot.y -= 360.0f;

	float foo = rot.y;

	for(int i=0; i < numsprites; i++)
	    if(foo+(180.0f/numsprites) > (360.0f/numsprites)*i &&
	       foo+(180.0f/numsprites) <= (360.0f/numsprites)*(i+1))
		glBindTexture(GL_TEXTURE_2D, sprites[i]);

	glRotatef(-rot.y, 0.0f, 1.0f, 0.0f);

	glBegin(GL_QUADS);
	float bbsize = Length * Maxlevel * (1.0f / (sqrt(threshold)/6.4f));
	float xscale = view[2]/512.0f, yscale = view[3]/512.0f;

	glTexCoord2f(0.0f, 1.0f);
	glVertex3f(-1.0f*bbsize*xscale, 2.0f*bbsize*yscale, 0.0f);

	glTexCoord2f(0.0f, 0.0f);
	glVertex3f(-1.0f*bbsize*xscale, 0.0f*bbsize*yscale, 0.0f);

	glTexCoord2f(1.0f, 0.0f);
	glVertex3f( 1.0f*bbsize*xscale, 0.0f*bbsize*yscale, 0.0f);

	glTexCoord2f(1.0f, 1.0f);
	glVertex3f( 1.0f*bbsize*xscale, 2.0f*bbsize*yscale, 0.0f);
	glEnd();

	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    }
    else
    {
	glCallList(treelist);
    }
}
