/*************************************************************************
MGLTree.h

Tree Geometry class.

Author   :   Baldur Karlsson

Written for More OpenGL Game Programming
*************************************************************************/

#ifndef MGLTREE_H_INCLUDED
#define MGLTREE_H_INCLUDED

#include "MGLMath.h"
#include "MGLMatrix.h"
#include "MGLVector.h"
#include "MGLTargaImage.h"

#include <vector>
using namespace::std;

struct Branch
{
    Branch() { left = middle = right = NULL; }
    ~Branch() { if(left)delete left;if(middle)delete middle;if(right)delete right; }
    float length;
    float start;
    float theta;
    float alpha;
    Branch *left, *middle, *right;
    MGL::Matrix4 m;
};

struct Line
{
    MGL::Vector3 s, e;
    int l;
    float len;
    int segs;
    MGL::Vector3 sgs[10];
};

struct Leaf
{
    MGL::Vector3 pos;
    MGL::Vector3 dir;
    bool bb;
};

class TreeGeometry
{
  public:
    TreeGeometry() { leavesort = new Leaf[2]; }
    bool SetupTex(const char *barkname, const char *leafname, int spritenum);
    void Generate(int maxlevel,
		  float anglemod,
		  float twist,
		  float length, float lengthmod,
		  float anglerand, float BranchWidth, float leafsize);
    void Draw();
  private:
    void TreeRecurse(Branch *b, MGL::Matrix4 m,
		     int level, int maxlevel,
		     float angle, float anglemod,
		     float twist,
		     float length, float lengthmod,
		     float anglerand);
    void RenderSprite(int sprite);
    void Render3D();

    void DrawBranch(Line l, float bottomrad, float toprad, float segs);

    static int LeafSortFunc(const void *a, const void *b);

    int Maxlevel;
    float Length;
    float AngleMod, LengthMod;
    float AngleRand, LeafSize;
    float Twist, branchwidth;

    vector<Line> lines;
    vector<Leaf> leaves;
    Leaf *leavesort;

    GLuint treelist;
    int numsprites;
    GLuint bark, leaf, *sprites;

    static MGL::Matrix4 modelview;
    int view[4];
};

#endif // MGLTREE_H_INCLUDED
