/*************************************************************************
tvector.cpp

Vector math library.

Author   :   Dimi Christopoulos

*************************************************************************/
#include "tvector.h"

// Make a unit vector
TVector32 &TVector32::unit() {
	if (isDefault()) {
		float rep = mag();
		if (rep < EPSILON) {
			_x = 0.0f;
			_y = 0.0f;
			_z = 0.0f;
		} else {
			float temp = 1.0f / rep;
			_x *= temp;
			_y *= temp;
			_z *= temp;
		}
		_Status = UNIT;
	}
	return *this;
}

// Make a default vector
TVector32 &TVector32::Default() {
	if (isUnit())
		_Status = DEFAULT;
	return *this;
}

TVector32 &TVector32::invert(const TVector32 &v1, TVector32 &result) {
	if (v1.isValid()) {
		result._x = -v1._x;
		result._y = -v1._y;
		result._z = -v1._z;
		result._Status = v1._Status;
	} else
		result = TVector32();
	return result;
}

TVector32 &TVector32::add(const TVector32 &v1, const TVector32 &v2, TVector32 &result) {
	if (v1.isValid() && v2.isValid()) {
		result._x = v1._x + v2._x;
		result._y = v1._y + v2._y;
		result._z = v1._z + v2._z;
		result._Status = DEFAULT;
	} else
		result = TVector32();
	return result;
}

TVector32 &TVector32::subtract(const TVector32 &v1, const TVector32 &v2, TVector32 &result) {
	if (v1.isValid() && v2.isValid()) {
		result._x = v1._x - v2._x;
		result._y = v1._y - v2._y;
		result._z = v1._z - v2._z;
		result._Status = DEFAULT;
	} else
		result = TVector32();
	return result;
}

TVector32 &TVector32::cross(const TVector32 &v1, const TVector32 &v2, TVector32 &result) {
	if (v1.isValid() && v2.isValid()) {
		result._x = v1._y * v2._z - v1._z * v2._y;
		result._y = v1._z * v2._x - v1._x * v2._z;
		result._z = v1._x * v2._y - v1._y * v2._x;
		result._Status = DEFAULT;
	} else
		result = TVector32();
	return result;
}

TVector32 &TVector32::multiply(const TVector32 &v1, const float &scale, TVector32 &result) {
	if (v1.isValid()) {
		result._x = v1._x * scale;
		result._y = v1._y * scale;
		result._z = v1._z * scale;
		result._Status = DEFAULT;
	} else
		result = TVector32();
	return result;
}





