/*************************************************************************
MGLApp.cpp

Main application class for SDL and Win32.

Author   :   Andrei Stojan, Dimi Christopoulos

Written for More OpenGL Game Programming
*************************************************************************/

#include "MGLApp.h"
#include "dk_grass.h"
#include "CGLFontClass.h"

unsigned grassTexture;
int     grassplane=1;
int     help=0;
int     info=0;
CGLFontClass font;

MGLApp::MGLApp()
{
        //Bucharest, 12 AM GMT, middle summer
	Latitude = 44.0f; 
	Longitude = 36.0f; 
	Time = 16.0f;
	Day = 180.0f;

	SkyRadius = 2000.0f;
}

MGLApp::~MGLApp()
{
}

bool MGLApp::Init()
{

	if (!GLeeInit())
		return false;

    Camera.PositionCamera(0,20,0, 66, 42, 43,0,1,0);

	SetupProjection(MGL_WINDOW_WIDTH, MGL_WINDOW_HEIGHT);
	if (Sky.Initialize(SkyRadius, 48, 32, false, 0.8)) return false;
	Sky.SetCoordinates(Latitude, Longitude, Time, Day);
    Sky.Update(0);

    glGenTextures(1, &grassTexture);
	glBindTexture(GL_TEXTURE_2D, grassTexture);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); /* scale linearly when image bigger than texture*/
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST); /* scale linearly when image smalled than texture*/
    gluBuild2DMipmaps(GL_TEXTURE_2D, 3, 64, 64, GL_RGB, GL_UNSIGNED_BYTE, dk_grass_array);

	unsigned e;		
	if ( (e = glGetError()) )
	{
		return false;
	}

    font.Load("font.tga","font.dat");

	return true;
}

bool MGLApp::Shutdown()
{
	Sky.Destroy();

	return true;
}

void MGLApp::SetupProjection(int width, int height)
{
	// avoid div by zero
	if( height == 0 )
		height = 1;					

	// set up GL viewport
	glViewport( 0, 0, width, height );

    font.setDimensions(width,height);

	// set up a perspective projection
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 60.0f, (GLfloat)width/(GLfloat)height, 1.0f, 5000.0f );
	glMatrixMode(GL_MODELVIEW);
}

void MGLApp::Update( double dt )
{
    bool updateDome=false;

#ifdef MGL_SDL
	unsigned char *Keys = SDL_GetKeyState(NULL);
	
	if (Keys[SDLK_1]) {Sky.SetColoringMode(COLOR_TABLE); updateDome=true;}
	if (Keys[SDLK_2]) {Sky.SetColoringMode(COLOR_TEXTURE);  updateDome=true;}
	if (Keys[SDLK_3]) {Sky.SetColoringMode(COLOR_MODEL);  updateDome=true;}
	if (Keys[SDLK_4]) {Sky.SetColoringMode(COLOR_SKYMAP);  updateDome=true;}
    if (Keys[SDLK_q]) {Latitude += 0.01f;  updateDome=true;}
	if (Keys[SDLK_a]) {Latitude -= 0.01f;  updateDome=true;}
	if (Keys[SDLK_w]) {Longitude += 0.01f;  updateDome=true;}
	if (Keys[SDLK_s]) {Longitude -= 0.01f;  updateDome=true;}
	if (Keys[SDLK_e]) {Time += 0.01f;  updateDome=true;}
	if (Keys[SDLK_d]) {Time -= 0.01f;  updateDome=true;}
	if (Keys[SDLK_r]) {Day += 0.1f;  updateDome=true;}
	if (Keys[SDLK_f]) {Day -= 0.1f;  updateDome=true;}
	if (Keys[SDLK_h]) {help = 1;}
	if (Keys[SDLK_j]) {help = 0;}
	if (Keys[SDLK_i]) {info = 1;}
	if (Keys[SDLK_o]) {info = 0;}
	if (Keys[SDLK_SPACE]) grassplane ^=1;
	
#else
	if (GetKeyState('1') & 0x80) {Sky.SetColoringMode(COLOR_TABLE); updateDome=true;}
	if (GetKeyState('2') & 0x80) {Sky.SetColoringMode(COLOR_TEXTURE);  updateDome=true;}
	if (GetKeyState('3') & 0x80) {Sky.SetColoringMode(COLOR_MODEL);  updateDome=true;}
	if (GetKeyState('4') & 0x80) {Sky.SetColoringMode(COLOR_SKYMAP);  updateDome=true;}
    if (GetKeyState('Q') & 0x80) {Latitude += 0.01f;  updateDome=true;}
	if (GetKeyState('A') & 0x80) {Latitude -= 0.01f;  updateDome=true;}
	if (GetKeyState('W') & 0x80) {Longitude += 0.01f;  updateDome=true;}
	if (GetKeyState('S') & 0x80) {Longitude -= 0.01f;  updateDome=true;}
	if (GetKeyState('E') & 0x80) {Time += 0.01f;  updateDome=true;}
	if (GetKeyState('D') & 0x80) {Time -= 0.01f;  updateDome=true;}
	if (GetKeyState('R') & 0x80) {Day += 0.1f;  updateDome=true;}
	if (GetKeyState('F') & 0x80) {Day -= 0.1f;  updateDome=true;}
	if (GetKeyState('H') & 0x80) {help = 1;}
	if (GetKeyState('J') & 0x80) {help = 0;}
	if (GetKeyState('I') & 0x80) {info = 1;}
	if (GetKeyState('O') & 0x80) {info = 0;}
	if (GetKeyState(' ') & 0x80) grassplane ^=1;
	if (GetKeyState('9') & 0x80)
        glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
    if (GetKeyState('0') & 0x80)
        glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

#endif

	if (Time < 0.0f) {Time = 24 + Time; Day--;}
	if (Time > 24) {Time -= 24; Day++;}
	Day = fmod(Day, 360.25f);

    if (updateDome)
    {
	    Sky.SetCoordinates(Latitude, Longitude, Time, Day);
        Sky.Update(0);
    }
}

void MGLApp::Render()
{
    glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();

	Camera.Update();

    //Uncomment these liens if you want Fog
	//Color4 FogColor = Sky.GetHorizonColor(Camera.GetCameraView());
	//glClearColor(FogColor.R, FogColor.G, FogColor.B, FogColor.A);	
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glColor3f(1.0f, 1.0f, 1.0f);

        Sky.Render(Camera.GetCameraPosition());


        glEnable(GL_TEXTURE_2D);			/* Enable Texture Mapping*/
        if (grassplane)
        {    
            // Render Ground plane
            glColor3f(1.f, 1.f, 1.f);
            glBindTexture(GL_TEXTURE_2D, grassTexture);   /* choose the texture to use.*/
            glBegin(GL_QUADS);		                /* begin drawing a cube*/
            glTexCoord2f(0.0f, 0.0f); glVertex3f(-SkyRadius, 0.0f, SkyRadius);	
            glTexCoord2f(20.0f, 0.0f); glVertex3f( SkyRadius, 0.0f, SkyRadius);	
            glTexCoord2f(20.0f, 15.0f); glVertex3f( SkyRadius, 0.0f,-SkyRadius);	
            glTexCoord2f(0.0f, 15.0f); glVertex3f(-SkyRadius, 0.0f,-SkyRadius);	
            glEnd();
        }

        if (info)
        {
            switch (Sky.GetColoringMode())
            {
            case COLOR_TEXTURE: font.Display("Using Static Dome Texture map",5,5);
                                break;
            case COLOR_TABLE: font.Display("Using Color Table Dome colouring",5,5);
                               break;
            case COLOR_SKYMAP: font.Display("Using Sky Map Dome colouring",5,5);
                               break;
            case COLOR_MODEL: font.Display("Using Analytical Physically Based Dome colouring",5,5);
                               break;
            };
            char temp[256];
            sprintf(temp, "Latitude:%.1f, Longitude:%.1f, Time:%.1f, Day:%.0f", Latitude,Longitude,Time,Day);
            font.Display(temp,5,35);
        }
        
        if (help)
        {
            font.Display("'1','2','3','4': Various SkyDome representation methods",5,90);
            font.Display("'q','a':Latitude change",5,115);
            font.Display("'w','s':Longitude change",5,140);
            font.Display("'e','d':Time change",5,175);
            font.Display("'r','f':Day change",5,205);
            font.Display("'i','o':Info on/off",5,235);
            font.Display("'h','j':Help on/off",5,265);
            font.Display("'Space':Grass plane on/off",5,295);
            font.Display("Navigate using Mouse and Cursor Keys",5,325);
            font.Display("Demo by Dimi Christopoulos and Andrei Stojan",5,355);
        }
}
