/*************************************************************************
MGLApp.h

Main application class for SDL and Win32.

Author   :   Andrei Stojan, Dimi Christopoulos

Written for More OpenGL Game Programming
*************************************************************************/

#ifndef MGLAPP_H_INCLUDED
#define MGLAPP_H_INCLUDED

#if defined(MGL_WIN32)
	#define WIN32_LEAN_AND_MEAN
	#define WIN32_EXTRA_LEAN
	#include <windows.h>
    #include "glee.h"
#elif defined(MGL_SDL)
    #ifdef WIN32
    #include <SDL.h>
    #include "glee.h"
    #else
    #include <SDL/SDL.h>
    #endif
#endif

#include <iostream>
#include <vector>
#include <algorithm>
using namespace std;

#include <stdlib.h>
#include <stdarg.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <math.h>

#include "MGLVector.h"
#include "MGLTypes.h"
#include "MGLTargaImage.h"
using namespace MGL;

#include "MGLVector.h"
#include "MGLColor.h"
#include "MGLCamera.h"
#include "MGLSkyDome.h"

#define SQR(x) ( (x) * (x) )

#ifdef _MSC_VER
#pragma warning (disable: 4244)
#pragma warning (disable: 4305)
#endif

// values used to set up the window and rendering context
const char MGL_APP_NAME[]	= "MOGLGP - SkyDome Rendering - Andrei Stojan, Dimi Christopoulos";

const int MGL_WINDOW_WIDTH = 1024;
const int MGL_WINDOW_HEIGHT = 768;

const int MGL_COLOR_BITS = 32;
const int MGL_DEPTH_BITS = 24;
const int MGL_STENCIL_BITS = 0;

const bool MGL_FORCE_FULLSCREEN = false;	

class MGLApp
{
public:
	MGLApp();
	virtual ~MGLApp();

	bool Init();
	bool Shutdown();
	void SetupProjection( int width, int height );
	void Update( double dt );
	void Render();

private:

	MGLCamera 	Camera;
	CSkyDome	Sky;
	float Latitude, Longitude, Time, Day;
	float SkyRadius;
};

#endif // MGLAPP_H_INCLUDED
