/*************************************************************************
MGLCamera.cpp

OpenGL Camera Implementation, for SDL and Win32.

Author   :   Andrei Stojan, Dimi Christopoulos

Written for More OpenGL Game Programming
*************************************************************************/
#include "MGLApp.h"

MGLCamera::MGLCamera()
{
	Reset();
	Speed = 10.5f;
}

void MGLCamera::Reset()
{
	Position	= Vector3(0.0, 0.0, 0.0);
	View		= Vector3(1.0, 0.0, 0.0);					
	UpVector	= Vector3(0.0, 1.0, 0.0);	
	
	CurrentRot = 0.0f;

    unsigned short middleX = MGL_WINDOW_WIDTH / 2;
	unsigned short middleY = MGL_WINDOW_HEIGHT / 2;
#ifdef MGL_SDL
	SDL_WarpMouse(middleX, middleY);
#else
	SetCursorPos(middleX, middleY);
#endif
}

void MGLCamera::PositionCamera(float positionX, float positionY, float positionZ,
                               float viewX,     float viewY,     float viewZ,
                               float upVectorX, float upVectorY, float upVectorZ)
{
	Position	= Vector3(positionX, positionY, positionZ);
	View		= Vector3(viewX, viewY, viewZ);
	UpVector	= Vector3(upVectorX, upVectorY, upVectorZ);
}


void MGLCamera::SetViewByMouse()
{
	unsigned short middleX = MGL_WINDOW_WIDTH / 2;
	unsigned short middleY = MGL_WINDOW_HEIGHT / 2;
	
	float angleY = 0.0f;
	float angleZ = 0.0f;
	
	int PosX, PosY;

#ifdef MGL_SDL
	SDL_GetMouseState(&PosX, &PosY);
#else
	POINT mouse;
	GetCursorPos(&mouse);
	PosX = mouse.x; PosY = mouse.y;
#endif

	if( ((PosX == middleX) && (PosY == middleY)) || ((PosX == 0) && (PosY == 0)))
            return;

#ifdef MGL_SDL
	SDL_WarpMouse(middleX, middleY);
#else
	SetCursorPos(middleX, middleY);
#endif
	
	angleY = (float)( (middleX - PosX) ) / 500.0f;		
	angleZ = (float)( (middleY - PosY) ) / 500.0f;		

	CurrentRot -= angleZ;  

	Vector3 vAxis = (View - Position).CrossProduct(UpVector);
	vAxis.Normalize();

	RotateView(angleY, 0, 1, 0);

	if(CurrentRot > 2.0f)
		CurrentRot = 2.0f;	
	else 
	if(CurrentRot < -2.0f)
		CurrentRot = -2.0f;	
	else
		RotateView(angleZ, vAxis.x, vAxis.y, vAxis.z);
}

void MGLCamera::RotateView(float angle, float x, float y, float z)
{
	Vector3 vNewView;
	Vector3 vView = View - Position;		

	float cosTheta = (float)cos(angle);
	float sinTheta = (float)sin(angle);

	vNewView.x  = (cosTheta + (1 - cosTheta) * x * x)		* vView.x;
	vNewView.x += ((1 - cosTheta) * x * y - z * sinTheta)	* vView.y;
	vNewView.x += ((1 - cosTheta) * x * z + y * sinTheta)	* vView.z;

	vNewView.y  = ((1 - cosTheta) * x * y + z * sinTheta)	* vView.x;
	vNewView.y += (cosTheta + (1 - cosTheta) * y * y)		* vView.y;
	vNewView.y += ((1 - cosTheta) * y * z - x * sinTheta)	* vView.z;

	vNewView.z  = ((1 - cosTheta) * x * z - y * sinTheta)	* vView.x;
	vNewView.z += ((1 - cosTheta) * y * z + x * sinTheta)	* vView.y;
	vNewView.z += (cosTheta + (1 - cosTheta) * z * z)		* vView.z;

	View = Position + vNewView;
}

void MGLCamera::StrafeCamera(float speed)
{	
	Position.x += Strafe.x * speed;
	Position.z += Strafe.z * speed;

	View.x += Strafe.x * speed;
	View.z += Strafe.z * speed;
}

void MGLCamera::MoveCamera(float speed)
{
	Vector3 vVector = View - Position;
	vVector.Normalize();

	Position.x += vVector.x * speed;
	Position.y += vVector.y * speed;
	Position.z += vVector.z * speed;
	View.x += vVector.x * speed;
	View.y += vVector.y * speed;
	View.z += vVector.z * speed;
}

void MGLCamera::CheckForMovement()
{	
#ifdef MGL_SDL
	unsigned char *Keys = SDL_GetKeyState(NULL);
	
	if (Keys[SDLK_UP])   MoveCamera(Speed);				
	if (Keys[SDLK_DOWN]) MoveCamera(-Speed);				
	if (Keys[SDLK_LEFT]) StrafeCamera(-Speed);
	if (Keys[SDLK_RIGHT]) StrafeCamera(Speed);
#else
	if (GetKeyState(VK_UP) & 0x80) MoveCamera(Speed);
	if (GetKeyState(VK_DOWN) & 0x80) MoveCamera(-Speed);
	if (GetKeyState(VK_LEFT) & 0x80) StrafeCamera(-Speed);
	if (GetKeyState(VK_RIGHT) & 0x80) StrafeCamera(Speed);
#endif
}

void MGLCamera::Update() 
{
	Strafe = (View - Position).CrossProduct(UpVector);
	Strafe.Normalize();

	CheckForMovement();
	SetViewByMouse();
	gluLookAt(Position.x, Position.y, Position.z,	
			  View.x,	 View.y,     View.z,	
			  UpVector.x, UpVector.y, UpVector.z);
}
