/*************************************************************************
MGLSkyColorTable.cpp

SkyColor Table iomplementation for Sky Dome Rendering.

Author   :   Andrei Stojan

Written for More OpenGL Game Programming
*************************************************************************/
#include "MGLApp.h"

SkyColorTable::SkyColorTable()
{

}

int SkyColorTable::Load()
{
	TargaImage img;

	if (img.Load("skymap.tga") == false)
	{
		printf("Cannot load skymap.tga\n");
		return 1;
	}
	Width = img.GetWidth();
	Height = img.GetHeight();
	ColorTable = new Color4[Width * Height];
	
	unsigned char bcolor[4];
	for (int i = 0; i < Height; i++)
		for (int j = 0; j < Width; j++)
		{
			 memcpy(bcolor, img.GetImage() + (i * Width + j) * 4, 4);
			 ColorTable[i * Width + j].R = bcolor[0] / 255.0f;
			 ColorTable[i * Width + j].G = bcolor[1] / 255.0f;
			 ColorTable[i * Width + j].B = bcolor[2] / 255.0f;
			 ColorTable[i * Width + j].A = bcolor[3] / 255.0f;
		}

	img.Release();

	return 0;
}

void SkyColorTable::GetColor(float TimeOfDay, float Phi, float Theta, Color4* ptr)
{
	float fX = TimeOfDay / 24.0f * Width;
	int iX = fX;
	int iXn = (iX >= Width - 1 ? iX : iX + 1);
	float fY = Theta / (PI / 2) * Height;
	int iY = fY;
	if (iY >= Height) iY = Height - 1;
	int iYn = (iY >= Height - 1 ? iY : iY + 1);

/*
	+---------------+
	|C10         C11|
	|               |
	|               |
	|               |    ^x
	|               |    |
	|C00         C01|    |     y
	+---------------+    +----->
*/
	
	Color4 Col00 = ColorTable[iY * Width + iX];
	Color4 Col01 = ColorTable[iY * Width + iXn];
	Color4 Col11 = ColorTable[iYn * Width + iXn];
	Color4 Col10 = ColorTable[iYn * Width + iX];

/*	    Horiz1
	+---X----------+=
	|   .          |1-vf
	|...R..........|-  Result
	|   .          |
	|   .          |
	|   .          |vf
	|   .          |
	+---X----------+=
    <hf>|<  1-hf   >
        Horiz0
*/
	float hf = fX - iX;
	Color4 Horiz0 = Col00 * (1.0f - hf) + Col01 * hf;
	Color4 Horiz1 = Col10 * (1.0f - hf) + Col11 * hf;
	float vf = fY - iY;
	*ptr = Horiz0 * (1.0f - vf) + Horiz1 * vf;
}