/*************************************************************************
MGLSkyDome.h

SkyDome Geometry computation and rendering.

Author   :   Dimi Christopoulos, Andrei Stojan

Written for More OpenGL Game Programming
*************************************************************************/
#ifndef _SKY_DOME_H
#define _SKY_DOME_H

#define COLOR_TEXTURE	100
#define COLOR_TABLE	101
#define COLOR_MODEL	102
#define COLOR_SKYMAP 	103

struct Vector2
{
	float x, y;
};

class SkyColorTable
{
public:
	SkyColorTable();

	int  Load();
	void GetColor(float TimeOfDay, float Phi, float Theta, Color4* ColorPtr);
	
private:

	Color4* ColorTable;
	int	Width, Height;
};

class CSkyColor
{
public: 
	
	CSkyColor();
	
	void GetVertexColor(float Theta, float Phi, float SunPhi, Color4* color);
	void SetInfo(float SunTheta, float Turbidity);


private:
	
	void ToRGBA(float x, float y, float Y, Color4* rgb);
	float Angle(float theta1, float phi1, float theta2, float phi2);
	
	inline float PerezFunction_x(float, float, float);
	inline float PerezFunction_y(float, float, float);
	inline float PerezFunction_Y(float, float, float);

	float Ax, Bx, Cx, Dx, Ex;
	float Ay, By, Cy, Dy, Ey;
	float AY, BY, CY, DY, EY;
	
	float T, suntheta, T2, suntheta3, suntheta2;	
	float xz, yz, Yz;
	float cosSTheta;
};

class CSkyDome
{
	public:
		CSkyDome();
	
		int  Initialize(float Radius, int NumSlices, int NumSides, bool exponential, float dampening=1);
		void SetCoordinates(float Latitude, float Longitude, float TimeOfDay, float JulianDay);
		void Render(Vector3 Camera);
		void RenderElements(bool initial);
		void Update(float deltatime);
		void Destroy();
		void SetColoringMode(int mode)
		{
			ColoringMode = mode;
		}

		int GetColoringMode()
		{
			return ColoringMode;
		}

		Vector3 GetSunVector()
		{
			return Vector3(sin(SunTheta) * cos(SunPhi) * SkyRadius,
			    			cos(SunTheta) * SkyRadius,
			    			sin(SunTheta) * sin(SunPhi) * SkyRadius);
		}
		
		Color4 GetHorizonColor(Vector3 CameraView);
		
	private:
			
		Vector3 *VertexBuffer;
		Color4 *ColorBuffer;
		Vector2 *TexCoordBuffer;
		Vector2 *SkyMapTexCoordBuffer;

		Vector3*	Stars;
		Color4*		StarColors;
		int		NumStars;

		void RenderDome();
		
		unsigned short *IndexBuffer;
		
		int ColoringMode;

		int NumSlices;
		int NumSides;
		int NumIndices;
		
		float SkyRadius;
		
		unsigned	FlareTex0;
		unsigned	MoonTex;
		unsigned	Texture;
		unsigned	SkyMap;

		float SunTheta, SunPhi;
		float MoonTheta, MoonPhi;
		
		float TimeOfDay, JulianDay, Latitude, Longitude;
		
		CSkyColor Shader;
		SkyColorTable ColorTable;
};

#endif
