/*************************************************************************
glCamera.cpp

Camera Class.

Author   :   Dimi Christopoulos

*************************************************************************/
#include "glCamera.h"
#include "tmatrix.h"
#include "mathex.h"

void GLCamera::setPos(float x, float y, float z)
{
    _CamPos = TVector32(x, y, z);
}

void GLCamera::setCamTarget(float x, float y, float z)
{
    // Compute direction vector
    _Direction = TVector32(x, y, z) - _CamPos;
    _Direction.unit();
    
    // Find the Up vector perpendicular to that direction vector.
    // Project direction vector to YZ or XZ plane and take the cross 
    // product to get first an axis then the up vector.
    if (fabs(_Direction.X()) < 0.00001f && fabs(_Direction.Z()) < 0.00001f)
    {
	// Project onto YZ plane
	TVector32 projTarget = TVector32(0.f, _Direction.Y(), _Direction.Z());

        // No need to get the perpendicular axis since its the x axis.
	// Get the up vector by cross product projected point and x axis.
	_Up = projTarget * TVector32(1.f, 0.f, 0.f);
	_Up.unit();
    }
    else
    {
	// Project onto XZ plane
	TVector32 projTarget = TVector32(_Direction.X(), 0.f, _Direction.Z());
	
	// Compute perpendicular axis with cross product
	TVector32 axis = projTarget * TVector32(0.f, 1.f, 0.f);
	
	// Get the up vector by cross product direction and axis.
        _Up = axis*_Direction;
	_Up.unit();
    }
}

void GLCamera::addPos(float x, float y, float z)
{
    _CamPos += TVector32(x, y, z);
}

void GLCamera::rotateY(const float angle)
{
    float angle_rad = DegToRad(angle);
    TMatrix33 RotY(0, angle_rad, 0);
	
    _Direction = RotY * _Direction;
    _Up = RotY * _Up;
}

void GLCamera::rotateAxis(const float angle)
{
    TVector32 Axis=(_Direction * _Up).unit();
    TMatrix33 RotAxis(Axis, angle);

    _Direction = RotAxis * _Direction;
    _Up = RotAxis * _Up;
}

void GLCamera::setHeadPitch (float heading, float pitch)
{
    _Direction = TVector32(0, 0, -1);
    _Up = TVector32(0, 1, 0);

    rotateY (heading);
    rotateAxis (pitch);
}


void GLCamera::move (float amount)
{
     _CamPos += _Direction * amount;
}


