#include <stdio.h>
#include <stdlib.h> 
#include <string.h>

///////////////////////////////////////////////////////////////////////////
// Functions to read SGI textures
///////////////////////////////////////////////////////////////////////////
void
bwtorgba(unsigned char *b,unsigned char *l,int n) {
    while(n--) {
	l[0] = *b;
	l[1] = *b;
	l[2] = *b;
	l[3] = 0xff;
	l += 4; b++;
    }
}

void
latorgba(unsigned char *b, unsigned char *a,unsigned char *l,int n) {
    while(n--) {
	l[0] = *b;
	l[1] = *b;
	l[2] = *b;
	l[3] = *a;
	l += 4; b++; a++;
    }
}

void
rgbtorgba(unsigned char *r,unsigned char *g,unsigned char *b,unsigned char *l,int n) {
    while(n--) {
	l[0] = r[0];
	l[1] = g[0];
	l[2] = b[0];
	l[3] = 0xff;
	l += 4; r++; g++; b++;
    }
}

void
rgbatorgba(unsigned char *r,unsigned char *g,unsigned char *b,unsigned char *a,unsigned char *l,int n) {
    while(n--) {
	l[0] = r[0];
	l[1] = g[0];
	l[2] = b[0];
	l[3] = a[0];
        l += 4; r++; g++; b++; a++;
    }
}

typedef struct _ImageRec {
    unsigned short imagic;
    unsigned short type;
    unsigned short dim;
    unsigned short xsize, ysize, zsize;
    unsigned int min, max;
    unsigned int wasteBytes;
    char name[80];
    unsigned long colorMap;
    FILE *file;
    unsigned char *tmp, *tmpR, *tmpG, *tmpB;
    unsigned long rleEnd;
    unsigned int *rowStart;
    int *rowSize;
} ImageRec;

static void
ConvertShort(unsigned short *array, long length) {
    unsigned b1, b2;
    unsigned char *ptr;

    ptr = (unsigned char *)array;
    while (length--) {
	b1 = *ptr++;
	b2 = *ptr++;
	*array++ = (b1 << 8) | (b2);
    }
}

static void
ConvertLong(unsigned *array, long length) {
    unsigned b1, b2, b3, b4;
    unsigned char *ptr;

    ptr = (unsigned char *)array;
    while (length--) {
	b1 = *ptr++;
	b2 = *ptr++;
	b3 = *ptr++;
	b4 = *ptr++;
	*array++ = (b1 << 24) | (b2 << 16) | (b3 << 8) | (b4);
    }
}

static ImageRec *ImageOpen(const char *fileName)
{
    union {
	int testWord;
	char testByte[4];
    } endianTest;
    ImageRec *image;
    int swapFlag;
    int x;

    endianTest.testWord = 1;
    if (endianTest.testByte[0] == 1) {
	swapFlag = 1;
    } else {
	swapFlag = 0;
    }
 
    //image = (ImageRec *)malloc(sizeof(ImageRec));
    image = new ImageRec;
    if (image == NULL) {
	fprintf(stderr, "Out of memory!\n");
	exit(1);
    }
    if ((image->file = fopen(fileName, "rb")) == NULL) {
	perror(fileName);
	exit(1);
    }

    fread(image, 1, 12, image->file);

    if (swapFlag) {
	ConvertShort(&image->imagic, 6);
    }

    /*image->tmp = (unsigned char *)malloc(image->xsize*256);
    image->tmpR = (unsigned char *)malloc(image->xsize*256);
    image->tmpG = (unsigned char *)malloc(image->xsize*256);
    image->tmpB = (unsigned char *)malloc(image->xsize*256);*/
    image->tmp = new unsigned char[image->xsize*256];
    image->tmpR = new unsigned char[image->xsize*256];
    image->tmpG = new unsigned char[image->xsize*256];
    image->tmpB = new unsigned char[image->xsize*256];
    if (image->tmp == NULL || image->tmpR == NULL || image->tmpG == NULL ||
	image->tmpB == NULL) {
	fprintf(stderr, "Out of memory!\n");
	exit(1);
    }

    if ((image->type & 0xFF00) == 0x0100) {
    /*x = image->ysize * image->zsize * sizeof(unsigned);
	image->rowStart = (unsigned *)malloc(x);
	image->rowSize = (int *)malloc(x);*/
    x=image->ysize * image->zsize;
    image->rowStart = new unsigned int[x];
	image->rowSize = new int[x];
    x = image->ysize * image->zsize * sizeof(unsigned);
	
	if (image->rowStart == NULL || image->rowSize == NULL) {
	    fprintf(stderr, "Out of memory!\n");
	    exit(1);
	}
	image->rleEnd = 512 + (2 * x);
	fseek(image->file, 512, SEEK_SET);
	fread(image->rowStart, 1, x, image->file);
	fread(image->rowSize, 1, x, image->file);
	if (swapFlag) {
	    ConvertLong(image->rowStart, x/(int)sizeof(unsigned));
	    ConvertLong((unsigned *)image->rowSize, x/(int)sizeof(int));
	}
    } else {
	image->rowStart = NULL;
	image->rowSize = NULL;
    }
    return image;
}

static void
ImageClose(ImageRec *image) {
    fclose(image->file);
    /*free(image->tmp);
    free(image->tmpR);
    free(image->tmpG);
    free(image->tmpB);
    free(image);*/
    delete[] image->tmp;
    delete[] image->tmpR;
    delete[] image->tmpG;
    delete[] image->tmpB;
    delete image;
}

static void
ImageGetRow(ImageRec *image, unsigned char *buf, int y, int z) {
    unsigned char *iPtr, *oPtr, pixel;
    int count;

    if ((image->type & 0xFF00) == 0x0100) {
	fseek(image->file, (long) image->rowStart[y+z*image->ysize], SEEK_SET);
	fread(image->tmp, 1, (unsigned int)image->rowSize[y+z*image->ysize],
	      image->file);

	iPtr = image->tmp;
	oPtr = buf;
	for (;;) {
	    pixel = *iPtr++;
	    count = (int)(pixel & 0x7F);
	    if (!count) {
		return;
	    }
	    if (pixel & 0x80) {
		while (count--) {
		    *oPtr++ = *iPtr++;
		}
	    } else {
		pixel = *iPtr++;
		while (count--) {
		    *oPtr++ = pixel;
		}
	    }
	}
    } else {
	fseek(image->file, 512+(y*image->xsize)+(z*image->xsize*image->ysize),
	      SEEK_SET);
	fread(buf, 1, image->xsize, image->file);
    }
}

unsigned char * read_rgbtexture(char *name, int *width, int *height, int *components) {

    unsigned char *base, *lptr;
    unsigned char *rbuf, *gbuf, *bbuf, *abuf;
    ImageRec *image;
    int y;

    image = ImageOpen(name);
    
    if(!image)
	return NULL;
    (*width)=image->xsize;
    (*height)=image->ysize;
    (*components)=image->zsize;
    /*base = (unsigned char *)malloc(4*image->xsize*image->ysize*sizeof(unsigned char));
    rbuf = (unsigned char *)malloc(image->xsize*sizeof(unsigned char));
    gbuf = (unsigned char *)malloc(image->xsize*sizeof(unsigned char));
    bbuf = (unsigned char *)malloc(image->xsize*sizeof(unsigned char));
    abuf = (unsigned char *)malloc(image->xsize*sizeof(unsigned char));*/
    base = new unsigned char[4*image->xsize*image->ysize];
    rbuf = new unsigned char[image->xsize];
    gbuf = new unsigned char[image->xsize];
    bbuf = new unsigned char[image->xsize];
    abuf = new unsigned char[image->xsize];
    if(!base || !rbuf || !gbuf || !bbuf)
      return NULL;
    lptr = base;
    for(y=0; y<image->ysize; y++) {
	if(image->zsize>=4) {
	    ImageGetRow(image,rbuf,y,0);
	    ImageGetRow(image,gbuf,y,1);
	    ImageGetRow(image,bbuf,y,2);
	    ImageGetRow(image,abuf,y,3);
	    rgbatorgba(rbuf,gbuf,bbuf,abuf,(unsigned char *)lptr,image->xsize);
	    lptr += 4*image->xsize;
	} else if(image->zsize==3) {
	    ImageGetRow(image,rbuf,y,0);
	    ImageGetRow(image,gbuf,y,1);
	    ImageGetRow(image,bbuf,y,2);
	    rgbtorgba(rbuf,gbuf,bbuf,(unsigned char *)lptr,image->xsize);
	    lptr += 4*image->xsize;
	} else if(image->zsize==2) {
	    ImageGetRow(image,rbuf,y,0);
	    ImageGetRow(image,abuf,y,1);
	    latorgba(rbuf,abuf,(unsigned char *)lptr,image->xsize);
	    lptr += 4*image->xsize;
	} else {
	    ImageGetRow(image,rbuf,y,0);
	    bwtorgba(rbuf,(unsigned char *)lptr,image->xsize);
	    lptr += 4*image->xsize;
	}
    }
    ImageClose(image);
    /*free(rbuf);
    free(gbuf);
    free(bbuf);
    free(abuf);*/
    delete[] rbuf; 
    delete[] gbuf; 
    delete[] bbuf; 
    delete[] abuf;

    return base;
}



///////////////////////////////////////////////////////////////////////////
// Functions to Write SGI textures
///////////////////////////////////////////////////////////////////////////
void putbyte(FILE *outf,unsigned char val)
{
    unsigned char buf[1];
    
    buf[0] = val;
    fwrite(buf,1,1,outf);
}
     
void putshort(FILE *outf,unsigned short val)
{
    unsigned char buf[2];
     
    buf[0] = (val>>8);
    buf[1] = (val>>0);
    fwrite(buf,2,1,outf);
}
     
int putlong(FILE *outf,unsigned long val)
{
    unsigned char buf[4];
     
    buf[0] = (val>>24);
    buf[1] = (val>>16);
    buf[2] = (val>>8);
    buf[3] = (val>>0);
    return fwrite(buf,4,1,outf);
}

int write_rgbtexture(char *name, unsigned char *buffer, int width, int height, int components) 
{
    /* Write texture in Verbatim non RLE mode, with 1 byte per pixel */
    FILE *of;
    char iname[80];
    unsigned char *outbuf=new unsigned char[width];
    int i, x, y, z;
    
    if (!outbuf)
    {
        fprintf(stderr,"sgiimage: can't allocate memory for image %s\n", name);
        return 0;
    }
    of = fopen(name,"w");
    if (!of) 
    {
        fprintf(stderr,"sgiimage: can't open output file %s\n", name);
        return 0;
    }
    /* Write Header */
    putshort(of,474);       /* MAGIC               */
    putbyte(of,0);          /* STORAGE is VERBATIM */
    putbyte(of,1);          /* BPC is 1            */
    if (components==1)      /* Single channel many scanlines */
        putshort(of,2);     /* DIMENSION is 2      */
    else                    /* Many channels many scanlines */
        putshort(of,3);     /* DIMENSION is 3      */
    putshort(of,width);     /* XSIZE               */
    putshort(of,height);    /* YSIZE               */
    putshort(of,components);/* ZSIZE               */
    putlong(of,0);          /* PIXMIN is 0         */
    putlong(of,255);        /* PIXMAX is 255       */
    for(i=0; i<4; i++)      /* DUMMY 4 bytes       */
        putbyte(of,0);
    strcpy(iname,"No Name");
    fwrite(iname,80,1,of);  /* IMAGENAME           */
    putlong(of,0);          /* COLORMAP is 0       */
    for(i=0; i<404; i++)    /* DUMMY 404 bytes     */
        putbyte(of,0);

    /* Write Image Data */
    /* For each component */
    for(z=0; z<components; z++)
    {
        for(y=0; y<height; y++) 
        {
            /* Write out all pixel for this component*/
            for(x=0; x<width; x++)
                 outbuf[x] = buffer[(x+y*width)*components + z];
            fwrite(outbuf,width,1,of);
        }
    }
    fclose(of);

    delete[] outbuf;
    return 1;
}
     

///////////////////////////////////////////////////////////////////////////
// Help function to convert RGBA to Grayscale
///////////////////////////////////////////////////////////////////////////
void  RGBAtoGrayscale(unsigned char *texbuffer, int width, int height, int components)
{
    unsigned char graycolor=0;
    
    // If no RGB/RGBA data
    if (components<3)
        return;

    // convert pixels: greyscale = 0.30 * R + 0.59 * G + 0.11 * B
    for (int i=0;i<height*width*components;i+=components)
    {
        graycolor = (unsigned char)(0.30 * texbuffer[i] + 0.59 * texbuffer[i+1] 
                                     + 0.11 * texbuffer[i+2]);
        texbuffer[i]   = graycolor;
        texbuffer[i+1] = graycolor;
        texbuffer[i+2] = graycolor;
    }
}


///////////////////////////////////////////////////////////////////////////
// Functions to Read BMP textures
///////////////////////////////////////////////////////////////////////////
/*
 * getint and getshort are help functions to load the bitmap byte by byte on 
 * SPARC platform (actually, just makes the thing work on platforms of either
 * endianness, not just Intel's little endian)
 */
static unsigned int getint(FILE *fp)
{
  int c, c1, c2, c3;

  /* get 4 bytes */
  c = getc(fp);  
  c1 = getc(fp);  
  c2 = getc(fp);  
  c3 = getc(fp);
  
  return ((unsigned int) c) +   
    (((unsigned int) c1) << 8) + 
    (((unsigned int) c2) << 16) +
    (((unsigned int) c3) << 24);
}

static unsigned int getshort(FILE *fp)
{
  int c, c1;
  
  /*get 2 bytes */
  c = getc(fp);  
  c1 = getc(fp);

  return ((unsigned int) c) + (((unsigned int) c1) << 8);
}

/* quick and dirty bitmap loader...for 24 bit bitmaps with 1 plane only.   */
/* See http://www.dcs.ed.ac.uk/~mxr/gfx/2d/BMP.txt for more info. */
unsigned char *read_bmptexture(char *filename, int *width, int *height, int *components) 
{
    FILE *file;
    unsigned long size;                 /* size of the image in bytes. */
    unsigned long i;                    /* standard counter. */
    unsigned short int planes;          /* number of planes in image (must be 1)  */
    unsigned short int bpp;             /* number of bits per pixel (must be 24) */
    unsigned char temp;                 /* used to convert bgr to rgb color. */

    /* make sure the file is there. */
    if ((file = fopen(filename, "rb"))==NULL) {
      printf("File Not Found : %s\n",filename);
      return 0;
    }
    
    /* seek through the bmp header, up to the width/height: */
    fseek(file, 18, SEEK_CUR);

    /* No 100% errorchecking anymore!!! */

    /* read the width */
    *width = getint (file);
    //printf("Width of %s: %lu\n", filename, width);
    
    /* read the height  */
    *height = getint (file);
    //printf("Height of %s: %lu\n", filename, height);
    
    /* calculate the size (assuming 24 bits or 3 bytes per pixel). */
    *components = 3;
    size = *width * *height * *components;

    /* read the planes */
    planes = getshort(file);
    if (planes != 1) {
	printf("Planes from %s is not 1: %u\n", filename, planes);
	return 0;
    }

    /* read the bpp */
    bpp = getshort(file);
    if (bpp != 24) {
      printf("Bpp from %s is not 24: %u\n", filename, bpp);
      return 0;
    }
	
    /* seek past the rest of the bitmap header. */
    fseek(file, 24, SEEK_CUR);

    /* read the data.  */
    unsigned char *data = new unsigned char[size];
    if (data == NULL) {
	printf("Error allocating memory for color-corrected image data");
	return 0;	
    }

    if ((i = fread(data, size, 1, file)) != 1) {
	printf("Error reading image data from %s.\n", filename);
	return 0;
    }

    for (i=0;i<size;i+=3) { /* reverse all of the colors.  (bgr -> rgb) */
	temp = data[i];
	data[i] = data[i+2];
	data[i+2] = temp;
    }

    /* we're done. */
    return data;
}
