/*************************************************************************
tmatrix.h

Matrix Library.

Author   :   Dimi Christopoulos

*************************************************************************/
#ifndef matrix32_h
#define matrix32_h

#include "mathex.h"

class TVector32;

class TMatrix33
{
	private:
		float _Mx[3][3];

	public:
		// Constructors
		TMatrix33();
		TMatrix33(const TVector32& a, float psi);
		TMatrix33(float Phi, float Theta, float Psi);
		TMatrix33(float mx00, float mx01, float mx02, float mx10, float mx11, float mx12, float mx20, float mx21, float mx22);

		// Selectors
		float operator()(int Row, int Column) const { return _Mx[Row][Column]; }
		float &operator()(int Row, int Column) { return _Mx[Row][Column]; }

		// Optimised artimetric methods
		static TMatrix33 &add(const TMatrix33 &m1, const TMatrix33 &m2, TMatrix33 &result);
		static TMatrix33 &subtract(const TMatrix33 &m1, const TMatrix33 &m2, TMatrix33 &result);
		static TMatrix33 &multiply(const TMatrix33 &m1, const TMatrix33 &m2, TMatrix33 &result);
		static TMatrix33 &multiply(const TMatrix33 &m1, const float &scale, TMatrix33 &result);
		static TVector32 &multiply(const TMatrix33 &m1, const TVector32 &v, TVector32 &result);

		// Matrix arithmetic
		TMatrix33 &operator+=(const TMatrix33 &m) { return add(*this, m, *this); }
		TMatrix33 &operator-=(const TMatrix33 &m) { return subtract(*this, m, *this); }
		TMatrix33 &operator*=(const TMatrix33 &m) { TMatrix33 tm(*this); return multiply(tm, m, *this); }
		TMatrix33 &operator*=(const float &scale) { return multiply(*this, scale, *this); }
		TMatrix33 operator+(const TMatrix33 &m) const { TMatrix33 tm; return add(*this, m, tm); }
		TMatrix33 operator-(const TMatrix33 &m) const { TMatrix33 tm; return subtract(*this, m, tm); }
		TMatrix33 operator*(const TMatrix33 &m) const { TMatrix33 tm; return multiply(*this, m, tm); }
		TMatrix33 operator*(const float &scale) const { TMatrix33 tm; return multiply(*this, scale, tm); }
		TVector32 operator*(const TVector32 &v) const;

		// Determinants
		float determinant() const;
		static float determinant(const TMatrix33 &m) { return m.determinant(); }

		// Transpose
		TMatrix33 &transpose();
		static TMatrix33 &transpose(const TMatrix33 &m, TMatrix33 &result) { result = m; return result.transpose(); }
		static TMatrix33 transpose(const TMatrix33 &m) { return TMatrix33(m).transpose(); }

		// Inverse
		static TMatrix33 &inverse(const TMatrix33 &m1, TMatrix33 &result);
		static TMatrix33 inverse(const TMatrix33 &m1) { TMatrix33 tm; return inverse(m1, tm); }
		TMatrix33 inverse() const { TMatrix33 tm; return inverse(*this, tm); }

};
#endif
