/*************************************************************************
MGLApp.cpp

Main application class.

Author   :   Dave Astle

Written for More OpenGL Game Programming
*************************************************************************/

#ifdef WIN32
	#define WIN32_LEAN_AND_MEAN
	#define WIN32_EXTRA_LEAN
	#include <windows.h>
#endif

#include "MGLApp.h"


MGLApp::MGLApp()
{
}

MGLApp::~MGLApp()
{
}

bool MGLApp::Init(HDC g_hDC)
{	
	ghDC = g_hDC;

	if (!GLeeInit())
		return false;

	// Load our font.
	if (!font.load("font.tga", "font.dat"))
		return false;

	// Initialize the heightmap by creating a midpoint displacement heightmap.
	heightmap.createHeightmap(CHeightClass::Midpoint);

	return true;
}

bool MGLApp::Shutdown()
{
	return true;
}

void MGLApp::SetupProjection(int width, int height)
{
	// avoid div by zero
	if( height == 0 )
		height = 1;					

	// set up GL viewport
	glViewport( 0, 0, width, height );

	// set up a perspective projection
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0f, (GLfloat)width/(GLfloat)height, 1.0f, 1000.0f );

	// Give the window dimentions to the font class.
	font.setDimentions(width, height);

	// Save the width and height for later use.
	sWidth = width;
	sHeight = height;
}

void MGLApp::Update( double dt )
{
	// If the '1' key is pressed, create a heightmap using the hill algorithm.
	if ((GetAsyncKeyState('1') & 0x8000))
	{
		glClear(GL_COLOR_BUFFER_BIT);
		font.display("Creating Heightmap With:", 1, 30);
		font.display("Hill Algorithm", 1, 70);
		SwapBuffers(ghDC);

		heightmap.createHeightmap(CHeightClass::Hill);
	}

	// If the '2' key is pressed, create a heightmap using the fault line algorithm.
	if ((GetAsyncKeyState('2') & 0x8000))
	{
		glClear(GL_COLOR_BUFFER_BIT);
		font.display("Creating Heightmap With:", 1, 30);
		font.display("Fault Line Algorithm", 1, 70);
		SwapBuffers(ghDC);

		heightmap.createHeightmap(CHeightClass::FaultLine);
	}

	// If the '3' key is pressed, create a heightmap using midpoint displacement.
	if ((GetAsyncKeyState('3') & 0x8000))
	{
		glClear(GL_COLOR_BUFFER_BIT);
		font.display("Creating Heightmap With:", 1, 30);
		font.display("Midpoint Algorithm", 1, 70);
		SwapBuffers(ghDC);

		heightmap.createHeightmap(CHeightClass::Midpoint);
	}

}

void MGLApp::Render()
{
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );


	// set up the camera
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();

	// Display the heightmap to the screen.
	heightmap.draw(sWidth, sHeight);
}
