/*************************************************************************
MGLApp.h

Main application class.

Author   :   Dave Astle

Written for More OpenGL Game Programming
*************************************************************************/

#ifndef MGLAPP_H_INCLUDED
#define MGLAPP_H_INCLUDED

#ifdef WIN32
  #define WIN32_LEAN_AND_MEAN
  #define WIN32_EXTRA_LEAN
  #include <windows.h>
#endif

#pragma comment(lib, "glee")

#include "glee.h"
#include <gl/glu.h>

#include "CGLFontClass.h"
#include "CHeightClass.h"
#include "CTerrainQuadLOD.h"

// values used to set up the window and rendering context
const char MGL_APP_NAME[]	= "MOGLGP - Chapter 10 - VIPM Terrain Mesh";

const int MGL_WINDOW_WIDTH = 800;
const int MGL_WINDOW_HEIGHT = 600;

const int MGL_COLOR_BITS = 32;
const int MGL_DEPTH_BITS = 32;
const int MGL_STENCIL_BITS = 0;

const bool MGL_FORCE_FULLSCREEN = false;


class MGLApp
{
public:
	MGLApp();
	virtual ~MGLApp();

	bool Init(HDC g_hDC);
	bool Shutdown();
	void SetupProjection( int width, int height );
	void Update( double dt );
	void Render();

private:

	HDC ghDC;

	// The applications font.
	CGLFontClass font;
	
	// Our heightmap.
	CHeightClass heightmap;

	// Our terrain mesh.
	CTerrainQuadLOD terrain;

	// Width and Height of the window
	int sWidth, sHeight;

	// Variable to toggle wireframe and bounding boxes.
	bool wireframe;

	int teselation;

	// Movement variables.
	float translateX, translateZ, rotateX, rotateY;

	// Variable to make sure input works smoothly.
	double timeSinceInput;
};

#endif // MGLAPP_H_INCLUDED