fn export_mesh n f =
(
	m = snapshotAsMesh n
	format "\tBEGIN_MESH\n" to:f
	format "\t\tNUM_VERTS %\n" (getNumVerts m) to:f
	
	-- verts list
	for i = 1 to  ((getNumVerts m)) do (
		v	= getVert m i
		format "\t\t%, %, %\n" v.x v.y v.z to:f
	)
	format "\n" to:f

	format "\t\tNUM_FACES %\n" (getNumFaces m) to:f
	-- faces list
	for i = 1 to ((getNumFaces m)) do (
		fa = getFace m i
		format "\t\t%, %, %\n" ((fa.x-1) as Integer) ((fa.y-1) as Integer) ((fa.z-1) as Integer) to:f
	)
	format "\tEND_MESH\n" to:f
)

fn export_all_portals f =
(
	deselect selection
	bs = select $tmp_portal_*
	format "BEGIN_PORTALS %\n" selection.count to:f
	if bs != undefined then
	(
		for i = 1 to selection.count do
		(
			format "\tPORTAL %\n" (substring selection[i].name 8 -1) to:f
			export_mesh selection[i] f
		)
	)
	format "END_PORTALS\n\n" to:f
	deselect selection
	delete $tmp_portal_*
)

fn export_all_occluders f =
(
	deselect selection
	bs = select $occluder_*
	format "BEGIN_OCCLUDERS %\n" selection.count to:f
	if bs != undefinded then
	(
		for i = 1 to selection.count do
		(
			format "\tOCCLUDER %\n" (substring selection[i].name 10 -1) to:f
			export_mesh selection[i] f
		)
	)
	format "END_OCCLUDERS\n\n" to:f
)

fn build_all_areas_mesh =
(
	if $Areas_Mesh != undefined then delete $Areas_Mesh
	
	deselect selection
	select $occluder_*
	occluders = #()
	for i = 1 to selection.count do
		append occluders selection[i]

	deselect selection
	select $portal_*
	portals = #()
	for i = 1 to selection.count do
		append portals selection[i]
	
	_str = "occluders: " + (occluders.count as string) + "\nportals: " + (portals.count as string) + "\n"
	print occluders
	print portals

	-- now proceed and build the areas mesh
	if occluders.count == 0 then return "ZERO occluder!!! Unable to proceed!!!"
	
	mo	= snapshotAsMesh occluders[1];
	
	-- boolean with union operator for occluders
	for i = 2 to occluders.count do
		mo = mo + (snapshotAsMesh occluders[i])

	for i = 1 to mo.numFaces do
	(
		f_n	= getFace mo i
		f_f = [f_n.z, f_n.y, f_n.x]
		setFace mo i f_f
	)
		
	-- boolean with A-B operator between occluders(A) and portals(B)
	for i = 1 to portals.count do (
		mp	= snapshotAsMesh portals[i]
		mp	= mp * mo

		_verts = #()
		_faces = #()

		for i = 1 to mp.numVerts do
			append _verts (getVert mp i)

		for i = 1 to mp.numFaces do
			append _faces (getFace mp i)
		
		np = mesh vertices: _verts faces: _faces--numVerts: _verts.count numFaces: _faces.count

		np.name = "TMP_" + portals[i].name
		--np.transform	= portals[i].transform
		mo = mo - (snapshotAsMesh portals[i])
	)

	for i = 1 to mo.numFaces do
	(
		f_n	= getFace mo i
		f_f = [f_n.z, f_n.y, f_n.x]
		setFace mo i f_f
	)

	_verts = #()
	_faces = #()

	for i = 1 to mo.numVerts do
		append _verts (getVert mo i)

	for i = 1 to mo.numFaces do
		append _faces (getFace mo i)
		
	areas_mesh = mesh vertices: _verts faces: _faces--numVerts: _verts.count numFaces: _faces.count

	areas_mesh.name	= "AREAS_MESH"
	convertToMesh areas_mesh
	return _str
)

fn export_all_areas_mesh f =
(

	format "BEGIN_AREAS_MESH\n" to:f
	export_mesh $areas_mesh f
	format "END_AREAS_MESH\n" to:f

	return _str	
)

rollout MapExporter " Map Exporter" width:162 height:159
(
	button btnExportMap "Export Map" pos:[8,8] width:144 height:16
	edittext edtExportLog "" pos:[8,32] width:144 height:112 enabled:true
	on btnExportMap pressed do
	(
		map_file_name = getSaveFileName caption:"Open A Test File:" filename:"map00.map" types:"Map(*.map)|*.map|All|*.*|"
		edtExportLog.text = edtExportLog.text + "Creating " + map_file_name + " ...\n"
		map_file = createFile map_file_name
		_str = build_all_areas_mesh()
		export_all_portals map_file
		export_all_occluders map_file
		export_all_areas_mesh map_file
		edtExportLog.text = edtExportLog.text + (_str)
		close map_file
		edtExportLog.text = edtExportLog.text + "DONE\n"
	)
)
createDialog MapExporter style:#(#style_toolwindow, #style_sysmenu)