/*************************************************************************
MGLApp.cpp

Main application class.

Copyright 2005(C) Wael El Oraiby based on MGLApp.cpp by Dave Astle

Written for More OpenGL Game Programming
*************************************************************************/

#ifdef WIN32
	#define WIN32_LEAN_AND_MEAN
	#define WIN32_EXTRA_LEAN
	#include <windows.h>
#endif

#include "MGLApp.h"

using namespace std;
using namespace SceneMap;

MGLApp::MGLApp() : mainWinWidth(0), mainWinHeight(0), mouseStartX(0), mouseStartY(0),
				viewAngle(-150.0f),	viewAngle2(30.0f), viewPos(0.0f, 0.0f, 0.0f),
				viewDir(0.0f, 1.0f, 0.0f), fontCourier(NULL), mouseMoving(false)
{
}

MGLApp::~MGLApp()
{
}

bool MGLApp::Init()
{	
	if (!GLeeInit())
		return false;

	if ( !GLEE_ARB_occlusion_query )
	{
		throw "your card or OpenGL driver doesn't support Occlusion queries\n" \
			"You could try updating your display driver to fix the problem...\n";
		return false;
	}

	fontCourier	= new Font("courier_rgba.tga");
	sceneMap.load("map01.map");

	sceneMap.setupVisSystem();
	sceneMap.issueVisQueries(640 / 2, 480 / 2, viewPos, vec3(0.0f, 0.0f, 0.0f), vec3(0.0f, 1.0f, 0.0f));

	return true;
}

bool MGLApp::Shutdown()
{
	delete[] fontCourier;

	return true;
}

void MGLApp::SetupProjection(int width, int height)
{
	float		ratio;
	if( height == 0 )
		height	= 1;

	mainWinWidth	= width;
	mainWinHeight	= height;
	ratio	= ((float)width)/((float)height);

	/* setup projection matrix */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glViewport(0, 0, width, height);
	glScissor(0, 0, width, height);

	gluPerspective(45.0f, ratio, 1.0f, 4.0f * sceneMap.getBoundingVolume().radius);

	/* setup model view matrix */
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	vec3	center	= sceneMap.getBoundingVolume().center;
	gluLookAt(0.0f, 3.0f * sceneMap.getBoundingVolume().radius, 0.0f,
		0.0f, 0.0f, 0.0f,
		//:center.x, center.y, center.z,
		0.0f, 0.0f, 1.0f);
}

void MGLApp::Update( double dt )
{
}

inline void glVertex(vec3 &pos) {
	glVertex3f(pos.x, pos.y, pos.z);
}

inline void glLine(vec3 &start, vec3 &end) {
	glBegin(GL_LINES); {
		glVertex(start);
		glVertex(end);
	} glEnd();
}


void drawPointer(float size, vec3 &pos, vec3 &dir)
{
	vec3	tmpPos	= pos + dir * size;

	glLine(pos, tmpPos);

	glPushMatrix();
	glTranslatef(tmpPos.x, tmpPos.y, tmpPos.z);
	//glutSolidSphere(size / 4, 8, 9);
	glPopMatrix();
}

/*
** render to a viewport
*/
void MGLApp::renderToViewport(int x, int y, int width, int height, vector<int> &visibleAreas)
{
	int				i, areaId;


	glViewport(x, y, width, height);
	glScissor(x, y, width, height);
	glEnable(GL_SCISSOR_TEST);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);


	// rotate
	glPushMatrix();
	{

		glRotatef(viewAngle2, 1.0f, 0.0f, 0.0f);
		glRotatef(viewAngle, 0.0f, 1.0f, 0.0f);

		// center the view
		glTranslatef(-sceneMap.getBoundingVolume().center.x,
			-sceneMap.getBoundingVolume().center.y,
			-sceneMap.getBoundingVolume().center.z);
		

		//renderBV(scn_map->getBoundingVolume());
		glPushMatrix();
		{
			vec3	pointerPos	= viewPos;// + sceneMap.getBoundingVolume().center;
			glColor3f(1.0f, 0.0f, 0.0f);
			glLineWidth(4.0f);
			drawPointer(20.0f, pointerPos, viewDir);
		} glPopMatrix();

		// render occluders
		glLineWidth(2.0f);

		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
		glColor3f(0.5f, 1.0f, 0.5f);
		for (int i = 0; i < sceneMap.getNumOccluders(); i++ )
			sceneMap.getOccluders()[i].mesh.glRender();

		glColor3f(0.0f, 0.0f, 1.0f);
		for (int i = 0; i < sceneMap.getNumPortals(); i++ )
			sceneMap.getPortals()[i].mesh.glRender();

		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
		for ( int i = 0; i < static_cast<int>(visibleAreas.size()); i++ ) {
			vec3	color	= sceneMap.getAreas()[visibleAreas[i]].color;
			glColor3f(color.x, color.y, color.z);
			sceneMap.getAreas()[visibleAreas[i]].glRender();
		}

		glDisable(GL_SCISSOR_TEST);
	}
	glPopMatrix();
}

void MGLApp::Render()
{
	stringstream	ss;

	glClearColor (0.5, 0.5, 0.5, 0.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);

		
	static vector<int>		visPortalsId, visAreasId;

	// render to the viewport the bird eye view
	renderToViewport(0, 0, mainWinWidth, mainWinHeight, visAreasId);

	sceneMap.getVisResults(visPortalsId, visAreasId);
	sceneMap.issueVisQueries(mainWinWidth / 4, mainWinHeight / 4, viewPos, viewDir, vec3(0.0f, 1.0f, 0.0f));
	
	// reset viewport
	glViewport(0, 0, mainWinWidth, mainWinHeight);
	glScissor(0, 0, mainWinWidth, mainWinHeight);
	glEnable(GL_SCISSOR_TEST);

	//
	// draw information about the level
	//
	stringstream	str;

	str << "Number of Areas : " << sceneMap.getNumAreas() << endl;
	str << "Number of Portals : " << sceneMap.getNumPortals() << endl;
	str << "Number of Occluders : " << sceneMap.getNumOccluders() << endl;
	str << "Current view pos : < " << viewPos.x << " , " << viewPos.y << " , " << viewPos.z << " >" << endl;
	str << "Current area : " << sceneMap.getPointArea(viewPos) << endl;

	str << "Visible Portals : ";
	for ( size_t i = 0; i < visPortalsId.size(); i++ ) {
		str << "/ " << visPortalsId[i];
	}
	str << endl << "Visible Areas : ";
	for ( size_t i = 0; i < visAreasId.size(); i++ ) {
		str << "/ " << visAreasId[i];
	}
	str << endl;

	textBegin(mainWinWidth, mainWinHeight);
	glColor3f(1.0f, 1.0f, 1.0f);
	fontCourier->setCursor(0.0f, 0.0f);
	fontCourier->setSize(16.0f);
	fontCourier->drawText(str.str().c_str());
	textEnd();
}

/*
** Name: motionFunc
** Desc: glut motion function
*/
void MGLApp::motionFunc(int x, int y)
{
  if ( mouseMoving ) {
    viewAngle		+= (x - mouseStartX);
    viewAngle2		+= (y - mouseStartY);
    mouseStartX		= x;
    mouseStartY		= y;
  }
}

/*
** Name: mouseFunc
** Desc: glut mouse function
*/
void MGLApp::mouseFunc(int button, int state, int x, int y)
{
  /* Rotate the scene with the left mouse button. */
  if (button == MGL_MOUSE_LEFT_BUTTON) {
    if (state == MGL_MOUSE_DOWN) {
      mouseMoving = true;
      mouseStartX = x;
      mouseStartY= y;
    }
    if (state == MGL_MOUSE_UP) {
      mouseMoving = false;
    }
  }
}

/*
** glut keyboard function
*/
void MGLApp::keybFunc(int key)
{
	vec3	rotVec	= vec3(0.0f, 1.0f, 0.0f);
	mat4	rotMat;

	switch (key)
	{
	case MGL_KEY_UP:
		viewPos += viewDir;
		break;
	case MGL_KEY_DOWN:
		viewPos -= viewDir;
		break;
	case MGL_KEY_LEFT:
		rotMat		= rotation3D(vec3(0.0f, 0.0f, 1.0f), -2.0f);
		viewDir	= viewDir * rotMat;
		break;
	case MGL_KEY_RIGHT:
		rotMat		= rotation3D(vec3(0.0f, 0.0f, 1.0f), 2.0f);
		viewDir	= viewDir * rotMat;
		break;
	}
}