/*************************************************************************
MGLApp.h

Main application class.

Author   :   Dave Astle

Written for More OpenGL Game Programming
*************************************************************************/

#ifndef MGLAPP_H_INCLUDED
#define MGLAPP_H_INCLUDED

#ifdef WIN32
  #define WIN32_LEAN_AND_MEAN
  #define WIN32_EXTRA_LEAN
  #include <windows.h>
#endif

#include "glee.h"
#include <gl/glu.h>

#include <vector>
#include <string>
#include <sstream>

#include "algebra3.h"
#include "geom.h"
#include "font.h"
#include "scene_map.h"

// values used to set up the window and rendering context
const char MGL_APP_NAME[]	= "MOGLGP - Chapter 12 - Scene Management";

const int MGL_WINDOW_WIDTH = 800;
const int MGL_WINDOW_HEIGHT = 600;

const int MGL_COLOR_BITS = 32;
const int MGL_DEPTH_BITS = 24;
const int MGL_STENCIL_BITS = 0;

enum
{
  MGL_FULLSCREEN_FALSE = 0,
  MGL_FULLSCREEN_TRUE = 1,
  MGL_FULLSCREEN_CHOOSE = 2
};

enum {
	MGL_MOUSE_LEFT_BUTTON,
	MGL_MOUSE_MIDDLE_BUTTON,
	MGL_MOUSE_RIGHT_BUTTON
};

enum {
	MGL_MOUSE_DOWN,
	MGL_MOUSE_UP
};

enum {
	MGL_KEY_UP,
	MGL_KEY_LEFT,
	MGL_KEY_DOWN,
	MGL_KEY_RIGHT
};

const int MGL_FULLSCREEN_MODE = MGL_FULLSCREEN_CHOOSE;


class MGLApp
{
public:
	MGLApp();
	virtual ~MGLApp();

	bool Init();
	bool Shutdown();
	void SetupProjection( int width, int height );
	void Update( double dt );
	void Render();
	
	void motionFunc(int x, int y);
	void mouseFunc(int button, int state, int x, int y);
	void keybFunc(int key);

private:

	void renderToViewport(int x, int y, int width, int height, std::vector<int> &visibleAreas);


	int				mainWinWidth;
	int				mainWinHeight;

	int				mouseStartX, mouseStartY;
	bool			mouseMoving;
	float			viewAngle;
	float			viewAngle2;

	vec3			viewPos;
	vec3			viewDir;

	Font*			fontCourier;
	SceneMap::Map	sceneMap;
};

#endif // MGLAPP_H_INCLUDED