/*
** geom.h
**
** Copyright (C) 2005 Wael EL ORAIBY
**
** geometry class
**
** Written for More OpenGL Game Programming
*/
#ifndef __GEOM__H__
#define __GEOM__H__

struct BoundingVolume {
	vec3	center;		// bounding volume center
	vec3	minV;		// bounding box min
	vec3	maxV;		// bounding box max
	float	radius;		// bounding sphere radius
	float	radius2;	// bounding sphere radius^2

	void	setup(int numVerts, vec3 *verts);
};

// 3D plane 
struct plane {
	union {
		float	p[4];
		struct {
			float	a, b, c, d;
		};

		struct {
			vec3	normal;
			float	dist;
		};
	};

	plane(): a(0.0f), b(0.0f), c(0.0f), d(0.0f){}
	plane(float _a, float _b, float _c, float _d): a(_a), b(_b), c(_c), d(_d)
	{}

	plane(float *_p): a(_p[0]), b(_p[1]), c(_p[2]), d(_p[3])
	{}

	plane(const vec3 &_normal, float _dist): normal(_normal), dist(_dist)
	{}

	plane(const vec3 &v0, const vec3 &v1, const vec3 &v2);
	
	void	normalize();
	float	signedDistance(const vec3 &v) const;
	float	distance(const vec3 &v) const;
	float	intersection(vec3 &int_point, const vec3 &start, const vec3 &end) const;
};

bool planes_are_equal(const plane &p0, const plane &p1);

// for maps when we have to find a plane
inline bool operator == ( const plane &p0, const plane &p1 )
{
	return planes_are_equal( p0, p1 );
}

extern float	plane_threshold;

// used for maps when planes are provided as keys (hash maps, etc...)
struct lessplane
{
	bool operator()( const plane &p0, const plane &p1 ) const;
};

#endif