/*************************************************************************/
/*|=====================================================================|*/
/*| Filename: CCamera.h                                                 |*/
/*| Author: Joel Villagrana  (joel.villagrana@gmail.com)                |*/
/*|                                                                     |*/
/*| Defined Classes:                                                    |*/
/*| CCamera                                                             |*/
/*|                                                                     |*/
/*|%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%|*/
/*| dd/mm/yy   | Programmer        | Modifications                      |*/ 
/*|=====================================================================|*/
/*| 11/09/04   | JVillagrana       | Initial version                    |*/
/*************************************************************************/

#ifndef _CAMERA_H
#define _CAMERA_H

#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <stdlib.h>
#include <math.h>
#include <glee/glee.h>
#include <gl/gl.h>
#include <gl/glu.h>

#include "../Common/MGLVector.h"
#include "../Math/helper_math.h"
#include "../Util/helper_defs.h"
#include "../Texture/textureManager.h"

/*$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
===================================================================================================
							DEFINITION OF THE CCamera CLASS
===================================================================================================
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$*/
class CCamera
{
private:

	// GLOBAL POINTER TO THE CAMERA
	static  CCamera	*m_pCamera;

	// VECTORS USED BY THE CAMERA
	MGL::Vector3	m_currentWorldPos;
	MGL::Vector3	m_lastWorldPos;
	MGL::Vector3	m_directionVector;
	MGL::Vector3	m_initialDirV;
	MGL::Vector3	m_newDirVector;
	MGL::Vector3	m_lookAtVector;		
	MGL::Vector3	m_upVector;
	MGL::Vector3	m_rightVector;

	// THE QUATERNION
	GLQuaternion	m_quaternion;

	// VIRTUAL OBSERVER REL POS
	float			m_virtualObs;

	// THE PITCH, YAW, AND ROLL ANGLES IN DEGREES AND RADS
	float			m_pitch, m_pitchRad,
					m_yaw, m_yawRad,
					m_roll, m_rollRad,
					m_Speed;

public:

	// CONSTRUCTORS AND DESTRUCTOR
	CCamera(){ InitAll(); }
	CCamera( MGL::Vector3 eye, float yaw, float pitch, float roll );
	~CCamera(){}
	
	// INITIALIZATION AND DE-INITIALIZATION
	bool	InitAll();
	void	DestroyAll(){}

	// ROTATE USING THE CURRENT INFO ON THE QUATERNION
	void	UpdateRotation();
	// UPDATE CAMERA EVERY FRAME
	void	Tick();
	void	UpdateCurrentWorldPos();

	// ZOOM, PAN CAMERA 
	void	MouseMove(bool left, bool right, float deltaX, float deltaY);
	void	ZoomCamera(float amtZ);
	void	PanCamera(MGL::Vector3 newMove);

	// YAW, PITCH, ROLL
	void	Set_YawPitchRoll( float yaw, float pitch, float roll );
	void	Get_YawPitchRoll( float *yaw, float *pitch, float *roll );

	// MODIFY THE SPEED OF THE CAMERA
	void	Set_Speed(float s)	{					m_Speed = s;			}
	void	UpdateSpeed(float s){	if((m_Speed+s)>0.0f)m_Speed += s;		}

	// GET THE CAMERA VECTORS
	MGL::Vector3 Get_currPos()		{	return	m_currentWorldPos;			}
	MGL::Vector3 Get_dirVector()	{	return	m_directionVector;			}
	MGL::Vector3 Get_upVector()		{	return	m_upVector;					}
	MGL::Vector3 Get_lookAtVector()	{	return	m_lookAtVector;				}

	// SET THE CAMERA VECTORS
	void	Set_currPos(MGL::Vector3 pos)	{	m_currentWorldPos= pos;		}	
	void	Set_dirVector(MGL::Vector3 dir)	{	m_directionVector= dir;		}
	void	Set_initialDirV(MGL::Vector3 dir){	m_initialDirV	= dir;		}
	void	Set_upVector(MGL::Vector3 upV)	{	m_upVector		= upV;		}
	void	Set_lookAtV(MGL::Vector3 lookAt){	m_lookAtVector	= lookAt;	}
	void	Set_rightVec(MGL::Vector3 right){	m_rightVector	= right;	}

	//  STATIC METHODS TO SET/GET THE GLOBAL POINTER 
	static CCamera* GetCamera()			{ return m_pCamera;	}
	static void SetCamera(CCamera* cam)	{ m_pCamera = cam;	} 
};
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ 

inline CCamera* _Camera()
{
	assert(CCamera::GetCamera() && "Error generated in class definition file, CCamera* _Camera()");
	return CCamera::GetCamera();
}
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ 


#endif _CAMERA_H

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% END OF FILE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%





