/*************************************************************************
MGLApp.h

Main application class.

Author   :   Dave Astle

Written for More OpenGL Game Programming
-------------------------------------------------------------------------
| dd/mm/yyyy  |  Programmer        | Explanation of change              |
-------------------------------------------------------------------------
| 09/10/2005  |  J.Villagrana   @1 | -Changed MGL_APP_NAME and added    | 
|             |                    |  some other stuff       			|
*************************************************************************/

#ifndef MGLAPP_H_INCLUDED
#define MGLAPP_H_INCLUDED

#ifdef WIN32
  #define WIN32_LEAN_AND_MEAN
  #define WIN32_EXTRA_LEAN
  #include <windows.h>
#endif

//#include <gl/glee.h> @1D
#include <GLee/glee.h> //@1A
#include <gl/glu.h>

#include "./Common/MGLTimer.h"						//@1A
#include "./Text/GLFont.h"							//@1A

// values used to set up the window and rendering context
const char MGL_APP_NAME[]	= "MOGLGP - Chapter 12 - Scene Management - Octrees - J.Villagrana"; //@1C

const int MGL_WINDOW_WIDTH = 800;
const int MGL_WINDOW_HEIGHT = 600;

const int MGL_COLOR_BITS = 32;
const int MGL_DEPTH_BITS = 32;
const int MGL_STENCIL_BITS = 0;

const bool MGL_FORCE_FULLSCREEN = false; //@1C - changed to false

const char SCENE_FILENAME[] = "Data/Scene3.obj"; //@1A

class MGLApp
{
public:
	MGLApp();
	virtual ~MGLApp();

	bool Init();
	bool Shutdown();
	void SetupProjection( int width, int height );
	void Update( double dt );
	void Render();
	void Switch_ShowInfo(){	m_showInfo = !m_showInfo;	}		//@1A
	void Switch_RenderNodes(){m_renderNodes = !m_renderNodes;}	//@1A
	void Switch_RenderGeometry(){m_renderGeometry = !m_renderGeometry;}	//@1A
	void Switch_RenderSplit(){m_renderSplitOnly = !m_renderSplitOnly;}	//@1A
	void Switch_RenderWireframe(){m_renderWireframe = !m_renderWireframe; }	//@1A

private:
	void Draw2DBegin();											//@1A
	void Draw2DEnd();											//@1A
	void RenderInfo(int visibleNodes, int totalNodes,			//@1A
			int trianglesRendered, int totalTriangles, int splitTriangles);	//@1A

	MGL::Timer			m_timer;								//@1A
	GLFont				m_font;									//@1A
	float				m_lastTimeFps;							//@1A
	float				m_lastFps;								//@1A
	bool				m_showInfo;								//@1A
	bool				m_renderNodes;							//@1A
	bool				m_renderGeometry;						//@1A
	bool				m_renderSplitOnly;						//@1A
	bool				m_renderWireframe;						//@1A
};

#endif // MGLAPP_H_INCLUDED

