/*************************************************************************/
/*|=====================================================================|*/
/*| Filename: helper_math.h                                             |*/
/*| Author: Joel Villagrana  (joel.villagrana@gmail.com)                |*/
/*|                                                                     |*/
/*| Classes Defined:                                                    |*/
/*| -GLLineSeg2D		                                                |*/
/*| -GLVector2                                                          |*/
/*| -GLVector3                                                          |*/
/*| -GLVector4                                                          |*/
/*| -GLPlane                                                            |*/
/*| -GLMatrix44                                                         |*/
/*| -GLBoundingBox                                                      |*/
/*| -GLQuaternion                                                       |*/
/*| -GLBoundingSphere                                                   |*/
/*| -GLRay                                                              |*/
/*| -GLPoint2D                                                          |*/
/*| -GLFrustum                                                          |*/
/*| -GLTriangle                                                         |*/
/*|                                                                     |*/
/*|%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%|*/
/*| dd/mm/yy   | Programmer        | Modifications                      |*/ 
/*|=====================================================================|*/
/*| 14/03/04   | JVillagrana       | Initial Version                    |*/
/*|---------------------------------------------------------------------|*/
/*| 24/03/04   | JVillagrana   @1  | - Added class GLTriangle for BSP   |*/
/*|            |                   |   purposes                         |*/
/*************************************************************************/

#ifndef _MATHHELPER_H
#define _MATHHELPER_H

#include <stdio.h>
#include <math.h>
#include <assert.h>

#include "../Util/helper_defs.h"

#define XPI			 3.14159265359
#define PI_2		 6.28318530718
#define PI_OVER_2	 1.57079632679
#define PI_OVER_180  0.01745329251
#define PI_UNDER_180 57.2957795131
#define EPSILON		 0.003125

/* $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
================================================================================
	Definition of a Line Class
================================================================================
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ */
class GLLineSeg2D
{
private:
	GLfloat m_origX;
	GLfloat m_origY;
	GLfloat m_destX;
	GLfloat m_destY;
	GLfloat m_slope;

public:
	GLLineSeg2D();
	GLLineSeg2D(GLfloat ox, GLfloat oy, GLfloat dx, GLfloat dy);
	~GLLineSeg2D();

	VOID InitAll();

	bool IntersectsLine(GLLineSeg2D &seg2);
};
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ 


/* $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
================================================================================
	Definition of a Vector Class in 2D
================================================================================
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ */
class GLVector2 
{
public:
		GLfloat x, y;
		GLfloat vecxy[2];

		GLVector2();
		GLVector2( GLfloat _x, GLfloat _y );
		GLVector2( GLVector2& v );
		~GLVector2();

		GLVector2	operator+(GLVector2& v);
		VOID		operator+=(GLVector2& v);
		GLVector2	operator-(GLVector2& v);
		VOID		operator-=(GLVector2& v);
		GLVector2	operator-(GLfloat c);
		VOID		operator=(GLVector2& v);
		GLVector2	operator*(GLfloat c);
		VOID		operator*=(GLfloat c);
		BOOL		operator==(GLVector2& v2);
		BOOL		operator!=(GLVector2& v2);
		GLVector2	operator/(GLfloat c);
		VOID		operator/=(GLfloat c);

		VOID		setValues( GLfloat _x, GLfloat _y );
		VOID		normalize();
		GLfloat		length(){	return (GLfloat)(sqrt((x*x)+(y*y)));	}
};
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ 

class GLVector4;

/* $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
================================================================================
	Definition of a Vector Class in 3D
================================================================================
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ */
class GLVector3 
{
public:
		GLfloat	x, y, z;

		GLVector3();
		GLVector3( GLfloat _x, GLfloat _y, GLfloat _z );
		GLVector3( GLVector3& v );
		~GLVector3();

		GLVector3	operator+(GLVector3& v);
		VOID		operator+=(GLVector3& v);
		VOID		operator+=(GLVector4& v);
		GLVector3	operator-(GLVector3& v);
		VOID		operator-=(GLVector3& v);
		VOID		operator-=(GLVector4& v);
		GLVector3	operator-(GLfloat c);
		VOID		operator=(GLVector3& v);
		GLVector3	operator*(GLfloat c);	
		VOID		operator*=(GLfloat c);
		BOOL		operator==(GLVector3& v2);
		BOOL		operator!=(GLVector3& v2);
		GLVector3	operator/(GLfloat c);
		VOID		operator/=(GLfloat c);

		VOID		setValues( GLfloat _x, GLfloat _y, GLfloat _z );
		VOID		setValues( GLVector3 &v2 );
		VOID		getValues( GLfloat *_x, GLfloat *_y, GLfloat *_z);
		GLVector3	crossProduct(GLVector3& v2);
		GLfloat		dotProduct(GLVector3& v2);
		GLfloat		distanceToVertex(GLVector3 &v2);
		VOID		normalize();
		GLfloat		length(){	return (GLfloat)(sqrt((x*x)+(y*y)+(z*z)));	}
		GLVector3	PlaneLineIntersection(GLVector3 n, float d, GLVector3 p2);
};
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ 


/* $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
================================================================================
	Definition of a Vector Class with 4 Components
================================================================================
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ */
class GLVector4
{
public:
		GLfloat x, y, z, n;
		
		GLVector4();
		GLVector4( GLfloat _x, GLfloat _y, GLfloat _z, GLfloat _n );
		GLVector4( GLfloat _x, GLfloat _y, GLfloat _z );
		GLVector4( GLVector4& v );
		~GLVector4();

		GLVector4	operator+(GLVector4& v);
		VOID		operator+=(GLVector4& v);
		GLVector4	operator+(GLVector3& v);
		VOID		operator+=(GLVector3& v);
		GLVector4	operator-(GLVector4& v);
		VOID		operator-=(GLVector4& v);
		GLVector4	operator-(GLfloat c);
		VOID		operator=(GLVector4& v);
		VOID		operator=(GLVector3& v);
		GLVector4	operator*(GLfloat c);
		VOID		operator*=(GLfloat c);
		BOOL		operator==(GLVector4& v2);
		BOOL		operator!=(GLVector4& v2);
		GLVector4	operator/(GLfloat c);
		VOID		operator/=(GLfloat c);

		VOID		setValues( GLfloat _x, GLfloat _y, GLfloat _z );
		VOID		setValues( GLVector4 &v2 );
		GLfloat		dotProduct(GLVector4& v2);
		GLVector4	crossProduct(GLVector4& v2);
		VOID		resetValuesToZero();
		GLfloat		angle(GLVector4& v2);
		VOID		normalize();
		BOOL		isZero(){	return ((x==0.0f)&&(y==0.0f)&&(z==0.0f));	}
		GLfloat		length(){	return (GLfloat)(sqrt((x*x)+(y*y)+(z*z)));	}
};
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ 

class GLBoundingBox;
class GLTriangle;

/* $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
================================================================================
	Definition of a Plane Class
================================================================================
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ */
class GLPlane
{
private:
	GLfloat		m_A, m_B, m_C, m_D;
	GLVector4	m_pointOnPlane;
public:
	GLPlane();
	GLPlane(GLfloat A, GLfloat B, GLfloat C, GLfloat D);
	GLPlane(GLfloat x[3], GLfloat y[3], GLfloat z[3]);
	~GLPlane();

	VOID		Set_A(GLfloat A){			m_A = A;				}
	GLfloat		Get_A()			{	return	m_A;					}
	VOID		Set_B(GLfloat B){			m_B = B;				}
	GLfloat		Get_B()			{	return	m_B;					}
	VOID		Set_C(GLfloat C){			m_C = C;				}
	GLfloat		Get_C()			{	return	m_C;					}
	VOID		Set_D(GLfloat D){			m_D = D;				}
	GLfloat		Get_D()			{	return	m_D;					}
	GLVector4	GetNormal()		{	return GLVector4(m_A, m_B, m_C);}

	VOID		SetCoefficientsFromPoints(GLfloat x[3], GLfloat y[3], GLfloat z[3]);

	VOID		DebugMe();
	VOID		Set_Coefficients(GLfloat A, GLfloat B, GLfloat C, GLfloat D);
	VOID		NormalizeGLPlane();
	HALFSPACE	ClassifyPoint(GLVector4 pt);
	GLfloat		SignedDistanceToPoint(GLVector4 pt);
	BOOL		AABBPointsInsidePlane(GLBoundingBox aabb);
	BOOL		PointInsidePlane(GLVector4 &point);
	BOOL		LineIntersectsPlane(GLVector4 &startPoint, GLVector4 &endPoint);
	VOID		operator=(GLPlane& p2);
	VOID		GetPointOnPlane(GLVector4 *point)	{ *point = m_pointOnPlane;	}
	VOID		SetPointOnPlane(GLVector4 point)	{ m_pointOnPlane = point;	}
	
	// Checks if a triangle is partitioned by a plane, and if it's, return the resulting 
	// triangles partitioned
	bool		PartitionTriWithPlane(GLTriangle tri, GLTriangle* partTris[3], 
												int *numPart, HALFSPACE hf[3]);

};
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ 

/* $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
================================================================================
	Definition of a 4x4 Matrix Class, OpenGL Style ([16] instead of [4][4])
================================================================================
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ */
class GLMatrix44
{
	public:
		GLfloat elem[16];

		GLMatrix44();
		GLMatrix44( GLfloat a1, GLfloat a2, GLfloat a3, GLfloat a4,
					GLfloat a5, GLfloat a6, GLfloat a7, GLfloat a8, 
					GLfloat a9, GLfloat a10, GLfloat a11, GLfloat a12, 
					GLfloat a13, GLfloat a14, GLfloat a15, GLfloat a16);
		~GLMatrix44();

		VOID Op_Identity();
		VOID		Set(GLfloat *e);
		VOID		Transpose(GLMatrix44 *destMatrix);

		GLMatrix44 operator+(GLMatrix44&);
		GLMatrix44 operator-(GLMatrix44&);
		GLVector4  operator*(GLVector4&);
		GLVector3  operator*(GLVector3&);
		VOID	   operator*=(GLMatrix44& m2);
		VOID	   operator[](int i);
		VOID	   operator=(GLMatrix44 mt);
};
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ 

/* $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
================================================================================
	Definition of a Bounding Box Class
================================================================================
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ */
class GLBoundingBox
{
private:
	GLfloat		m_magX, m_magY, m_magZ;
	GLVector4	m_center, m_c1, m_c2, m_c3, m_c4, m_c5, m_c6, m_c7, m_c8;
	GLVector4	m_trcenter, m_trc1, m_trc2, m_trc3, m_trc4;
	GLVector4	m_trc5, m_trc6, m_trc7, m_trc8;
	GLPlane		m_planes[6];

public:
	GLBoundingBox();
	GLBoundingBox(GLVector4& v, GLfloat mag);
	GLBoundingBox(GLVector4& v, GLfloat magX, GLfloat magY, GLfloat magZ);
	~GLBoundingBox();

	VOID			Clear();
	HRESULT			InitAll(GLVector4 &v, GLfloat mag);
	HRESULT			InitAll(GLVector3 &v, GLfloat mag);
	HRESULT			InitAll(float x, float y, float z, GLfloat mag);
	HRESULT			InitAll(GLVector4 &v, GLfloat magX, GLfloat magY, GLfloat magZ);
	BOOL			CollidesBB(GLBoundingBox& bb2);
	VOID			MoveBB(GLVector4& addVector);
	VOID			MoveBB_PointsOnly(GLVector4& addVector);
	VOID			UpdateSizeBB();
	VOID			ComputePlanes();
	BOOL			PointInsideBBPlanes(GLVector4& point);
	BOOL			PointIntersectsBB(GLVector4 pt, int XZ, int YZ, int XY);
	BOOL			PointIntersectsFace(GLVector4, GLVector4 v0, GLVector4 v1, GLVector4 v2, 
																		int XZ, int YZ, int XY);
	int				GetClosestSideToPoint(GLVector4& point);
	VOID			UpdateCornerPointsWithMatrix(GLMatrix44& matrix);
	VOID			UpdateWithMatrix_Acc(GLMatrix44& matrix);

	VOID			Set_Center(GLVector4& v){			m_center=v; m_trcenter=v;	}
	GLVector4		Get_Center()			{	return	m_trcenter;					}
	VOID			Set_magX(GLfloat mx)	{			m_magX = mx;				}
	GLfloat			Get_magX()				{	return	m_magX;						}
	VOID			Set_magY(GLfloat my)	{			m_magY = my;				}
	GLfloat			Get_magY()				{	return	m_magY;						}
	VOID			Set_magZ(GLfloat mz)	{			m_magZ = mz;				}
	GLfloat			Get_magZ()				{	return	m_magZ;						}

	GLVector4		Get_point1()			{	return	m_trc1;						}
	GLVector4		Get_point2()			{	return	m_trc2;						}
	GLVector4		Get_point3()			{	return	m_trc3;						}
	GLVector4		Get_point4()			{	return	m_trc4;						}
	GLVector4		Get_point5()			{	return	m_trc5;						}
	GLVector4		Get_point6()			{	return	m_trc6;						}
	GLVector4		Get_point7()			{	return	m_trc7;						}
	GLVector4		Get_point8()			{	return	m_trc8;						}
	VOID			GetAllPoints(GLVector4 *p1, GLVector4 *p2, GLVector4 *p3, 
								 GLVector4 *p4, GLVector4 *p5, GLVector4 *p6,
								 GLVector4 *p7, GLVector4 *p8);

	VOID			operator=(GLBoundingBox& b2);
};
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ 

/* $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
================================================================================
	Definition of a Quaternion Class
================================================================================
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ */
class GLQuaternion
{
	public:
		GLfloat w, x, y, z;
		
		GLQuaternion();
		GLQuaternion( GLfloat _w, GLfloat _x, GLfloat _y, GLfloat _z );
		GLQuaternion( GLMatrix44& glM );
		GLQuaternion( GLQuaternion& q2 );
		GLQuaternion( GLVector4& v, GLfloat angle );
		GLQuaternion( GLfloat pitch, GLfloat yaw, GLfloat roll );

		~GLQuaternion();

		VOID			Reset();

		GLQuaternion	operator+(GLQuaternion& q);
		VOID			operator=(GLQuaternion& q);
		GLQuaternion	operator/(GLfloat c);
		GLQuaternion	operator*(GLQuaternion& q);
		GLQuaternion	operator*(GLfloat s);
		VOID			operator*=(GLQuaternion& q);
		VOID			operator*=(GLfloat s);
		VOID			operator+=(GLQuaternion& q);
		VOID			operator-=(GLQuaternion& q);

		GLQuaternion	Get_Conjugate();
		GLQuaternion	Get_Inverse();
		VOID			Negate();

		GLQuaternion	Get_Interpolate(GLQuaternion& b, GLfloat b_amt);
		GLMatrix44		Get_MatrixFromQuaternion();
		GLVector4		Get_AxisOfRotation();
		GLfloat			Get_AngleRadOfRotation();
		GLVector4		Get_DirectionVector();

		GLVector4		Get_xDir(){return GLVector4(1-2*(y*y+z*z),  2*(x*y+w*z),  2*(x*z-w*y));}
		GLVector4		Get_yDir(){return GLVector4(  2*(x*y-w*z),1-2*(x*x+z*z),  2*(y*z+w*x));}
		GLVector4		Get_zDir(){return GLVector4(  2*(x*z+w*y),  2*(y*z-w*x),1-2*(x*x+y*y));}
		//matrix		getmatrix(){return matrix(xdir(),ydir(),zdir());}

		VOID			Normalize();
		GLfloat			Get_Norm(){		return (GLfloat)((w*w)+(x*x)+(y*y)+(z*z));	 }
		VOID			Set_Euler(GLfloat pitch, GLfloat yaw, GLfloat roll);
		VOID			Set_Values(GLfloat _w, GLfloat _x, GLfloat _y, GLfloat _z);

		GLfloat			Dot_Product(GLQuaternion& q2);

		friend GLQuaternion	operator*(GLVector4& v, GLQuaternion& q)
		{
			GLQuaternion t(v, 0);
			return (t*q);
		}

};
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ 

/* $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
================================================================================
	Definition of a Bounding Sphere Class
================================================================================
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ */
class GLBoundingSphere
{
private:
	GLVector4	m_center;
	GLfloat		m_radius, m_radiusSq;

public:
	GLBoundingSphere();
	GLBoundingSphere(GLVector4& center);
	GLBoundingSphere(GLVector4& center, GLfloat rad);
	~GLBoundingSphere();

	VOID		InitAll(GLVector4& v1, GLVector4& v2, GLVector4& v3);
	VOID		Clear();
	BOOL		CollidesBS(GLBoundingSphere& bs2);
	BOOL		CollidesBB(GLBoundingBox &bb2);

	VOID		operator=(GLBoundingSphere& b);

	VOID		Set_Center(GLVector4& v){			m_center = v;					}
	GLVector4	Get_Center(){				return	m_center;						}
	VOID		Set_Radius(GLfloat r){				m_radius = r;					}
	GLfloat		Get_Radius(){				return	m_radius;						}
	VOID		Set_RadiusSq(){						m_radiusSq=m_radius*m_radius;	}
	GLfloat		Get_RadiusSq(){				return	m_radiusSq;						}

//	VOID		FindBestRadius(VERTEX3D polygons[], int numPolygons);
};
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ 

/* $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
================================================================================
	Definition of a Ray Class
================================================================================
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ */
class GLRay
{
private:
	GLVector4	m_origin, 
				m_end, 
				m_direction;
	GLfloat		m_distance;

public:
	GLRay();
	GLRay(GLVector4& origPt, GLVector4& dirVec);
	GLRay(GLVector4& origPt, GLVector4& dirVec, GLfloat dist);
	~GLRay();

	VOID		Set_Origin(GLVector4& v){			m_origin = v;				}
	GLVector4	Get_Origin(){				return	m_origin;					}
	VOID		Set_Direction(GLVector4& v){		m_direction = v;			}
	GLVector4	Get_Direction(){			return	m_direction;				}
	VOID		Set_Distance(GLfloat d){			m_distance = d;				}
	GLfloat		Get_Distance(){				return	m_distance;					}
	VOID		Set_End(GLVector4& v){				m_end = v;					}
	GLVector4	Get_End(){					return	m_end;						}

	BOOL		IntersectsSphere(GLBoundingSphere& sphere);
};
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ 


/* $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
================================================================================
	Definition of a 2DPoint Class
================================================================================
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ */
class GLPoint2D
{
private:
	GLfloat m_X;
	GLfloat m_Y;

public:
	GLPoint2D();
	GLPoint2D(GLfloat x, GLfloat y);
	~GLPoint2D();

	VOID InitAll(GLfloat x, GLfloat y);
	GLfloat	Get_X(){	return m_X;		}
	GLfloat Get_Y(){	return m_Y;		}
};
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ 


/* $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
================================================================================
	Definition of a Frustum Class
================================================================================
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ */
class GLFrustum
{
private:
	GLPlane		m_topPlane, m_bottomPlane, 
				m_rightPlane, m_leftPlane,
				m_nearPlane, m_farPlane;

public:
	GLFrustum();
	~GLFrustum();

	VOID	Set_topPlane(GLPlane topP){				m_topPlane = topP;		}
	GLPlane	Get_topPlane(){					return	m_topPlane;				}
	VOID	Set_bottomPlane(GLPlane bottomP){		m_bottomPlane = bottomP;}
	GLPlane	Get_bottomPlane(){				return	m_bottomPlane;			}
	VOID	Set_rightPlane(GLPlane rightP){			m_rightPlane = rightP;	}
	GLPlane	Get_rightPlane(){				return	m_rightPlane;			}
	VOID	Set_leftPlane(GLPlane leftP){			m_leftPlane = leftP;	}
	GLPlane	Get_leftPlane(){				return	m_leftPlane;			}
	VOID	Set_nearPlane(GLPlane nearP){			m_nearPlane = nearP;	}
	GLPlane	Get_nearPlane(){				return	m_nearPlane;			}
	VOID	Set_farPlane(GLPlane farP){				m_farPlane = farP;		}
	GLPlane	Get_farPlane(){					return	m_farPlane;				}

	VOID	Update(GLMatrix44 modelViewMatrix, GLMatrix44 projectionMatrix);
	BOOL	AABBinFrustum(GLBoundingBox aabb);
	BOOL	BSinFrustum(GLBoundingSphere bs);
	BOOL	PointInFrustum(GLfloat x, GLfloat y, GLfloat z);
};
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ 


/* $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
================================================================================
	Definition of a Triangle Class - @1A BEGINS
================================================================================
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ */
class GLTriangle
{
private:
	GLVector4		m_vertex1;
	GLVector4		m_vertex2;
	GLVector4		m_vertex3;

	GLVector4		m_normal;

	GLPlane			m_triPlane;

public:
	GLTriangle();
	GLTriangle( GLVector4 v1, GLVector4 v2, GLVector4 v3 );
	GLTriangle( float x[3], float y[3], float z[3] );				
	~GLTriangle();
	
	// CLASSIFY tri2 AGAINST THIS TRIANGLE
	POLYGON_SIDE	CalculateSide(GLTriangle tri2);

	// CLASSIFY A 3D POINT AGAINST THIS TRIANGLE
	POLYGON_SIDE	Point_CalculateSide(GLVector4 point);

	// SET ALL VERTICES TO 0
	VOID			Set_0_Values();

	// COMPUTE TRIANGLE PLANE
	VOID			ComputePlane();

	// COMPUTE TRIANGLE NORMAL
	VOID			ComputeNormal();

	// ASSIGN OPERATOR 
	VOID			operator=(GLTriangle& tri2);

	// SETTER AND GETTER METHODS
	VOID			Get_Vertices(GLVector4 *v1, GLVector4 *v2, GLVector4 *v3)
					{	*v1 = m_vertex1;	*v2 = m_vertex2;	*v3 = m_vertex3;}

	VOID			Get_Vertices(float x[3], float y[3], float z[3]);

	VOID			Set_Vertices(GLVector4 v1, GLVector4 v2, GLVector4 v3)
					{	m_vertex1 = v1; m_vertex2 = v2; m_vertex3 = v3;			}

	VOID			Set_Vertices(float x[3], float y[3], float z[3]);

//	VOID			Set_Triangle(GLTriangle &tri2)
//					{ tri2.Get_Vertices(&m_vertex1, &m_vertex2, &m_vertex3);	}

	VOID			Get_Triangle(GLTriangle *tri2)
					{	tri2->Set_Vertices(m_vertex1, m_vertex2, m_vertex3);	}	

//	GLTriangle		Get_Triangle()
//					{ return GLTriangle(m_vertex1, m_vertex2, m_vertex3);		}

	VOID			Get_Normal(GLVector4 *n)
					{	n->x = m_normal.x; n->y = m_normal.y; n->z = m_normal.z;}

	GLPlane			Get_Plane()	{	return	m_triPlane;							}

};
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$ @1A ENDS $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ 



#endif _MATHHELPER_H

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% END OF FILE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%





