/*************************************************************************/
/*|=====================================================================|*/
/*| Filename: Scene.h                                                   |*/
/*| Author: Joel Villagrana   (joel.villagrana@gmail.com)               |*/
/*|                                                                     |*/
/*| Classes Defined:                                                    |*/
/*| CScene                                                              |*/
/*|                                                                     |*/
/*|%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%|*/
/*| dd/mm/yy   | Programmer        | Modifications                      |*/ 
/*|=====================================================================|*/
/*| 11/03/04   | JVillagrana       | -Initial Version                   |*/
/*************************************************************************/

#ifndef _SCENE_H
#define _SCENE_H

#include <windows.h>
#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <stdlib.h>
#include <glee/glee.h>
#include <gl/gl.h> 
#include <gl/glu.h>

#include <vector>
using namespace std;

#include "../Octree/Octree.h"

#include "../Util/helper_defs.h"
#include "../Common/MGLVector.h"
#include "../Math/helper_math.h"
#include "../Texture/textureManager.h"

#define MAX_TRIS_IN_OCTREE_NODE  250

/*$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
================================================================================
					DEFINITION OF THE CScene CLASS			
================================================================================
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$*/

class CScene
{

private:

	// GLOBAL POINTER TO THE SCENE
	static  CScene	*m_pScene;

	// RAW MESH DATA... SHOULD BE STORED IN ITS OWN CLASS THOUGH...
	int		*m_indices, *m_newIndices, *m_allIndices, m_numIndices, m_numNewIndices;
	int		m_numVertices, m_numNewVertices, *m_visibleIndices;
	MGL::Vector3 *m_vertices, *m_newVertices, *m_allVertices;
	MGL::Vector3 *m_vertexNormals;

	// OCTREE
	Octree		m_Octree;	
	bool		m_OctreeBuilt; 
	GLFrustum	m_viewFrustum;

public:

	CScene(){}
	~CScene(){}

	// INIT AND CLEANUP
	bool		InitAll();
	void		DestroyAll();

	// RENDER SCENE
	void		RenderScene(int *visibleNodes, int *totalNodes,	int *trianglesRendered, 
							int *totalTriangles, bool renderNodes, int *splitTriangles,
							bool renderGeometry, bool renderSplitOnly, bool renderWireframe);

	// LOAD AN OBJ MESH
	bool		LoadOBJMesh(const char filename[]);

	void		MakeGridList();	

	bool		BuildOctreeForScene();	
	void		CalculateNormalsForMesh();
	void		CalculateNormalForFace(int i, int j, MGL::Vector3 *faceNormals);
	void		CalculateNormalForVertex(int i, MGL::Vector3 *faceNormals);
	int			GetNumberOfFacesSharingVertex(int vertexIndex);
	int			GetFaceIndicesSharingVertex(int vertexIndex, int *indices, int maxF);
	bool		OctreeCreated(){	return m_OctreeBuilt;	} 
	void		Set_OctreeRenderEmptyAABB(bool r){m_Octree.Set_renderEmptyAABB(r);		}
	bool		Get_OctreeRenderEmptyAABB(){	return m_Octree.Get_renderEmptyAABB();	}	

	//  STATIC METHODS TO SET/GET THE GLOBAL POINTER 
	static CScene* GetScene()			{ return m_pScene;	}
	static void SetScene(CScene* sc)	{ m_pScene = sc;	} 
};
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ 

inline CScene* _Scene()
{
	assert(CScene::GetScene() && "Error generated in class definition file, CScene* _Scene()");
	return CScene::GetScene();
}
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ 


#endif _SCENE_H


// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% END OF FILE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

