/*************************************************************************/
/*|=====================================================================|*/
/*| Filename: GLFont.cpp												|*/
/*| Author: Joel Villagrana    (joel.villagrana@gmail.com)              |*/
/*|                                                                     |*/
/*| Classes Implementer:                                                |*/
/*| GLFont																|*/
/*| (mostly font stuff taken from NeHe)                                 |*/
/*|%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%|*/
/*| dd/mm/yy   | Programmer        | Modifications                      |*/ 
/*|=====================================================================|*/
/*| 11/03/04   | JVillagrana       | Initial Version                    |*/
/*************************************************************************/

#include <windows.h>
#include "GLFont.h"

GLFont::GLFont()
{
	InitAll();
}

GLFont::~GLFont()
{
	DestroyAll();
}

HRESULT GLFont::InitAll()
{
	m_fontList = 0;
	m_textureID = -1;

	return S_OK;
}

HRESULT GLFont::DestroyAll()
{
	glDeleteLists(m_fontList, 95);

	return S_OK;
}


bool GLFont::CreateGLFont(char *texturePath)
{
	int loop;

	m_textureID = _TextureManager()->LoadTexture(texturePath, GL_LINEAR);

	if(m_textureID >= 0)
	{
		// ENABLE TEXTURE MAPPING
		glEnable(GL_TEXTURE_2D);

		// GENERATE LIST IDS
		m_fontList = glGenLists(95);
		assert(m_fontList!=0 && "Could not create bitmap font list");

		// BIND TEXTURE
		glBindTexture(GL_TEXTURE_2D, _TextureManager()->Get_texture(m_textureID));

		// BUILD 95 CHARACTERS
		for(loop=0; loop<95; ++loop)
		{
			GLfloat cx = (GLfloat)((loop%16)/16.0f);
			GLfloat cy = (GLfloat)((loop/16)/8.0f);

			glNewList(m_fontList+loop, GL_COMPILE);
				glBegin(GL_QUADS);
					glTexCoord2f(cx,         1.0f-cy-0.120f); glVertex2i(0, 0);	// Texture / Vertex Coord (Bottom Left)
					glTexCoord2f(cx+0.0625f, 1.0f-cy-0.120f); glVertex2i(11,0);	// Texutre / Vertex Coord (Bottom Right)
					glTexCoord2f(cx+0.0625f, 1.0f-cy);		  glVertex2i(11,11);// Texture / Vertex Coord (Top Right)
					glTexCoord2f(cx,         1.0f-cy);		  glVertex2i(0, 11);// Texture / Vertex Coord (Top Left)
				glEnd();
				glTranslated(8,0,0);
			glEndList();
		}
	}
	else
		return false;

	return true;
}

// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
// ==================================================================================
// DISPLAY A STRING OF CHARACTERS ON THE SCREEN
// ==================================================================================
// $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
VOID GLFont::PrintScreen(GLfloat r, GLfloat g, GLfloat b, GLfloat a, 
											GLint x, GLint y, const char *fmt, ...)
{
	char	text[256];	

	if((m_fontList>0)&&(m_textureID>=0))
	{
		// POINTER TO LIST OF ARGUMENTS
		va_list	ap;	

		// MAKE SURE THE STRING IS NOT NULL
		if(fmt==NULL)return;			

		// PARSES THE STRING FOR VARIABLES AND CONVERTS SYMBOLS TO ACTUAL NUMBERS
		// RESULTS ARE STORED IN TEXT
		va_start(ap, fmt);					
		    vsprintf(text, fmt, ap);		
		va_end(ap);							

		// ENABLE TEXTURE MAPPING
		if(!glIsEnabled(GL_TEXTURE_2D))
			glEnable(GL_TEXTURE_2D);

		// SELECT TEXTURE
		glBindTexture(GL_TEXTURE_2D, _TextureManager()->Get_texture(m_textureID));

		// SET THE COLOR OF THE TEXT
		glColor4f(r, g, b, a);
		// ENABLE BLENDING
		glEnable(GL_BLEND);
		// SET THE BLENDING MODE
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);		
		//glBlendFunc(GL_ONE_MINUS_DST_COLOR, GL_ONE);

		// DRAW THE CHARACTERS
		glPushMatrix();
			glLoadIdentity();
			glTranslated(x,y,0);
			glListBase(m_fontList-32);			
			glCallLists(strlen(text), GL_UNSIGNED_BYTE, text);
		glPopMatrix();

		glDisable(GL_BLEND);
	}
}
