/*************************************************************************/
/*|=====================================================================|*/
/*| Filename: CColor.h                                                  |*/
/*| Author: Joel Villagrana   (joel.villagrana@gmail.com)               |*/
/*|                                                                     |*/
/*| Classes Defined:                                                    |*/
/*| CColour                                                             |*/
/*|                                                                     |*/
/*|%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%|*/
/*| dd/mm/yy   | Programmer        | Modifications                      |*/ 
/*|=====================================================================|*/
/*| 11/03/04   | JVillagrana       | Initial Version                    |*/
/*************************************************************************/

#ifndef _COLOURHELPER_H
#define _COLOURHELPER_H

#include <stdio.h>
#include <assert.h>
#include <gl/gl.h>

/* $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
================================================================================
	Definition of the CColour Class
================================================================================
$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ */
class CColour
{
public:

	// COLOUR COMPONENTS
	GLuint m_red, m_green, m_blue, m_alpha;
	GLfloat m_glRed, m_glGreen, m_glBlue, m_glAlpha;	

	// CONSTRUCTORS
	CColour();
	CColour( GLuint r, GLuint g, GLuint b );
	CColour( GLuint r, GLuint g, GLuint b, GLuint a );
	~CColour();

	VOID	operator =(CColour& c2);

	VOID	Set_ColourRGB( GLuint r, GLuint g, GLuint b );
	VOID	Set_ColourRGBA( GLuint r, GLuint g, GLuint b, GLuint a );

	VOID	GenerateRandomColour();
	VOID	ConvertToOGLFloat();
	VOID	UpdateAlpha(int a);	

	// GET THE COLOUR (INTEGERS)
	GLuint	Get_RedRGBA()	{	return m_red;			}
	GLuint  Get_GreenRGBA()	{	return m_green;			}
	GLuint  Get_BlueRGBA()	{	return m_blue;			}
	GLuint  Get_AlphaRGBA()	{	return m_alpha;			}

	// GET THE COLOUR (FLOATS)
	GLfloat	Get_RedGL()		{	return m_glRed;			}
	GLfloat Get_GreenGL()	{	return m_glGreen;		}
	GLfloat Get_BlueGL()	{	return m_glBlue;		}
	GLfloat Get_AlphaGL()	{	return m_glAlpha;		}
	
	VOID	Get_RGB(GLfloat *r, GLfloat *g, GLfloat *b)	{ *r=m_glRed; 
											*g=m_glGreen; *b=m_glBlue; }
	VOID	Get_RGBA(GLfloat *r, GLfloat *g, GLfloat *b, GLfloat *a){ *r=m_glRed; 
											*g=m_glGreen; *b=m_glBlue; *a=m_glAlpha; }

};

#endif _COLOURHELPER_H

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% END OF FILE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



