#ifndef __PRIM_H__
#define __PRIM_H__

#ifdef WIN32
	#define WIN32_LEAN_AND_MEAN
	#define WIN32_EXTRA_LEAN
	#include <windows.h>
#endif

#include "glee.h"


// Structure for holding a primitive model
struct Primitive
{
	GLenum nElemType;	// GL_TRIANGLES, etc
	int nVertices;		// total unique vertex count
	int nTriangles;		// poly count
	GLfloat *pVert;		// x,y,z
	GLubyte *pCol;		// r,g,b,a
	GLfloat *pNormal;	// x,y,z
	GLfloat *pTexCoord;	// u,v
	GLushort *pIdx;

	Primitive( GLenum nType, int nVerts, int nTris );
	~Primitive();

	// Sets up clientstate and renders the primitive
	void Draw( bool bWire = false, 
	           bool bNormalArray = true, 
	           bool bColorArray = true, 
	           bool bTexCoordArray = true );

};

// Primitive factory class
class CPrimitiveFactory
{
public:

	// Deletes a primitive object that was allocated by a factory
	static void DeletePrimitive( Primitive **ppPrim );

	// Generates a patch on the xz plane
	static Primitive* MakePatch( float fWidth, float fHeight, int nCols, int nRows );

	// Loads a primitive from a text file
	static Primitive* LoadPrimitive (const char* fname);

};


#endif //__PRIM_H__